package cn.fw.dalaran.common.utils;

import java.util.concurrent.*;
import java.util.concurrent.locks.LockSupport;
import java.util.logging.Logger;

/**
 * @author wmy3969
 * @version 1.0
 * @date 2022/8/24 16:06
 * @Description 带自动过期的本地缓存
 */
public class AutoExpireLocalCache {
    private final DelayQueue<Entity> delayQueue = new DelayQueue<>();// 延迟队列
    private final ConcurrentHashMap<String, String> cacheMap = new ConcurrentHashMap<>();// 缓存map
    public static final Logger logger = Logger.getLogger(AutoExpireLocalCache.class.getName());// 日志

    /**
     * AutoExpireLocalCache对象构造方法, 开启一个线程, 默认100ms一个周期, 监听过期key
     */
    public AutoExpireLocalCache() {
        this(100);
    }

    /**
     * AutoExpireLocalCache对象构造方法, 开启一个线程, 指定一个周期(不能小于10ms), 监听过期key
     *
     * @param scanCycle 监听过期key, 扫描周期
     */
    public AutoExpireLocalCache(long scanCycle) {
        long cycle = scanCycle <= 10 ? 10 : scanCycle;
        new Thread(() -> {
            while (true) {
                while (!delayQueue.isEmpty()) {
                    try {
                        Entity entity = delayQueue.take();
                        final String waitRemoveKey = entity.getKey();
                        logger.info("已移除key为: " + waitRemoveKey + ", 队列长度为: " + delayQueue.size() + ", 缓存map元素个数为: " + (cacheMap.size() - 1));
                        cacheMap.remove(waitRemoveKey);
                    } catch (InterruptedException e) {
                        logger.warning("error: " + e.getMessage());
                        Thread.currentThread().interrupt();
                    }
                }
                LockSupport.parkNanos(TimeUnit.MILLISECONDS.toNanos(cycle));
            }
        }).start();
    }

    /**
     * 缓存map放数据, 默认30s过期
     *
     * @param key   待缓存key
     * @param value 待缓存value
     */
    public void put(String key, String value) {
        this.removeKeyIfAbsent(key);
        delayQueue.put(new Entity(key));
        cacheMap.put(key, value);
    }

    /**
     * 缓存map放数据
     *
     * @param key        待缓存key
     * @param value      待缓存value
     * @param saveMillis 保存时间(ms)
     */
    public void put(String key, String value, long saveMillis) {
        this.removeKeyIfAbsent(key);
        delayQueue.put(new Entity(key, saveMillis));
        cacheMap.put(key, value);
    }

    /**
     * 根据key获取value
     *
     * @param key 缓存key
     * @return 缓存value
     */
    public String get(String key) {
        return cacheMap.get(key);
    }

    /**
     * 获取整个缓存map
     *
     * @return 整个缓存map
     */
    public ConcurrentMap<String, String> getCacheMap() {
        return cacheMap;
    }

    /**
     * 移除缓存key相关内容
     *
     * @param key 缓存key
     */
    public void removeKey(String key) {
        if (this.removeKeyIfAbsent(key))
            cacheMap.remove(key);
    }

    /**
     * 获取AutoExpireLocalCache对象中队列长度和map元素个数
     *
     * @return 队列长度和map元素个数
     */
    public String getSizeMsg() {
        return "队列长度为: " + delayQueue.size() + ", 缓存map元素个数为: " + cacheMap.size();
    }

    /**
     * 销毁AutoExpireLocalCache对象中队列和缓存map
     */
    public void clear() {
        delayQueue.clear();
        cacheMap.clear();
    }

    /**
     * 如果key存在于队列中, 则移除该实体(用作锁续命, 重新指定过期时间)
     *
     * @param key 待缓存的key
     */
    private boolean removeKeyIfAbsent(String key) {
        Entity entity = null;
        if (cacheMap.containsKey(key)) {
            for (Entity e : delayQueue) {
                if (e.getKey().equals(key)) {
                    entity = e;
                    break;
                }
            }
            if (entity != null)
                delayQueue.remove(entity);
            return true;
        }
        return false;
    }

    static class Entity implements Delayed {
        private final String key;// key
        private final long createMillis;// 创建时间戳
        private final long delayMillis;// 延迟时间(ms)
        private final long expireMillis;// 到期时间戳

        public String getKey() {
            return key;
        }

        /**
         * 不指定key延期时间(默认30s过期)
         *
         * @param key 缓存key
         */
        public Entity(String key) {
            this(key, 30000);
        }

        /**
         * 指定key延期时间
         *
         * @param key         缓存key
         * @param delayMillis 延期时间(ms)
         */
        public Entity(String key, long delayMillis) {
            this.key = key;
            this.createMillis = System.currentTimeMillis();
            this.delayMillis = delayMillis;
            this.expireMillis = this.createMillis + delayMillis;
        }

        public long getDelay(TimeUnit unit) {
            return unit.convert(this.expireMillis - System.currentTimeMillis(), TimeUnit.MILLISECONDS);
        }

        public int compareTo(Delayed delayed) {
            return (int) (this.getDelay(TimeUnit.MILLISECONDS) - delayed.getDelay(TimeUnit.MILLISECONDS));
        }

        @Override
        public String toString() {
            return "Entity{" +
                    "key='" + key + '\'' +
                    ", createMillis=" + createMillis +
                    ", delayMillis=" + delayMillis +
                    ", expireMillis=" + expireMillis +
                    '}';
        }
    }
}
