package cn.fw.dalaran.domain.db.config;

import cn.fw.dalaran.common.utils.PublicUtil;
import cn.fw.dalaran.domain.dto.common.ShopDto;
import cn.fw.dalaran.domain.enums.config.WorksTypeEnum;
import cn.fw.dalaran.domain.vo.ActivityThemeVo;
import cn.fw.dalaran.domain.vo.activity.TopicVo;
import cn.fw.dalaran.domain.vo.config.EvlVo;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableLogic;
import lombok.Data;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author wmy3969
 * @version 1.0
 * @date 2022/1/11 23:46
 * @Description 活动主题实体
 */
@Data
public class ActivityTheme {
    /**
     * 主键id
     */
    @TableId(type = IdType.AUTO)
    private Long id;
    /**
     * 所属配置组id
     */
    private Long configGroupId;
    /**
     * 集团id(冗余字段)
     */
    private Long groupId;
    /**
     * 活动主题
     */
    private String theme;
    /**
     * 活动话题
     */
    private String topic;
    /**
     * 主题开始时间
     */
    private Date startTime;
    /**
     * 主题结束时间
     */
    private Date endTime;
    /**
     * 全部门店(0:否, 1:是)(冗余字段)
     */
    private Integer allShop;
    /**
     * 门店范围
     */
    private String shopJson;
    /**
     * 数据有效性过滤配置
     */
    private String validConfigJson;
    /**
     * 指标评分配置
     */
    private String evlConfigJson;
    /**
     * 最低有效直播场次(主题得分前置条件)
     */
    private Integer liveValidTimes;
    /**
     * 最低有效短视频场次(主题得分前置条件)
     */
    private Integer videoValidTimes;
    /**
     * 粉丝数要求
     */
    private String fansLimit;
    /**
     * 是否完成初始化检查发送参与主题待办(0:否, 1:是)
     */
    private Integer initCheck;
    /**
     * 逻辑删除
     */
    @TableLogic(value = "0", delval = "1")
    private Integer logicDel;

    /**
     * 获取主题门店范围（报表使用）
     *
     * @param shopIds
     * @return
     */
    public List<Long> getShopIdByReport(List<Long> shopIds) {
        List<Long> themeShopRange = JSON.parseArray(this.getShopJson(), ShopDto.class)
                .stream().map(ShopDto::getShopId).collect(Collectors.toList());
        if (PublicUtil.isNotEmpty(shopIds))
            themeShopRange.retainAll(shopIds);
        return themeShopRange;
    }

    /**
     * db->vo
     */
    public static ActivityThemeVo toVO(ActivityTheme db) {
        final ActivityThemeVo vo = new ActivityThemeVo();
        vo.setId(db.getId());
        vo.setConfigGroupId(db.getConfigGroupId());
        vo.setGroupId(db.getGroupId());
        vo.setTheme(db.getTheme());
        vo.setTopic(JSON.parseArray(db.getTopic(), TopicVo.class).stream()
                .sorted(Comparator.comparing(TopicVo::getForcedCheck).reversed())
                .collect(Collectors.toList())
        );
        vo.setStartTime(db.getStartTime());
        vo.setEndTime(db.getEndTime());
        vo.setAllShop(db.getAllShop());
        if (StringUtils.hasText(db.getShopJson())) {
            List<ShopDto> shopsDto;
            try {
                shopsDto = JSON.parseArray(db.getShopJson(), ShopDto.class);
            } catch (Exception e) {
                shopsDto = new ArrayList<>();
            }
            vo.setShopIds(shopsDto.stream()
                    .map(ShopDto::getShopId)
                    .collect(Collectors.toList())
            );
            vo.setShopNames(shopsDto.stream()
                    .map(ShopDto::getShopName)
                    .collect(Collectors.toList())
            );
        }
        return vo;
    }

    /**
     * 综合得分占比
     */
    public Map<Integer, BigDecimal> getRate() {
        List<EvlVo> evlVoList = JSON.parseArray(this.evlConfigJson, EvlVo.class);
        if (CollectionUtils.isEmpty(evlVoList))
            return new HashMap<>();
        return evlVoList.stream()
                .collect(Collectors.toMap(EvlVo::getType, EvlVo::getRate));
    }

    /**
     * 直播综合得分占比
     */
    public BigDecimal getLiveRate() {
        return this.getRate().getOrDefault(WorksTypeEnum.LIVE.getValue(), BigDecimal.ZERO)
                .divide(BigDecimal.valueOf(100), 4, RoundingMode.HALF_DOWN);

    }

    /**
     * 短视频综合得分占比
     */
    public BigDecimal getVideoRate() {
        return this.getRate().getOrDefault(WorksTypeEnum.VIDEO.getValue(), BigDecimal.ZERO)
                .divide(BigDecimal.valueOf(100), 4, RoundingMode.HALF_DOWN);
    }

    /**
     * 获取主题包含门店id列表
     *
     * @return 主题包含门店id列表
     */
    public List<Long> getShopIdByReport() {
        return JSON.parseArray(this.getShopJson(), ShopDto.class)
                .stream()
                .map(ShopDto::getShopId)
                .collect(Collectors.toList())
                ;
    }
}
