package cn.fw.dalaran.domain.vo.report.score;

import cn.fw.dalaran.domain.enums.report.ReportListTypeEnum;
import cn.fw.dalaran.domain.vo.report.UserCalRuleVo;
import com.alibaba.fastjson.annotation.JSONField;
import com.fasterxml.jackson.annotation.JsonIgnore;
import lombok.Data;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author wmy3969
 * @version 1.0
 * @date 2023/11/20 11:27
 * @Description 作品目标达成详情
 */
@Data
public class WorksCompleteDetails {
    /**
     * 报表类型
     * {@link ReportListTypeEnum}
     */
    @JsonIgnore
    @JSONField(serialize = false)
    private Integer rowType;
    /**
     * 主题id
     */
    private Long themeId;
    /**
     * 主题名
     */
    private String themeName;
    /**
     * 主题需要最佳直播场次
     */
    @JsonIgnore
    @JSONField(serialize = false)
    private Integer themeNeedBestLiveCnt;
    /**
     * 主题需要最佳短视频条数
     */
    @JsonIgnore
    @JSONField(serialize = false)
    private Integer themeNeedBestVideoCnt;
    /**
     * 主题涉及各门店人员目标达成情况
     */
    private List<ShopCompleteDetails> shopDetails = new ArrayList<>();
    /**
     * 主题涉及所有人员目标达成情况
     */
    @JsonIgnore
    @JSONField(serialize = false)
    private List<UserCalRuleVo> allUserDetails = new ArrayList<>();
    /**
     * 目标及计分情况结果
     */
    @JsonIgnore
    @JSONField(serialize = false)
    private TargetResult targetResult;

    /**
     * 门店人员目标达成信息
     */
    @Data
    public static class ShopCompleteDetails {
        /**
         * 门店id
         */
        private Long shopId;
        /**
         * 门店名
         */
        private String shopName;
        /**
         * 门店下各个人员目标完成情况
         */
        private List<UserCalRuleVo> userDetails = new ArrayList<>();

        /**
         * 按人员分组
         *
         * @return K:userId, V:门店人员目标达成情况
         */
        @JsonIgnore
        @JSONField(serialize = false)
        public Map<Long, UserCalRuleVo> getUserDetailMap() {
            if (CollectionUtils.isEmpty(userDetails))
                return new HashMap<>();
            return userDetails.stream()
                    .collect(Collectors.toMap(UserCalRuleVo::getUserId, Function.identity()));
        }
    }

    /**
     * 目标及计分情况结果
     */
    @Data
    public static class TargetResult {
        /**
         * 需要最佳短视频条数
         */
        private Integer needBestVideoCnt = 0;
        /**
         * 需要最佳直播场次
         */
        private Integer needBestLiveCnt = 0;
        /**
         * 当前最佳短视频条数
         */
        private Integer currentBestVideoCnt = 0;
        /**
         * 需要最佳直播场次
         */
        private Integer currentBestLiveCnt = 0;
        /**
         * 最佳短视频占比
         */
        private BigDecimal bestVideoRate = BigDecimal.ZERO;
        /**
         * 最佳直播率占比
         */
        private BigDecimal bestLiveRate = BigDecimal.ZERO;
        /**
         * 销售顾问数量
         */
        private Integer consultantCnt = 0;
        /**
         * 是否计算综合得分
         */
        private Boolean calcCompositeScore = false;

        public BigDecimal getBestVideoRate() {
            if (Objects.equals(this.needBestVideoCnt, 0) || Objects.equals(this.currentBestVideoCnt, 0))
                return BigDecimal.ZERO;
            if (Objects.isNull(this.needBestVideoCnt) || Objects.isNull(this.currentBestVideoCnt))
                return BigDecimal.ZERO;
            return BigDecimal.valueOf(this.currentBestVideoCnt)
                    .divide(BigDecimal.valueOf(this.needBestVideoCnt), 4, RoundingMode.HALF_UP);
        }

        public BigDecimal getBestLiveRate() {
            if (Objects.equals(this.needBestLiveCnt, 0) || Objects.equals(this.currentBestLiveCnt, 0))
                return BigDecimal.ZERO;
            if (Objects.isNull(this.needBestLiveCnt) || Objects.isNull(this.currentBestLiveCnt))
                return BigDecimal.ZERO;
            return BigDecimal.valueOf(this.currentBestLiveCnt)
                    .divide(BigDecimal.valueOf(this.needBestLiveCnt), 4, RoundingMode.HALF_UP);
        }
    }

    /**
     * 按门店分组
     *
     * @return K:shopId, V:门店人员目标达成情况
     */
    @JsonIgnore
    @JSONField(serialize = false)
    public Map<Long, ShopCompleteDetails> getShopDetailMap() {
        if (CollectionUtils.isEmpty(shopDetails))
            return new HashMap<>();
        return shopDetails.stream()
                .collect(Collectors.toMap(ShopCompleteDetails::getShopId, Function.identity()));
    }

    /**
     * 根据门店id获取门店人员目标达成情况
     *
     * @param shopId 门店id
     * @return 门店人员目标达成情况
     */
    public ShopCompleteDetails getShopDetailByShopId(Long shopId) {
        return this.getShopDetailMap().getOrDefault(shopId, new ShopCompleteDetails());
    }

    /**
     * 根据门店id与人员id获取该门店该人员目标达成情况
     *
     * @param shopId 门店id
     * @param userId 人员id
     * @return 该门店该人员目标达成情况
     */
    public UserCalRuleVo getUserDetailByUserId(Long shopId, Long userId) {
        ShopCompleteDetails shop = this.getShopDetailByShopId(shopId);
        if (Objects.isNull(shop.getShopId()))
            return new UserCalRuleVo();
        return shop.getUserDetailMap().getOrDefault(userId, new UserCalRuleVo());
    }

    /**
     * 根据人员id获取该人员目标达成情况
     *
     * @param userId 用户id
     * @return 该人员目标达成情况
     */
    public UserCalRuleVo getUserDetailByUserId(Long userId) {
        for (UserCalRuleVo user : allUserDetails) {
            if (Objects.equals(userId, user.getUserId()))
                return user;
        }
        UserCalRuleVo vo = new UserCalRuleVo();
        vo.setNeedLiveValidTimes(this.themeNeedBestLiveCnt);
        vo.setNeedVideoValidTimes(this.themeNeedBestVideoCnt);
        return vo;
    }

    /**
     * 组装目标达成结果
     *
     * @return 目标达成结果
     */
    public TargetResult getTargetResult() {
        TargetResult result = new TargetResult();
        List<ShopCompleteDetails> shopDetails = this.getShopDetails();
        List<UserCalRuleVo> userDetails = new ArrayList<>();
        if (!CollectionUtils.isEmpty(shopDetails)) {
            shopDetails.forEach(item -> userDetails.addAll(item.getUserDetails()));
            result.setNeedBestLiveCnt(this.sumInt(userDetails, UserCalRuleVo::getNeedLiveValidTimes));
            result.setNeedBestVideoCnt(this.sumInt(userDetails, UserCalRuleVo::getNeedVideoValidTimes));
            result.setCurrentBestLiveCnt(this.sumInt(userDetails, UserCalRuleVo::getCurrentLiveValidTimes));
            result.setCurrentBestVideoCnt(this.sumInt(userDetails, UserCalRuleVo::getCurrentVideoValidTimes));
            result.setCalcCompositeScore(userDetails.stream()
                    .anyMatch(UserCalRuleVo::getCalcCompositeScore)
            );
            result.setConsultantCnt(userDetails.size());
        }
        return result;
    }

    /**
     * 累加int字段
     *
     * @param list  T对象列表
     * @param filed 待累加字段
     * @param <T>   泛型
     * @return 所有T对象对应字段总和
     */
    public <T> Integer sumInt(List<T> list, Function<T, Integer> filed) {
        return list.stream()
                .filter(Objects::nonNull)
                .map(filed)
                .filter(Objects::nonNull)
                .mapToInt(item -> item)
                .sum()
                ;
    }
}
