package cn.fw.dalaran.rpc.backlog;

import cn.fw.backlog.sdk.api.BacklogItemApi;
import cn.fw.backlog.sdk.api.param.*;
import cn.fw.backlog.sdk.api.result.FailBacklogItem;
import cn.fw.dalaran.common.domain.dto.ToDoDto;
import cn.fw.dalaran.common.utils.PublicUtil;
import cn.fw.data.base.domain.common.Message;
import com.alibaba.fastjson.JSON;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @author: lym
 * @date: 2023/4/20
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class TodoRpc {
    private final BacklogItemApi backlogItemApi;

    /**
     * 新增待办
     *
     * @param toDoDto
     */
    public void toDoAdd(ToDoDto toDoDto) {
        try {
            BacklogItemPlanReq planReq = new BacklogItemPlanReq();
            planReq.setUserId(toDoDto.getUserId());
            planReq.setShopId(toDoDto.getShopId());
            planReq.setRoleCode(toDoDto.getRoleCode());
            planReq.setItemCode(toDoDto.getTodoCode());
            planReq.setDataId(toDoDto.getDataId());
            planReq.setPlanTime(toDoDto.getPlanTime());
            planReq.setExpireTime(toDoDto.getExpireTime());
            planReq.setDynamicMap(toDoDto.getDynamicMap());
            planReq.setExtraData(toDoDto.getExtraData());
            log.info("新增待办 TodoRpc.toDoAdd 传入参数:{}", JSON.toJSONString(planReq));
            Message<Void> message = backlogItemApi.planBacklogItem(planReq);
            log.info("新增待办 TodoRpc.toDoAdd 返回结果：{}", JSON.toJSONString(message));
            PublicUtil.assertTrue(message.isSuccess(), message.getResult());
        } catch (Exception e) {
            log.error("TodoRpc.toDoAdd 新增待办异常:" + e);
            throw e;
        }
    }

    /**
     * 新增待办
     *
     * @param toDoDtoList
     */
    public List<FailBacklogItem> batchToDoAdd(String itemCode, List<ToDoDto> toDoDtoList) {
        try {
            BacklogBatchPlanReq backlogBatchPlanReq = new BacklogBatchPlanReq();
            backlogBatchPlanReq.setItemCode(itemCode);
            List<BacklogBatchPlanItemReq> reqList = toDoDtoList.stream().map(toDoDto -> {
                BacklogBatchPlanItemReq planReq = new BacklogBatchPlanItemReq();
                planReq.setUserId(toDoDto.getUserId());
                planReq.setShopId(toDoDto.getShopId());
                planReq.setRoleCode(toDoDto.getRoleCode());
                planReq.setDataId(toDoDto.getDataId());
                planReq.setPlanTime(toDoDto.getPlanTime());
                planReq.setExpireTime(toDoDto.getExpireTime());
                planReq.setDynamicMap(toDoDto.getDynamicMap());
                planReq.setExtraData(toDoDto.getExtraData());
                return planReq;
            }).collect(Collectors.toList());
            backlogBatchPlanReq.setItemList(reqList);
            log.info("新增待办 TodoRpc.batchToDoAdd 传入参数:{}", JSON.toJSONString(reqList));
            Message<List<FailBacklogItem>> message = backlogItemApi.batchPlanBacklogItem(backlogBatchPlanReq);
            log.info("新增待办 TodoRpc.batchToDoAdd 返回结果：{}", JSON.toJSONString(message));
            PublicUtil.assertTrue(message.isSuccess(), message.getResult());
            return message.getData();
        } catch (Exception e) {
            log.error("TodoRpc.batchToDoAdd 新增待办异常:" + e);
            throw e;
        }
    }

    /**
     * 完成待办
     *
     * @param toDoDto
     */
    public void toDoComplete(ToDoDto toDoDto) {
        try {
            BacklogItemCommitReq commitReq = new BacklogItemCommitReq();
            commitReq.setUserId(toDoDto.getUserId());
            commitReq.setItemCode(toDoDto.getTodoCode());
            commitReq.setDataId(toDoDto.getDataId());
            commitReq.setCommitTime(toDoDto.getCommitTime());
            log.info("完成待办 TodoRpc.toDoComplete 传入参数:{}", JSON.toJSONString(commitReq));
            Message<Void> message = backlogItemApi.commitBacklogItem(commitReq);
            log.info("完成待办 TodoRpc.toDoComplete 返回结果：{}", JSON.toJSONString(message));
            PublicUtil.assertTrue(message.isSuccess(), message.getResult());
        } catch (Exception e) {
            log.error("TodoRpc.toDoComplete 完成待办异常:" + e);
            throw e;
        }
    }

    /**
     * 批量完成待办
     *
     * @param toDoDtoList
     */
    public List<FailBacklogItem> batchToDoComplete(String itemCode, List<ToDoDto> toDoDtoList) {
        try {
            BacklogBatchCommitReq backlogBatchCommitReq = new BacklogBatchCommitReq();
            backlogBatchCommitReq.setItemCode(itemCode);
            List<BacklogBatchCommitItemReq> commitItemReqs = toDoDtoList.stream().map(r -> {
                BacklogBatchCommitItemReq commitItemReq = new BacklogBatchCommitItemReq();
                commitItemReq.setUserId(r.getUserId());
                commitItemReq.setDataId(r.getDataId());
                commitItemReq.setCommitTime(r.getCommitTime());
                return commitItemReq;
            }).collect(Collectors.toList());
            backlogBatchCommitReq.setItemList(commitItemReqs);
            log.info("完成待办 TodoRpc.batchToDoComplete 传入参数:{}", JSON.toJSONString(backlogBatchCommitReq));
            Message<List<FailBacklogItem>> message = backlogItemApi.batchCommitBacklogItem(backlogBatchCommitReq);
            log.info("完成待办 TodoRpc.batchToDoComplete 返回结果：{}", JSON.toJSONString(message));
            PublicUtil.assertTrue(message.isSuccess(), message.getResult());
            return message.getData();
        } catch (Exception e) {
            log.error("TodoRpc.batchToDoComplete 完成待办异常:" + e);
            throw e;
        }
    }

    /**
     * 取消待办
     */
    public void toDoCancel(ToDoDto toDoDto) {
        try {
            BacklogItemCancelReq cancelReq = new BacklogItemCancelReq();
            cancelReq.setUserId(toDoDto.getUserId());
            cancelReq.setItemCode(toDoDto.getTodoCode());
            cancelReq.setDataId(toDoDto.getDataId());
            log.info("取消待办 TodoRpc.toDoCancel 传入参数:{}", JSON.toJSONString(cancelReq));
            Message<Void> message = backlogItemApi.cancelBacklogItem(cancelReq);
            log.info("取消待办 TodoRpc.toDoCancel 返回结果：{}", JSON.toJSONString(message));
            PublicUtil.assertTrue(message.isSuccess(), message.getResult());
        } catch (Exception e) {
            log.error("TodoRpc.toDoCancel 取消待办异常:" + e);
            throw e;
        }
    }

    /**
     * 批量取消待办
     *
     * @param itemCode
     * @param toDoDtoList
     */
    public List<FailBacklogItem> batchToDoCancel(String itemCode, List<ToDoDto> toDoDtoList) {
        try {
            BacklogBatchCancelReq backlogBatchCancelReq = new BacklogBatchCancelReq();
            backlogBatchCancelReq.setItemCode(itemCode);
            List<BacklogBatchCancelItemReq> cancelItemReqList = toDoDtoList.stream().map(r -> {
                BacklogBatchCancelItemReq cancelItemReq = new BacklogBatchCancelItemReq();
                cancelItemReq.setUserId(r.getUserId());
                cancelItemReq.setDataId(r.getDataId());
                return cancelItemReq;
            }).collect(Collectors.toList());
            backlogBatchCancelReq.setItemList(cancelItemReqList);
            log.info("取消待办 TodoRpc.toDoCancel 传入参数:{}", JSON.toJSONString(backlogBatchCancelReq));
            Message<List<FailBacklogItem>> message = backlogItemApi.batchCancelBacklogItem(backlogBatchCancelReq);
            log.info("取消待办 TodoRpc.toDoCancel 返回结果：{}", JSON.toJSONString(message));
            PublicUtil.assertTrue(message.isSuccess(), message.getResult());
            return message.getData();
        } catch (Exception e) {
            log.error("TodoRpc.toDoCancel 取消待办异常:" + e);
            throw e;
        }
    }
}
