package cn.fw.dalaran.rpc.ehr;

import cn.fw.common.exception.BusinessException;
import cn.fw.dalaran.common.utils.PublicUtil;
import cn.fw.dalaran.rpc.dto.StaffBaseInfoRpc;
import cn.fw.dalaran.rpc.dto.StaffInfoRpc;
import cn.fw.dalaran.rpc.dto.StaffShopSelectRpc;
import cn.fw.data.base.db.query.Page;
import cn.fw.data.base.domain.common.Message;
import cn.fw.ehr.sdk.api.PostApi;
import cn.fw.ehr.sdk.api.StaffApi;
import cn.fw.ehr.sdk.api.dto.ManagerScopeCheckDto;
import cn.fw.ehr.sdk.api.result.PostInfo;
import cn.fw.ehr.sdk.api.result.StaffBaseInfo;
import cn.fw.ehr.sdk.api.result.StaffInfo;
import cn.fw.ehr.sdk.api.result.StaffShopSelectVo;
import cn.hutool.json.JSONObject;
import com.alibaba.fastjson.JSON;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.*;

/**
 * @author wmy3969
 * @version 1.0
 * @date 2021/9/17 10:01
 * @Description 远程调用人事系统
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class EhrRpc {

    private final PostApi postApi;
    private final StaffApi staffApi;

    /**
     * @param staffIds    需要收到公示的人员的id集合
     * @param publisherId 发布公示的人员的id
     * @return 每个员工的是否在该发布者的管理范围的封装信息
     */
    public List<StaffShopSelectRpc> checkBeyondManage(Set<Long> staffIds, Long publisherId) {
        try {
            ManagerScopeCheckDto managerScopeCheckDto = new ManagerScopeCheckDto();// 准备远程调用人事系统, 封装请求需要的参数
            managerScopeCheckDto.setStaffIdList(new ArrayList<>(staffIds));// 设置staffIdList参数
            managerScopeCheckDto.setStaffId(publisherId);// 设置发布公示的职工的id
            Message<List<StaffShopSelectVo>> StaffShopSelectList = staffApi.checkManagerScope(managerScopeCheckDto);
            PublicUtil.assertTrue(StaffShopSelectList.isSuccess(), StaffShopSelectList.getResult());
            log.info("远程调用人事服务 staffApi.checkManagerScope 成功, 传入参数为 managerScopeCheckDto: {} 结果为: {}", managerScopeCheckDto, JSON.toJSONString(StaffShopSelectList));
            return PublicUtil.copyList(StaffShopSelectList.getData(), StaffShopSelectRpc.class);
        } catch (Exception e) {
            log.error("远程调用人事服务 staffApi.checkManagerScope 发生异常, 传入参数为 staffIds: {}, publisherId: {} 失败!", staffIds, publisherId);
        }
        return null;
    }

    /**
     * 获取集团员工列表
     *
     * @param groupId 集团id
     */
    public List<StaffBaseInfo> getGroupStaffList(Long groupId) {
        String func = "调用人事系统获取集团员工列表";
        if (Objects.isNull(groupId)) {
            throw new IllegalArgumentException();
        }

        try {
            Message<Page<StaffBaseInfo>> msg = staffApi.getGroupStaffList(groupId, 1, 10000, null);
            log.info("{}，groupId：{}，result：{}", func, groupId, JSON.toJSONString(msg));
            if (msg.isSuccess()) {
                return msg.getData().getData();
            }
        } catch (Exception e) {
            log.error("{}失败，groupId：{}", func, groupId, e);
        }
        throw new BusinessException(func + "失败");
    }

    /**
     * 获取员工基础信息
     *
     * @param staffId 员工id
     * @return 人脸图片
     */
    public StaffBaseInfo getStaffBaseInfo(Long staffId) {
        try {
            Message<StaffBaseInfo> staffBaseInfo = staffApi.getStaffBaseInfo(staffId);
            log.info("调用人事系统获取员工基础信息成功, result:{}", JSON.toJSONString(staffBaseInfo));
            if (staffBaseInfo.isSuccess()) {
                return staffBaseInfo.getData();
            }
            return null;
        } catch (Exception e) {
            log.error("调用人事系统获取员工基础信息失败, 入参staffId:{}", staffId, e);
            return null;
        }
    }

    /**
     * 查询指定岗位ID的岗位信息
     *
     * @param postId 岗位ID
     * @return 岗位信息
     */
    public PostInfo getPostInfo(final Long postId) {
        if (Objects.isNull(postId)) {
            return null;
        }
        try {
            final Message<List<PostInfo>> message = postApi.getPostList(Collections.singletonList(postId));
            log.info("调用人事系统查询指定岗位ID的岗位信息：result[{}],code[{}],data[{}]", message.getResult()
                    , message.getCode(), message.getData());
            if (message.isSuccess() && Objects.nonNull(message.getData())) {
                return message.getData().get(0);
            }
            if (!message.isSuccess()) {
                throw new BusinessException(message.getResult());
            }
            return null;
        } catch (Exception e) {
            log.error("调用人事系统查询指定岗位ID的岗位信息失败", e);
        }
        return null;
    }

    /**
     * 根据职工id集合批量获取职工基础信息
     *
     * @param staffIds 职工的id的集合
     * @return 职工的基础信息集合
     */
    public List<StaffBaseInfoRpc> getStaffInfoBatch(List<Long> staffIds) {
        try {
            Message<List<StaffBaseInfo>> batchStaffBaseInfoList = staffApi.getBatchStaffBaseInfoList(staffIds);
            log.info("远程调用 staffApi.getBatchStaffBaseInfoList 成功, 传入参数为 staffIds: {} 结果为: {}", staffIds, JSON.toJSONString(batchStaffBaseInfoList));
            PublicUtil.assertTrue(batchStaffBaseInfoList.isSuccess(), batchStaffBaseInfoList.getResult());
            return PublicUtil.copyList(batchStaffBaseInfoList.getData(), StaffBaseInfoRpc.class);
        } catch (Exception e) {
            log.error("调用人事系统 staffApi.getBatchStaffBaseInfoList 发生异常, 传入参数为 staffId: {} 异常!", staffIds);
        }
        return null;
    }

    /**
     * 根据职工id获取职工信息
     *
     * @param staffId 职工的id
     * @return 该职工其他信息
     */
    public StaffInfoRpc getStaffOtherInfo(Long staffId) {
        try {
            Message<StaffInfo> staffInfo = staffApi.getStaffInfoById(staffId);
            log.info("远程调用 staffApi.getStaffInfoById 成功, 传入参数为 staffId: {} 结果为: {}", staffId, JSON.toJSONString(staffInfo));
            PublicUtil.assertTrue(staffInfo.isSuccess(), staffInfo.getResult());
            return PublicUtil.copy(staffInfo.getData(), StaffInfoRpc.class);
        } catch (Exception e) {
            log.error("调用人事系统 staffApi.getStaffInfoById 发生异常, 传入参数为 staffId: {} 异常!", staffId);
        }
        return null;
    }

    /**
     * 根据 门店id 获取职工信息
     *
     * @param shopId 门店id
     * @return 该门店下所有员工信息
     */
    public List<StaffBaseInfoRpc> getShopStaffList(Long shopId) {
        try {
            Message<List<StaffBaseInfo>> postShopStaffList = staffApi.getPostShopStaffList(null, shopId);
            log.info("远程调用 staffApi.getPostShopStaffList 成功, 传入参数为 shopId: {} 结果为: {}", shopId, JSON.toJSONString(postShopStaffList));
            PublicUtil.assertTrue(postShopStaffList.isSuccess(), postShopStaffList.getResult());
            return PublicUtil.copyList(postShopStaffList.getData(), StaffBaseInfoRpc.class);
        } catch (Exception e) {
            log.error("调用人事系统 staffApi.getPostShopStaffList 发生异常, 传入参数为 shopId: {} 异常!", shopId);
        }
        return null;
    }

    /**
     * 根据 岗位id 获取职工信息
     *
     * @param postId 门店id
     * @return 该岗位的所有员工信息
     */
    public List<StaffBaseInfoRpc> getPostStaffList(Long postId) {
        try {
            Message<List<StaffBaseInfo>> postShopStaffList = staffApi.getPostShopStaffList(postId, null);
            log.info("远程调用 staffApi.getPostShopStaffList 成功, 传入参数为 postId: {} 结果为: {}", postId, JSON.toJSONString(postShopStaffList));
            PublicUtil.assertTrue(postShopStaffList.isSuccess(), postShopStaffList.getResult());
            return PublicUtil.copyList(postShopStaffList.getData(), StaffBaseInfoRpc.class);
        } catch (Exception e) {
            log.error("调用人事系统 staffApi.getPostShopStaffList 发生异常, 传入参数为 postId: {} 异常!", postId);
        }
        return null;
    }

    /**
     * 根据 岗位id+门店id 获取职工信息
     *
     * @param postId 岗位id
     * @param shopId 门店id
     * @return 该 岗位+门店 的所有员工信息
     */
    public List<StaffBaseInfoRpc> getPostShopStaffList(Long postId, Long shopId) {
        try {
            Message<List<StaffBaseInfo>> postShopStaffList = staffApi.getPostShopStaffList(postId, shopId);
            log.info("远程调用 staffApi.getPostShopStaffList 成功, 传入参数为 postId: {} shopId: {} 结果为: {}", postId, shopId, JSON.toJSONString(postShopStaffList));
            PublicUtil.assertTrue(postShopStaffList.isSuccess(), postShopStaffList.getResult());
            return PublicUtil.copyList(postShopStaffList.getData(), StaffBaseInfoRpc.class);
        } catch (Exception e) {
            log.error("调用人事系统 staffApi.getPostShopStaffList 发生异常, 传入参数为 postId: {} shopId: {} 异常!", postId, shopId);
        }
        return null;
    }

    /**
     * 批量获取人员信息
     *
     * @param userIds 人员id列表
     */
    public List<StaffBaseInfo> getBatchStaffShopList(List<Long> userIds) {
        if (CollectionUtils.isEmpty(userIds)) {
            return new ArrayList<>();
        }
        final String msg = "调用EHR系统[批量获取人员门店信息]参数：getBatchUsersByIds";
        Message<List<StaffBaseInfo>> staffMessage = staffApi.getBatchStaffBaseInfoList(userIds);
        log.info("{}, 返回信息:{}", msg, new JSONObject(staffMessage).toString());
        if (staffMessage.isSuccess()) {
            return staffMessage.getData();
        } else {
            throw new RuntimeException(staffMessage.getResult());
        }
    }

}
