package cn.fw.dalaran.rpc.oop;

import cn.fw.common.exception.BusinessException;
import cn.fw.dalaran.common.utils.PublicUtil;
import cn.fw.dalaran.rpc.oop.dto.CarBrandDTO;
import cn.fw.dalaran.rpc.oop.dto.GroupDTO;
import cn.fw.dalaran.rpc.oop.dto.ShopDTO;
import cn.fw.data.base.domain.common.Message;
import cn.fw.oop.sdk.CarBrandApiService;
import cn.fw.oop.sdk.GroupApiService;
import cn.fw.oop.sdk.ShopApiService;
import cn.fw.oop.sdk.enums.BizTypeEnum;
import cn.fw.oop.sdk.result.BrandSaleShopVo;
import cn.fw.oop.sdk.result.CarBrandInfoVo;
import cn.fw.oop.sdk.result.GroupInfoVo;
import cn.fw.oop.sdk.result.ShopInfoVo;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * OOP RPC
 * <p>
 * create at 2019-07-24
 *
 * @author zhangjj
 */
@Slf4j
@Component
@SuppressWarnings("unchecked")
public class OopRpc {
    /**
     * 门店api
     */
    private final ShopApiService shopApiService;
    /**
     * 集团api
     */
    private final GroupApiService groupApiService;
    /**
     * 品牌api
     */
    private final CarBrandApiService brandApiService;

    /**
     * constructor for container
     */
    public OopRpc(final ShopApiService shopApiService,
                  final GroupApiService groupApiService,
                  final CarBrandApiService brandApiService) {
        this.shopApiService = shopApiService;
        this.groupApiService = groupApiService;
        this.brandApiService = brandApiService;
    }

    /**
     * 查询所有集团
     *
     * @return 门店列表
     */
    public List<GroupDTO> getAllGroups() {
        Message<List<GroupInfoVo>> result = groupApiService.getGroupInfoList();
        log.info("调用groupApiService.getGroupInfoList()原始数据: {}", JSON.toJSONString(result));
        PublicUtil.assertTrue(result.isSuccess(), result.getResult());
        return result.getData().stream()
                .map(GroupDTO::with)
                .collect(Collectors.toList());
    }

    /**
     * 查询集团在售品牌
     *
     * @return 品牌列表
     */
    public List<CarBrandDTO> getGroupBrands(final Long groupId) {
        Message<List<CarBrandInfoVo>> result = brandApiService.getBrandList(groupId);
        log.info("调用brandApiService.getBrandList()原始数据: {}", JSON.toJSONString(result));
        PublicUtil.assertTrue(result.isSuccess(), result.getResult());
        return result.getData().stream()
                .map(CarBrandDTO::with)
                .collect(Collectors.toList());
    }

    /**
     * 查询集团下所有门店
     *
     * @param groupId 集团id
     * @return 门店列表
     */
    public List<ShopDTO> getGroupShops(final Long groupId, Integer bizType) {
        Message<List<ShopInfoVo>> result = shopApiService.getShopInfoList(groupId, null, bizType);
        log.info("调用shopApiService.getShopInfoList()原始数据: {}", JSON.toJSONString(result));
        PublicUtil.assertTrue(result.isSuccess(), result.getResult());
        return result.getData().stream()
                .map(ShopDTO::with)
                .collect(Collectors.toList());
    }

    /**
     * 查询集团售前门店信息
     *
     * @param groupId 集团id
     * @param shopIds 门店id列表
     * @return 门店列表
     */
    public List<ShopDTO> getPreSaleShopList(final Long groupId, final Set<Long> shopIds) {
        if (CollectionUtils.isEmpty(shopIds)) {
            return Collections.emptyList();
        }
        final String msg = "调用OOP系统[查询集团门店信息]";
        try {
            log.info("{}, shopIds:{}", msg, shopIds);
            final Message<List<ShopInfoVo>> result = shopApiService.getShopInfoList(groupId, null, BizTypeEnum.PRE_SALE.getValue());
            if (result.isSuccess()) {
                log.info("{}成功, shopIds:{}, result:{}", msg, shopIds, result.getData());
                return Objects.isNull(result.getData())
                        ? Collections.emptyList()
                        : result.getData().stream()
                        .filter(v -> shopIds.contains(v.getId())).map(ShopDTO::with).collect(Collectors.toList());
            } else {
                throw new BusinessException(result.getResult());
            }
        } catch (Exception e) {
            log.warn("{}失败, shopIds:{}", msg, shopIds, e);
        }
        return Collections.emptyList();
    }

    /**
     * 批量获取门店信息
     *
     * @param shopIds 门店id列表
     * @return 门店列表
     */
    public List<ShopInfoVo> getShopByIds(List<Long> shopIds) {
        final String msg = "调用OOP系统[查询集团门店信息]";
        try {
            log.info("{}, shopIds:{}", msg, shopIds);
            if (CollectionUtils.isEmpty(shopIds)) {
                return Collections.emptyList();
            }
            final Message<List<ShopInfoVo>> result = shopApiService.getShopInfoList(shopIds);
            PublicUtil.assertTrue(result.isSuccess(), "调用OOP系统[查询集团门店信息]失败");
            log.info("{}成功, shopIds:{}, result:{}", msg, shopIds, result.getData());
            return result.getData();
        } catch (Exception e) {
            log.warn("{}失败, shopIds:{}", msg, shopIds, e);
        }
        return Collections.emptyList();
    }

    /**
     * 获取某个集团下所有门店
     */
    public List<ShopInfoVo> getAllShopsByGroup(Long groupId) {
        log.info("获取某个集团下所有门店 OopRpc.getAllShopsByGroup groupId：{}", groupId);
        Message<List<ShopInfoVo>> message = shopApiService.getExistShopInfoList(groupId, null, null);
        log.info("获取某个集团下所有门店 OopRpc.getAllShopsByGroup  返回数据 门店总数：{}", message.getData().size());
        if (!message.isSuccess())
            throw new BusinessException("获取某个集团下所有门店失败");
        return message.getData();
    }

    /**
     * 获取集团下品牌映射门店信息
     *
     * @param groupId  集团id
     * @param brandIds 带查询品牌
     * @return 品牌映射门店信息
     */
    public List<BrandSaleShopVo> getBrandShopBindInfo(Long groupId, List<Long> brandIds) {
        log.info("获取集团下品牌映射门店信息 shopApiService.getBrandShopList() 入参: brandIds; {} groupId：{}",
                JSON.toJSONString(brandIds), groupId);
        Message<List<BrandSaleShopVo>> result = shopApiService.getBrandShopList(brandIds, groupId);
        PublicUtil.assertTrue(result.isSuccess(), result.getResult());
        return result.getData();
    }
}
