package cn.fw.dalaran.server.aspect;

import cn.fw.dalaran.common.annotation.IgnoreLog;
import cn.fw.dalaran.common.domain.pojo.LogInfo;
import cn.fw.dalaran.common.exception.IgnoreLevel;
import cn.fw.dalaran.common.utils.PublicUtil;
import cn.fw.security.auth.client.enums.CookieEnum;
import cn.fw.security.auth.client.interceptor.LoginContext;
import cn.fw.security.auth.client.interceptor.PassportContext;
import cn.fw.security.auth.client.util.CookieUtils;
import cn.fw.security.auth.client.util.StringUtils;
import com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.HttpMethod;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.lang.reflect.Method;
import java.util.Arrays;
import java.util.Map;
import java.util.Objects;
import java.util.UUID;

/**
 * 统一打印日志
 *
 * @author Devin
 * @date: 2018/12/4 20:10
 */
@Slf4j
@Aspect
@Configuration
public class HttpLogAspect {

    /**
     * 定义切点Pointcut
     */
    @Pointcut("execution(* cn.fw.dalaran.server..*.*(..))" +
            "&&(@annotation(org.springframework.web.bind.annotation.RequestMapping)" +
            "|| @annotation(org.springframework.web.bind.annotation.PostMapping)" +
            "||@annotation(org.springframework.web.bind.annotation.GetMapping))")
    public void controllerAspect() {
    }

    /**
     * controller 层出入参日志记录
     *
     * @param joinPoint 切点
     * @return
     */
    @Around(value = "controllerAspect()")
    public Object controllerLogAround(ProceedingJoinPoint joinPoint) throws Throwable {
        long startTime = System.currentTimeMillis();
        // 获取 request 中包含的请求参数
        String requestId = UUID.randomUUID().toString().replace("-", "");
        HttpServletRequest request = ((ServletRequestAttributes) Objects.requireNonNull(RequestContextHolder.getRequestAttributes())).getRequest();
        // 获取切点请求参数(class,method)
        MethodSignature signature = (MethodSignature) joinPoint.getSignature();
        Method method = signature.getMethod();
        IgnoreLog ignoreLog = method.getAnnotation(IgnoreLog.class);
        IgnoreLevel level = IgnoreLevel.NONE;
        if (PublicUtil.isNotEmpty(ignoreLog)) {
            level = ignoreLog.level();
        }
        String params = getRequestParams(request, joinPoint);
        String authToken = getAuthToken(request);
        String userInfo = getRequestUserInfo();
        if (!(IgnoreLevel.ALL.equals(level) || IgnoreLevel.REQUEST.equals(level))) {
            // 入参日志
            log.info("服务器收到请求 \n\trequestId: {}\n\tauthToken: {}\n\trequestUser: {}\n\tcontentType:{}" +
                            "\n\trequestUrl: {}\n\trequestMethod: {}\n\trequestParams: {}\n\ttargetClassAndMethod: {}#{}",
                    requestId,
                    authToken,
                    userInfo,
                    request.getHeader("Content-Type"),
                    request.getRequestURI(),
                    request.getMethod(),
                    params,
                    method.getDeclaringClass().getName(),
                    method.getName());
        }
        Object result = joinPoint.proceed();
        // 出参日志
        if (!(IgnoreLevel.ALL.equals(level) || IgnoreLevel.RETURN.equals(level))) {
            log.info("服务器已处理完请求 \n\trequestId: {} \n\tcostTime：{} ms \n\t返回数据: {}", requestId,
                    System.currentTimeMillis() - startTime, JSON.toJSONString(result));
        } else {
            log.info("服务器已处理完请求 \n\trequestId: {} \n\tcostTime：{} ms 「已忽略返回JSON」", requestId,
                    System.currentTimeMillis() - startTime);
        }
        return result;
    }

    /**
     * 获取请求参数
     *
     * @param request
     * @param joinPoint
     * @return
     */
    private String getRequestParams(HttpServletRequest request, ProceedingJoinPoint joinPoint) {
        StringBuilder params = new StringBuilder();
        // 获取 request parameter 中的参数
        Map<String, String[]> parameterMap = request.getParameterMap();
        if (parameterMap != null && !parameterMap.isEmpty()) {
            for (Map.Entry<String, String[]> entry : parameterMap.entrySet()) {
                params.append(entry.getKey()).append(" = ").append(Arrays.toString(entry.getValue())).append(";");
            }
        }
        if (HttpMethod.POST.name().equalsIgnoreCase(request.getMethod()) ||
                HttpMethod.PUT.name().equalsIgnoreCase(request.getMethod())) {
            // 获取非 request parameter 中的参数
            Object[] objects = joinPoint.getArgs();
            for (Object arg : objects) {
                if (arg == null) {
                    break;
                }
                String className = arg.getClass().getName().toLowerCase();
                String contentType = request.getHeader("Content-Type");
                // 文件参数,上传文件信息
                if (className.contains("MultipartFile".toLowerCase())) {
                    MultipartFile multipartFile = (MultipartFile) arg;
                    params.append("fileSize = ").append(multipartFile.getSize()).append(";");
                    params.append("fileContentType = ").append(multipartFile.getContentType()).append(";");
                    params.append("fieldName = ").append(multipartFile.getName()).append(";");
                    params.append("fileOriginalName = ").append(multipartFile.getOriginalFilename()).append(";");
                }
                if (contentType != null && contentType.contains("application/json")) {
                    try {
                        // json 参数
                        params.append(JSON.toJSONString(arg));
                    } catch (Exception e) {
                        log.info("无法序列化请求入参类型：{}", arg.getClass().getName());
                    }
                }
            }
        }
        return params.toString();
    }

    /**
     * 获取请求用户信息
     *
     * @return
     */
    public String getRequestUserInfo() {
        StringBuilder requestUserInfo = new StringBuilder();
        if (PublicUtil.isNotEmpty(LoginContext.getLoginContext())) {
            LoginContext context = LoginContext.getLoginContext();
            requestUserInfo.append("B端用户 ").append(LogInfo.builder()
                    .userId(context.getUserId())
                    .userName(context.getUserName())
                    .groupId(context.getGroupId())
                    .build().toString());
        } else if (PublicUtil.isNotEmpty(PassportContext.getPassPortContext())) {
            requestUserInfo.append("C端用户 userId=").append(PassportContext.getPassPortContext().getUserId());
        } else {
            requestUserInfo.append("未知");
        }
        return requestUserInfo.toString();
    }

    /**
     * 获取授权token
     *
     * @param request
     * @return
     */
    private String getAuthToken(HttpServletRequest request) {
        String loginValue = request.getHeader("Authorization");
        if (StringUtils.isEmpty(loginValue)) {
            loginValue = request.getHeader("authorization");
        }
        if (StringUtils.isEmpty(loginValue)) {
            loginValue = request.getParameter("token");
        }
        if (StringUtils.isEmpty(loginValue)) {
            loginValue = CookieUtils.getCookieValue(request, CookieEnum.fwTkValue.getKey());
        }
        return loginValue;
    }
}
