package cn.fw.dalaran.server.controller.app.report;

import cn.fw.common.web.annotation.ControllerMethod;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.common.web.auth.annotation.CurrentUser;
import cn.fw.dalaran.common.page.PageHelper;
import cn.fw.dalaran.domain.param.report.base.ReportBaseQueryParam;
import cn.fw.dalaran.domain.param.report.live.*;
import cn.fw.dalaran.domain.vo.LivePoolVo;
import cn.fw.dalaran.domain.vo.report.RateAnalysisListVo;
import cn.fw.dalaran.domain.vo.report.live.ListVo;
import cn.fw.dalaran.domain.vo.report.live.SummaryVo;
import cn.fw.dalaran.domain.vo.report.live.TrendVo;
import cn.fw.dalaran.service.biz.report.live.LiveReportBizService;
import cn.fw.data.base.domain.common.Message;
import cn.fw.security.auth.client.annotation.Authorization;
import cn.fw.security.auth.client.annotation.IgnoreAuth;
import cn.fw.security.auth.client.enums.AuthType;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.math.BigDecimal;
import java.util.List;

import static cn.fw.common.web.util.ResultBuilder.success;

/**
 * 直播报表控制器
 *
 * @author wmy3969
 * @version 1.0
 * @date 2023/5/9 10:22
 * @Description 直播报表控制器
 */
@Slf4j
@Validated
@IgnoreAuth
@RestController
@RequiredArgsConstructor
@Authorization(AuthType.APP)
@RequestMapping("/report/live")
public class LiveReportController {
    private final LiveReportBizService liveReportBizService;

    /**
     * 获取直播汇总报表
     *
     * @param param 查询参数
     * @return 直播汇总报表
     */
    @GetMapping("/getSummary")
    @ControllerMethod("获取直播考核汇总统计")
    public Message<SummaryVo> getSummary(@Valid ReportBaseQueryParam param) {
        return success(liveReportBizService.getSummary(param));
    }

    /**
     * 获取直播趋势报表
     *
     * @param param 查询参数
     * @return 直播趋势报表
     */
    @GetMapping("/getTrend")
    @ControllerMethod("获取直播趋势")
    public Message<TrendVo<BigDecimal>> getTrend(@Valid TrendQueryParam param) {
        return success(liveReportBizService.getTrend(param));
    }

    /**
     * 获取直播列表报表
     *
     * @param user            当前操作用户
     * @param param           查询参数
     * @param secondDimension 二次分析维度(可不传)
     * @return 直播列表报表
     * @ignoreParams user
     */
    @GetMapping("/getList")
    @ControllerMethod("获取直播列表")
    //@IgnoreLog(level = IgnoreLevel.RETURN)
    public Message<List<ListVo>> getList(@CurrentUser LoginAuthBean user, @Valid ListQueryParam param,
                                         Integer secondDimension) {
        param.setGroupId(user.getGroupId());
        return success(liveReportBizService.getListHaveSecondAnalysis(param, secondDimension));
    }

    /**
     * 列表数据分析(门店/销顾/平台...分析)
     *
     * @param user  当前操作用户
     * @param param 分析维度参数
     * @return 分析结果
     */
    @GetMapping("/normal/analysis")
    //@IgnoreLog(level = IgnoreLevel.RETURN)
    public Message<List<ListVo>> analysis(@CurrentUser LoginAuthBean user, @Valid AnalysisParams param) {
        param.setGroupId(user.getGroupId());
        return success(liveReportBizService.getAnalysis(param));
    }

    /**
     * 列表数据占比分析
     *
     * @param user  当前操作用户
     * @param param 分析维度参数
     * @return 占比分析结果
     */
    @GetMapping("/rate/analysis")
    //@IgnoreLog(level = IgnoreLevel.RETURN)
    public Message<List<RateAnalysisListVo>> getRateAnalysis(@CurrentUser LoginAuthBean user,
                                                             @Valid RatioAnalysisParams param) {
        param.setGroupId(user.getGroupId());
        return success(liveReportBizService.getRateAnalysis(param));
    }

    /**
     * 获取直播池
     *
     * @param user  当前操作用户
     * @param param 池查询条件参数
     * @return 直播池
     */
    @GetMapping("/getPool")
    @ControllerMethod("获取直播池")
    @IgnoreAuth
    //@IgnoreLog(level = IgnoreLevel.RETURN)
    public Message<PageHelper<LivePoolVo>> getPool(@CurrentUser LoginAuthBean user, @Valid PoolQueryParam param) {
        param.setGroupId(user.getGroupId());
        return success(liveReportBizService.getPool(param));
    }
}
