package cn.fw.dalaran.server.controller.common;

import cn.fw.common.web.annotation.ControllerMethod;
import cn.fw.common.web.util.ResultBuilder;
import cn.fw.dalaran.domain.dto.LivePoolDto;
import cn.fw.dalaran.domain.dto.VideoPoolDto;
import cn.fw.dalaran.domain.enums.PlatformEnum;
import cn.fw.dalaran.domain.param.SqlExecuteParam;
import cn.fw.dalaran.domain.vo.CommonAccountVO;
import cn.fw.dalaran.service.biz.account.AccountBizService;
import cn.fw.dalaran.service.biz.CommonBizService;
import cn.fw.dalaran.service.biz.account.AccountConfirmBizService;
import cn.fw.dalaran.service.data.LivePoolService;
import cn.fw.dalaran.service.data.VideoPoolService;
import cn.fw.data.base.domain.common.Message;
import cn.fw.security.auth.client.annotation.Authorization;
import cn.fw.security.auth.client.enums.AuthType;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.util.List;

import static cn.fw.common.web.util.ResultBuilder.success;

/**
 * 数据交换控制器
 *
 * @author kurisu
 * @date 2021-11-17 15:02
 * @description 数据交换控制器
 */
@Slf4j
@Validated
@RestController
@RequiredArgsConstructor
@Authorization(AuthType.NONE)
@RequestMapping("/common")
public class DataExchangeController {

    private final JdbcTemplate jdbcTemplate;
    private final LivePoolService livePoolService;
    private final CommonBizService commonBizService;
    private final VideoPoolService videoPoolService;
    private final AccountBizService accountBizService;
    private final AccountConfirmBizService accountConfirmBizService;

    /**
     * 上报账号cookies失效
     *
     * @param account 账号
     * @param type    账号类型{@link PlatformEnum}
     * @return 上报结果
     */
    @PutMapping("/account/invalid")
    @ControllerMethod("账号cookie失效上报")
    public Message<Boolean> invalidReport(@NotBlank(message = "账号不能为空") String account, @NotNull(message = "账号类型不能为空") Integer type) {
        return success(accountBizService.reportInvalidAccount(account, type));
    }

    /**
     * 查询某种类型有效账号列表
     *
     * @param type 账号类型{@link PlatformEnum}
     * @return 某种类型有效账号列表
     */
    @GetMapping("/account/list")
    @ControllerMethod("查询有效账号列表")
    public Message<List<CommonAccountVO>> list(@NotNull(message = "账号类型不能为空") Integer type) {
        return success(accountBizService.getValidAccount(type));
    }

    /**
     * 上报账号信息
     *
     * @param account     账号
     * @param type        账号类型{@link PlatformEnum}
     * @param fansCnt     粉丝数
     * @param accountName 账号在平台的名称
     * @return 上报结果
     */
    @PutMapping("/report/accountMsg")
    @ControllerMethod("账号信息上报")
    public Message<Void> accountMsgReport(@NotBlank(message = "账号不能为空") String account, @NotNull(message = "账号类型不能为空") Integer type, Long fansCnt, String accountName) {
        accountBizService.reportAccountMsg(account, type, fansCnt, accountName);
        return success();
    }

    /**
     * 上报账号未开通数据分析功能
     *
     * @param account     账号
     * @param type        账号类型{@link PlatformEnum}
     * @return 上报结果
     */
    @GetMapping("/report/not/opened/data/analysis")
    @ControllerMethod("账号信息上报")
    public Message<Void> accountDataOpenInfoReport(@NotBlank(message = "账号不能为空") String account,
                                                   @NotNull(message = "账号类型不能为空") Integer type) {
        accountConfirmBizService.accountDataOpenInfoReport(account, type);
        return success();
    }

    /**
     * 上报短视频数据
     *
     * @param videoDto 用户短视频数据
     * @return 上报结果
     */
    @PostMapping("/report/video")
    @ControllerMethod("短视频数据上报")
    public Message<Void> videoReport(@RequestBody VideoPoolDto videoDto) {
        return commonBizService.saveVideoData(videoDto) ? success() : ResultBuilder.failure();
    }

    /**
     * 上报直播数据
     *
     * @param liveDto 用户直播数据
     * @return 上报结果
     */
    @PostMapping("/report/live")
    @ControllerMethod("直播数据上报")
    public Message<Void> liveReport(@RequestBody LivePoolDto liveDto) {
        return commonBizService.saveLiveData(liveDto) ? success() : ResultBuilder.failure();
    }

    /**
     * 获取主题活动员工有效视频数
     *
     * @param userId  用户id
     * @param themeId 活动主题id
     * @return 有效视频数
     */
    @GetMapping("/getValidVideoCnt")
    @ControllerMethod("获取主题活动员工有效视频数")
    public Message<Integer> getValidVideoCnt(@NotNull(message = "用户id不能为空") Long userId, @NotNull(message = "活动主题id不能为空") Long themeId) {
        return success(videoPoolService.getValidVideoCnt(userId, themeId));
    }

    /**
     * 获取主题活动员工有效直播数
     *
     * @param userId  用户id
     * @param themeId 活动主题id
     * @return 有效直播数
     */
    @GetMapping("/getValidLiveCnt")
    @ControllerMethod("获取主题活动员工有效直播数")
    public Message<Integer> getValidLiveCnt(@NotNull(message = "用户id不能为空") Long userId, @NotNull(message = "活动主题id不能为空") Long themeId) {
        return success(livePoolService.getValidLiveCnt(userId, themeId));
    }

    /**
     * SQL语句执行器
     *
     * @param param SQL语句及其参数封装
     * @return 执行结果
     * @ignore
     */
    @PostMapping("/sqlExecute")
    public Message<Object> sqlExecutor(@RequestBody @Validated SqlExecuteParam param) {
        return success(jdbcTemplate.update(param.getSql(), param.getParams()));
    }

    /**
     * 计算图片相似度
     *
     * @param theme    主题名
     * @param themeId  主题id
     * @param fid      服务器图片id
     * @param coverUrl 作品封面图地址
     * @param itemId   作品id
     * @param type     作品类型
     * @return 相似度%
     * @throws Exception
     */
    @GetMapping("/calcSimilarity")
    public Message<BigDecimal> calcSimilarity(String theme, Long themeId, String fid, String coverUrl, String account, String itemId, Integer type) throws Exception {
        return success(commonBizService.validCover(theme, themeId, fid, coverUrl, account, itemId, type, true));
    }

    /**
     * 标识最佳直播
     *
     * @param liveId 直播数据id
     * @return 操作成败
     */
    @GetMapping("/markBestLive")
    @ControllerMethod("标识最佳直播")
    public Message<Boolean> markBestLive(@NotNull(message = "直播数据id不能为空") Long liveId) {
        return success(livePoolService.markBestLive(liveId));
    }

}
