package cn.fw.dalaran.server.controller.web;

import cn.fw.common.page.AppPage;
import cn.fw.common.web.annotation.ControllerMethod;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.common.web.auth.annotation.CurrentUser;
import cn.fw.dalaran.common.page.PageHelper;
import cn.fw.dalaran.domain.db.Account;
import cn.fw.dalaran.domain.dto.AccountDTO;
import cn.fw.dalaran.domain.dto.AccountQueryDTO;
import cn.fw.dalaran.domain.dto.UpdateAccountParam;
import cn.fw.dalaran.domain.vo.AccountStatusVo;
import cn.fw.dalaran.domain.vo.AccountVO;
import cn.fw.dalaran.service.biz.account.AccountBizService;
import cn.fw.data.base.domain.common.Message;
import cn.fw.security.auth.client.annotation.Authorization;
import cn.fw.security.auth.client.annotation.IgnoreAuth;
import cn.fw.security.auth.client.annotation.RepeatSubmit;
import cn.fw.security.auth.client.enums.AuthType;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotNull;
import java.util.List;

import static cn.fw.common.web.util.ResultBuilder.success;

/**
 * 账号配置接口
 *
 * @author kurisu
 * @date 2021-11-17 15:02
 */
@Slf4j
@Validated
@IgnoreAuth
@RestController
@RequiredArgsConstructor
@Authorization(AuthType.ERP)
@RequestMapping("/web/account")
public class AccountController {
    private final AccountBizService accountBizService;

    /**
     * 分页查询账号列表
     *
     * @param user  当前操作用户
     * @param query 查询条件
     * @return 账号列表
     * @ignoreParams user pageHelper
     */
    @GetMapping("/list")
    @ControllerMethod("查询账号列表")
    public Message<PageHelper<AccountVO>> list(@CurrentUser LoginAuthBean user, PageHelper<Account> pageHelper,
                                            AccountQueryDTO query) {
        return success(accountBizService.page(user.getGroupId(), pageHelper, query));
    }

    /**
     * 保存账号信息
     *
     * @param user       当前操作用户
     * @param accountDTO 账号参数
     * @return 是否添加成功
     * @ignoreParams user
     */
    @RepeatSubmit
    @PostMapping("/add")
    public Message<Boolean> add(@CurrentUser LoginAuthBean user, @RequestBody @Validated final AccountDTO accountDTO) {
        accountDTO.setGroupId(user.getGroupId());
        return success(accountBizService.saveAccount(accountDTO));
    }

    /**
     * 删除账号
     *
     * @param id 账号id
     * @return 是否删除成功
     */
    @DeleteMapping("/delete")
    @ControllerMethod("删除账号")
    public Message<Boolean> delete(@NotNull(message = "id不能为空") Long id) {
        return success(accountBizService.deleteAccount(id));
    }

    /**
     * 上报账号状态
     *
     * @param accountId 账号id
     * @param valid     cookies是否有效
     * @return 是否上报成功
     */
    @PutMapping("/authorized/report")
    @ControllerMethod("上报账号状态")
    public Message<Boolean> report(@NotNull(message = "账号id不能为空") Long accountId, @NotNull(message = "账号状态不能为空") Boolean valid) {
        return success(accountBizService.reportAccountStatus(accountId, valid));
    }

    /**
     * 查询门店账号状态详情列表
     *
     * @param user 当前操作用户
     * @return 门店账号状态详情列表
     * @ignoreParams user
     */
    @GetMapping("/status")
    @ControllerMethod("查询门店账号状态详情列表")
    public Message<List<AccountStatusVo.ShopDetails>> status(@CurrentUser LoginAuthBean user) {
        return success(accountBizService.status(user.getGroupId()));
    }

    /**
     * 批量修改账号
     *
     * @param param 账号参数
     * @return 是否修改成功
     * @ignore
     */
    @PostMapping("/alert")
    @ControllerMethod("批量修改账号")
    public Message<Boolean> alert(@RequestBody @Validated final UpdateAccountParam param) {
        return success(accountBizService.alert(param));
    }

}
