package cn.fw.dalaran.server.controller.web.config;

import cn.fw.common.web.annotation.ControllerMethod;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.common.web.auth.annotation.CurrentUser;
import cn.fw.dalaran.common.page.PageHelper;
import cn.fw.dalaran.domain.db.config.ActivityTheme;
import cn.fw.dalaran.domain.dto.activity.ActivityQueryDto;
import cn.fw.dalaran.domain.dto.activity.ActivitySaveDto;
import cn.fw.dalaran.domain.vo.ActivityThemeVo;
import cn.fw.dalaran.domain.vo.BrandVo;
import cn.fw.dalaran.domain.vo.UserThemeCompleteDetailsVo;
import cn.fw.dalaran.domain.vo.activity.ActivityDetailVo;
import cn.fw.dalaran.domain.vo.activity.ActivityVo;
import cn.fw.dalaran.domain.vo.common.PlatformTypeVo;
import cn.fw.dalaran.service.biz.config.ActivityBizService;
import cn.fw.data.base.domain.common.Message;
import cn.fw.security.auth.client.annotation.Authorization;
import cn.fw.security.auth.client.annotation.IgnoreAuth;
import cn.fw.security.auth.client.enums.AuthType;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import javax.validation.constraints.NotNull;
import java.util.Date;
import java.util.List;
import java.util.Objects;

import static cn.fw.common.web.util.ResultBuilder.success;

/**
 * 活动主题配置接口
 *
 * @author: lym
 * @date: 2023/4/25
 */
@Slf4j
@Validated
@IgnoreAuth
@RestController
@RequiredArgsConstructor
@Authorization(AuthType.ERP)
@RequestMapping("/activity/theme/setting")
public class ActivityController {
    private final ActivityBizService activityBizService;

    /**
     * 查询活动主题列表
     *
     * @param queryParam 查询参数
     * @return 符合条件的活动列表
     * @ignoreParams pageHelper user
     */
    @GetMapping("/list")
    @ControllerMethod("查询活动主题列表")
    public Message<PageHelper<ActivityVo>> list(@CurrentUser LoginAuthBean user,
                                                PageHelper<ActivityTheme> pageHelper,
                                                @Valid ActivityQueryDto queryParam) {
        return success(activityBizService.getActivityList(pageHelper, user, queryParam));
    }

    /**
     * 查看活动主题详情
     *
     * @param themeId 活动主题id
     */
    @GetMapping("/detail")
    @ControllerMethod("查看活动主题详情")
    public Message<ActivityDetailVo> detail(@NotNull(message = "活动主题id不能为空") Long themeId) {
        return success(activityBizService.detail(themeId));
    }

    /**
     * 保存活动主题
     *
     * @param param 活动主题配置参数
     * @return 符合条件的活动列表
     * @ignoreParams user
     */
    @PostMapping("/save")
    @ControllerMethod("保存活动主题")
    public Message<Void> save(@CurrentUser LoginAuthBean user, @RequestBody @Valid ActivitySaveDto param) {
        activityBizService.save(user, param);
        return success();
    }

    /**
     * 删除活动主题
     *
     * @param id 活动id
     * @return 符合条件的活动列表
     */
    @GetMapping("/del")
    @ControllerMethod("查询活动主题列表")
    public Message<Void> del(@NotNull(message = "活动id不能为空") Long id) {
        activityBizService.del(id);
        return success();
    }

    /**
     * 查询月度主题活动列表(报表使用)
     *
     * @param user      当前登录用户
     * @param timeStamp 指定月之内的时间戳
     * @param brandId   品牌id
     * @return 符合条件的活动列表
     */
    @GetMapping("/listByMonth")
    @ControllerMethod("查询指定月活动主题列表")
    public Message<List<ActivityThemeVo>> listByMonth(@CurrentUser LoginAuthBean user, Long timeStamp,
                                                      @RequestParam List<Long> shopIds, Long brandId) {
        return success(activityBizService.listByMonth(user, Objects.isNull(timeStamp) ?
                System.currentTimeMillis() : timeStamp, shopIds, brandId));
    }

    /**
     * 刷新主题封面缓存
     *
     * @param themeId 主题id
     * @return void
     */
    @GetMapping("/flushThemeCoverCache")
    public Message<Void> flushThemeCoverCache(@NotNull(message = "主题id不能为空") Long themeId) {
        activityBizService.flushThemeCoverCache(themeId);
        return success();
    }

    /**
     * 获取用户当前参与的进行中的主题有效作品完成进度详情
     *
     * @param user 当前操作用户
     * @return 用户当前参与的进行中的主题有效作品完成进度详情
     */
    @GetMapping("/getUserAllGoingThemeCompleteDetails")
    public Message<List<UserThemeCompleteDetailsVo>> getUserAllGoingThemeCompleteDetails(
            @CurrentUser LoginAuthBean user) {
        return success(activityBizService.getUserAllGoingThemeCompleteDetails(user));
    }

    /**
     * 获取用户某主题有效作品完成进度详情
     *
     * @param user    当前操作用户
     * @param themeId 主题id
     * @return 用户某主题有效作品完成进度详情
     */
    @GetMapping("/getUserThemeCompleteDetails")
    public Message<UserThemeCompleteDetailsVo> getUserThemeCompleteDetails(
            @CurrentUser LoginAuthBean user,
            @NotNull(message = "主题id不能为空") Long themeId) {
        return success(activityBizService.getUserThemeCompleteDetails(user, themeId));
    }

    /**
     * 获取活动主题最低能配置的开始时间
     *
     * @param configGroupId 配置组id
     * @return 活动主题最低能配置的开始时间
     */
    @GetMapping("/getThemeCanStartMinTime")
    public Message<Date> getThemeCanStartMinTime(@NotNull(message = "配置组id不能为空") Long configGroupId) {
        return success(activityBizService.getThemeCanStartMinTime(configGroupId));
    }

    /**
     * 获取可选的品牌列表
     *
     * @param user 当前用户
     * @return 可选的品牌列表
     * @ignoreParams user
     */
    @GetMapping("/getCanSelectBrands")
    public Message<List<BrandVo>> getCanSelectBrands(@CurrentUser LoginAuthBean user) {
        return success(activityBizService.getCanSelectBrands(user));
    }

}
