package cn.fw.dalaran.server.rocketMQ;

import cn.fw.dalaran.common.constants.Constants;
import cn.fw.dalaran.common.constants.ErpTaskConstant;
import cn.fw.dalaran.domain.db.Account;
import cn.fw.dalaran.domain.db.config.ActivityTheme;
import cn.fw.dalaran.domain.dto.common.ShopDto;
import cn.fw.dalaran.service.biz.config.ConfigGroupBizService;
import cn.fw.dalaran.service.data.AccountService;
import cn.fw.dalaran.service.data.ActivityThemeService;
import cn.fw.rocketmq.annotation.RocketListeners;
import cn.fw.rocketmq.annotation.RocketMQListener;
import cn.fw.starter.redis.redis.RedisUtil;
import com.alibaba.fastjson.JSON;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.Objects;
import java.util.stream.Collectors;

/**
 * erp定时任务消费
 *
 * @author: lym
 * @date: 2023/5/22
 */
@Slf4j
@Service
@RequiredArgsConstructor
@RocketListeners(topic = ErpTaskConstant.TOPIC, consumerGroup = "${spring.application.name}_" + ErpTaskConstant.TOPIC)
public class ErpTaskConsumer {
    private final RedisUtil redisUtil;
    private final AccountService accountService;
    private final ActivityThemeService activityThemeService;
    private final ConfigGroupBizService configGroupBizService;

    /**
     * 配置组调整更新活动主题缓存
     */
    @RocketMQListener(messageClass = Long.class, tag = Constants.CHANGE_CONFIG_GROUP_TO_THEME)
    public void handleChangeConfigGroupToTheme(Long configGroupId) {
        String key = "fw:dalaran:task:config:group:change:" + configGroupId;
        if (!redisUtil.getLock(key))
            return;
        try {
            log.info("处理配置组调整更新活动主题缓存定时任务 configGroupId:{}", configGroupId);
            configGroupBizService.syncGroupConfigToTheme(configGroupId);
        } catch (Exception e) {
            log.error("处理配置组调整更新活动主题缓存定时任务失败");
            throw e;
        } finally {
            redisUtil.releaseLock(key);
        }
    }

    /**
     * 新主题开始, 初始化自媒体系统员工账号状态
     */
    @RocketMQListener(messageClass = Long.class, tag = Constants.INIT_ACCOUNT_STATUS)
    public void handleInitStaffWorkStatus(Long themeId) {
        String key = "fw:dalaran:task:init:account:status" + themeId;
        if (!redisUtil.getLock(key))
            return;
        try {
            ActivityTheme theme;
            if (Objects.nonNull(theme = activityThemeService.getById(themeId))) {
                if (Objects.equals(theme.getAllShop(), 0)) {
                    accountService.lambdaUpdate()
                            .eq(Account::getAccountStatus, 1)
                            .in(Account::getShopId, JSON.parseArray(theme.getShopJson(), ShopDto.class)
                                    .stream()
                                    .map(ShopDto::getShopId)
                                    .collect(Collectors.toList())
                            )
                            .set(Account::getAccountStatus, 0)
                            .update();
                } else
                    accountService.lambdaUpdate()
                            .eq(Account::getAccountStatus, 1)
                            .set(Account::getAccountStatus, 0)
                            .update();
            } else {
                log.info(String.format("id为: %s的活动主题已不存在, 不进行初始化员工账号状态操作", themeId));
            }
        } catch (Exception e) {
            log.error("处理配置组调整更新活动主题缓存定时任务失败", e);
        } finally {
            redisUtil.releaseLock(key);
        }
    }
}
