package cn.fw.dalaran.server.schedule;

import cn.fw.backlog.sdk.api.result.MeddleItem;
import cn.fw.dalaran.common.domain.enums.ThemeChangeTypeEnum;
import cn.fw.dalaran.common.exception.InterveneTypeEnum;
import cn.fw.dalaran.common.utils.PublicUtil;
import cn.fw.dalaran.domain.db.Account;
import cn.fw.dalaran.domain.db.config.ActivityTheme;
import cn.fw.dalaran.domain.enums.account.AccountEnableAnalysisEnum;
import cn.fw.dalaran.rpc.backlog.TodoInterveneRpc;
import cn.fw.dalaran.rpc.oop.OopRpc;
import cn.fw.dalaran.rpc.oop.dto.GroupDTO;
import cn.fw.dalaran.service.Common;
import cn.fw.dalaran.service.biz.account.AccountConfirmBizService;
import cn.fw.dalaran.service.biz.config.ActivityBizService;
import cn.fw.dalaran.service.data.AccountService;
import cn.fw.dalaran.service.data.ActivityThemeService;
import cn.fw.starter.redis.redis.RedisUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.io.File;
import java.util.*;
import java.util.function.Consumer;
import java.util.stream.Collectors;

/**
 * 系统其他定时任务
 *
 * @author wmy3969
 * @version 1.0
 * @date 2022/7/27 09:04
 */
@Slf4j
@Component
@ConditionalOnProperty(prefix = "task", name = "switch", havingValue = "on")
@RequiredArgsConstructor
public class OtherTask {
    private final Common common;
    private final OopRpc oopRpc;
    private final RedisUtil redisUtil;
    private final AccountService accountService;
    private final TodoInterveneRpc todoInterveneRpc;
    private final ActivityBizService activityBizService;
    private final ActivityThemeService activityThemeService;// 活动主题业务
    private final AccountConfirmBizService accountConfirmBizService;


    /**
     * 每天都找已经停止抓数据7天及其更久的主题, 删除缓存封面图
     */
    @Scheduled(fixedRate = 60 * 60 * 1000, initialDelay = 5 * 1000)
    public void deleteCoverFiles() {
        final List<ActivityTheme> themeList = activityThemeService.lambdaQuery()
                .list()
                .stream()
                .filter(item -> activityThemeService.themeHaveStopCatchData(item.getEndTime(), 7))
                .collect(Collectors.toList());// 找到需要删除的主题
        themeList.addAll(activityThemeService.getHasDelActivityTheme());// 找到已经逻辑删除的主题
        if (CollectionUtils.isEmpty(themeList))
            return;
        String tempDir = common.getActivityThemeCoverDir();
        StringBuilder sb = new StringBuilder();
        themeList.forEach(item -> {
            sb.append(tempDir);
            sb.append("activityTheme");
            sb.append(File.separator);
            sb.append(item.getId());
            String dirPath = sb.toString();
            File file = new File(dirPath);
            if (file.isDirectory())
                common.deleteAllFile(dirPath, file);
            sb.setLength(0);
        });
    }

    /**
     * 每天08:30触发账号确认干预待办
     */
    @Scheduled(cron = "0 30 8 * * ?")
    public void accountConfirmIntervene() {
        String lockKey = "fw:dalaran:todo:make:up:cancel";
        this.commonLock(lockKey, key -> {
            List<GroupDTO> allGroups = oopRpc.getAllGroups();
            for (GroupDTO group : allGroups) {
                List<MeddleItem> thresholdList = todoInterveneRpc.getThresholdList(group.getId(),
                        InterveneTypeEnum.ACCOUNT_CONFIRM.getValue());
                if (PublicUtil.isEmpty(thresholdList)) {
                    continue;
                }
                final List<MeddleItem> sortList = thresholdList.stream()
                        .sorted(Comparator.comparing(MeddleItem::getThreshold).reversed()).collect(Collectors.toList());
                List<Account> needConfirmList = accountService.list(Wrappers.<Account>lambdaQuery()
                        .eq(Account::getGroupId, group.getId())
                        .in(Account::getEnableAnalysis, AccountEnableAnalysisEnum.ADD, AccountEnableAnalysisEnum.THEME_ZERO));
                needConfirmList.forEach(account -> accountConfirmBizService.interveneCheck(account, sortList));
            }
        });
    }

    /**
     * 每天00:05检查并给对应人员发送参加活动主题待办
     */
    @Scheduled(cron = "0 5 0 * * ?")
    public void sendJoinThemeBacklog() {
        this.commonLock("sendJoinThemeBacklog", key -> {
            activityThemeService.lambdaQuery()
                    .and(wrapper -> wrapper
                            .eq(ActivityTheme::getInitCheck, 0)
                            .or()
                            .isNull(ActivityTheme::getInitCheck)
                    )
                    .lt(ActivityTheme::getStartTime, new Date())
                    .gt(ActivityTheme::getEndTime, new Date())
                    .list()
                    .forEach(item -> {
                        if (Objects.isNull(item.getInitCheck()))
                            item.setInitCheck(0);
                        activityBizService.checkJoinThemeBacklogSend(item,
                                Collections.singletonList(ThemeChangeTypeEnum.FIRST_CONFIG));
                    })
            ;
        });
    }

    public void commonLock(String lockKey, Consumer<String> consumer) {
        if (!redisUtil.getLock(lockKey)) {
            log.info("加锁失败, lockKey: {}", lockKey);
            return;
        }
        try {
            consumer.accept(lockKey);
        } catch (Exception e) {
            log.error(e.getMessage());
        } finally {
            redisUtil.unlock(lockKey);
        }
    }
}
