package cn.fw.dalaran.server.schedule;

import cn.fw.dalaran.common.domain.enums.ToDoTypeEnum;
import cn.fw.dalaran.common.utils.DateUtil;
import cn.fw.dalaran.domain.db.common.TodoHistory;
import cn.fw.dalaran.service.biz.common.LiveCheckTaskBizService;
import cn.fw.dalaran.service.biz.common.ToDoBizService;
import cn.fw.dalaran.service.data.common.TodoHistoryService;
import cn.fw.starter.redis.redis.RedisUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;

/**
 * @author kurisu
 * @date 2021-12-02 16:34
 * @description 待办定时任务
 */
@Slf4j
@Component
//@ConditionalOnProperty(prefix = "task", name = "switch", havingValue = "on")
@RequiredArgsConstructor
public class TodoTask {
    private final RedisUtil redisUtil;
    private final ToDoBizService toDoBizService;
    private final TodoHistoryService todoHistoryService;
    private final LiveCheckTaskBizService liveCheckTaskBizService;

    /**
     * 定时异步发送待办
     */
    @Scheduled(fixedRate = 15 * 1000, initialDelay = 5 * 1000)
    public void sendTodo() {
        String lockKey = "fw:dalaran:todo:make:up:add";
        if (!redisUtil.getLock(lockKey, 31))
            return;
        try {
            //找到需要推送待办的数据项(时间7天内)
            List<TodoHistory> needSendList = todoHistoryService.list(Wrappers.<TodoHistory>lambdaQuery()
                    .eq(TodoHistory::getSend, Boolean.FALSE)
                    .eq(TodoHistory::getDone, ToDoTypeEnum.ADD.getValue())
                    .ge(TodoHistory::getCreateTime, DateUtil.minusDays(new Date(), 7)));
            needSendList.forEach(toDoBizService::pushToDo);
        } catch (Exception e) {
            log.error("定时补偿待办推送异常", e);
        } finally {
            redisUtil.unlock(lockKey);
        }
    }

    /**
     * 完成待办
     */
    @Scheduled(fixedRate = 15 * 1000, initialDelay = 5 * 1000)
    public void completeTodo() {
        String lockKey = "fw:dalaran:todo:make:up:complete";
        if (!redisUtil.getLock(lockKey, 31))
            return;
        try {
            //找到需要推送待办的数据项(时间7天内)
            List<TodoHistory> needCompleteList = todoHistoryService.list(Wrappers.<TodoHistory>lambdaQuery()
                    .eq(TodoHistory::getSend, Boolean.TRUE)
                    .eq(TodoHistory::getDone, ToDoTypeEnum.COMPLETE.getValue())
                    .eq(TodoHistory::getTodoDone, Boolean.FALSE)
                    .ge(TodoHistory::getCreateTime, DateUtil.minusDays(new Date(), 7)));
            needCompleteList.forEach(toDoBizService::finishToDo);
        } catch (Exception e) {
            log.error("定时补偿待办推送异常", e);
        } finally {
            redisUtil.unlock(lockKey);
        }
    }

    /**
     * 取消待办
     */
    @Scheduled(fixedRate = 15 * 1000, initialDelay = 5 * 1000)
    public void cancelTodo() {
        String lockKey = "fw:dalaran:todo:make:up:cancel";
        if (!redisUtil.getLock(lockKey, 31))
            return;
        try {
            //找到需要推送待办的数据项(时间7天内)
            List<TodoHistory> needCompleteList = todoHistoryService.list(Wrappers.<TodoHistory>lambdaQuery()
                    .eq(TodoHistory::getSend, Boolean.TRUE)
                    .eq(TodoHistory::getDone, ToDoTypeEnum.CANCEL.getValue())
                    .eq(TodoHistory::getTodoDone, Boolean.FALSE)
                    .ge(TodoHistory::getCreateTime, DateUtil.minusDays(new Date(), 7)));
            needCompleteList.forEach(toDoBizService::cancelToDo);
        } catch (Exception e) {
            log.error("定时补偿待办推送异常", e);
        } finally {
            redisUtil.unlock(lockKey);
        }
    }

    /**
     * 每天都找需要审计的主题
     */
    @Scheduled(fixedRate = 15 * 60 * 1000, initialDelay = 5 * 1000)
    public void sendLiveCheckBacklog() {
        liveCheckTaskBizService.sendLiveCheckBacklog(System.currentTimeMillis());
    }
}
