package cn.fw.dalaran.service.biz;

import cn.fw.dalaran.common.domain.enums.RoleMsgEnum;
import cn.fw.dalaran.common.utils.AutoExpireLocalCache;
import cn.fw.dalaran.common.utils.DateUtil;
import cn.fw.dalaran.common.utils.ImageUtils;
import cn.fw.dalaran.common.utils.PublicUtil;
import cn.fw.dalaran.domain.db.*;
import cn.fw.dalaran.domain.db.config.ActivityTheme;
import cn.fw.dalaran.domain.db.config.ValidConfigNew;
import cn.fw.dalaran.domain.dto.LivePoolDto;
import cn.fw.dalaran.domain.dto.VideoPoolDto;
import cn.fw.dalaran.domain.dto.common.ShopDto;
import cn.fw.dalaran.domain.enums.DisqualificationEnum;
import cn.fw.dalaran.domain.enums.FileTypeEnum;
import cn.fw.dalaran.domain.enums.PlatformEnum;
import cn.fw.dalaran.domain.enums.config.*;
import cn.fw.dalaran.domain.enums.report.InvalidReasonEnum;
import cn.fw.dalaran.domain.vo.activity.TopicVo;
import cn.fw.dalaran.domain.vo.config.ValidConfigNewVo;
import cn.fw.dalaran.service.Common;
import cn.fw.dalaran.service.data.*;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.Data;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.io.File;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDate;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.ReentrantLock;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author kurisu
 * @date 2021-12-04 14:29
 * @description 通用服务
 */
@Slf4j
@Service
@RequiredArgsConstructor
@SuppressWarnings("Duplicates")
public class CommonBizService {

    @Value("${spring.profiles.active}")
    private String env;// 获取系统当前环境
    private final transient ReentrantLock lock = new ReentrantLock();

    private final Common common;
    private final AccountService accountService;
    private final LivePoolService livePoolService;
    private final ThemeFileService themeFileService;
    private final VideoPoolService videoPoolService;
    private final LocalCacheService localCacheService;
    private final AutoExpireLocalCache autoExpireLocalCache;// 本地自动过期缓存
    private final ActivityThemeService activityThemeService;// 主题配置
    private final ValidConfigNewService validConfigNewService;// 直播/视频有效性
    private final DisqualificationRecordBizService disqualificationRecordBizService;

    /**
     * 话题验证结果
     */
    @Data
    public static class TagCheckResult {
        /**
         * 是否包含指定话题
         */
        private Boolean containsTag = false;
        /**
         * 是否匹配到多个主题
         */
        private Boolean matchMultipleTheme = false;
        /**
         * 主题id
         */
        private Long themeId;
        /**
         * 主题名
         */
        private String themeName;
        /**
         * 命中主题详情
         */
        private ActivityTheme hitTheme;
        /**
         * 所属配置组id
         */
        private Long configGroupId;
        /**
         * 系统配置话题
         */
        private Set<String> configTags = new HashSet<>();
        /**
         * 作品包含话题
         */
        private Set<String> worksTags = new HashSet<>();
        /**
         * 验证有效话题
         */
        private Set<TopicVo> validTags = new HashSet<>();
    }

    /**
     * 储存短视频数据
     *
     * @param videoPoolDTO 自媒体服务器传过来的视频池
     */
    public boolean saveVideoData(VideoPoolDto videoPoolDTO) {
        String account = videoPoolDTO.getAccount();
        Integer type = videoPoolDTO.getType();
        List<Account> accountList = queryAccList(account, type);
        if (CollectionUtils.isEmpty(accountList)) {
            log.info(String.format("不存在[%s]平台, 账号为: %s的账号, 视频保存失败", PlatformEnum.getNameByVale(type), account));
            return false;
        }
        List<VideoPoolDto.VideoDTO> videoList = videoPoolDTO.getVideoList();
        if (CollectionUtils.isEmpty(videoList)) {
            return true;
        }
        if (!this.acceptData()) {
            log.info(String.format("%s 的视频数据提交时间已超当日期限, 数据将不保存到视频池", account));
            return true;
        }
        for (Account db : accountList) {
            this.saveVideoPoolList(db, videoList);
        }
        return true;
    }

    /**
     * 储存直播数据
     *
     * @param livePoolDTO 自媒体服务器传过来的直播池
     */
    public boolean saveLiveData(LivePoolDto livePoolDTO) {
        String account = livePoolDTO.getAccount();
        Integer type = livePoolDTO.getType();
        List<Account> accountList = queryAccList(account, type);
        if (CollectionUtils.isEmpty(accountList)) {
            log.info(String.format("不存在[%s]平台, 账号为: %s的账号, 直播保存失败", PlatformEnum.getNameByVale(type), account));
            return false;
        }
        List<LivePoolDto.LiveDTO> liveList = livePoolDTO.getLiveList();
        if (CollectionUtils.isEmpty(liveList)) {
            return true;
        }
        if (!this.acceptData()) {
            log.info(String.format("%s 的直播数据提交时间已超当日期限, 数据将不保存到直播池", account));
            return true;
        }
        for (Account db : accountList) {
            this.saveLivePoolList(db, liveList);
        }
        return true;
    }

    /**
     * 是否保存数据入池
     */
    public boolean acceptData() {
        /*if (env.contains("prd")) {
            long acceptDataMaxTime = DateUtil.getBeginInTime(new Date()).getTime() + 27 * 1800 * 1000L;
            return System.currentTimeMillis() - acceptDataMaxTime <= 0;
        }
        return true;*/
        return true;
    }

    /**
     * 查询指定类型的账号列表
     *
     * @param account 账户号
     * @param type    账户类型
     * @return
     */
    private List<Account> queryAccList(String account, Integer type) {
        PlatformEnum platformEnum = PlatformEnum.ofValue(type);
        if (Objects.isNull(platformEnum)) {
            return new ArrayList<>();
        }
        List<Account> accountList = accountService.list(Wrappers.<Account>lambdaQuery()
                .eq(Account::getAccount, account)
                .eq(Account::getType, platformEnum)
                .eq(Account::getYn, Boolean.TRUE)
        );
        return CollectionUtils.isEmpty(accountList) ? new ArrayList<>() : accountList;
    }

    /**
     * 保存视频池列表
     *
     * @param account   账户
     * @param videoList 该账户的视频信息
     */
    private void saveVideoPoolList(Account account, List<VideoPoolDto.VideoDTO> videoList) {
        List<VideoPool> waitSaveWorks = new ArrayList<>();
        LocalDate today = LocalDate.now();
        Long groupId = account.getGroupId();
        Long shopId = account.getShopId();
        String accountNo = account.getAccount();
        PlatformEnum platformEnum = account.getType();
        String platformName = platformEnum.getName();
        Map<String, List<ActivityTheme>> themeMap = this.getGroupActivityTheme(groupId, shopId);
        Date now = new Date();
        Date yesterdayNow = DateUtil.minusDays(now, 1);
        List<ActivityTheme> thisMonthThemes = themeMap.get(DateUtil.format(yesterdayNow, DateUtil.FORMAT_1));
        List<ActivityTheme> lastMonthThemes = themeMap.get(DateUtil.format(DateUtil.minusMonths(yesterdayNow, 1), DateUtil.FORMAT_1));
        if (PublicUtil.isEmpty(themeMap)) {
            log.error("账号：{}({})门店{}({})近期2个月暂无活动", account.getAccount(), account.getId(),
                    account.getShopName(), account.getShopId());
            return;
        }
        List<ActivityTheme> themeList;
        List<String> userDisqualificationWorkIds = disqualificationRecordBizService
                .getUserDisqualificationRecords(accountNo, WorksTypeEnum.VIDEO, DateUtil.minusDays(now, 1))
                .stream()
                .map(DisqualificationRecordEntity::getWorksId)
                .collect(Collectors.toList());
        Map<String, VideoPool> recent2MonthVideo = videoPoolService.getRecent2MonthVideo();// 最近2个月视频归属详情
        List<String> invalidStatusVideoIds = videoPoolService.getRecent2MonthInvalidStatusVideoIds();// 获取最近2个月被标记无效视频id列表
        for (VideoPoolDto.VideoDTO videoDTO : videoList) {
            String videoId = videoDTO.getVideoId();// 获取视频id
            Date publishTime = videoDTO.getPublishTime();// 获取视频发布时间
            String videoCover = videoDTO.getPreview();
            String baseInfo = String.format("账户号为: %s, id为: %s, 发布时间为: %s, %s平台的视频", accountNo, videoId,
                    DateUtil.format(publishTime, DateUtil.FORMAT_4), platformName);
            if (invalidStatusVideoIds.contains(videoId)) {
                log.info(String.format(" %s, 由于删账号导致被标记为无效视频, 不进入视频池", baseInfo));
                continue;
            }
            if (DateUtil.sameMonth(yesterdayNow, publishTime))
                themeList = thisMonthThemes;
            else
                themeList = lastMonthThemes;
            List<ActivityTheme> joinThemes = this.getWorksJoinThemes(publishTime, themeList);// 作品发布时间和活动主题生效时间匹配的活动主题
            if (CollectionUtils.isEmpty(joinThemes)) {
                log.info(String.format(" %s, 找不到作品发布时间期间参加的主题, 不进入视频池", baseInfo));
                continue;
            }
            VideoPool existVideo = recent2MonthVideo.get(videoId);
            VideoPool videoPool = this.setVideoBelong(account, existVideo);// 设置视频归属
            Set<String> worksTag = processTags(videoDTO.getTitle());// 作品包含话题
            List<Integer> invalidReasons = new ArrayList<>();
            TagCheckResult result = this.calcHitTheme(worksTag, joinThemes, shopId, WorksTypeEnum.VIDEO);
            /* 话题验证 */
            if (!result.getContainsTag()) {
                if (!userDisqualificationWorkIds.contains(videoId)) {
                    disqualificationRecordBizService.saveDisqualificationRecord(
                            disqualificationRecordBizService.createVideoDisqualificationRecord(account, result,
                                    videoDTO, DisqualificationEnum.TAGS)
                    );
                }
                log.info(String.format("%s, 不包含主题指定的话题, 视作无效视频", baseInfo));
                invalidReasons.add(result.getMatchMultipleTheme() ?
                        InvalidReasonEnum.MATCH_MULTIPLE_THEME.getValue() : InvalidReasonEnum.TAG.getValue());
            }
            ActivityTheme activityTheme = result.getHitTheme();
            Long configGroupId = result.getConfigGroupId();// 配置组id
            if (Objects.nonNull(configGroupId)) {
                /* 指标验证 */
                List<Integer> invalidIndexes = this.newValidVideo(account, configGroupId, videoDTO);
                invalidReasons.addAll(invalidIndexes);
                if (!CollectionUtils.isEmpty(invalidIndexes))
                    log.info(String.format("%s, 指标数据不满足集团标准, 视作无效视频", baseInfo));
                /* 封面图验证 */
                boolean validCover = false;
                BigDecimal rate = this.getCoverConfigValue(configGroupId);
                List<ThemeFile> list = themeFileService.lambdaQuery()
                        .eq(ThemeFile::getThemeId, activityTheme.getId())
                        .eq(ThemeFile::getType, FileTypeEnum.THEME_COVER.getValue())
                        .list();
                for (ThemeFile themeFile : list) {
                    try {
                        BigDecimal coverSimilarity = this.validCover(activityTheme.getTheme(), themeFile.getThemeId(),
                                themeFile.getFileId(), videoCover, accountNo, videoId, 1, false);
                        if (validCover = coverSimilarity.compareTo(rate) >= 0)
                            break;
                    } catch (Exception e) {
                        log.error(String.format("%s, 验证封面图出现异常", baseInfo), e);
                    }
                }
                if (!validCover) {
                    if (!userDisqualificationWorkIds.contains(videoId)) {
                        log.info(String.format("%s, 封面图不匹配, 视作无效视频", baseInfo));
                        disqualificationRecordBizService.saveDisqualificationRecord(
                                disqualificationRecordBizService.createVideoDisqualificationRecord(account, result,
                                        videoDTO, DisqualificationEnum.COVER)
                        );
                    }
                    invalidReasons.add(InvalidReasonEnum.COVER.getValue());
                }
            }
            videoPool.setAccountId(account.getId());
            videoPool.setPlatform(platformEnum);
            videoPool.setVideoId(videoId);
            videoPool.setPlayUrl(videoDTO.getPlayUrl());
            videoPool.setTitle(videoDTO.getTitle());
            videoPool.setPreview(videoCover);
            videoPool.setUv(videoDTO.getUv());
            videoPool.setFullUv(videoDTO.getFullUv());
            videoPool.setLikeNum(videoDTO.getLikeNum());
            videoPool.setShareNum(videoDTO.getShareNum());
            videoPool.setCommentNum(videoDTO.getCommentNum());
            videoPool.setNewFanNum(videoDTO.getNewFanNum());
            videoPool.setDuration(videoDTO.getDuration());
            videoPool.setThemeId(Objects.nonNull(result.getThemeId()) ? result.getThemeId() : -1);
            videoPool.setTheme(result.getThemeName());
            videoPool.setTags(collection2String(worksTag));
            videoPool.setPublishTime(publishTime);
            videoPool.setLatestRecord(1);
            if (CollectionUtils.isEmpty(invalidReasons)) {
                videoPool.setValidVideo(1);
            } else {
                videoPool.setValidVideo(0);
                videoPool.setInvalidReason(this.invalidReason2String(invalidReasons));// 无效理由
            }
            if (Objects.nonNull(activityTheme)) {
                long maxMills = ((11 + 24) * 60 + 30) * 60 * 1000;// 1天零11.5小时的毫秒数
                Date themeEndTime = activityTheme.getEndTime();// 主题活动结束时间
                if (activityThemeService.themeHaveStopCatchData(themeEndTime, 0) &&// 主题已经停止数据抓取
                        DateUtil.dayBegin(now).getTime() - publishTime.getTime() > maxMills) {// 作品是主题结束那天12:30前发布的
                    log.info(String.format("%s, 不在目前最新活动时间内, 尝试查询历史记录", baseInfo));
                    VideoPool latestVideo = videoPoolService.getLatestVideo(videoId);
                    if (Objects.isNull(latestVideo) || !DateUtil.sameMonth(latestVideo.getPublishTime(), yesterdayNow)) {
                        log.info(String.format("%s, 不在目前最新活动时间内, 且无历史记录, 不进入视频池", baseInfo));
                        continue;
                    }
                    videoPool = latestVideo;
                    videoPool.setLatestRecord(1);
                }
            } else {
                if (!DateUtil.sameMonth(publishTime, yesterdayNow)) {
                    log.info(String.format("%s, 未匹配到主题, 且无历史记录, 不进入视频池", baseInfo));
                    continue;
                }
            }
            videoPool.setCollectDay(LocalDate.now().minusDays(1));
            waitSaveWorks.add(videoPool);
            videoPoolService.markVideoIsOld(videoId);// 将历史数据标记为老数据
        }
        videoPoolService.removeAccountDataByDate(account.getId(), today);
        videoPoolService.saveBatch(waitSaveWorks);
    }

    /**
     * 保存直播池列表
     *
     * @param account  账户
     * @param liveList 该账户的直播信息
     */
    private void saveLivePoolList(Account account, List<LivePoolDto.LiveDTO> liveList) {
        List<LivePool> waitSaveWorks = new ArrayList<>();
        LocalDate today = LocalDate.now();
        Long groupId = account.getGroupId();
        Long shopId = account.getShopId();
        String accountNo = account.getAccount();
        PlatformEnum platformEnum = account.getType();
        String platformName = platformEnum.getName();
        Map<String, List<ActivityTheme>> themeMap = this.getGroupActivityTheme(groupId, shopId);
        List<ActivityTheme> thisMonthThemes = themeMap.get(DateUtil.format(DateUtil.minusDays(new Date(), 1), DateUtil.FORMAT_1));
        if (PublicUtil.isEmpty(themeMap)) {
            log.error("账号：{}({})门店{}({})近期2个月暂无活动", account.getAccount(), account.getId(),
                    account.getShopName(), account.getShopId());
            return;
        }
        List<String> userDisqualificationWorkIds = disqualificationRecordBizService
                .getUserDisqualificationRecords(accountNo, WorksTypeEnum.LIVE, DateUtil.minusDays(new Date(), 1))
                .stream()
                .map(DisqualificationRecordEntity::getWorksId)
                .collect(Collectors.toList());
        for (LivePoolDto.LiveDTO liveDTO : liveList) {
            String roomNo = liveDTO.getRoomNo();
            String liveCover = liveDTO.getCover();
            Date liveStartTime = liveDTO.getLiveStartTime();
            String baseInfo = String.format("账户号为: %s, id为: %s, 开播时间为: %s, %s平台的直播", accountNo, roomNo,
                    DateUtil.format(liveStartTime, DateUtil.FORMAT_4), platformName);
            Set<String> worksTag = processTags(liveDTO.getTitle());// 直播标题包含话题
            List<ActivityTheme> joinThemes = this.getWorksJoinThemes(liveStartTime, thisMonthThemes);// 作品发布时间和活动主题生效时间匹配的活动主题
            if (CollectionUtils.isEmpty(joinThemes)) {
                log.info(String.format(" %s, 找不到开播时间期间参加的主题, 不进入直播池", baseInfo));
                continue;
            }
            List<Integer> invalidReasons = new ArrayList<>();
            LivePool livePool = this.setLiveBelong(account);// 设置直播归属
            TagCheckResult result = this.calcHitTheme(worksTag, joinThemes, shopId, WorksTypeEnum.LIVE);
            /* 话题验证 */
            if (!result.getContainsTag()) {
                if (!userDisqualificationWorkIds.contains(roomNo)) {
                    disqualificationRecordBizService.saveDisqualificationRecord(
                            disqualificationRecordBizService.createLiveDisqualificationRecord(account, result,
                                    liveDTO, DisqualificationEnum.TAGS)
                    );
                }
                log.info(String.format("%s, 不包含主题指定的话题, 视作无效直播", baseInfo));
                livePool.setValidLive(0);
                invalidReasons.add(result.getMatchMultipleTheme() ?
                        InvalidReasonEnum.MATCH_MULTIPLE_THEME.getValue() : InvalidReasonEnum.TAG.getValue());
            }
            /* 有无回播验证 */
            String playbackUrl = liveDTO.getPlaybackUrl();
            if (!Objects.equals(platformEnum, PlatformEnum.DY) && !StringUtils.hasText(playbackUrl)) {// 非 抖音 平台都校验回播
                if (!userDisqualificationWorkIds.contains(roomNo)) {
                    disqualificationRecordBizService.saveDisqualificationRecord(
                            disqualificationRecordBizService.createLiveDisqualificationRecord(account, result,
                                    liveDTO, DisqualificationEnum.LOSE_PLAYBACK)
                    );
                }
                log.info(String.format("%s, 无直播回放, 视作无效直播", baseInfo));
                invalidReasons.add(InvalidReasonEnum.LOSE_PLAYBACK.getValue());
                livePool.setValidLive(0);
            }
            Long configGroupId = result.getConfigGroupId();// 获取主题所处配置组id
            if (Objects.nonNull(configGroupId)) {// 只有找到了唯一主题, 才能进行'指标/封面/付费'验证环节
                ActivityTheme activityTheme = result.getHitTheme();// 作品命中活动主题
                /* 指标验证 */
                List<Integer> invalidIndexes = this.newValidLive(account, configGroupId, liveDTO);// 指标不满足原因
                invalidReasons.addAll(invalidIndexes);// 是否满足指标
                //boolean pay = this.validPay(liveDTO, configGroupId);
                if (!CollectionUtils.isEmpty(invalidIndexes)) {
                    livePool.setValidLive(0);
                    log.info(String.format("%s, 指标数据不满足集团标准, 视作无效直播", baseInfo));
                }
                /* 封面图验证 */
                if (!Objects.equals(platformEnum, PlatformEnum.DY)) {// 非 抖音 平台, 执行封面图校验(抖音平台将封面图处理了无法校验)
                    boolean validCover = false;
                    BigDecimal rate = this.getCoverConfigValue(configGroupId);
                    List<ThemeFile> list = themeFileService.lambdaQuery()
                            .eq(ThemeFile::getThemeId, activityTheme.getId())
                            .eq(ThemeFile::getType, FileTypeEnum.THEME_COVER.getValue())
                            .list();
                    int compareCnt = 0;
                    for (ThemeFile themeFile : list) {
                        try {
                            BigDecimal coverSimilarity = this.validCover(activityTheme.getTheme(), activityTheme.getId(),
                                    themeFile.getFileId(), liveCover, accountNo, roomNo, 2,
                                    StringUtils.hasText(playbackUrl) && Objects.equals(++compareCnt, 1));
                            if (validCover = coverSimilarity.compareTo(rate) >= 0)
                                break;
                        } catch (Exception e) {
                            log.error(String.format("%s, 验证封面图出现异常", baseInfo), e);
                            livePool.setValidLive(0);
                        }
                    }
                    if (!validCover) {
                        if (!userDisqualificationWorkIds.contains(roomNo)) {
                            disqualificationRecordBizService.saveDisqualificationRecord(
                                    disqualificationRecordBizService.createLiveDisqualificationRecord(account, result,
                                            liveDTO, DisqualificationEnum.COVER)
                            );
                        }
                        log.info(String.format("%s, 封面图不匹配, 视作无效直播", baseInfo));
                        invalidReasons.add(InvalidReasonEnum.COVER.getValue());
                        livePool.setValidLive(0);
                    }
                }
                /* 是否付费直播验证 */
                Optional<ValidConfigNew> first = this.getValidConfigList(configGroupId, ValidConfigTypeEnum.LIVE)
                        .stream()
                        .filter(item -> Objects.equals(item.getIndexType(), ValidLiveEnum.WATCH_USER_PEAK.getValue()))
                        .findFirst();// 获取观看巅峰配置
                if (liveDTO.getUvPeak() >= liveDTO.getUv() &&
                        liveDTO.getUvPeak() >= (first.isPresent() ? first.get().getMinValue() : 30)) {
                    invalidReasons.add(InvalidReasonEnum.PAY.getValue());
                    livePool.setValidLive(-1);
                    log.info(String.format("%s, 系统判定直播观看巅峰数据异常, 视作无效直播", baseInfo));
                }
            }
            livePool.setAccountId(account.getId());
            livePool.setPlatform(platformEnum);
            livePool.setRoomNo(roomNo);
            livePool.setPlaybackUrl(liveDTO.getPlaybackUrl());
            livePool.setUserNick(liveDTO.getUserNick());
            livePool.setTitle(liveDTO.getTitle());
            livePool.setCover(liveCover);
            livePool.setUv(liveDTO.getUv());
            livePool.setUvPeak(liveDTO.getUvPeak());
            livePool.setLikeNum(liveDTO.getLikeNum());
            livePool.setShareNum(liveDTO.getShareNum());
            livePool.setCommentNum(liveDTO.getCommentNum());
            livePool.setNewFanNum(liveDTO.getNewFanNum());
            livePool.setReceiveNum(liveDTO.getReceiveNum());
            livePool.setReceiveAmount(liveDTO.getReceiveAmount());
            livePool.setLiveDuration(liveDTO.getLiveDuration());
            livePool.setLiveStartTime(liveStartTime);
            livePool.setLiveEndTime(liveDTO.getLiveEndTime());
            livePool.setThemeId(Objects.nonNull(result.getThemeId()) ? result.getThemeId() : -1);
            livePool.setTheme(result.getThemeName());
            livePool.setTags(collection2String(worksTag));
            if (CollectionUtils.isEmpty(invalidReasons))
                livePool.setValidLive(1);
            else
                livePool.setInvalidReason(this.invalidReason2String(invalidReasons));// 无效理由
            waitSaveWorks.add(livePool);
        }
        livePoolService.removeAccountDataByDate(account.getId(), today);
        livePoolService.saveBatch(waitSaveWorks);
    }

    /**
     * 将无效原因数组转为字符串
     */
    public String invalidReason2String(List<Integer> invalidType) {
        return collection2String(invalidType.stream()
                .sorted()
                .collect(Collectors.toList())
        );
    }

    /**
     * 计算作品发布时间正在参与的活动主题列表
     *
     * @param publishTime 作品发布时间
     * @param themeList   月活动主题列表
     * @return 作品发布时间正在参与的活动主题列表
     */
    private List<ActivityTheme> getWorksJoinThemes(Date publishTime, List<ActivityTheme> themeList) {
        return themeList.stream()
                .filter(item -> DateUtil.timePointInRange(publishTime, item.getStartTime(), item.getEndTime()))
                .collect(Collectors.toList());// 作品发布时间和活动主题生效时间匹配的活动主题
    }

    /**
     * 计算作品命中主题
     *
     * @param worksTag   作品话题
     * @param joinThemes 门店此刻参加的活动主题列表
     * @param shopId     门店id
     * @param worksType  作品类型
     * @return 话题校验结果及命中主题
     */
    private TagCheckResult calcHitTheme(Set<String> worksTag, List<ActivityTheme> joinThemes,
                                        Long shopId, WorksTypeEnum worksType) {
        TagCheckResult result = new TagCheckResult();
        if (CollectionUtils.isEmpty(joinThemes))
            return result;
        List<TagCheckResult> checkResults = joinThemes.stream()
                .map(item -> this.validTags(item, worksTag, shopId, worksType))
                .collect(Collectors.toList());
        if (Objects.equals(checkResults.size(),1))
            return checkResults.get(0);
        List<TagCheckResult> validResult = checkResults.stream()
                .filter(TagCheckResult::getContainsTag)
                .collect(Collectors.toList());
        if (Objects.equals(validResult.size(), 1)) {
            result = validResult.get(0);
        } else if (validResult.size() > 1) {
            result.setMatchMultipleTheme(true);// 检测匹配到多主题
        } else {
            List<TagCheckResult> results = checkResults.stream()
                    .filter(item -> !CollectionUtils.isEmpty(item.getValidTags()))
                    .sorted(Comparator.comparing(item -> item.getValidTags().size()))
                    .collect(Collectors.toList());
            if (!CollectionUtils.isEmpty(results))
                result = results.get(results.size() - 1);
        }
        return result;
    }

    /**
     * new校验该视频是否符合集团配置的标准
     *
     * @param configGroupId 所属配置组id
     * @param video         视频信息
     * @return 该条视频是否符合集团配置的标准
     */
    public List<Integer> newValidVideo(Account account, Long configGroupId, VideoPoolDto.VideoDTO video) {
        String accountNo = account.getAccount();
        PlatformEnum platformEnum = account.getType();
        String platformName = platformEnum.getName();
        String videoId = video.getVideoId();
        Map<Integer, ValidConfigNew> configMap = this.getValidConfigList(configGroupId, ValidConfigTypeEnum.VIDEO)
                .stream()
                .collect(Collectors.toMap(ValidConfigNew::getIndexType, Function.identity()));// 获取配置组的短视频检验最低标准指标
        LocalDate now = LocalDate.now();
        String format = String.format("%s 账户号为: %s, id为: %s, %s平台的视频", now, accountNo, videoId, platformName);
        List<Integer> videoCheckResult = new ArrayList<>();
        ValidConfigNew playCntConfig = configMap.get(ValidVideoEnum.PLAY_CNT.getValue());// 播放数
        if (Objects.nonNull(playCntConfig)) {
            if (playCntConfig.getValid()) {
                Integer playCnt = video.getUv();
                if (Objects.isNull(playCnt)) {
                    log.error(String.format("%s[播放数]数据为空", format));
                } else {
                    Integer minValue = playCntConfig.getMinValue();
                    if (playCnt < minValue) {
                        log.info(String.format("%s[播放数]数据不满足集团配置标准(最低: %s)", format, minValue));
                        videoCheckResult.add(InvalidReasonEnum.PLAY_CNT.getValue());
                    }
                }
            }
        }
        ValidConfigNew fullPlayCntConfig = configMap.get(ValidVideoEnum.FULL_PLAY_CNT.getValue());// 完整播放数
        if (Objects.nonNull(fullPlayCntConfig)) {
            if (fullPlayCntConfig.getValid()) {
                Integer fullPlayCnt = video.getFullUv();
                if (Objects.isNull(fullPlayCnt)) {
                    log.error(String.format("%s[完整播放数]数据为空", format));
                } else {
                    Integer minValue = fullPlayCntConfig.getMinValue();
                    if (fullPlayCnt < minValue) {
                        log.info(String.format("%s[完整播放数]数据不满足集团配置标准(最低: %s)", format, minValue));
                        videoCheckResult.add(InvalidReasonEnum.FULL_PLAY_CNT.getValue());
                    }
                }
            }
        }
        ValidConfigNew likeCntConfig = configMap.get(ValidVideoEnum.LIKE_CNT.getValue());// 点赞数
        if (Objects.nonNull(likeCntConfig)) {
            if (likeCntConfig.getValid()) {
                Integer likeCnt = video.getLikeNum();
                if (Objects.isNull(likeCnt)) {
                    log.error(String.format("%s[点赞数]数据为空", format));
                } else {
                    Integer minValue = likeCntConfig.getMinValue();
                    if (likeCnt < minValue) {
                        log.info(String.format("%s[点赞数]数据不满足集团配置标准(最低: %s)", format, minValue));
                        videoCheckResult.add(InvalidReasonEnum.LIKE_CNT.getValue());
                    }
                }
            }
        }
        ValidConfigNew commentCntConfig = configMap.get(ValidVideoEnum.COMMENT_CNT.getValue());// 评论数
        if (Objects.nonNull(commentCntConfig)) {
            if (commentCntConfig.getValid()) {
                Integer commentCnt = video.getCommentNum();
                if (Objects.isNull(commentCnt)) {
                    log.error(String.format("%s[评论数]数据为空", format));
                } else {
                    Integer minValue = commentCntConfig.getMinValue();
                    if (commentCnt < minValue) {
                        log.info(String.format("%s[评论数]数据不满足集团配置标准(最低: %s)", format, minValue));
                        videoCheckResult.add(InvalidReasonEnum.COMMENT_CNT.getValue());
                    }
                }
            }
        }
        ValidConfigNew shareCntConfig = configMap.get(ValidVideoEnum.SHARE_CNT.getValue());// 分享数
        if (Objects.nonNull(shareCntConfig)) {
            if (shareCntConfig.getValid()) {
                Integer shareCnt = video.getShareNum();
                if (Objects.isNull(shareCnt)) {
                    log.error(String.format("%s[分享数]数据为空", format));
                } else {
                    Integer minValue = shareCntConfig.getMinValue();
                    if (shareCnt < minValue) {
                        log.info(String.format("%s[分享数]数据不满足集团配置标准(最低: %s)", format, minValue));
                        videoCheckResult.add(InvalidReasonEnum.SHARE_CNT.getValue());
                    }
                }
            }
        }
        if (!Objects.equals(PlatformEnum.BILIBILI.getValue(), platformEnum.getValue())) {// bilibili视频[不过滤]增粉指标
            ValidConfigNew newFansCntConfig = configMap.get(ValidVideoEnum.NEW_FANS_CNT.getValue());// 增粉数
            if (Objects.nonNull(newFansCntConfig)) {
                if (newFansCntConfig.getValid()) {
                    Integer newFansCnt = video.getNewFanNum();
                    if (Objects.isNull(newFansCnt)) {
                        log.error(String.format("%s[增粉数]数据为空", format));
                    } else {
                        Integer minValue = newFansCntConfig.getMinValue();
                        if (newFansCnt < minValue) {
                            log.info(String.format("%s[增粉数]数据不满足集团配置标准(最低: %s)", format, minValue));
                            videoCheckResult.add(InvalidReasonEnum.NEW_FANS_CNT.getValue());
                        }
                    }
                }
            }
        }
        Integer duration = video.getDuration();// 短视频时长
        if (Objects.isNull(duration)) {
            log.error(String.format("%s[短视频时长]数据为空", format));
        } else {
            Integer minValue = 20;
            if (duration < minValue) {
                log.info(String.format("%s[短视频时长]数据不满足集团配置标准(最低: %s)", format, minValue));
                videoCheckResult.add(InvalidReasonEnum.VIDEO_DURATION.getValue());
            }
        }
        return videoCheckResult;
    }

    /**
     * 观看巅峰数据异常判定
     *
     * @param live          直播数据
     * @param configGroupId 配置组id
     * @return 数据是否异常
     */
    /*private boolean validPay(LivePoolDto.LiveDTO live, Long configGroupId) {
        ValidConfigNew config = validConfigNewService.getOne(Wrappers.<ValidConfigNew>lambdaQuery()
                .eq(ValidConfigNew::getConfigGroupId, configGroupId)
                .eq(ValidConfigNew::getType, WorksTypeEnum.LIVE.getValue())
                .eq(ValidConfigNew::getIndexType, ValidLiveEnum.PAY_LIVE_RATE.getValue()));
        if (Objects.equals(config.getValid(), 0))
            return false;
        else
            return live.getUvPeak() * config.getMinValue() >= live.getUv();
    }*/

    /**
     * new校验该直播是否符合集团配置的标准
     *
     * @param account       账户信息
     * @param configGroupId 所属配置组id
     * @param live          直播信息
     * @return 该次直播是否符合集团配置的标准
     */
    public List<Integer> newValidLive(Account account, Long configGroupId, LivePoolDto.LiveDTO live) {
        String accountNo = account.getAccount();
        PlatformEnum platformEnum = account.getType();
        String platformName = platformEnum.getName();
        String roomNo = live.getRoomNo();
        Map<Integer, ValidConfigNew> configMap = this.getValidConfigList(configGroupId, ValidConfigTypeEnum.LIVE)
                .stream()
                .collect(Collectors.toMap(ValidConfigNew::getIndexType, Function.identity()));// 获取配置组的直播检验最低标准指标
        String format = String.format("账户号为: %s, id为: %s, %s平台的直播", accountNo, roomNo, platformName);
        List<Integer> liveCheckResult = new ArrayList<>();
        if (!Objects.equals(PlatformEnum.BILIBILI.getValue(), platformEnum.getValue())) {// bilibili直播[不过滤]观看人数
            ValidConfigNew watchCntConfig = configMap.get(ValidLiveEnum.WATCH_USER.getValue());// 观看人数
            if (Objects.nonNull(watchCntConfig)) {
                if (watchCntConfig.getValid()) {
                    Integer watchCnt = live.getUv();
                    if (Objects.isNull(watchCnt)) {
                        log.error(String.format("%s[观看人数]数据为空", format));
                    } else {
                        Integer minValue = watchCntConfig.getMinValue();
                        if (watchCnt < minValue) {
                            log.info(String.format("%s[观看人数]数据不满足集团配置标准(最低: %s)", format, minValue));
                            liveCheckResult.add(InvalidReasonEnum.WATCH_USER.getValue());
                        }
                    }
                }
            }
        }
        if (!Objects.equals(PlatformEnum.BILIBILI.getValue(), platformEnum.getValue())) {// bilibili直播[不过滤]观看人数巅峰
            ValidConfigNew watchPeakCntConfig = configMap.get(ValidLiveEnum.WATCH_USER_PEAK.getValue());// 观看人数峰值
            if (Objects.nonNull(watchPeakCntConfig)) {
                if (watchPeakCntConfig.getValid()) {
                    Integer watchCntPeak = live.getUvPeak();
                    if (Objects.isNull(watchCntPeak)) {
                        log.error(String.format("%s[观看人数巅峰]数据为空", format));
                    } else {
                        Integer minValue = watchPeakCntConfig.getMinValue();
                        if (watchCntPeak < minValue) {
                            log.info(String.format("%s[观看人数峰值]数据不满足集团配置标准(最低: %s)", format, minValue));
                            liveCheckResult.add(InvalidReasonEnum.WATCH_USER_PEAK.getValue());
                        }
                    }
                }
            }
        }
        if (Objects.equals(PlatformEnum.KS.getValue(), platformEnum.getValue())) {// 快手直播[才过滤]点赞指标
            ValidConfigNew likeCntConfig = configMap.get(ValidLiveEnum.LIKE_CNT.getValue());// 点赞数
            if (Objects.nonNull(likeCntConfig)) {
                if (likeCntConfig.getValid()) {
                    Integer likeCnt = live.getLikeNum();
                    if (Objects.isNull(likeCnt)) {
                        log.error(String.format("%s[点赞数]数据为空", format));
                    } else {
                        Integer minValue = likeCntConfig.getMinValue();
                        if (likeCnt < minValue) {
                            log.info(String.format("%s[点赞数]数据不满足集团配置标准(最低: %s)", format, minValue));
                            liveCheckResult.add(InvalidReasonEnum.LIKE_CNT.getValue());
                        }
                    }
                }
            }
        }
        ValidConfigNew commentCntConfig = configMap.get(ValidLiveEnum.COMMENT_CNT.getValue());// 评论数
        if (Objects.nonNull(commentCntConfig)) {
            if (commentCntConfig.getValid()) {
                Integer commentCnt = live.getCommentNum();
                if (Objects.isNull(commentCnt)) {
                    log.error(String.format("%s[评论数]数据为空", format));
                } else {
                    Integer minValue = commentCntConfig.getMinValue();
                    if (commentCnt < minValue) {
                        log.info(String.format("%s[评论数]数据不满足集团配置标准(最低: %s)", format, minValue));
                        liveCheckResult.add(InvalidReasonEnum.COMMENT_CNT.getValue());
                    }
                }
            }
        }
        if (Objects.equals(PlatformEnum.KS.getValue(), platformEnum.getValue())) {// 快手直播[才过滤]分享指标
            ValidConfigNew shareCntConfig = configMap.get(ValidLiveEnum.SHARE_CNT.getValue());// 分享数
            if (Objects.nonNull(shareCntConfig)) {
                if (shareCntConfig.getValid()) {
                    Integer shareCnt = live.getShareNum();
                    if (Objects.isNull(shareCnt)) {
                        log.error(String.format("%s[分享数]数据为空", format));
                    } else {
                        Integer minValue = shareCntConfig.getMinValue();
                        if (shareCnt < minValue) {
                            log.info(String.format("%s[分享数]数据不满足集团配置标准(最低: %s)", format, minValue));
                            liveCheckResult.add(InvalidReasonEnum.SHARE_CNT.getValue());
                        }
                    }
                }
            }
        }
        ValidConfigNew newFansCntConfig = configMap.get(ValidLiveEnum.NEW_FANS_CNT.getValue());// 增粉数
        if (Objects.nonNull(newFansCntConfig)) {
            if (newFansCntConfig.getValid()) {
                Integer newFanCnt = live.getNewFanNum();
                if (Objects.isNull(newFanCnt)) {
                    log.error(String.format("%s[增粉数]数据为空", format));
                } else {
                    Integer minValue = newFansCntConfig.getMinValue();
                    if (newFanCnt < minValue) {
                        log.info(String.format("%s[增粉数]数据不满足集团配置标准(最低: %s)", format, minValue));
                        liveCheckResult.add(InvalidReasonEnum.NEW_FANS_CNT.getValue());
                    }
                }
            }
        }
        ValidConfigNew durationCntConfig = configMap.get(ValidLiveEnum.LIVE_DURATION.getValue());// 直播时长
        if (Objects.nonNull(durationCntConfig)) {
            if (durationCntConfig.getValid()) {
                Integer liveDuration = live.getLiveDuration();
                if (Objects.isNull(liveDuration)) {
                    log.error(String.format("%s[直播时长]数据为空", format));
                } else {
                    Integer minValue = durationCntConfig.getMinValue();
                    if (liveDuration < minValue) {
                        log.info(String.format("%s[直播时长]数据不满足集团配置标准(最低: %s)", format, minValue));
                        liveCheckResult.add(InvalidReasonEnum.LIVE_DURATION.getValue());
                    }
                }
            }
        }
        return liveCheckResult;
    }

    /**
     * 设置视频归属信息
     *
     * @param account    账户实体
     * @param existVideo 该id对应视频实体首次数据
     * @return 视频归属信息
     */
    private VideoPool setVideoBelong(Account account, VideoPool existVideo) {
        VideoPool videoPool = new VideoPool();
        if (Objects.nonNull(existVideo)) {
            videoPool.setBelongUserId(existVideo.getBelongUserId());
            videoPool.setBelongShopId(existVideo.getBelongShopId());
            videoPool.setBelongUserName(existVideo.getBelongUserName());
            videoPool.setFirstManagerId(existVideo.getFirstManagerId());
            videoPool.setSecondManagerId(existVideo.getSecondManagerId());
            videoPool.setThirdManagerId(existVideo.getThirdManagerId());
            videoPool.setFourthManagerId(existVideo.getFourthManagerId());
            videoPool.setScore(existVideo.getScore());
            videoPool.setThemeScore(existVideo.getThemeScore());
            videoPool.setScoreCalJson(existVideo.getScoreCalJson());
        } else {
            Long groupId = account.getGroupId();
            Long shopId = account.getShopId();
            videoPool.setBelongUserId(account.getUserId());
            videoPool.setBelongUserName(account.getUserName());
            videoPool.setBelongShopId(shopId);
            videoPool.setFirstManagerId(localCacheService.getBelongManagerId(groupId, shopId, RoleMsgEnum.XSZG));
            videoPool.setSecondManagerId(localCacheService.getBelongManagerId(groupId, shopId, RoleMsgEnum.XSJL));
            videoPool.setThirdManagerId(localCacheService.getBelongManagerId(groupId, shopId, RoleMsgEnum.XSZJL));
            videoPool.setFourthManagerId(localCacheService.getBelongManagerId(groupId, shopId, RoleMsgEnum.XSSJGL));
            videoPool.setScore(BigDecimal.ZERO);
            videoPool.setThemeScore(BigDecimal.ZERO);
        }
        return videoPool;
    }

    /**
     * 设置直播归属信息
     *
     * @param account 账户实体
     * @return 直播归属信息
     */
    private LivePool setLiveBelong(Account account) {
        LivePool livePool = new LivePool();
        Long groupId = account.getGroupId();
        Long shopId = account.getShopId();
        livePool.setBelongUserId(account.getUserId());
        livePool.setBelongShopId(shopId);
        livePool.setFirstManagerId(localCacheService.getBelongManagerId(groupId, shopId, RoleMsgEnum.XSZG));
        livePool.setSecondManagerId(localCacheService.getBelongManagerId(groupId, shopId, RoleMsgEnum.XSJL));
        livePool.setThirdManagerId(localCacheService.getBelongManagerId(groupId, shopId, RoleMsgEnum.XSZJL));
        livePool.setFourthManagerId(localCacheService.getBelongManagerId(groupId, shopId, RoleMsgEnum.XSSJGL));
        return livePool;
    }

    /**
     * 将标题中的话题拿出来
     *
     * @param title 标题
     * @return 话题集合
     */
    private static Set<String> processTags(String title) {
        title = title
                .replace("＃", "#")
                .replace(" ", "")
                .replace("\n", "")
                .replace("—", "一");
        String tagsStr = String.join("", cn.fw.dalaran.common.utils.StringUtils.matchs(title, cn.fw.dalaran.common.utils.StringUtils.tagsPattern));
        return Arrays.stream(tagsStr.split("#"))
                .map(item -> item.replace("#", ""))
                .filter(item -> item.length() > 0)
                .collect(Collectors.toSet());
    }

    /**
     * 集合转换成字符串
     *
     * @param collection 输入集合
     * @return 字符串类型的集合
     */
    public static <Q> String collection2String(Collection<Q> collection) {
        if (CollectionUtils.isEmpty(collection))
            return "";
        return JSON.toJSONString(collection)
                .replace("[", "")
                .replace("]", "")
                .replace("\"", "");
    }

    /**
     * 验证标题的话题
     *
     * @param joinTheme 参加的活动主题列表
     * @param worksTag  作品包含话题集合
     * @param shopId    门店id
     * @param worksType 作品类型
     * @return 该视频/直播包含话题是否符合主题配置的话题
     */
    private TagCheckResult validTags(ActivityTheme joinTheme, Set<String> worksTag,
                                     Long shopId, WorksTypeEnum worksType) {
        ValidConfigNew config = this.getValidConfig(joinTheme.getConfigGroupId(), ValidConfigTypeEnum.TOPIC,
                ValidIndexTypeEnum.TOPIC_SIMILARITY);// 获取话题校验相关配置
        TagCheckResult result = new TagCheckResult();
        result.setThemeId(joinTheme.getId());// 主题id
        result.setThemeName(joinTheme.getTheme());// 主题名
        result.setHitTheme(joinTheme);// 主题
        result.setConfigGroupId(joinTheme.getConfigGroupId());// 所属配置组id
        if (Objects.nonNull(config) && !config.getValid()) {// 配置不过滤话题
            result.setContainsTag(true);
            return result;
        }
        Set<Long> shopIds = new HashSet<>();
        Set<TopicVo> configTags = new HashSet<>();// 主题配置话题
        Set<TopicVo> forceTags = new HashSet<>();// 主题强校验话题
        Set<TopicVo> validTags = new HashSet<>();// 匹配率达标话题
        // 找到活动主题的所有话题, 覆盖的所有门店
        JSON.parseArray(joinTheme.getTopic(), TopicVo.class).forEach(item -> {
            configTags.add(item);
            if (Objects.equals(item.getForcedCheck(), true))
                forceTags.add(item);
        });
        if (Objects.equals(joinTheme.getAllShop(), 0) && StringUtils.hasText(joinTheme.getShopJson()))
            shopIds.addAll(JSON.parseArray(joinTheme.getShopJson(), ShopDto.class)
                    .stream()
                    .map(ShopDto::getShopId)
                    .collect(Collectors.toSet())
            );
        if (joinTheme.getAllShop() > 0 || shopIds.contains(shopId)) {
            BigDecimal rate = PublicUtil.isEmpty(config.getMinValue()) ? BigDecimal.ZERO :
                    BigDecimal.valueOf(config.getMinValue());// 话题配置匹配率
            boolean completeMatching = Objects.equals(rate.compareTo(BigDecimal.valueOf(100)), 0);// 100%匹配
            for (TopicVo configTag : configTags) {
                for (String tag : worksTag) {
                    if (completeMatching) {// 全匹配
                        if (Objects.equals(configTag.getTopic(), tag)) {
                            validTags.add(configTag);
                            break;
                        }
                    } else {// LCS计算话题匹配度
                        if (this.LCSCalcTopicSimilarity(rate, configTag.getTopic(), tag)) {
                            validTags.add(configTag);
                            break;
                        }
                    }
                }
            }
            boolean containsTag = true;
            if (CollectionUtils.isEmpty(forceTags)) {
                containsTag = !CollectionUtils.isEmpty(validTags);
            } else {
                if (Objects.equals(worksType, WorksTypeEnum.VIDEO)) {// 短视频校验所有强制话题
                    for (TopicVo forceTag : forceTags) {// 遍历强制校验话题
                        boolean nonMatch = validTags.stream()// 遍历匹配率达标话题
                                .noneMatch(item -> Objects.equals(item.getTopic(), forceTag.getTopic()));
                        if (nonMatch) {
                            containsTag = false;
                            break;
                        }
                    }
                } else if (Objects.equals(worksType, WorksTypeEnum.LIVE)) {// 直播校验任一强制话题
                    containsTag = false;
                    for (TopicVo forceTag : forceTags) {// 遍历强制校验话题
                        boolean anyMatch = validTags.stream()// 遍历匹配率达标话题
                                .anyMatch(item -> Objects.equals(item.getTopic(), forceTag.getTopic()));
                        if (anyMatch) {
                            containsTag = true;
                            break;
                        }
                    }
                }
            }
            result.setContainsTag(containsTag);
        } else {
            result.setContainsTag(false);
        }
        result.setConfigTags(configTags.stream()
                .map(TopicVo::getTopic)
                .collect(Collectors.toSet())
        );// 主题配置tags
        result.setWorksTags(worksTag);// 作品包含tags
        result.setValidTags(validTags);// 匹配率达标话题
        return result;
    }

    /**
     * LCS计算话题匹配度
     *
     * @param setSimilarity 设置匹配度
     * @param setTopic      主题设置话题
     * @param userTopic     用户话题
     * @return 话题匹配度是否合格
     */
    private boolean LCSCalcTopicSimilarity(BigDecimal setSimilarity, String setTopic, String userTopic) {
        int length = setTopic.length();
        BigDecimal single = BigDecimal.valueOf(100).divide(BigDecimal.valueOf(length), 4, RoundingMode.HALF_UP);// 单个字占比
        int m = setTopic.length();
        int n = userTopic.length();
        int[][] c = new int[m + 1][n + 1];
        for (int i = 1; i < m + 1; i++) {
            for (int j = 1; j < n + 1; j++) {
                if (setTopic.charAt(i - 1) == userTopic.charAt(j - 1))
                    c[i][j] = c[i - 1][j - 1] + 1;
                else
                    c[i][j] = Math.max(c[i - 1][j], c[i][j - 1]);
            }
        }
        return c[setTopic.length()][userTopic.length()] >= setSimilarity.divideAndRemainder(single)[0].intValue();
    }

    /**
     * 验证封面匹配度
     *
     * @param theme      主题名
     * @param themeId    主题id
     * @param fileId     文件id(主题配置指定封面图文件)
     * @param coverUrl   用户作品封面图地址
     * @param account    账户号
     * @param itemId     作品id
     * @param type       类型(1:视频, 2:直播)
     * @param latestLive 最新直播数据(已经拿到回播)
     * @return 封面图匹配度
     * @throws Exception
     */
    public BigDecimal validCover(String theme, Long themeId, String fileId, String coverUrl, String account,
                                 String itemId, Integer type, Boolean latestLive) throws Exception {
        int[][] imagePixArr1;// 图片1的像素数组
        int[][] imagePixArr2;// 图片2的像素数组
        File imageFile1 = this.findLocalCacheImageTheme(/*theme,*/ themeId, fileId);
        imagePixArr1 = ImageUtils.readImagePixel(imageFile1);
        File imageFile2;
        File localCacheImage = this.findLocalCacheImageUser(/*theme,*/ themeId, account, itemId, type, latestLive);
        if (Objects.nonNull(localCacheImage)) {
            imageFile2 = localCacheImage;
        } else {
            imageFile2 = ImageUtils.convertFileByUrl(coverUrl, itemId);
            String tempDir = common.getActivityThemeCoverDir();
            File file = new File(ImageUtils.modifyResolution1(imageFile2.getPath(),
                    tempDir + "activityTheme" + File.separator + themeId /*+ "#" + theme*/ + File.separator + account + File.separator + (Objects.equals(type, 1) ? "video" : "live"),
                    "fw_theme_cover_" + itemId,
                    512, 512));
            if (imageFile2.delete()) {
                imageFile2 = file;
            }
        }
        imagePixArr2 = ImageUtils.readImagePixel(imageFile2);
        String msg = String.format("\n 主题id: %s \n 主题名: %s\n 设置封面文件id: %s\n 账户号: %s\n %s%s\n", themeId, theme, fileId, account, Objects.equals(type, 1) ? "短视频id: " : "直播间号: ", itemId);
        return BigDecimal.valueOf(ImageUtils.calcSimilarity(ImageUtils.getFingerprint(imagePixArr1), ImageUtils.getFingerprint(imagePixArr2), msg));
    }

    /**
     * 获取本地已下载的封面图(用户作品封面图)
     *
     * @param themeId    主题id
     * @param account    账号
     * @param itemId     视频id/直播间id
     * @param type       1:视频, 2:直播
     * @param latestLive 最新直播数据(已经拿到回播)
     */
    private File findLocalCacheImageUser(/*String theme,*/ Long themeId, String account, String itemId, Integer type,
                                                           Boolean latestLive) {
        String tempDir = common.getActivityThemeCoverDir();
        File file = new File(tempDir + File.separator + "activityTheme" + File.separator + themeId /*+ "#" + theme*/ +
                File.separator + account + File.separator + (Objects.equals(type, 1) ? "video" : "live"));
        if (!file.exists()) {
            boolean mkdirs = file.mkdirs();
            if (mkdirs)
                return null;
        }
        File imageFile = null;
        for (File item : Objects.requireNonNull(file.listFiles())) {
            if (item.getName().startsWith("fw_theme_cover_" + itemId)) {
                imageFile = item;
                break;
            }
        }
        if (latestLive && Objects.nonNull(imageFile) && file.exists()) {
            imageFile.delete();
            return null;
        }
        return imageFile;
    }

    /**
     * 获取本地已下载的主题背景图(主题预设封面图)
     *
     * @param themeId 主题id
     * @param fileId  文件id(主题配置指定封面图文件)
     */
    private File findLocalCacheImageTheme(/*String theme,*/ Long themeId, String fileId) {
        String tempDir = common.getActivityThemeCoverDir();
        File file = new File(tempDir + "activityTheme" + File.separator + themeId /*+ "#" + theme*/ + File.separator + "settingCover");
        if (!file.exists()) {
            boolean mkdirs = file.mkdirs();
            if (mkdirs)
                return null;
        }
        File imageFile = null;
        for (File item : Objects.requireNonNull(file.listFiles())) {
            if (item.getName().startsWith("fw_theme_cover_" + fileId)) {
                imageFile = item;
                break;
            }
        }
        if (Objects.isNull(imageFile)) {

        }
        return imageFile;
    }

    /**
     * 获取集团对应门店近期2个月参加的活动主题列表(优先查缓存)
     *
     * @param groupId 集团id
     * @param shopId  门店id
     * @return K:yyyy-MM, V:集团活动主题(全集团, 该门店)
     */
    public Map<String, List<ActivityTheme>> getGroupActivityTheme(Long groupId, Long shopId) {
        String key = groupId + "#" + shopId;// 构建缓存key
        String themeMapJson = autoExpireLocalCache.get(key);
        Map<String, List<ActivityTheme>> themeMap;
        if (StringUtils.hasText(themeMapJson))
            return this.string2Map(themeMapJson);
        final ReentrantLock lock = this.lock;
        lock.lock();
        try {
            themeMapJson = autoExpireLocalCache.get(key);// 双重检查, 防止多次查数据库
            if (StringUtils.hasText(themeMapJson))
                return this.string2Map(themeMapJson);
            themeMap = this.getGroupActivityThemeFromDb(groupId, shopId);
            autoExpireLocalCache.put(key, JSON.toJSONString(themeMap), TimeUnit.SECONDS.toMillis(120));// 2分钟过期
        } finally {
            lock.unlock();
        }
        return themeMap;
    }

    /**
     * json字符串转为Map<String, List<ActivityTheme>>
     *
     * @param themeMapJson 待转换json字符串
     * @return 结果
     */
    @SuppressWarnings("unchecked")
    private Map<String, List<ActivityTheme>> string2Map(String themeMapJson) {
        Map<String, JSONArray> map = JSON.parseObject(themeMapJson, Map.class);
        Map<String, List<ActivityTheme>> themeMap = new HashMap<>();
        if (!CollectionUtils.isEmpty(map))
            map.forEach((k, v) -> themeMap.put(k, v.toJavaList(ActivityTheme.class)));
        return themeMap;
    }

    /**
     * 获取集团对应门店近期2个月参加的活动主题列表(查数据库)
     *
     * @param groupId 集团id
     * @param shopId  门店id
     * @return K:yyyy-MM, V:集团活动主题(全集团, 该门店)
     */
    public Map<String, List<ActivityTheme>> getGroupActivityThemeFromDb(Long groupId, Long shopId) {
        Date now = new Date();
        Date yesterdayNow = DateUtil.minusDays(now, 1);
        Date monthBegin = DateUtil.monthBegin(DateUtil.minusMonths(yesterdayNow, 1));
        Date monthEnd = DateUtil.monthEnd(yesterdayNow);
        log.info("monthBegin: {}  monthEnd: {}", DateUtil.format(monthBegin, DateUtil.FORMAT_4),
                DateUtil.format(monthEnd, DateUtil.FORMAT_4));
        return activityThemeService.lambdaQuery()
                .select(ActivityTheme::getId, ActivityTheme::getConfigGroupId, ActivityTheme::getGroupId,
                        ActivityTheme::getTheme, ActivityTheme::getTopic, ActivityTheme::getStartTime,
                        ActivityTheme::getEndTime, ActivityTheme::getAllShop, ActivityTheme::getShopJson)
                .eq(ActivityTheme::getGroupId, groupId)
                .between(ActivityTheme::getEndTime, monthBegin, monthEnd)
                .list()
                .stream()
                .filter(item -> item.getStartTime().before(now))// 已经开始了的主题
                .filter(item -> Objects.equals(item.getAllShop(), 1) || JSON.parseArray(item.getShopJson(), ShopDto.class)
                        .stream()
                        .anyMatch(item1 -> Objects.equals(item1.getShopId(), shopId))// 找出门店要参加的活动主题
                )
                .collect(Collectors.groupingBy(item -> DateUtil.format(item.getStartTime(), DateUtil.FORMAT_1)))
                ;
    }

    /**
     * 获取有效性判断配置列表(优先查缓存)
     *
     * @param configGroupId 配置组id
     * @param type          类型
     * @return 有效性判断配置列表
     */
    private List<ValidConfigNew> getValidConfigList(Long configGroupId, ValidConfigTypeEnum type) {
        String key = configGroupId + "->" + type.getValue();// 构建缓存key
        String configJson = autoExpireLocalCache.get(key);
        List<ValidConfigNew> configs;
        if (StringUtils.hasText(configJson)) {
            configs = JSON.parseArray(configJson, ValidConfigNew.class);
            if (!CollectionUtils.isEmpty(configs))
                return configs;
        }
        final ReentrantLock lock = this.lock;
        lock.lock();
        try {
            configJson = autoExpireLocalCache.get(key);// 双重检查, 防止多次查数据库
            if (StringUtils.hasText(configJson)) {
                lock.unlock();
                configs = JSON.parseArray(configJson, ValidConfigNew.class);
                if (!CollectionUtils.isEmpty(configs))
                    return configs;
            }
            configs = validConfigNewService
                    .queryList(configGroupId, type.getValue())
                    .getDetails()
                    .stream()
                    .map(ValidConfigNewVo::toDB)
                    .collect(Collectors.toList());
            autoExpireLocalCache.put(key, JSON.toJSONString(configs), TimeUnit.SECONDS.toMillis(120));// 2分钟过期
        } finally {
            lock.unlock();
        }
        return configs;
    }

    /**
     * 获取封面/话题校验相关配置(优先查缓存)
     */
    private ValidConfigNew getValidConfig(Long configGroupId, ValidConfigTypeEnum type, ValidIndexTypeEnum subType) {
        String key = configGroupId + "->" + type.getValue() + "#" + subType.getValue();// 构建缓存key
        String configJson = autoExpireLocalCache.get(key);
        List<ValidConfigNew> validConfigNews;
        if (StringUtils.hasText(configJson)) {
            validConfigNews = JSON.parseArray(configJson, ValidConfigNew.class);
            if (!CollectionUtils.isEmpty(validConfigNews))
                return validConfigNews.get(0);
        }
        final ReentrantLock lock = this.lock;
        lock.lock();
        try {
            configJson = autoExpireLocalCache.get(key);// 双重检查, 防止多次查数据库
            if (StringUtils.hasText(configJson)) {
                lock.unlock();
                validConfigNews = JSON.parseArray(configJson, ValidConfigNew.class);
                if (!CollectionUtils.isEmpty(validConfigNews))
                    return validConfigNews.get(0);
            }
            validConfigNews = validConfigNewService.list(Wrappers.<ValidConfigNew>lambdaQuery()
                    .eq(ValidConfigNew::getConfigGroupId, configGroupId)
                    .eq(ValidConfigNew::getType, type.getValue())
                    .eq(ValidConfigNew::getIndexType, subType.getValue()));
            autoExpireLocalCache.put(key, JSON.toJSONString(validConfigNews), TimeUnit.SECONDS.toMillis(120));// 2分钟过期
        } finally {
            lock.unlock();
        }
        if (!CollectionUtils.isEmpty(validConfigNews))
            return validConfigNews.get(0);
        return null;
    }

    /**
     * 获取集团封面有效性配置
     *
     * @param configGroupId 所属配置组id
     * @return 封面匹配相似度
     */
    private BigDecimal getCoverConfigValue(Long configGroupId) {
        Integer rate = Objects.requireNonNull(this.getValidConfig(configGroupId, ValidConfigTypeEnum.COVER,
                ValidIndexTypeEnum.COVER_SIMILARITY)).getMinValue();
        return PublicUtil.isEmpty(rate) ? BigDecimal.ZERO : BigDecimal.valueOf(rate);

    }

    public static void main(String[] args) {
        /*System.out.println(System.getProperty("java.io.tmpdir"));
        String workTitle = "#现车加\\\"免\\\"开新过新年";
        String setTopic = "#温暖回家路提现车过新年,#现车加\\免\\开新过新年,#温暖回家路春节不打烊,#开年有礼大展宏兔,#智电超能体验,#开新年夜FUN,#温暖回家路一路有你,#展厅实车带货UNI-K iDD".replace(",", "");
        Set<String> strings1 = processTags(workTitle);
        Set<String> strings2 = processTags(setTopic);
        System.out.println(CollectionUtils.containsAny(strings1, strings2));*/
    }
}
