package cn.fw.dalaran.service.biz;

import cn.fw.dalaran.common.page.PageHelper;
import cn.fw.dalaran.common.utils.DateUtil;
import cn.fw.dalaran.common.utils.PageUtil;
import cn.fw.dalaran.domain.db.Account;
import cn.fw.dalaran.domain.db.DisqualificationRecordEntity;
import cn.fw.dalaran.domain.dto.LivePoolDto;
import cn.fw.dalaran.domain.dto.VideoPoolDto;
import cn.fw.dalaran.domain.enums.DisqualificationEnum;
import cn.fw.dalaran.domain.enums.config.WorksTypeEnum;
import cn.fw.dalaran.domain.param.DisqualificationRecordParam;
import cn.fw.dalaran.domain.vo.DisqualificationRecordVo;
import cn.fw.dalaran.service.data.DisqualificationRecordService;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * @author wmy3969
 * @version 1.0
 * @date 2023/3/30 9:28
 * @Description 作品不合格原因记录业务
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class DisqualificationRecordBizService {

    private final DisqualificationRecordService disqualificationRecordService;// 作品不合格原因记录

    /**
     * 根据查询条件分页获取满足条件的记录列表
     *
     * @param page  分页参数
     * @param param 查询条件
     * @return 满足条件的记录列表
     */
    public PageHelper<DisqualificationRecordVo> getPage(PageHelper<Void> page, DisqualificationRecordParam param) {
        Long userId = param.getUserId();
        String userName = param.getUserName();
        String account = param.getAccount();
        Date startTime = param.getStartTime();
        Date endTime = param.getEndTime();
        Integer platformType = param.getPlatformType();
        Integer worksType = param.getWorksType();
        Integer reason = param.getReason();
        return PageUtil.toPage(disqualificationRecordService.page(PageUtil.helper2Page(page),
                Wrappers.<DisqualificationRecordEntity>lambdaQuery()
                        .eq(Objects.nonNull(userId), DisqualificationRecordEntity::getUserId, userId)
                        .like(Objects.nonNull(userName), DisqualificationRecordEntity::getUserName, userName)
                        .like(Objects.nonNull(account), DisqualificationRecordEntity::getAccount, account)
                        .ge(Objects.nonNull(startTime), DisqualificationRecordEntity::getWorksStartTime, startTime)
                        .le(Objects.nonNull(endTime), DisqualificationRecordEntity::getWorksEndTime, endTime)
                        .eq(Objects.nonNull(platformType), DisqualificationRecordEntity::getPlatformType, platformType)
                        .eq(Objects.nonNull(worksType), DisqualificationRecordEntity::getWorksType, worksType)
                        .eq(Objects.nonNull(reason), DisqualificationRecordEntity::getReason, reason)
        ), DisqualificationRecordEntity::toVo);
    }

    /**
     * 创建短视频不合格原因记录
     *
     * @param account  账户
     * @param result   话题验证结果
     * @param videoDTO 短视频源数据
     * @param type     不合格类型
     * @return 短视频不合格原因记录
     */
    public DisqualificationRecordEntity createVideoDisqualificationRecord(Account account, CommonBizService.TagCheckResult result,
                                                                          VideoPoolDto.VideoDTO videoDTO,
                                                                          DisqualificationEnum type) {
        DisqualificationRecordEntity db = copyToRecord(account, result);
        db.setWorksType(WorksTypeEnum.VIDEO);
        db.setWorksId(videoDTO.getVideoId());
        db.setWorksStartTime(videoDTO.getPublishTime());
        db.setWorksCover(videoDTO.getPreview());
        db.setWorksTitle(videoDTO.getTitle());
        db.setReason(type);
        return db;
    }

    /**
     * 创建直播不合格原因记录
     *
     * @param account 账户
     * @param result  话题验证结果
     * @param liveDTO 直播源数据
     * @param type    不合格类型
     * @return 直播不合格原因记录
     */
    public DisqualificationRecordEntity createLiveDisqualificationRecord(Account account, CommonBizService.TagCheckResult result,
                                                                         LivePoolDto.LiveDTO liveDTO,
                                                                         DisqualificationEnum type) {
        DisqualificationRecordEntity db = copyToRecord(account, result);
        db.setWorksType(WorksTypeEnum.LIVE);
        db.setWorksId(liveDTO.getRoomNo());
        db.setWorksStartTime(liveDTO.getLiveStartTime());
        db.setWorksEndTime(liveDTO.getLiveEndTime());
        db.setWorksCover(liveDTO.getCover());
        db.setWorksTitle(liveDTO.getTitle());
        db.setReason(type);
        return db;
    }

    /**
     * 参数转换成记录
     *
     * @param account 账号实体
     * @param result  话题验证结果
     * @return 不合格记录
     */
    private DisqualificationRecordEntity copyToRecord(Account account, CommonBizService.TagCheckResult result) {
        DisqualificationRecordEntity db = new DisqualificationRecordEntity();
        db.setGroupId(account.getGroupId());
        db.setUserId(account.getUserId());
        db.setUserName(account.getUserName());
        db.setAccount(account.getAccount());
        db.setPlatformType(account.getType());
        db.setThemeId(result.getThemeId());
        db.setConfigTags(CommonBizService.collection2String(result.getConfigTags()));
        db.setWorksTags(CommonBizService.collection2String(result.getWorksTags()));
        return db;
    }

    /**
     * 保存作品不合格原因记录
     *
     * @param db 作品不合格原因记录
     */
    public void saveDisqualificationRecord(DisqualificationRecordEntity db) {
        try {
            disqualificationRecordService.save(db);
        } catch (Exception e) {
            log.warn("保存作品不合格原因异常, 请检查是否是数据库unique键导致", e);
        }
    }

    /**
     * 获取账户某月作品不合格历史记录
     *
     * @param account   账号
     * @param worksType 作品类型
     * @param month     哪个月的记录
     * @return 账户某月作品不合格历史记录
     */
    public List<DisqualificationRecordEntity> getUserDisqualificationRecords(String account, WorksTypeEnum worksType,
                                                                             Date month) {
        return disqualificationRecordService.lambdaQuery()
                .eq(DisqualificationRecordEntity::getAccount, account)
                .eq(DisqualificationRecordEntity::getWorksType, worksType)
                .between(DisqualificationRecordEntity::getWorksStartTime, DateUtil.monthBegin(month),
                        DateUtil.monthEnd(month))
                .list();
    }

}
