package cn.fw.dalaran.service.biz;

import cn.fw.dalaran.common.domain.enums.RoleMsgEnum;
import cn.fw.dalaran.common.utils.PublicUtil;
import cn.fw.dalaran.domain.db.snapshot.ConsultantSnapshot;
import cn.fw.dalaran.domain.dto.ManagerInfo;
import cn.fw.dalaran.rpc.ehr.EhrRpc;
import cn.fw.dalaran.rpc.erp.ErpRpc;
import cn.fw.dalaran.rpc.oop.OopRpc;
import cn.fw.dalaran.service.data.IConsultantSnapshotService;
import cn.fw.ehr.sdk.api.result.StaffBaseInfo;
import cn.fw.erp.sdk.api.result.UserRoleDataRange;
import cn.fw.erp.sdk.api.result.UserRoleInfo;
import cn.fw.oop.sdk.result.ShopInfoVo;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicReference;
import java.util.concurrent.locks.LockSupport;
import java.util.stream.Collectors;

/**
 * @author wmy3969
 * @version 1.0
 * @date 2022/8/5 15:45
 * @Description 公共业务
 */
@Slf4j
@Component
@RequiredArgsConstructor
public class LocalCacheService {

    private final EhrRpc ehrRpc;// 人事系统rpc
    private final OopRpc oopRpc;// 商家系统rpc
    private final ErpRpc erpRpc;// 基础系统rpc
    private final IConsultantSnapshotService consultantSnapshotService;// 人员快照

    private final ConcurrentHashMap<Long, List<ShopInfoVo>> groupShopMap = new ConcurrentHashMap<>();// 集团门店缓存
    private final ConcurrentHashMap<Long, List<UserRoleInfo>> groupConsultantMap = new ConcurrentHashMap<>();// 集团销售顾问缓存
    private final ConcurrentHashMap<Long, List<ManagerInfo>> groupManagerMap = new ConcurrentHashMap<>();// 集团管理层缓存
    private final ConcurrentHashMap<Long, StaffBaseInfo> groupStaffMap = new ConcurrentHashMap<>();// 集团员工缓存

    private final ConcurrentHashMap<Long, Map<Long, ShopInfoVo>> groupShopList = new ConcurrentHashMap<>();// 集团门店缓存
    private final ConcurrentHashMap<Long, Map<Long, UserRoleInfo>> groupConsultantList = new ConcurrentHashMap<>();// 集团销售顾问缓存

    private static final int DIVISION_VALUE = 50;// 分度值(ms)
    private static final int TRY_TIMES = 20;// 最大尝试获取次数

    /**
     * id-name的K-V封装
     */
    @Data
    @NoArgsConstructor
    @AllArgsConstructor
    public static class KV {
        /**
         * 行数据id
         */
        private Long id;
        /**
         * 行数据名
         */
        private String name;
    }

    /**
     * 定时获取集团门店信息, 集团销售顾问信息
     */
    @Scheduled(fixedRate = 15 * 60 * 1000, initialDelay = 5000)
    public void flushLocalCache() {
        groupShopMap.clear();
        groupConsultantMap.clear();
        groupManagerMap.clear();
        groupStaffMap.clear();
        Optional.ofNullable(oopRpc.getAllGroups())
                .orElse(new ArrayList<>())
                .forEach(item -> {
                            Long groupId = item.getId();
                            List<ShopInfoVo> groupAllShops = oopRpc.getAllShopsByGroup(groupId);
                            if (!CollectionUtils.isEmpty(groupAllShops)) {
                                List<Long> shopIds = groupAllShops
                                        .stream()
                                        .map(ShopInfoVo::getId)
                                        .collect(Collectors.toList());// 获取门店集合
                                groupShopMap.put(groupId, groupAllShops);
                                List<UserRoleInfo> userRoleInfoList = erpRpc.getUserRoleInfoList(RoleMsgEnum.XSGW, shopIds);
                                groupConsultantMap.put(groupId, userRoleInfoList);
                                groupShopList.put(groupId, groupAllShops.stream().collect(Collectors.toMap(ShopInfoVo::getId, r -> r)));
                                groupConsultantList.put(groupId, userRoleInfoList.stream().collect(Collectors.toMap(UserRoleInfo::getUserId, r -> r)));
                            }
                            List<ManagerInfo> managerInfos = this.extractGroupManagerInfo(groupId);
                            if (Objects.nonNull(managerInfos))
                                groupManagerMap.put(groupId, managerInfos);
                            List<StaffBaseInfo> groupStaffList = ehrRpc.getGroupStaffList(groupId);
                            if (!CollectionUtils.isEmpty(groupStaffList))
                                groupStaffList.forEach(item1 -> groupStaffMap.put(item1.getId(), item1));
                        }
                )
        ;
    }

    /**
     * 获取集团门店列表
     *
     * @param groupId 集团id
     * @return 集团下门店列表
     */
    public List<ShopInfoVo> getGroupShops(Long groupId) {
        int tryCnt = 0;
        List<ShopInfoVo> shopInfoVos;
        while (CollectionUtils.isEmpty(shopInfoVos = this.groupShopMap.get(groupId)) && tryCnt < TRY_TIMES) {
            tryCnt++;
            LockSupport.parkNanos(TimeUnit.MILLISECONDS.toNanos(DIVISION_VALUE));
        }
        if (Objects.isNull(shopInfoVos)) {
            shopInfoVos = oopRpc.getAllShopsByGroup(groupId);
            if (Objects.nonNull(shopInfoVos))
                this.groupShopMap.put(groupId, shopInfoVos);
        }
        return shopInfoVos;
    }

    /**
     * 获取集团某个门店信息
     *
     * @param groupId 集团id
     * @param shopId  门店id
     * @return 集团某个门店信息
     */
    public ShopInfoVo getShopInfo(Long groupId, Long shopId) {
        return this.getGroupShops(groupId)
                .stream()
                .filter(item -> Objects.equals(item.getId(), shopId))
                .findFirst()
                .orElse(new ShopInfoVo())
                ;
    }

    /**
     * 获取集团销售顾问
     *
     * @param groupId 集团id
     * @return 集团销售顾问
     */
    public List<UserRoleInfo> getGroupConsultant(Long groupId) {
        int tryCnt = 0;
        List<UserRoleInfo> consultantRoleInfos;
        while (CollectionUtils.isEmpty(consultantRoleInfos = this.groupConsultantMap.get(groupId)) && tryCnt < TRY_TIMES) {
            tryCnt++;
            LockSupport.parkNanos(TimeUnit.MILLISECONDS.toNanos(DIVISION_VALUE));
        }
        if (Objects.isNull(consultantRoleInfos)) {
            consultantRoleInfos = erpRpc.getUserRoleInfoList(RoleMsgEnum.XSGW, this.getGroupShops(groupId)
                    .stream()
                    .map(ShopInfoVo::getId)
                    .collect(Collectors.toList())
            );
            if (Objects.nonNull(consultantRoleInfos))
                this.groupConsultantMap.put(groupId, consultantRoleInfos);
        }
        return consultantRoleInfos;
    }

    /**
     * 获取集团员工信息
     *
     * @param staffId 员工id
     * @return 集团员工信息
     */
    public StaffBaseInfo getStaffBaseInfo(Long staffId) {
        StaffBaseInfo staffInfo = this.groupStaffMap.get(staffId);
        if (Objects.nonNull(staffInfo)) {
            return staffInfo;
        } else {
            staffInfo = ehrRpc.getStaffBaseInfo(staffId);
            if (Objects.isNull(staffInfo)/* && staffId < 0*/) {
                ConsultantSnapshot snapshot = consultantSnapshotService.getOne(Wrappers
                        .<ConsultantSnapshot>lambdaQuery()
                        .eq(ConsultantSnapshot::getStaffId, staffId)
                        .orderByDesc(ConsultantSnapshot::getThemeId)
                        .last(" limit 1"));
                if (Objects.nonNull(snapshot)) {
                    StaffBaseInfo info = new StaffBaseInfo();
                    info.setId(snapshot.getStaffId());
                    info.setName(snapshot.getStaffName());
                    staffInfo = info;
                }
            }
            if (Objects.nonNull(staffInfo))
                this.groupStaffMap.put(staffId, staffInfo);
        }
        return staffInfo;
    }

    /**
     * 获取集团所有员工信息
     *
     * @param groupId 员工id
     * @return 集团所有员工信息
     */
    public List<StaffBaseInfo> getGroupStaffs(Long groupId) {
        List<StaffBaseInfo> staffs = new ArrayList<>();
        this.groupStaffMap.forEach((k, v) -> {
            if (Objects.equals(v.getGroupId(), groupId))
                staffs.add(v);
        });
        if (staffs.isEmpty()) {
            List<StaffBaseInfo> groupStaffList = ehrRpc.getGroupStaffList(groupId);
            if (Objects.nonNull(groupStaffList) && !groupStaffList.isEmpty())
                groupStaffList.forEach(item -> this.groupStaffMap.put(item.getId(), item));
            return groupStaffList;
        }
        return staffs;
    }

    /**
     * 获取集团管理层信息
     *
     * @param groupId 集团id
     * @return 集团管理层信息
     */
    public List<ManagerInfo> getGroupManager(Long groupId) {
        int tryCnt = 0;
        List<ManagerInfo> managerInfos;
        while (CollectionUtils.isEmpty(managerInfos = this.groupManagerMap.get(groupId)) && Objects.isNull(managerInfos) && tryCnt < TRY_TIMES) {
            tryCnt++;
            LockSupport.parkNanos(TimeUnit.MILLISECONDS.toNanos(DIVISION_VALUE));
        }
        if (Objects.isNull(managerInfos)) {
            managerInfos = this.extractGroupManagerInfo(groupId);
            if (Objects.nonNull(managerInfos))
                this.groupManagerMap.put(groupId, managerInfos);
        }
        return managerInfos;
    }

    /**
     * 获取员工所属管理层人员id
     *
     * @param groupId 员工所属集团
     * @param shopId  员工所属门店
     * @param level   管理层级枚举
     * @return 员工所属管理层人员id
     */
    public Long getBelongManagerId(Long groupId, Long shopId, RoleMsgEnum level) {
        List<ManagerInfo> groupManager = this.getGroupManager(groupId);
        AtomicReference<ManagerInfo.RangeInfo> user = new AtomicReference<>(new ManagerInfo.RangeInfo());
        groupManager.stream()
                .filter(item -> Objects.equals(item.getRoleCode(), level.getCode()))
                .findFirst()
                .flatMap(item -> item.getUsers()
                        .stream()
                        .filter(item1 -> item1.getShopIds().contains(shopId))
                        .findFirst()
                )
                .ifPresent(user::set);
        return user.get().getUserId();
    }

    /**
     * 获取员工所属管理层人员
     *
     * @param groupId 员工所属集团
     * @param shopId  员工所属门店
     * @param level   管理层级枚举
     * @return 员工所属管理层人员id
     */
    public ManagerInfo.RangeInfo getBelongManager(Long groupId, Long shopId, RoleMsgEnum level) {
        List<ManagerInfo> groupManager = this.getGroupManager(groupId);
        AtomicReference<ManagerInfo.RangeInfo> user = new AtomicReference<>(new ManagerInfo.RangeInfo());
        groupManager.stream()
                .filter(item -> Objects.equals(item.getRoleCode(), level.getCode()))
                .findFirst()
                .flatMap(item -> item.getUsers()
                        .stream()
                        .filter(item1 -> item1.getShopIds().contains(shopId))
                        .findFirst()
                )
                .ifPresent(user::set);
        return user.get();
    }

    /**
     * 获取集团门店map
     *
     * @param groupId 集团id
     * @return 集团下门店
     */
    public Map<Long, ShopInfoVo> getGroupShopMap(Long groupId) {
        Map<Long, ShopInfoVo> shopMap = this.groupShopList.getOrDefault(groupId, new HashMap<>());
        if (PublicUtil.isEmpty(shopMap)) {
            List<ShopInfoVo> allShopsByGroup = oopRpc.getAllShopsByGroup(groupId);
            allShopsByGroup.forEach(shop -> shopMap.put(shop.getId(), shop));
            this.groupShopList.put(groupId, shopMap);
        }
        return shopMap;
    }

    /**
     * 获取集团销售顾问map
     *
     * @param groupId 集团id
     * @return 集团销售顾问
     */
    public Map<Long, UserRoleInfo> getGroupConsultantMap(Long groupId) {
        Map<Long, UserRoleInfo> consultantMap = this.groupConsultantList.getOrDefault(groupId, new HashMap<>());
        if (PublicUtil.isEmpty(consultantMap)) {
            List<UserRoleInfo> userRoleInfoList = erpRpc.getUserRoleInfoList(RoleMsgEnum.XSGW, this.getGroupShops(groupId)
                    .stream().map(ShopInfoVo::getId).collect(Collectors.toList()));
            userRoleInfoList.forEach(staff -> consultantMap.put(staff.getUserId(), staff));
            this.groupConsultantList.put(groupId, consultantMap);
        }
        return consultantMap;
    }

    /**
     * 获取集团门店销售顾问map
     *
     * @param groupId 集团id
     * @param shopId  门店id
     * @return 集团该门店销售顾问
     */
    public Map<Long, UserRoleInfo> getShopConsultantMap(Long groupId, Long shopId) {
        Map<Long, UserRoleInfo> groupConsultants = this.getGroupConsultantMap(groupId);
        if (groupConsultants.isEmpty())
            return new HashMap<>();
        HashMap<Long, UserRoleInfo> consultantMap = new HashMap<>();
        groupConsultants.forEach((k, v) -> {
            if (v.getDataRangeList().stream()
                    .map(UserRoleDataRange::getRangeValue)
                    .collect(Collectors.toList())
                    .contains(shopId)) {
                consultantMap.put(k, v);
            }
        });
        return consultantMap;
    }

    /**
     * 获取集团门店列表中销售顾问map
     *
     * @param groupId 集团id
     * @param shopIds 门店id列表
     * @return 集团该门店列表中销售顾问
     */
    public Map<Long, UserRoleInfo> getShopsConsultantMap(Long groupId, List<Long> shopIds) {
        Map<Long, UserRoleInfo> consultants = new HashMap<>();
        shopIds.forEach(item -> {
            Map<Long, UserRoleInfo> shopConsultantMap = this.getShopConsultantMap(groupId, item);
            if (!shopConsultantMap.isEmpty())
                shopConsultantMap.forEach(consultants::put);
        });
        return consultants;
    }

    /**
     * 获取集团某管理层管理门店列表
     *
     * @param groupId   集团id
     * @param roleCode  管理层角色编码
     * @param managerId 管理人员id
     * @return 该管理层管理门店列表
     */
    public List<KV> getManageShops(Long groupId, String roleCode, Long managerId) {
        List<ManagerInfo> groupManager = this.getGroupManager(groupId);
        if (Objects.isNull(groupManager) || groupManager.isEmpty())
            return new ArrayList<>();
        AtomicReference<ArrayList<KV>> shops = new AtomicReference<>(new ArrayList<>());
        groupManager.stream()
                .filter(item -> Objects.equals(roleCode, item.getRoleCode()))
                .findFirst()
                .flatMap(item -> item.getUsers()
                        .stream()
                        .filter(item1 -> Objects.equals(item1.getUserId(), managerId))
                        .findFirst()
                )
                .ifPresent(item2 -> shops.set(new ArrayList<>(item2.getShopIds()
                        .stream()
                        .map(item3 -> new KV(item3, this.getShopInfo(groupId, managerId).getShopName()))
                        .collect(Collectors.toList())
                )));
        return shops.get();
    }

    /**
     * 获取集团某管理层管理销顾列表
     *
     * @param groupId   集团id
     * @param roleCode  管理层角色编码
     * @param managerId 管理人员id
     * @return 该管理层管理门店列表
     */
    public List<KV> getManageConsultants(Long groupId, String roleCode, Long managerId) {
        List<ManagerInfo> groupManager = this.getGroupManager(groupId);
        if (Objects.isNull(groupManager) || groupManager.isEmpty())
            return new ArrayList<>();
        List<KV> kvs = new ArrayList<>();
        Optional<ManagerInfo.RangeInfo> rangeInfo = groupManager.stream()
                .filter(item -> Objects.equals(roleCode, item.getRoleCode()))
                .findFirst()
                .flatMap(item -> item.getUsers()
                        .stream()
                        .filter(item1 -> Objects.equals(item1.getUserId(), managerId))
                        .findFirst()
                );
        if (rangeInfo.isPresent()) {
            ManagerInfo.RangeInfo rangeInfo1 = rangeInfo.get();
            rangeInfo1.getShopIds().forEach(item -> {
                Map<Long, UserRoleInfo> shopConsultantMap = this.getShopConsultantMap(groupId, item);
                shopConsultantMap.forEach((k, v) -> {
                    if (v.getDataRangeList()
                            .stream()
                            .map(UserRoleDataRange::getRangeValue)
                            .collect(Collectors.toList())
                            .contains(item)) {
                        kvs.add(new KV(k, v.getUserName()));
                    }
                });
            });
        }
        return kvs;
    }

    /**
     * 获取门店管理层信息
     */
    public List<ManagerInfo> extractShopManagerInfo(List<Long> shopIds, Long... groupId) {
        List<ManagerInfo> managerInfos = RoleMsgEnum.getSaleManageList().stream()
                .map(item -> this.getUserRoleInfoList(item, shopIds))
                .filter(Objects::nonNull)
                .collect(Collectors.toList());
        log.info(String.format("集团id为: %s, 门店ids为: %s, 管理层人员: %s", groupId.length > 0 ? groupId[0] : -1L,
                JSON.toJSONString(shopIds), JSON.toJSONString(managerInfos)));
        return managerInfos;
    }

    /**
     * 获取门店列表中拥有对应角色的人
     *
     * @param roleMsg 角色枚举
     * @param shopIds 门店列表
     * @return 门店列表中拥有对应角色的人
     */
    private ManagerInfo getUserRoleInfoList(RoleMsgEnum roleMsg, List<Long> shopIds) {
        List<UserRoleInfo> userRoleInfoList = erpRpc.getUserRoleInfoList(roleMsg, shopIds);// 新车四级管理
        if (!CollectionUtils.isEmpty(userRoleInfoList)) {
            ManagerInfo info = new ManagerInfo();
            info.setRoleCode(roleMsg.getCode());
            info.setUsers(userRoleInfoList.stream().map(item -> {
                ManagerInfo.RangeInfo managerInfo = new ManagerInfo.RangeInfo();
                managerInfo.setUserId(item.getUserId());
                managerInfo.setUserName(item.getUserName());
                managerInfo.setShopIds(item.getDataRangeList()
                        .stream()
                        .map(UserRoleDataRange::getRangeValue)
                        .sorted()
                        .collect(Collectors.toList()));
                return managerInfo;
            }).collect(Collectors.toList()));
            return info;
        }
        return null;
    }

    /**
     * 获取集团管理层信息
     */
    public List<ManagerInfo> extractGroupManagerInfo(Long groupId) {
        List<ShopInfoVo> shopList = this.getGroupShops(groupId)
                .stream()
                .filter(item -> Objects.equals(item.getBizType(), 1))
                .collect(Collectors.toList());
        if (CollectionUtils.isEmpty(shopList))
            return new ArrayList<>();
        List<ManagerInfo> managerInfos = new ArrayList<>(this.extractShopManagerInfo(shopList.stream()
                .map(ShopInfoVo::getId)
                .sorted()
                .collect(Collectors.toList()), groupId));
        if (CollectionUtils.isEmpty(managerInfos))
            return new ArrayList<>();
        List<StaffBaseInfo> userInfos = null;
        try {
            List<Long> userIds = new ArrayList<>();
            for (ManagerInfo managerInfo : managerInfos) {
                for (ManagerInfo.RangeInfo user : managerInfo.getUsers()) {
                    userIds.add(user.getUserId());
                }
            }
            userInfos = ehrRpc.getBatchStaffShopList(userIds);
        } catch (Exception e) {
            userInfos = new ArrayList<>();
        } finally {
            for (ManagerInfo managerInfo : managerInfos) {
                for (ManagerInfo.RangeInfo user : managerInfo.getUsers()) {
                    assert userInfos != null;
                    StaffBaseInfo userInfo = userInfos.stream()
                            .filter(item -> Objects.equals(item.getId(), user.getUserId()))
                            .findAny()
                            .orElse(null);
                    user.setShopId(Objects.nonNull(userInfo) ? userInfo.getShopId() : 8848L);
                    user.setShopName(Objects.nonNull(userInfo) ? userInfo.getShopName() : String.valueOf(UUID.randomUUID()));
                }
            }
        }
        return managerInfos.stream()
                .distinct()
                .collect(Collectors.toList());
    }

}
