package cn.fw.dalaran.service.biz.account;

import cn.fw.common.web.annotation.DisLock;
import cn.fw.dalaran.common.domain.AccountEventDetailsMsg;
import cn.fw.dalaran.common.domain.enums.EventTypeEnum;
import cn.fw.dalaran.common.domain.enums.RoleMsgEnum;
import cn.fw.dalaran.common.domain.enums.ToDoTypeEnum;
import cn.fw.dalaran.common.domain.enums.TodoCodeEnum;
import cn.fw.dalaran.common.exception.InterveneTypeEnum;
import cn.fw.dalaran.common.page.PageHelper;
import cn.fw.dalaran.common.utils.DateUtil;
import cn.fw.dalaran.common.utils.PublicUtil;
import cn.fw.dalaran.domain.db.Account;
import cn.fw.dalaran.domain.db.common.InterveneRecord;
import cn.fw.dalaran.domain.db.common.TodoHistory;
import cn.fw.dalaran.domain.dto.AccountDTO;
import cn.fw.dalaran.domain.dto.AccountQueryDTO;
import cn.fw.dalaran.domain.dto.UpdateAccountParam;
import cn.fw.dalaran.domain.enums.PlatformEnum;
import cn.fw.dalaran.domain.enums.account.AccountStatusEnum;
import cn.fw.dalaran.domain.enums.account.AccountTypeEnum;
import cn.fw.dalaran.domain.vo.AccountStatusVo;
import cn.fw.dalaran.domain.vo.AccountVO;
import cn.fw.dalaran.domain.vo.CommonAccountVO;
import cn.fw.dalaran.rpc.ehr.EhrRpc;
import cn.fw.dalaran.rpc.erp.UserRoleRpcService;
import cn.fw.dalaran.rpc.erp.dto.UserInfoDTO;
import cn.fw.dalaran.service.biz.config.ConfigCommonBizService;
import cn.fw.dalaran.service.data.AccountService;
import cn.fw.dalaran.service.data.IInterveneRecordService;
import cn.fw.dalaran.service.data.common.TodoHistoryService;
import cn.fw.ehr.sdk.api.result.StaffBaseInfo;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.stereotype.Service;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.TransactionDefinition;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

import static cn.fw.common.businessvalidator.Validator.BV;

@Service
@Slf4j
@RequiredArgsConstructor
public class AccountBizService {
    private final EhrRpc ehrRpc;
    private final AccountService accountService;// 账号业务
    private final ApplicationEventPublisher publisher;// spring事件发布器
    private final TodoHistoryService todoHistoryService;// 待办业务
    private final UserRoleRpcService userRoleRpcService;
    private final ConfigCommonBizService configCommonBizService;
    private final IInterveneRecordService interveneRecordService;
    private final AccountChangeBizService accountChangeBizService;
    private final AccountConfirmBizService accountConfirmBizService;
    /* 手动管理事务 */
    private final TransactionDefinition transactionDefinition;
    private final PlatformTransactionManager platformTransactionManager;

    /**
     * 分页查询账号信息
     *
     * @param groupId 集团id
     * @param query   查询参数
     * @return 账号列表
     */
    public PageHelper<AccountVO> page(Long groupId, PageHelper<Account> pageHelper, AccountQueryDTO query) {
        IPage<Account> page = accountService.page(PublicUtil.toPage(pageHelper), Wrappers.<Account>lambdaQuery()
                .eq(Account::getGroupId, groupId)
                .eq(Objects.nonNull(query.getAccountType()), Account::getAccountType, query.getAccountType())
                .eq(Objects.nonNull(query.getType()), Account::getType, query.getType())
                .eq(Objects.nonNull(query.getValid()), Account::getValid, query.getValid())
                .eq(Objects.nonNull(query.getShopId()), Account::getShopId, query.getShopId())
                .eq(Objects.nonNull(query.getUserId()), Account::getUserId, query.getUserId())
                .like(Objects.nonNull(query.getUserName()), Account::getUserName, query.getUserName())
                .eq(Account::getYn, Boolean.TRUE));
        return PublicUtil.toPage(page, account -> {
            AccountVO vo = PublicUtil.copy(account, AccountVO.class);
            vo.setId(account.getId());
            vo.setCreateTime(account.getCreateTime());
            return vo;
        });
    }

    /**
     * 添加账号(已手动管理事务, 请勿加@Transactional注解)
     *
     * @param dto 账号参数
     * @return 操作结果
     */
    public boolean saveAccount(AccountDTO dto) {
        TransactionStatus transaction = platformTransactionManager.getTransaction(transactionDefinition);// 开启事务
        try {
            boolean hasAffirmRecord = this.preCheck(dto);
            Account oldAccount = Objects.isNull(dto.getId())
                    ? null : accountService.getById(dto.getId());
            Account account = this.buildAccount(oldAccount, dto, hasAffirmRecord);
            //增加账号/删除账号, 处理参与主题待办
            configCommonBizService.handleUserJoinThemeBacklog(account.getGroupId(), account.getShopId(),
                    account.getUserId(), true, false);
            accountService.saveOrUpdate(account);
            //账号调整重推账号分析功能开通确认待办
            if (PublicUtil.isNotEmpty(dto.getId())) {
                accountConfirmBizService.accountChange(oldAccount, account);
            }
            // 发布账号新增事件
            AccountEventDetailsMsg event = new AccountEventDetailsMsg();
            event.setAccountId(account.getId());
            event.setEventType(EventTypeEnum.ACCOUNT_INSERT);
            platformTransactionManager.commit(transaction);
            publisher.publishEvent(event);
            return true;
        } catch (Exception e) {
            platformTransactionManager.rollback(transaction);
            return false;
        }
    }

    /**
     * 账号参数检查
     */
    private boolean preCheck(AccountDTO accountParam) {
        //账号重复添加判断
        List<Account> accountList = accountService.getByAccountNo(accountParam.getGroupId(), accountParam.getAccount(), accountParam.getType());
        accountList.stream()
                .filter(r -> Objects.isNull(accountParam.getId()) || !r.getId().equals(accountParam.getId()))
                .forEach(account -> PublicUtil.assertFalse(account.getYn(), "该账号已存在，不允许重复添加"));
        StaffBaseInfo staffBaseInfo = ehrRpc.getStaffBaseInfo(accountParam.getUserId());
        //个人账号
        if (AccountTypeEnum.STAFF.getValue().equals(accountParam.getAccountType())) {
            accountParam.setShopId(staffBaseInfo.getShopId());
            accountParam.setShopName(staffBaseInfo.getShopName());
        }
        //门店账号
        if (AccountTypeEnum.SHOP.getValue().equals(accountParam.getAccountType())) {
            PublicUtil.assertNotEmpty(accountParam.getShopId(), "账号未指定归属门店");
            PublicUtil.assertTrue(staffBaseInfo.getShopId().equals(accountParam.getShopId()),
                    "账号运营人员在职门店与账号归属门店不匹配，请重新选择人员");
        }
        //单平台最大账号数量限制检查
        List<Account> userAccountList = accountService.list(Wrappers.<Account>lambdaQuery()
                .eq(Account::getUserId, accountParam.getUserId())
                .eq(Account::getType, accountParam.getType())
                .ne(Objects.nonNull(accountParam.getId()), Account::getId, accountParam.getId())
                .eq(Account::getYn, Boolean.TRUE));
        Set<Integer> accountTypeSet = userAccountList.stream().map(Account::getAccountType).collect(Collectors.toSet());
        PublicUtil.assertFalse(accountTypeSet.contains(accountParam.getAccountType()),
                String.format("当前人员在[%s]平台已存在[%s],不可重复配置", accountParam.getType().getName(),
                        Objects.requireNonNull(AccountTypeEnum.ofValue(accountParam.getAccountType())).getName()));
        //判断账号曾经是否认证过
        return accountList.stream().anyMatch(Account::getAffirm);
    }

    /**
     * 组装账号实体
     */
    private Account buildAccount(Account oldAccount, AccountDTO accountParam, boolean hasAffirmRecord) {
        Account nowAccount;
        if (PublicUtil.isEmpty(accountParam.getId())) {
            nowAccount = new Account();
            nowAccount.setGroupId(accountParam.getGroupId());
            nowAccount.setYn(Boolean.TRUE);
            nowAccount.setEnableAnalysis(true);
            nowAccount.setCreateTime(DateUtil.localDateTime2Date(LocalDateTime.now().plusDays(-1)));
        } else {
            PublicUtil.assertNotEmpty(oldAccount, "账号信息不存在");
            PublicUtil.assertFalse(oldAccount.getAffirm(), "账号已授权登录成功,不支持调整");
            nowAccount = PublicUtil.copy(oldAccount, Account.class);
        }
        PublicUtil.copy(accountParam, nowAccount);
        nowAccount.setAffirm(hasAffirmRecord);
        return nowAccount;
    }

    /**
     * 删除账号
     *
     * @param id 账号id
     * @return 操作结果
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean deleteAccount(Long id) {
        // 账号删除完成登录待办
        todoHistoryService.lambdaUpdate()
                .eq(TodoHistory::getDataId, id)
                .in(TodoHistory::getTodoCode, TodoCodeEnum.ACCOUNT_INVALID.getCode(),
                        TodoCodeEnum.ACCOUNT_CONFIG_CONFIRM.getCode(), TodoCodeEnum.ACCOUNT_CONFIG_INTERVENE.getCode())
                .set(TodoHistory::getDone, ToDoTypeEnum.CANCEL.getValue())
                .update();
        //完成干预记录
        interveneRecordService.update(Wrappers.<InterveneRecord>lambdaUpdate()
                .eq(InterveneRecord::getType, InterveneTypeEnum.ACCOUNT_CONFIRM.getValue())
                .eq(InterveneRecord::getDataId, id)
                .set(InterveneRecord::getComplete, true));
        Account account = accountService.getById(id);
        //删除账号
        accountService.lambdaUpdate()
                .eq(Account::getId, id)
                .set(Account::getYn, 0)
                .set(Account::getId, id)
                .set(Account::getUpdateTime, new Date())
                .update();
        accountChangeBizService.accountDataFreeze(account, null, null);
        configCommonBizService.handleUserJoinThemeBacklog(account.getGroupId(), account.getShopId(),
                account.getUserId(), false, false);
        return true;
    }

    /**
     * 上报cookie失效账号
     *
     * @param account
     * @param type
     * @return 操作结果
     */
    @Transactional(rollbackFor = Exception.class)
    @DisLock(prefix = "':' + #this.getClass().getSimpleName()", key = "#type + ':' + #account", message = "请勿重复提交")
    public boolean reportInvalidAccount(String account, Integer type) {
        PlatformEnum platformEnum = PlatformEnum.ofValue(type);
        BV.notNull(platformEnum, () -> "平台类型不能为空");
        List<Account> list = accountService.list(Wrappers.<Account>lambdaQuery()
                .eq(Account::getAccount, account)
                .eq(Account::getType, platformEnum)
                .eq(Account::getValid, Boolean.TRUE)
                .eq(Account::getAffirm, Boolean.TRUE)
                .eq(Account::getYn, Boolean.TRUE)
        );// 找到失效的账号
        if (CollectionUtils.isEmpty(list))
            return true;
        boolean result = accountService.updateBatchById(list.stream()
                .peek(item -> item.setValid(Boolean.FALSE))
                .collect(Collectors.toList())
        );// 更新账号状态为失效
        List<TodoHistory> todos = new ArrayList<>();
        for (Account account1 : list) {
            TodoHistory history = new TodoHistory();
            history.setSend(Boolean.FALSE);
            history.setDone(ToDoTypeEnum.ADD.getValue());
            history.setDataId(account1.getId());
            history.setShopId(account1.getShopId());
            history.setUserId(account1.getUserId());
            history.setTodoCode(TodoCodeEnum.ACCOUNT_INVALID.getCode());
            Map<String, String> dynamicParam = this.getAccountInvalidDynamicParam(account1);
            history.setDynamicInfo(JSON.toJSONString(dynamicParam));
            todos.add(history);
            List<UserInfoDTO> users = userRoleRpcService.getUsers(account1.getShopId(), RoleMsgEnum.MKT_HDCZ.getCode());
            if (!CollectionUtils.isEmpty(users)) {
                Long managerId = users.get(0).getUserId();
                TodoHistory manageTodo = new TodoHistory();
                BeanUtils.copyProperties(history, manageTodo);
                manageTodo.setUserId(managerId);
                history.setExtra(String.valueOf(managerId));
                todos.add(manageTodo);
            }
        }
        List<Long> waitSendIds = list.stream()
                .map(Account::getId)
                .collect(Collectors.toList());// 失败账号id
        List<TodoHistory> todayHasSendList = todoHistoryService.lambdaQuery()
                .eq(TodoHistory::getTodoCode, TodoCodeEnum.ACCOUNT_INVALID.getCode())
                .in(!CollectionUtils.isEmpty(waitSendIds), TodoHistory::getDataId, waitSendIds)
                .last(" and DATEDIFF(create_time, NOW()) = 0")
                .list();
        if (todayHasSendList.stream().noneMatch(item -> waitSendIds.contains(item.getDataId())))
            todoHistoryService.saveBatch(todos);// 如果今天之内没有发送过待办, 才保存todo_history
        return result;
    }

    /**
     * 获取账号失效待办动态参数
     *
     * @param account
     * @return
     */
    private Map<String, String> getAccountInvalidDynamicParam(Account account) {
        Map<String, String> dynamicMap = new HashMap<>(4);
        dynamicMap.put("name", account.getType().getName());
        dynamicMap.put("shopName", account.getShopName());
        dynamicMap.put("account", account.getAccount());
        dynamicMap.put("userName", account.getUserName());
        return dynamicMap;
    }

    /**
     * 上报账号状态
     *
     * @param accountId 账号id
     * @param valid     状态
     * @return 操作结果
     */
    @Transactional(rollbackFor = Exception.class)
    @DisLock(prefix = "':' + #this.getClass().getSimpleName()", key = "'accountReport:' + #accountId", message = "请勿重复提交")
    public boolean reportAccountStatus(Long accountId, Boolean valid) {
        Account account = accountService.getById(accountId);
        if (Objects.isNull(account))
            return true;
        if (Objects.equals(valid, account.getValid()))
            return true;
        account.setValid(valid);
        account.setAffirm(Boolean.TRUE);
        boolean result = accountService.updateById(account);// 修改账号状态为指定状态
        List<TodoHistory> userList = todoHistoryService.lambdaQuery()
                .eq(TodoHistory::getDone, ToDoTypeEnum.ADD.getValue())
                .eq(TodoHistory::getDataId, accountId)
                .list();
        List<String> collect = userList.stream()
                .map(TodoHistory::getExtra)
                .filter(Objects::nonNull)
                .collect(Collectors.toList());// 查找是否有管理人员
        if (!CollectionUtils.isEmpty(collect)) {
            List<TodoHistory> managerList = todoHistoryService.lambdaQuery()
                    .eq(TodoHistory::getDone, ToDoTypeEnum.ADD.getValue())
                    .eq(TodoHistory::getDataId, accountId)
                    .in(TodoHistory::getUserId, collect)
                    .list();
            userList.addAll(managerList);
        }
        todoHistoryService.updateBatchById(new HashSet<>(userList).stream()
                .peek(item -> item.setDone(ToDoTypeEnum.COMPLETE.getValue()))
                .collect(Collectors.toList())
        );
        return result;
    }

    /**
     * 上报账户信息
     *
     * @param account
     * @param type
     * @param fans
     * @param accountName
     */
    public void reportAccountMsg(String account, Integer type, Long fans, String accountName) {
        PlatformEnum platformEnum = PlatformEnum.ofValue(type);
        if (Objects.isNull(platformEnum) || Objects.isNull(fans)) {
            return;
        }
        List<Account> list = accountService.list(Wrappers.<Account>lambdaQuery()
                .eq(Account::getAccount, account)
                .eq(Account::getType, platformEnum)
                .eq(Account::getYn, Boolean.TRUE)
                .eq(Account::getValid, Boolean.TRUE)
        );
        if (CollectionUtils.isEmpty(list)) {
            return;
        }
        list.forEach(item -> {
            item.setFansNum(fans);
            item.setAccountName(accountName);
        });
        accountService.updateBatchById(list);
    }

    /**
     * 获取有效账号
     *
     * @param type
     * @return
     */
    public List<CommonAccountVO> getValidAccount(Integer type) {
        PlatformEnum platformEnum = PlatformEnum.ofValue(type);
        BV.notNull(platformEnum, () -> "账号类型不正确");
        List<Account> list = accountService.list(Wrappers.<Account>lambdaQuery()
                .eq(Account::getAffirm, Boolean.TRUE)
                .eq(Account::getValid, Boolean.TRUE)
                .eq(Account::getType, platformEnum)
                .ne(Account::getAccountStatus, AccountStatusEnum.HAS_LEAVE.getValue())
                //.eq(Account::getAccountStatus, AccountStatusEnum.IN_WORK.getValue())
                .eq(Account::getYn, Boolean.TRUE)
        );
        if (CollectionUtils.isEmpty(list)) {
            return new ArrayList<>();
        }
        return list.stream()
                .map(item -> CommonAccountVO.builder()
                        .account(item.getAccount())// 账号
                        .playbackSearchKey(item.getPlaybackSearchKey())// 回播搜索关键词
                        .groupId(item.getGroupId())// 集团id
                        .build()
                )
                .collect(Collectors.toList());
    }

    /**
     * 查询个每个门店账号状态详情
     *
     * @param groupId 集团id
     * @return 每个门店账号状态详情视图
     */
    public List<AccountStatusVo.ShopDetails> status(Long groupId) {
        final AccountStatusVo vo = new AccountStatusVo();
        final List<Account> accountList = accountService.lambdaQuery()
                .eq(Account::getGroupId, groupId)
                .list();
        final Map<Long, List<Account>> shopAccountsMap = accountList.stream()
                .collect(Collectors.groupingBy(Account::getShopId));// 根据门店分组账号
        List<AccountStatusVo.ShopDetails> shopDetailsList = new ArrayList<>();
        shopAccountsMap.forEach((k, v) -> {
            AccountStatusVo.ShopDetails shopDetails = new AccountStatusVo.ShopDetails();
            shopDetails.setShopId(k);// 门店id
            shopDetails.setShopName(v.get(0).getShopName());// 门店名
            shopDetails.setAdvisorCnt((int) v.stream()
                    .map(Account::getUserName)
                    .distinct()
                    .count());// 销售顾问人数
            List<AccountStatusVo.PlatformDetails> accountTypeList = new ArrayList<>();
            v.stream()
                    .collect(Collectors.groupingBy(Account::getType))
                    .forEach((k1, v1) -> {
                        AccountStatusVo.PlatformDetails platFormDetails = new AccountStatusVo.PlatformDetails();
                        platFormDetails.setPlatform(k1);// 平台
                        platFormDetails.setPlatformName(k1.getName());// 平台名
                        v1.stream()
                                .collect(Collectors.groupingBy(Account::getValid))
                                .forEach((k2, v2) -> {
                                    if (!k2) {
                                        platFormDetails.setHasExpireCnt(v2.size());// 授权失效账号数
                                    } else {
                                        platFormDetails.setHasLoginCnt(v2.size());// 登陆成功账号数
                                    }
                                });
                        accountTypeList.add(platFormDetails);
                    });
            shopDetails.setPlatformDetailsList(accountTypeList);
            shopDetailsList.add(shopDetails);
        });
        vo.setShopDetailsList(shopDetailsList);
        //return vo;
        return shopDetailsList;
    }

    public boolean alert(UpdateAccountParam param) {
        final String auth = param.getAuth();
        if (Objects.equals(auth, "fw@2017")) {
            param.getAccounts().forEach(item -> {
                if (Objects.isNull(item.getId()))
                    return;
                accountService.alertAccount(item);
            });
            return true;
        }
        return false;
    }

}
