package cn.fw.dalaran.service.biz.account;

import cn.fw.dalaran.common.domain.enums.RoleMsgEnum;
import cn.fw.dalaran.common.utils.PublicUtil;
import cn.fw.dalaran.domain.db.Account;
import cn.fw.dalaran.domain.db.LivePool;
import cn.fw.dalaran.domain.db.VideoPool;
import cn.fw.dalaran.domain.db.config.ActivityTheme;
import cn.fw.dalaran.domain.dto.common.ShopDto;
import cn.fw.dalaran.domain.enums.account.AccountTypeEnum;
import cn.fw.dalaran.service.biz.LocalCacheService;
import cn.fw.dalaran.service.data.AccountService;
import cn.fw.dalaran.service.data.ActivityThemeService;
import cn.fw.dalaran.service.data.LivePoolService;
import cn.fw.dalaran.service.data.VideoPoolService;
import com.alibaba.fastjson.JSON;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * 账号变动实现
 *
 * @author: juzi
 * @date: 2023/9/13 14:35
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class AccountChangeBizService {
    private final AccountService accountService;
    private final LivePoolService livePoolService;
    private final VideoPoolService videoPoolService;
    private final LocalCacheService localCacheService;
    private final ActivityThemeService activityThemeService;

    /**
     * 恢复账号对应进行中直播、短视频数据
     *
     * @param account 账号
     */
    @Transactional(rollbackFor = Exception.class)
    public void accountDataRestore(Account account, List<ActivityTheme> themes, List<Account> accounts) {
        themes = PublicUtil.isEmpty(themes) ? activityThemeService.getGoingActivityTheme() : themes;
        accounts = PublicUtil.isEmpty(accounts) ? accountService.getByAccountNoIgnoreLogicDel(account.getAccount()) : accounts;
        if (PublicUtil.isEmpty(accounts) || PublicUtil.isEmpty(themes)) {
            return;
        }
        List<Long> accountIds = accounts.stream().filter(item -> Objects.equals(item.getType(), account.getType())).map(Account::getId).collect(Collectors.toList());
        List<Long> themeIds = themes.stream().map(ActivityTheme::getId).collect(Collectors.toList());
        List<LivePool> livePools = livePoolService.lambdaQuery()
                .eq(LivePool::getStatus, 0)
                .in(LivePool::getThemeId, themeIds)
                .in(LivePool::getAccountId, accountIds).list();// 查询出被标记无效的直播数据(未关单的主题, 且数据归属于该账号的)
        List<VideoPool> videoPools = videoPoolService.lambdaQuery()
                .eq(VideoPool::getStatus, 0)
                .in(VideoPool::getThemeId, themeIds)
                .in(VideoPool::getAccountId, accountIds).list();// 查询出被标记无效的短视频数据(未关单的主题, 且数据归属于该账号的)
        ActivityTheme activityTheme = this.getBelongActivityTheme(themes, account.getShopId());
        if (PublicUtil.isEmpty(activityTheme)) {
            return;
        }
        this.changeLiveBelong(account, livePools, activityTheme);
        this.changeVideoBelong(account, videoPools, activityTheme);
    }

    /**
     * 找到门店对应主题
     *
     * @param themes 正在进行中的活动主题列表
     * @return 对应主题详情
     */
    private ActivityTheme getBelongActivityTheme(List<ActivityTheme> themes, Long shopId) {
        for (ActivityTheme theme : themes) {
            List<ShopDto> shops = JSON.parseArray(theme.getShopJson(), ShopDto.class);
            if (shops.stream().anyMatch(r -> r.getShopId().equals(shopId))) {
                return theme;
            }
        }
        return null;
    }

    /**
     * 改变短视频数据归属信息
     *
     * @param account       账号实体(本次添加的最新账号)
     * @param videoPools    短视频数据列表
     * @param activityTheme 正在进行中的活动主题列表
     */
    @Transactional(rollbackFor = Exception.class)
    public void changeVideoBelong(Account account, List<VideoPool> videoPools, ActivityTheme activityTheme) {
        Long manager1Id = localCacheService.getBelongManagerId(account.getGroupId(), account.getShopId(), RoleMsgEnum.XSZG);
        Long manager2Id = localCacheService.getBelongManagerId(account.getGroupId(), account.getShopId(), RoleMsgEnum.XSJL);
        Long manager3Id = localCacheService.getBelongManagerId(account.getGroupId(), account.getShopId(), RoleMsgEnum.XSZJL);
        Long manager4Id = localCacheService.getBelongManagerId(account.getGroupId(), account.getShopId(), RoleMsgEnum.XSSJGL);
        List<Long> shopIds = activityTheme.getShopIdByReport();// 主题包含门店列表
        boolean shopAccount = AccountTypeEnum.SHOP.getValue().equals(account.getAccountType());
        List<VideoPool> needRestoreVideo = videoPools.stream().filter(r -> shopAccount ? Objects.isNull(r.getBelongUserId()) : account.getUserId().equals(r.getBelongUserId())).filter(r -> shopIds.contains(r.getBelongShopId())).peek(video -> {
            video.setAccountId(account.getId());
            video.setStatus(1);
            video.setBelongShopId(account.getShopId());
            video.setBelongUserId(shopAccount ? -1L : account.getUserId());
            video.setFirstManagerId(manager1Id);
            video.setSecondManagerId(manager2Id);
            video.setThirdManagerId(manager3Id);
            video.setFourthManagerId(manager4Id);
        }).collect(Collectors.toList());
        videoPoolService.updateBatchById(needRestoreVideo);
    }

    /**
     * 改变直播数据归属信息
     *
     * @param account       账号实体(本次添加的最新账号)
     * @param activityTheme 正在进行中的活动主题列表
     */
    @Transactional(rollbackFor = Exception.class)
    public void changeLiveBelong(Account account, List<LivePool> livePoolList, ActivityTheme activityTheme) {
        Long manager1Id = localCacheService.getBelongManagerId(account.getGroupId(), account.getShopId(), RoleMsgEnum.XSZG);
        Long manager2Id = localCacheService.getBelongManagerId(account.getGroupId(), account.getShopId(), RoleMsgEnum.XSJL);
        Long manager3Id = localCacheService.getBelongManagerId(account.getGroupId(), account.getShopId(), RoleMsgEnum.XSZJL);
        Long manager4Id = localCacheService.getBelongManagerId(account.getGroupId(), account.getShopId(), RoleMsgEnum.XSSJGL);
        List<Long> shopIds = activityTheme.getShopIdByReport();// 主题包含门店列表
        boolean shopAccount = AccountTypeEnum.SHOP.getValue().equals(account.getAccountType());
        List<LivePool> needRestoreLive = livePoolList.stream().filter(r -> shopAccount ? Objects.isNull(r.getBelongUserId()) : account.getUserId().equals(r.getBelongUserId())).filter(r -> shopIds.contains(r.getBelongShopId())).peek(live -> {
            live.setAccountId(account.getId());
            live.setStatus(1);
            live.setBelongShopId(account.getShopId());
            live.setBelongUserId(shopAccount ? -1L : account.getUserId());
            live.setFirstManagerId(manager1Id);
            live.setSecondManagerId(manager2Id);
            live.setThirdManagerId(manager3Id);
            live.setFourthManagerId(manager4Id);
        }).collect(Collectors.toList());
        livePoolService.updateBatchById(needRestoreLive);
    }

    /**
     * 冻结账号进行中直播、短视频数据
     *
     * @param account 账号
     */
    @Transactional(rollbackFor = Exception.class)
    public void accountDataFreeze(Account account, List<ActivityTheme> themes, List<Account> accounts) {
        if (PublicUtil.isEmpty(account.getAffirm()) || !account.getAffirm()) {
            return;
        }
        themes = PublicUtil.isEmpty(themes) ? activityThemeService.getGoingActivityTheme() : themes;
        accounts = PublicUtil.isEmpty(accounts) ? accountService.getByAccountNoIgnoreLogicDel(account.getAccount()) : accounts;
        if (PublicUtil.isEmpty(themes) || PublicUtil.isEmpty(accounts)) {
            return;
        }
        List<Long> accountIds = accounts.stream().filter(item -> Objects.equals(item.getType(), account.getType())).map(Account::getId).collect(Collectors.toList());
        List<Long> themeIds = themes.stream().map(ActivityTheme::getId).collect(Collectors.toList());
        // 标记归属短视频为无效
        this.freezeAccountVideoData(account, themeIds, accountIds);
        // 标记归属直播为无效
        this.freezeAccountLiveData(account, themeIds, accountIds);
    }

    /**
     * 冻结账号正在进行中的视频数据
     */
    @Transactional(rollbackFor = Exception.class)
    public void freezeAccountVideoData(Account account, List<Long> themeIds, List<Long> accountIds) {
        videoPoolService.lambdaUpdate()
                .set(VideoPool::getStatus, 0)
                .in(VideoPool::getThemeId, themeIds)
                .in(VideoPool::getAccountId, accountIds)
                .eq(VideoPool::getPlatform, account.getType())
                .set(VideoPool::getUpdateTime, new Date())
                .update();
    }

    /**
     * 冻结账号正在进行中的视频数据
     */
    @Transactional(rollbackFor = Exception.class)
    public void freezeAccountLiveData(Account account, List<Long> themeIds, List<Long> accountIds) {
        livePoolService.lambdaUpdate()
                .set(LivePool::getStatus, 0)
                .in(LivePool::getThemeId, themeIds)
                .in(LivePool::getAccountId, accountIds)
                .eq(LivePool::getPlatform, account.getType())
                .set(LivePool::getUpdateTime, new Date())
                .update();
    }
}

