package cn.fw.dalaran.service.biz.account;

import cn.fw.backlog.sdk.api.result.MeddleItem;
import cn.fw.dalaran.common.domain.enums.ToDoTypeEnum;
import cn.fw.dalaran.common.domain.enums.TodoCodeEnum;
import cn.fw.dalaran.common.exception.InterveneTypeEnum;
import cn.fw.dalaran.common.utils.DateUtil;
import cn.fw.dalaran.common.utils.PublicUtil;
import cn.fw.dalaran.domain.db.Account;
import cn.fw.dalaran.domain.db.common.InterveneRecord;
import cn.fw.dalaran.domain.db.common.TodoHistory;
import cn.fw.dalaran.domain.dto.config.AccountInterveneDto;
import cn.fw.dalaran.domain.enums.PlatformEnum;
import cn.fw.dalaran.domain.enums.account.AccountEnableAnalysisEnum;
import cn.fw.dalaran.domain.vo.AccountConfirmDetailVo;
import cn.fw.dalaran.rpc.erp.ErpRpc;
import cn.fw.dalaran.service.data.AccountService;
import cn.fw.dalaran.service.data.IInterveneRecordService;
import cn.fw.dalaran.service.data.common.TodoHistoryService;
import cn.fw.erp.sdk.api.result.UserRoleInfo;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.*;

/**
 * 账号后台分析功能确认开通实现
 *
 * @author: lym
 * @date: 2023/4/19
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class AccountConfirmBizService {
    private final ErpRpc erpRpc;
    private final AccountService accountService;
    private final TodoHistoryService todoHistoryService;
    private final IInterveneRecordService interveneRecordService;

    /**
     * 获取账号配置确认详情
     *
     * @param accountId 账号id
     * @return 详情
     */
    public AccountConfirmDetailVo getDetail(Long accountId) {
        Account account = accountService.getById(accountId);
        PublicUtil.assertNotEmpty(account, "账号信息不存在");
        AccountConfirmDetailVo detailVo = new AccountConfirmDetailVo();
        detailVo.setAccountNo(account.getAccount());
        detailVo.setAccountName(account.getAccount());
        detailVo.setPlatform(account.getType().getName());
        detailVo.setShopName(account.getShopName());
        detailVo.setStaffName(account.getUserName());
        List<String> urlList = new LinkedList<>();
        //有顺序要求
        if (PlatformEnum.KS.equals(account.getType())) {
            urlList.add("https://feewee-1255997270.cos.ap-chengdu.myqcloud.com/fw_fds_file//media_account_config_confirm/test1684918313741_kuaishou-1.png");
            urlList.add("https://feewee-1255997270.cos.ap-chengdu.myqcloud.com/fw_fds_file//media_account_config_confirm/test1684918313754_kuaishou-2.png");
        }
        if (PlatformEnum.DY.equals(account.getType())) {
            urlList.add("https://feewee-1255997270.cos.ap-chengdu.myqcloud.com/fw_fds_file//media_account_config_confirm/test1684918313767_douyin-2.png");
            urlList.add("https://feewee-1255997270.cos.ap-chengdu.myqcloud.com/fw_fds_file//media_account_config_confirm/test1684918313782_douyin-1.png");
        }
        detailVo.setGuideUrl(urlList);
        return detailVo;
    }

    /**
     * 主动确认已开通数据分析功能
     *
     * @param accountId 账号id
     */
    @Transactional(rollbackFor = Exception.class)
    public void confirm(Long accountId) {
        Account account = accountService.getById(accountId);
        PublicUtil.assertNotEmpty(account, "账号信息不存在");
        if (account.getEnableAnalysis()) {
            return;
        }
        //todo  判断实际是否开通
        account.setEnableAnalysis(true);
        accountService.updateById(account);
        //完成干预记录
        interveneRecordService.update(Wrappers.<InterveneRecord>lambdaUpdate()
                .eq(InterveneRecord::getType, InterveneTypeEnum.ACCOUNT_CONFIRM.getValue())
                .eq(InterveneRecord::getDataId, accountId)
                .set(InterveneRecord::getComplete, true));
        //完成待办
        List<TodoHistory> unCompleteList = todoHistoryService.list(Wrappers.<TodoHistory>lambdaQuery()
                .in(TodoHistory::getTodoCode, TodoCodeEnum.ACCOUNT_CONFIG_CONFIRM.getCode(), TodoCodeEnum.ACCOUNT_CONFIG_INTERVENE.getCode())
                .eq(TodoHistory::getDataId, accountId)
                .eq(TodoHistory::getDone, ToDoTypeEnum.ADD)
                .eq(TodoHistory::getTodoDone, Boolean.FALSE));
        unCompleteList.forEach(r -> r.setDone(ToDoTypeEnum.COMPLETE.getValue()));
        if (PublicUtil.isNotEmpty(unCompleteList)) {
            todoHistoryService.updateBatchById(unCompleteList);
        }
    }

    /**
     * 上报未开通数据分析功能
     *
     * @param accountNo 账号id
     * @param type      平台类型
     */
    @Transactional(rollbackFor = Exception.class)
    public void accountDataOpenInfoReport(String accountNo, Integer type) {
        Account account = accountService.getOne(Wrappers.<Account>lambdaQuery()
                .eq(Account::getAccount, accountNo)
                .eq(Account::getType, type));
        if (PublicUtil.isEmpty(account)) {
            return;
        }
        account.setEnableAnalysis(false);
        accountService.updateById(account);
        this.triggerConfirmTodo(account);
    }

    /**
     * 推送账号配置确认待办
     */
    @Transactional(rollbackFor = Exception.class)
    public void triggerConfirmTodo(Account account) {
        if (account.getEnableAnalysis()) {
            return;
        }
        TodoHistory one = todoHistoryService.getOne(Wrappers.<TodoHistory>lambdaQuery()
                .eq(TodoHistory::getTodoCode, TodoCodeEnum.ACCOUNT_CONFIG_CONFIRM.getCode())
                .eq(TodoHistory::getDataId, account.getId().toString())
                .eq(TodoHistory::getDone, ToDoTypeEnum.ADD.getValue())
                .last("limit 1"));
        if (PublicUtil.isNotEmpty(one)) {
            return;
        }
        TodoHistory todo = this.createBaseTodo(account.getId(), account.getShopId(), account.getUserId(),
                TodoCodeEnum.ACCOUNT_CONFIG_CONFIRM);
        Map<String, String> dynamicMap = this.createDynamicMap(account, AccountEnableAnalysisEnum.ADD.getReason());
        todo.setDynamicInfo(JSON.toJSONString(dynamicMap));
        todoHistoryService.save(todo);
    }

    /**
     * 账号信息变更
     *
     * @param oldAccount 原信息
     * @param nowAccount 信息信息
     */
    @Transactional(rollbackFor = Exception.class)
    public void accountChange(Account oldAccount, Account nowAccount) {
        if (PublicUtil.isEmpty(oldAccount) || oldAccount.getEnableAnalysis()) {
            return;
        }
        boolean platformSame = oldAccount.getType().equals(nowAccount.getType());
        boolean shopSame = oldAccount.getShopId().equals(nowAccount.getShopId());
        boolean staffSame = oldAccount.getUserId().equals(nowAccount.getUserId());
        if (platformSame && shopSame && staffSame) {
            return;
        }
        //完成干预记录
        interveneRecordService.update(Wrappers.<InterveneRecord>lambdaUpdate()
                .eq(InterveneRecord::getType, InterveneTypeEnum.ACCOUNT_CONFIRM.getValue())
                .eq(InterveneRecord::getDataId, oldAccount.getId())
                .set(InterveneRecord::getComplete, true));
        //取消原待办
        List<TodoHistory> unCompleteList = todoHistoryService.list(Wrappers.<TodoHistory>lambdaQuery()
                .in(TodoHistory::getTodoCode, TodoCodeEnum.ACCOUNT_CONFIG_CONFIRM.getCode(), TodoCodeEnum.ACCOUNT_CONFIG_INTERVENE.getCode())
                .eq(TodoHistory::getDataId, oldAccount.getId())
                .eq(TodoHistory::getDone, ToDoTypeEnum.ADD.getValue())
                .eq(TodoHistory::getTodoDone, Boolean.FALSE));
        unCompleteList.forEach(r -> r.setDone(ToDoTypeEnum.CANCEL.getValue()));
        if (PublicUtil.isNotEmpty(unCompleteList)) {
            todoHistoryService.updateBatchById(unCompleteList);
        }
        //重推新确认待办
        this.triggerConfirmTodo(nowAccount);
    }

    /**
     * 推送账号配置确认待办超期干预待办
     *
     * @param account 账号
     */
    @Transactional(rollbackFor = Exception.class)
    public void interveneCheck(Account account, List<MeddleItem> meddleItemList) {
        try {
            AccountInterveneDto handleParam = this.getHandleParam(account, meddleItemList);
            if (handleParam.getPass()) {
                return;
            }
            List<UserRoleInfo> userRoleInfos = erpRpc.queryUserRoleInfos(handleParam.getRoleCode(), account.getShopId());
            if (PublicUtil.isEmpty(userRoleInfos)) {
                log.error("账号所属门店{}下无{}人员", account.getShopName(), handleParam.getRoleName());
                return;
            }
            InterveneRecord interveneRecord = this.createInterveneRecord(account, userRoleInfos.get(0));
            interveneRecord.setRoleCode(handleParam.getRoleCode());
            interveneRecordService.save(interveneRecord);
            //推送待办
            TodoHistory todoHistory = this.createBaseTodo(account.getId(), interveneRecord.getShopId(),
                    interveneRecord.getStaffId(), TodoCodeEnum.ACCOUNT_CONFIG_INTERVENE);
            Map<String, String> dynamicMap = this.createDynamicMap(account, "该账号自媒体[账号配置确认]待办已超期,请及时干预处理");
            todoHistory.setDynamicInfo(JSON.toJSONString(dynamicMap));
            Map<String, Object> extraMap = new HashMap<>(1);
            extraMap.put("accountId", account.getId());
            todoHistory.setExtra(JSON.toJSONString(extraMap));
            todoHistoryService.save(todoHistory);
        } catch (Exception e) {
            log.error("推送账号配置确认超期待办异常accountId:{}", account.getId(), e);
        }
    }

    /**
     * 获取待办超期干预处理参数
     */
    private AccountInterveneDto getHandleParam(Account account, List<MeddleItem> meddleItemList) {
        AccountInterveneDto dto = new AccountInterveneDto();
        if (account.getEnableAnalysis()) {
            return dto;
        }
        TodoHistory history = todoHistoryService.getOne(Wrappers.<TodoHistory>lambdaQuery()
                .eq(TodoHistory::getDataId, account.getId())
                .eq(TodoHistory::getTodoCode, TodoCodeEnum.ACCOUNT_CONFIG_CONFIRM.getCode())
                .eq(TodoHistory::getUserId, account.getUserId())
                .eq(TodoHistory::getDone, ToDoTypeEnum.ADD.getValue())
                .last("limit 1"));
        if (PublicUtil.isEmpty(history)) {
            return dto;
        }
        dto.setHistory(history);
        dto.setDisDay(DateUtil.getDisDay(history.getPlanTime(), new Date()));
        MeddleItem item = meddleItemList.stream()
                .filter(r -> r.getThreshold().compareTo(BigDecimal.valueOf(dto.getDisDay())) == 0)
                .findFirst().orElse(null);
        if (PublicUtil.isEmpty(item)) {
            return dto;
        }
        InterveneRecord hasIntervene = interveneRecordService.getOne(Wrappers.<InterveneRecord>lambdaQuery()
                .eq(InterveneRecord::getType, InterveneTypeEnum.ACCOUNT_CONFIRM.getValue())
                .eq(InterveneRecord::getDataId, account.getId().toString())
                .eq(InterveneRecord::getRoleCode, item.getRoleCode())
                .eq(InterveneRecord::getComplete, false));
        if (PublicUtil.isNotEmpty(hasIntervene)) {
            return dto;
        }
        dto.setPass(false);
        dto.setThreshold(item.getThreshold());
        dto.setRoleCode(item.getRoleCode());
        dto.setRoleName(item.getRoleName());
        return dto;
    }


    /**
     * 创建干预记录
     */
    private InterveneRecord createInterveneRecord(Account account, UserRoleInfo userRoleInfo) {
        InterveneRecord interveneRecord = new InterveneRecord();
        interveneRecord.setGroupId(account.getGroupId());
        interveneRecord.setType(InterveneTypeEnum.ACCOUNT_CONFIRM.getValue());
        interveneRecord.setDataId(account.getId());
        interveneRecord.setRoleCode(userRoleInfo.getRoleCode());
        interveneRecord.setShopId(account.getShopId());
        interveneRecord.setShopName(account.getShopName());
        interveneRecord.setStaffId(userRoleInfo.getUserId());
        interveneRecord.setStaffName(userRoleInfo.getUserName());
        interveneRecord.setComplete(false);
        interveneRecord.setYn(Boolean.TRUE);
        return interveneRecord;
    }

    /**
     * 待办基础信息
     */
    private TodoHistory createBaseTodo(Long dataId, Long shopId, Long staffId, TodoCodeEnum codeEnum) {
        TodoHistory todo = new TodoHistory();
        todo.setSend(false);
        todo.setDone(ToDoTypeEnum.ADD.getValue());
        todo.setTodoDone(false);
        todo.setTodoCode(codeEnum.getCode());
        todo.setDataId(dataId);
        todo.setShopId(shopId);
        todo.setUserId(staffId);
        return todo;
    }

    /**
     * 创建动态参数
     */
    private Map<String, String> createDynamicMap(Account account, String reason) {
        Map<String, String> dynamicMap = new HashMap<>(5);
        dynamicMap.put("accountName", account.getAccount());
        dynamicMap.put("platform", account.getType().getName());
        dynamicMap.put("userStaffName", account.getUserName());
        dynamicMap.put("shopName", account.getShopName());
        dynamicMap.put("reason", reason);
        return dynamicMap;
    }


}
