package cn.fw.dalaran.service.biz.account;

import cn.fw.dalaran.common.domain.enums.RoleMsgEnum;
import cn.fw.dalaran.common.utils.PublicUtil;
import cn.fw.dalaran.domain.db.Account;
import cn.fw.dalaran.domain.db.config.ActivityTheme;
import cn.fw.dalaran.rpc.ehr.EhrRpc;
import cn.fw.dalaran.service.biz.config.ConfigCommonBizService;
import cn.fw.dalaran.service.data.AccountService;
import cn.fw.dalaran.service.data.ActivityThemeService;
import cn.fw.ehr.sdk.api.mq.StaffPostChangeEvent;
import cn.fw.ehr.sdk.api.result.PostInfo;
import cn.fw.ehr.sdk.api.result.StaffBaseInfo;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * 账号定时任务
 *
 * @author: LYM
 * @date: 2024/4/8 11:24
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class AccountTaskBizService {
    private final EhrRpc ehrRpc;
    private final AccountService accountService;
    private final ActivityThemeService activityThemeService;
    private final ConfigCommonBizService configCommonBizService;
    private final AccountChangeBizService accountChangeBizService;

    /**
     * 处理人员离职事件
     *
     * @param staffId 员工id
     */
    @Transactional(rollbackFor = Exception.class)
    public void handleStaffLeave(Long staffId) {
        this.removeAccount(staffId);
    }

    /**
     * 处理调岗
     *
     * @param event 员工id
     */
    @Transactional(rollbackFor = Exception.class)
    public void handleStaffChangePost(StaffPostChangeEvent event) {
        PostInfo postInfo = ehrRpc.getPostInfo(event.getNewPostId());
        if (!RoleMsgEnum.XSGW.getCode().equals(postInfo.getRoleCode())) {
            this.removeAccount(event.getStaffId());
            return;
        }
        this.changeShop(event);
    }

    /**
     * 删除账号
     *
     * @param staffId 员工id
     */
    @Transactional(rollbackFor = Exception.class)
    public void removeAccount(Long staffId) {
        List<Account> accountList = accountService.list(Wrappers.<Account>lambdaQuery()
                .eq(Account::getUserId, staffId));
        if (PublicUtil.isEmpty(accountList)) {
            return;
        }
        List<Long> ids = accountList.stream().map(Account::getId).collect(Collectors.toList());
        accountService.remove(Wrappers.<Account>lambdaQuery().in(Account::getId, ids));
        accountList.forEach(account -> accountChangeBizService.accountDataFreeze(account, null, null));
    }

    /**
     * 处理人员门店变动
     *
     * @param event 员工id
     */
    @Transactional(rollbackFor = Exception.class)
    public void changeShop(StaffPostChangeEvent event) {
        List<Account> accountList = accountService.list(Wrappers.<Account>lambdaQuery()
                .eq(Account::getUserId, event.getStaffId())
                .eq(Account::getYn, 1));
        if (PublicUtil.isEmpty(accountList)) {
            log.info("未配置账号");
            return;
        }
        //刷新账号
        Long staffId = event.getStaffId();
        StaffBaseInfo staffInfo = ehrRpc.getStaffBaseInfo(event.getStaffId());
        Long oldShopId = accountList.get(0).getShopId();
        Long newShopId = event.getNewShopId();
        if (!Objects.equals(oldShopId, newShopId)) {
            configCommonBizService.handleUserJoinThemeBacklog(staffInfo.getGroupId(), oldShopId, staffId,
                    false, true);
            configCommonBizService.handleUserJoinThemeBacklog(staffInfo.getGroupId(), newShopId, staffId,
                    true, true);
        }
        List<Account> hasChangeAccount = accountList.stream()
                .filter(account -> !account.getShopId().equals(newShopId))
                .peek(account -> {
                    account.setShopId(newShopId);
                    account.setShopName(event.getNewShopName());
                }).collect(Collectors.toList());
        if (PublicUtil.isEmpty(hasChangeAccount)) {
            return;
        }
        accountService.updateBatchById(hasChangeAccount);
        //调整进行中数据归属
        List<ActivityTheme> themes = activityThemeService.getGoingActivityTheme();
        if (PublicUtil.isNotEmpty(themes)) {
            hasChangeAccount.forEach(account -> {
                accountChangeBizService.accountDataFreeze(account, themes, Collections.singletonList(account));
                accountChangeBizService.accountDataRestore(account, themes, Collections.singletonList(account));
            });
        }
    }
}

