package cn.fw.dalaran.service.biz.common;

import cn.fw.dalaran.common.constants.Constants;
import cn.fw.dalaran.common.domain.enums.ToDoTypeEnum;
import cn.fw.dalaran.common.domain.enums.TodoCodeEnum;
import cn.fw.dalaran.domain.db.Account;
import cn.fw.dalaran.domain.db.LivePool;
import cn.fw.dalaran.domain.db.common.TodoHistory;
import cn.fw.dalaran.domain.db.config.ActivityTheme;
import cn.fw.dalaran.domain.enums.PlatformEnum;
import cn.fw.dalaran.domain.vo.ActivityThemeVo;
import cn.fw.dalaran.rpc.erp.UserRoleRpcService;
import cn.fw.dalaran.rpc.erp.dto.UserInfoDTO;
import cn.fw.dalaran.service.data.AccountService;
import cn.fw.dalaran.service.data.ActivityThemeService;
import cn.fw.dalaran.service.data.LivePoolService;
import cn.fw.dalaran.service.data.common.TodoHistoryService;
import com.alibaba.fastjson.JSON;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 定时检查直播审计待办实现
 *
 * @author: lym
 * @date: 2023/5/19
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class LiveCheckTaskBizService {
    private final AccountService accountService;
    private final LivePoolService livePoolService;
    private final TodoHistoryService todoHistoryService;
    private final UserRoleRpcService userRoleRpcService;
    private final ActivityThemeService activityThemeService;

    /**
     * 发送直播审计待办
     *
     * @param timeStamp 指定时间
     */
    public void sendLiveCheckBacklog(Long timeStamp) {
        final List<ActivityTheme> themeList = activityThemeService.lambdaQuery()
                .gt(ActivityTheme::getEndTime, new Date(timeStamp - 7 * 24 * 3600 * 1000))
                .list()
                .stream()
                .filter(item -> {
                    final long timeSub = timeStamp - item.getEndTime().getTime();
                    return 24 * 3600 * 1000 < timeSub && timeSub < 2 * 24 * 3600 * 1000;
                })
                .collect(Collectors.toList());// 找到(理论上)需要审计的主题
        if (CollectionUtils.isEmpty(themeList))
            return;
        final List<LivePool> bestLives = livePoolService.lambdaQuery()
                .in(LivePool::getThemeId, themeList.stream()
                        .map(ActivityTheme::getId)
                        .collect(Collectors.toList())
                )
                .ne(LivePool::getPlatform, PlatformEnum.DY.getValue())// 抖音直播不审计
                .eq(LivePool::getValidLive, 11)
                .list();// 找到主题对应的所有人的最佳直播
        final List<Long> waitCheckThemeIds = bestLives.stream()
                .map(LivePool::getThemeId)
                .distinct()
                .collect(Collectors.toList());// 根据最佳直播分布, 找到(真正)需要审核的主题id集合
        if (CollectionUtils.isEmpty(waitCheckThemeIds))
            return;
        final List<Long> usersInShopIds = accountService.lambdaQuery()
                .in(Account::getId, bestLives.stream()
                        .collect(Collectors.groupingBy(LivePool::getAccountId))
                        .keySet()
                )// 所有账户id
                .list()
                .stream()
                .map(Account::getShopId)
                .distinct()// 祛除重复门店
                .collect(Collectors.toList());// 最佳直播的账户分布在对应的门店id集合
        final List<ActivityThemeVo> themeVos = themeList.stream()
                .filter(item -> waitCheckThemeIds.contains(item.getId()))// 过滤出(真正)需要审核的主题
                .map(ActivityTheme::toVO)
                .collect(Collectors.toList());// 将主题转换成vo
        if (CollectionUtils.isEmpty(themeVos))
            return;
        List<TodoHistory> hasSaveTodo = todoHistoryService.lambdaQuery()
                .eq(TodoHistory::getTodoCode, TodoCodeEnum.CHECK_LIVE.getCode())
                .in(TodoHistory::getDataId, themeVos.stream()
                        .map(ActivityThemeVo::getId)
                        .collect(Collectors.toList())
                )
                .list();// 查询需要审核的主题已经保存的待办信息
        final SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        ArrayList<TodoHistory> collect = usersInShopIds
                .stream()
                .map(item -> {// 遍历每个门店
                    TodoHistory todo = new TodoHistory();
                    List<UserInfoDTO> users = userRoleRpcService.getUsers(item, Constants.ZBSJ_ROLE_CODE);// 获取门店拥有'直播审计'角色的人
                    if (!users.isEmpty()) {
                        todo.setSend(false);
                        todo.setDone(ToDoTypeEnum.ADD.getValue());
                        todo.setTodoDone(false);
                        todo.setTodoCode(TodoCodeEnum.CHECK_LIVE.getCode());
                        ActivityThemeVo activityThemeVo = themeVos.stream()
                                .filter(item1 -> item1.getShopIds().contains(item))
                                .collect(Collectors.toList())
                                .get(0);
                        todo.setDataId(activityThemeVo.getId());
                        todo.setShopId(item);
                        todo.setUserId(users.get(0).getUserId());
                        Map<String, String> dynamicParam = this.getLiveCheckDynamicParam(activityThemeVo, sdf);
                        todo.setDynamicInfo(JSON.toJSONString(dynamicParam));
                    }
                    return todo;
                })
                .filter(item -> Objects.nonNull(item.getTodoCode()))
                .collect(Collectors.collectingAndThen(Collectors.toCollection(() ->
                        new TreeSet<>(Comparator.comparing(TodoHistory::getRemoveDuplicatesCondition))), ArrayList::new)
                );
        if (!Objects.equals(hasSaveTodo.size(), 0) && Objects.equals(collect.size(), hasSaveTodo.size()))
            return;
        List<Long> themeIds = hasSaveTodo.stream()
                .map(TodoHistory::getDataId)
                .collect(Collectors.toList());
        todoHistoryService.saveBatch(collect.stream()
                .filter(item -> !themeIds.contains(item.getDataId()))
                .collect(Collectors.toList())
        );// 保存待办
    }

    /**
     * 获取直播审计待办展示参数
     *
     * @param activityThemeVo
     * @param sdf
     * @return
     */
    private Map<String, String> getLiveCheckDynamicParam(ActivityThemeVo activityThemeVo, SimpleDateFormat sdf) {
        Map<String, String> dynamicMap = new HashMap<>(4);
        dynamicMap.put("theme", activityThemeVo.getTheme());
        dynamicMap.put("startTime", sdf.format(activityThemeVo.getStartTime()));
        dynamicMap.put("endTime", sdf.format(activityThemeVo.getEndTime()));
        return dynamicMap;
    }
}
