package cn.fw.dalaran.service.biz.config;

import cn.fw.common.exception.BusinessException;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.dalaran.common.ConfigProperties;
import cn.fw.dalaran.common.constants.Constants;
import cn.fw.dalaran.common.constants.ErpTaskConstant;
import cn.fw.dalaran.common.domain.dto.ToDoDto;
import cn.fw.dalaran.common.domain.enums.ThemeChangeTypeEnum;
import cn.fw.dalaran.common.domain.enums.TodoCodeEnum;
import cn.fw.dalaran.common.page.PageHelper;
import cn.fw.dalaran.common.utils.DateUtil;
import cn.fw.dalaran.common.utils.ImageUtils;
import cn.fw.dalaran.common.utils.PageUtil;
import cn.fw.dalaran.common.utils.PublicUtil;
import cn.fw.dalaran.domain.db.Account;
import cn.fw.dalaran.domain.db.LivePool;
import cn.fw.dalaran.domain.db.ThemeFile;
import cn.fw.dalaran.domain.db.VideoPool;
import cn.fw.dalaran.domain.db.config.ActivityTheme;
import cn.fw.dalaran.domain.db.config.ConfigGroup;
import cn.fw.dalaran.domain.dto.activity.ActivityQueryDto;
import cn.fw.dalaran.domain.dto.activity.ActivitySaveDto;
import cn.fw.dalaran.domain.dto.activity.TopicDto;
import cn.fw.dalaran.domain.dto.common.ShopDto;
import cn.fw.dalaran.domain.enums.FileTypeEnum;
import cn.fw.dalaran.domain.enums.activity.ActivityStatusEnum;
import cn.fw.dalaran.domain.vo.ActivityThemeVo;
import cn.fw.dalaran.domain.vo.BrandVo;
import cn.fw.dalaran.domain.vo.UserThemeCompleteDetailsVo;
import cn.fw.dalaran.domain.vo.activity.ActivityDetailVo;
import cn.fw.dalaran.domain.vo.activity.ActivityVo;
import cn.fw.dalaran.domain.vo.activity.TopicVo;
import cn.fw.dalaran.domain.vo.common.ShopVo;
import cn.fw.dalaran.domain.vo.config.EvlVo;
import cn.fw.dalaran.domain.vo.config.FansLimitVo;
import cn.fw.dalaran.domain.vo.config.ValidVo;
import cn.fw.dalaran.rpc.backlog.TodoRpc;
import cn.fw.dalaran.rpc.dfs.FileRpc;
import cn.fw.dalaran.rpc.erp.ErpTaskRpc;
import cn.fw.dalaran.rpc.erp.dto.TaskSetReq;
import cn.fw.dalaran.rpc.oop.OopRpc;
import cn.fw.dalaran.service.Common;
import cn.fw.dalaran.service.data.*;
import cn.fw.dfs.sdk.api.result.FileInfo;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.imageio.ImageIO;
import javax.validation.Valid;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 活动主题配置实现
 *
 * @author: lym
 * @date: 2023/4/25
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class ActivityBizService {
    private final Common common;
    private final OopRpc oopRpc;
    private final TodoRpc todoRpc;
    private final FileRpc fileRpc;
    private final ErpTaskRpc erpTaskRpc;
    private final AccountService accountService;
    private final LivePoolService livePoolService;
    private final VideoPoolService videoPoolService;
    private final ThemeFileService themeFileService;
    private final ConfigProperties configProperties;
    private final ConfigGroupService configGroupService;
    private final ActivityThemeService activityThemeService;
    private final ConfigCommonBizService configCommonBizService;

    /**
     * 查询活动列表
     *
     * @param user       当前操作用户
     * @param queryParam 查询参数
     * @return 符合条件的活动列表
     */
    public PageHelper<ActivityVo> getActivityList(PageHelper<ActivityTheme> pageHelper, LoginAuthBean user,
                                                  ActivityQueryDto queryParam) {
        IPage<ActivityTheme> iPage = activityThemeService.page(PageUtil.helper2Page(pageHelper),
                Wrappers.<ActivityTheme>lambdaQuery()
                        .eq(ActivityTheme::getConfigGroupId, queryParam.getConfigGroupId())
                        .like(Objects.nonNull(queryParam.getTheme()), ActivityTheme::getTheme, queryParam.getTheme())
                        .like(Objects.nonNull(queryParam.getTopic()), ActivityTheme::getTopic, queryParam.getTopic())
                        .eq(ActivityTheme::getGroupId, user.getGroupId())
                        .orderByDesc(ActivityTheme::getId));
        if (PublicUtil.isEmpty(iPage.getRecords()))
            return new PageHelper<>(pageHelper.getCurrent(), pageHelper.getPageSize());
        List<Long> activityIds = iPage.getRecords().stream()
                .map(ActivityTheme::getId)
                .collect(Collectors.toList());
        List<ThemeFile> allFileList = themeFileService.list(Wrappers.<ThemeFile>lambdaQuery()
                .in(ThemeFile::getThemeId, activityIds)
                .eq(ThemeFile::getType, FileTypeEnum.THEME_COVER.getValue()));
        Map<Long, List<ThemeFile>> activityFileMap = allFileList.stream()
                .collect(Collectors.groupingBy(ThemeFile::getThemeId));
        Date now = new Date();
        return PageUtil.toPage(iPage, item -> {
            ActivityVo activityVo = PublicUtil.copy(item, ActivityVo.class);
            activityVo.setId(item.getId());
            if (now.before(item.getStartTime())) {
                activityVo.setStatus(ActivityStatusEnum.WAIT.getValue());
            } else {
                int status = now.compareTo(item.getEndTime()) >= 0
                        ? ActivityStatusEnum.COMPLETE.getValue() : ActivityStatusEnum.ONGOING.getValue();
                activityVo.setStatus(status);
            }
            activityVo.setCovers(activityFileMap.getOrDefault(item.getId(), new ArrayList<>()).stream()
                    .map(ThemeFile::getFileId)
                    .collect(Collectors.toList())
            );
            activityVo.setTopic(JSON.parseArray(item.getTopic(), TopicVo.class).stream()
                    .sorted(Comparator.comparing(TopicVo::getForcedCheck).reversed())
                    .collect(Collectors.toList())
            );// 设置话题, 强制校验话题放前面
            //账号粉丝要求
            activityVo.setFansLimit(Objects.nonNull(item.getFansLimit())
                    ? JSON.parseArray(item.getFansLimit(), FansLimitVo.class) : Collections.emptyList());
            return activityVo;
        });
    }

    /**
     * 查询活动详情
     *
     * @param themeId 活动主题id
     */
    public ActivityDetailVo detail(Long themeId) {
        ActivityTheme theme = activityThemeService.getById(themeId);
        PublicUtil.assertNotEmpty(theme, "该主题不存在");
        ActivityDetailVo detailVo = PublicUtil.copy(theme, ActivityDetailVo.class);
        detailVo.setId(theme.getId());
        detailVo.setStatus(new Date().before(theme.getStartTime())
                ? ActivityStatusEnum.WAIT.getValue()
                : new Date().compareTo(theme.getEndTime()) >= 0
                ? ActivityStatusEnum.COMPLETE.getValue() : ActivityStatusEnum.ONGOING.getValue());
        //封面图
        List<ThemeFile> fileList = themeFileService.list(Wrappers.<ThemeFile>lambdaQuery()
                .eq(ThemeFile::getThemeId, themeId)
                .eq(ThemeFile::getType, FileTypeEnum.THEME_COVER.getValue()));
        detailVo.setCovers(fileList.stream().map(ThemeFile::getFileId).collect(Collectors.toList()));
        //话题
        detailVo.setTopic(JSON.parseArray(theme.getTopic(), TopicVo.class).stream()
                .sorted(Comparator.comparing(TopicVo::getForcedCheck).reversed())
                .collect(Collectors.toList())
        );
        //指标
        List<EvlVo> evlVoList = JSON.parseArray(theme.getEvlConfigJson(), EvlVo.class);
        detailVo.setEvlItems(evlVoList.stream().peek(oldVo -> {
            List<EvlVo.EvlSubIndexVo> newItems = oldVo.getEvlSubItems().stream()
                    .filter(r -> Objects.nonNull(r.getRate()) && r.getRate().compareTo(BigDecimal.ZERO) > 0)
                    .collect(Collectors.toList());
            oldVo.setEvlSubItems(newItems);
        }).collect(Collectors.toList()));
        //有效性
        List<ValidVo> validVoList = JSON.parseArray(theme.getValidConfigJson(), ValidVo.class);
        detailVo.setValidItems(validVoList.stream()
                .filter(r -> PublicUtil.isNotEmpty(r.getValidSubItems()))
                .peek(oldVo -> {
                    List<ValidVo.ValidSubConfigVo> collect = oldVo.getValidSubItems().stream()
                            .filter(r -> Objects.nonNull(r.getValid()) && r.getValid())
                            .collect(Collectors.toList());
                    oldVo.setValidSubItems(collect);
                }).collect(Collectors.toList()));
        //门店范围
        detailVo.setShopRange(PublicUtil.isNotEmpty(theme.getShopJson())
                ? JSON.parseArray(theme.getShopJson(), ShopVo.class) : null);
        //账号粉丝要求
        detailVo.setFansLimit(Objects.nonNull(theme.getFansLimit())
                        ? JSON.parseArray(theme.getFansLimit(), FansLimitVo.class) : Collections.emptyList());
        return detailVo;
    }

    /**
     * 保存活动
     *
     * @param param 活动主题配置参数
     */
    @Transactional(rollbackFor = Exception.class)
    public void save(LoginAuthBean user, @Valid ActivitySaveDto param) {
        List<ThemeChangeTypeEnum> changeTypes = new ArrayList<>();
        boolean add = PublicUtil.isEmpty(param.getId());
        ConfigGroup configGroup = configGroupService.getById(param.getConfigGroupId());
        this.paramCheck(configGroup, param);
        ActivityTheme activityTheme = this.createActivityTheme(configGroup, param, changeTypes);
        configCommonBizService.validThemeTagConflict(activityTheme);
        activityThemeService.saveOrUpdate(activityTheme);
        List<ThemeFile> coverList = this.createThemeFile(activityTheme.getId(), param.getCovers(), changeTypes);// 保存图片文件
        /* 后端校验封面图格式及高宽比例 */
        /*List<String> errMsg = this.preCheckCovers(coverList);
        if (!CollectionUtils.isEmpty(errMsg)) {
            CompletableFuture.runAsync(() -> coverList.forEach(item -> fileRpc.delete(item.getFileId())));
            throw new BusinessException(String.join("#", errMsg));
        }*/
        themeFileService.saveBatch(coverList);
        this.downLoadCover(activityTheme.getId(), coverList);// 缓存图片
        if (Objects.isNull(activityTheme.getInitCheck()))
            activityTheme.setInitCheck(0);
        if (add) {// 推送初始化自媒体账号任务
            erpTaskRpc.setTask(TaskSetReq.<Long>builder()
                    .executeTime(DateUtil.plusDays(activityTheme.getStartTime(), 1))
                    .system(Constants.APPLICATION_NAME)
                    .param(activityTheme.getId())
                    .tag(ErpTaskConstant.INIT_ACCOUNT_STATUS)
                    .build()
            );
            this.checkJoinThemeBacklogSend(activityTheme, Collections.singletonList(ThemeChangeTypeEnum.FIRST_CONFIG));
        } else {
            this.checkJoinThemeBacklogSend(activityTheme, changeTypes);
        }
    }

    /**
     * 检查主题是否给人员发送参与待办
     *
     * @param theme       主题详情
     * @param changeTypes 变更类型集合
     */
    public void checkJoinThemeBacklogSend(ActivityTheme theme, List<ThemeChangeTypeEnum> changeTypes) {
        if (CollectionUtils.isEmpty(changeTypes))
            return;// 没有变更, 直接返回
        boolean haveChange = CollectionUtils.containsAny(changeTypes, ThemeChangeTypeEnum.getHaveChangeEnum());// 有增加封面/话题/变更时间
        if (Objects.equals(theme.getInitCheck(), 1) && !haveChange)
            return;// 已发送待办
        List<ToDoDto> todos = configCommonBizService.getUserJoinThemeBacklogParams(theme, changeTypes);
        if (CollectionUtils.isEmpty(todos))
            return;// 找不到收待办的人, 直接返回
        Date now = new Date();// 当前时间
        Date startTime = theme.getStartTime();// 主题开始时间
        if (now.before(startTime)) {
            todoRpc.batchToDoCancel(TodoCodeEnum.NEED_JOIN_THEME.getCode(), todos);// 取消之前待办
            todoRpc.batchToDoCancel(TodoCodeEnum.THEME_CHANGE.getCode(), todos);// 取消之前待办
            return;// 当前时间在主题开始时间之前(提前配置的之后的主题||主题开始时间延后)
        }
        if (haveChange) {
            todoRpc.batchToDoCancel(TodoCodeEnum.NEED_JOIN_THEME.getCode(), todos);// 取消之前待办
            todoRpc.batchToDoCancel(TodoCodeEnum.THEME_CHANGE.getCode(), todos);// 取消之前待办
        }
        todoRpc.batchToDoAdd(TodoCodeEnum.NEED_JOIN_THEME.getCode(), todos);// 重新发送待办
        todoRpc.batchToDoAdd(TodoCodeEnum.THEME_CHANGE.getCode(), todos);// 重新发送待办
        theme.setInitCheck(1);
        activityThemeService.updateById(theme);
    }

    /**
     * 前置校验封面图比例是否合规
     *
     * @param coverList 指定封面图
     * @return 错误信息
     */
    private CopyOnWriteArrayList<String> preCheckCovers(List<ThemeFile> coverList) {
        CopyOnWriteArrayList<String> errMsg = new CopyOnWriteArrayList<>();
        List<CompletableFuture<Void>> completableFutures = coverList.stream()
                .map(item -> CompletableFuture.runAsync(() -> this.checkSingleImage(item, errMsg)))
                .collect(Collectors.toList());
        CompletableFuture<?>[] threads = new CompletableFuture[completableFutures.size()];
        try {
            CompletableFuture.allOf(completableFutures.toArray(threads)).get();
        } catch (Exception e) {
            errMsg.add("多线程执行出错");
        }
        return errMsg;
    }

    /**
     * 校验单张封面图是否合规
     *
     * @param file   封面图对象
     * @param errMsg 错误信息
     */
    private void checkSingleImage(ThemeFile file, CopyOnWriteArrayList<String> errMsg) {
        String fileId = file.getFileId();
        File downloadFile = ImageUtils.convertFileByUrl(configProperties.getShow() + fileId, fileId);
        int scale = 12;// 精度
        String fileName = "";
        try {
            FileInfo fileInfo = fileRpc.queryInfo(fileId);
            fileName += fileInfo.getFileName();
            BufferedImage image = ImageIO.read(downloadFile);
            int height = image.getHeight();
            int width = image.getWidth();
            log.info("图片: {} 高度为: {} 像素, 宽度为: {} 像素", fileName, height, width);
            BigDecimal strand = BigDecimal.valueOf(16)
                    .divide(BigDecimal.valueOf(9), scale, RoundingMode.HALF_UP);
            BigDecimal upload = BigDecimal.valueOf(height)
                    .divide(BigDecimal.valueOf(width), scale, RoundingMode.HALF_UP);
            StringBuilder sb = new StringBuilder();
            if (!fileInfo.getExt().endsWith("png")) {
                sb.append("文件: ")
                        .append(fileName)
                        .append(" 不是png格式; ");
            }
            if (!Objects.equals(strand.compareTo(upload), 0)) {
                if (Objects.equals(sb.length(), 0))
                    sb.append("文件: ")
                            .append(fileName);
                sb.append(" 高宽比例不满足16:9; ");
            }
            if (!Objects.equals(sb.length(), 0)) {
                errMsg.add(sb.toString());
                //fileRpc.delete(fileId);
                sb.setLength(0);
            }
        } catch (IOException e) {
            errMsg.add(String.format("检查图片: %s 出错", fileName));
        } finally {
            downloadFile.delete();
        }
    }

    /**
     * 下载缓存封面图
     */
    public void downLoadCover(Long activityThemeId, List<ThemeFile> coverList) {
        String tempDir = common.getActivityThemeCoverDir();
        for (ThemeFile themeFile : coverList) {
            try {
                String fileId = themeFile.getFileId();
                File downloadFile = ImageUtils.convertFileByUrl(configProperties.getShow() + fileId, fileId);
                ImageUtils.modifyResolution1(downloadFile.getPath(),
                        tempDir + "activityTheme" + File.separator + activityThemeId /*+ "#" + activityTheme.getTheme()*/
                                + File.separator + "settingCover",
                        "fw_theme_cover_" + fileId, 512, 512);
                boolean result = downloadFile.delete();
            } catch (Exception e) {
                log.error("下载转换封面图失败", e);
            }
        }
    }

    /**
     * 参数检查
     */
    private void paramCheck(ConfigGroup configGroup, ActivitySaveDto param) {
        param.setStartTime(DateUtil.dayBegin(param.getStartTime()));
        param.setEndTime(DateUtil.dayEnd(param.getEndTime()));
        if (!DateUtil.sameMonth(param.getStartTime(), param.getEndTime()))
            throw new BusinessException("不支持配置跨月活动主题, 请调整活动起止时间为同一月");
        PublicUtil.assertTrue(param.getEndTime().compareTo(param.getStartTime()) > 0,
                "活动结束时间必须在开始时间之后");
        List<ActivityTheme> themeList = activityThemeService.lambdaQuery()
                .eq(ActivityTheme::getConfigGroupId, param.getConfigGroupId())
                .ge(ActivityTheme::getGroupId, configGroup.getGroupId())
                .ne(Objects.nonNull(param.getId()), ActivityTheme::getId, param.getId())
                .gt(ActivityTheme::getEndTime, new Date())
                .list();// 当前还未结束的活动主题(同配置组的)
        /*List<Long> currentShopIds = JSON.parseArray(configGroup.getShopJson(), ShopDto.class).stream()
                .map(ShopDto::getShopId)
                .collect(Collectors.toList());// 当前配置组指定门店id集合(当前待保存活动包含门店id集合)
        //循环检查
        for (ActivityTheme goingTheme : themeList) {
            if (DateUtil.validDateOverlap(goingTheme.getStartTime(), goingTheme.getEndTime(),
                    param.getStartTime(), param.getEndTime())) {
                List<ShopDto> themeShops = JSON.parseArray(goingTheme.getShopJson(), ShopDto.class);
                Set<Long> hasConfigShopIds = themeShops.stream()
                        .map(ShopDto::getShopId)
                        .collect(Collectors.toSet());
                PublicUtil.assertFalse(currentShopIds.stream().anyMatch(hasConfigShopIds::contains),
                        "指定时间段内[部分门店]已配置活动主题");
            }
        }*/
        for (ActivityTheme goingTheme : themeList) {
            PublicUtil.assertFalse(DateUtil.validDateOverlap(goingTheme.getStartTime(), goingTheme.getEndTime(),
                    param.getStartTime(), param.getEndTime()), "同时间段该配置组存在进行中的活动");
        }
    }

    /**
     * 创建活动主题
     */
    private ActivityTheme createActivityTheme(ConfigGroup configGroup, ActivitySaveDto param,
                                              List<ThemeChangeTypeEnum> changeTypes) {
        ActivityTheme activityTheme;
        if (PublicUtil.isEmpty(param.getId())) {
            activityTheme = PublicUtil.copy(param, ActivityTheme.class);
            activityTheme.setGroupId(configGroup.getGroupId());
            activityTheme.setConfigGroupId(configGroup.getId());
            activityTheme.setLogicDel(0);
            changeTypes.add(ThemeChangeTypeEnum.ADD_TOPIC);// 标识'新增了话题'
        } else {
            activityTheme = activityThemeService.getById(param.getId());
            PublicUtil.assertNotEmpty(activityTheme, "活动主题不存在");
            PublicUtil.assertFalse(activityThemeService.themeHaveStopCatchData(activityTheme.getEndTime(), 0), "活动已经结束");
            if (!Objects.equals(activityTheme.getStartTime(), param.getStartTime()) ||
                    !Objects.equals(activityTheme.getEndTime(), param.getEndTime()))
                changeTypes.add(ThemeChangeTypeEnum.CHANGE_TIME_RANGE);// 标识'活动主题时间范围变更'
            activityTheme.setStartTime(param.getStartTime());
        }
        activityTheme.setEndTime(param.getEndTime());
        activityTheme.setAllShop(configGroup.getAllShop());
        activityTheme.setShopJson(configGroup.getShopJson());
        activityTheme.setValidConfigJson(configCommonBizService.getValidConfigJson(configGroup.getId()));
        activityTheme.setEvlConfigJson(configCommonBizService.getEvlConfigJson(configGroup.getId()));
        //转换话题
        List<TopicDto> topicParam = param.getTopic().stream()
                .peek(item -> item.setTopic(item.getTopic().replace("#", "")))
                .collect(Collectors.toList());
        /* 校验不能删除老话题 */
        if (StringUtils.hasText(activityTheme.getTopic())) {
            List<TopicVo> oldTopics = JSON.parseArray(activityTheme.getTopic(), TopicVo.class);
            List<String> newTopics = param.getTopic().stream()
                    .map(TopicDto::getTopic)
                    .collect(Collectors.toList());// 本次提交话题
            oldTopics.forEach(item -> PublicUtil.assertTrue(newTopics.contains(item.getTopic()), "不可删除原有话题"));
            if (newTopics.size() > oldTopics.size())
                changeTypes.add(ThemeChangeTypeEnum.ADD_TOPIC);// 标识'新增了话题'
        }
        activityTheme.setTopic(JSON.toJSONString(topicParam));
        //主题得分前置条件
        activityTheme.setLiveValidTimes(param.getLiveValidTimes());
        activityTheme.setVideoValidTimes(param.getVideoValidTimes());
        activityTheme.setFansLimit(PublicUtil.isNotEmpty(param.getFansLimit())
                ? JSON.toJSONString(param.getFansLimit()) : null);
        return activityTheme;
    }

    /**
     * 创建封面图信息
     */
    private List<ThemeFile> createThemeFile(Long themeId, List<String> covers, List<ThemeChangeTypeEnum> changeTypes) {
        List<ThemeFile> themeFiles = themeFileService.list(Wrappers.<ThemeFile>lambdaQuery()
                .eq(ThemeFile::getThemeId, themeId)
                .eq(ThemeFile::getType, FileTypeEnum.THEME_COVER.getValue()));
        Set<String> oldCoverSet = themeFiles.stream()
                .map(ThemeFile::getFileId)
                .collect(Collectors.toSet());
        oldCoverSet.forEach(item -> PublicUtil.assertTrue(covers.contains(item), "不可删除原有封面"));
        if (covers.size() > oldCoverSet.size())
            changeTypes.add(ThemeChangeTypeEnum.ADD_COVER);// 标识'新增了封面'
        return covers.stream()
                .filter(item -> !oldCoverSet.contains(item))
                .map(item -> {
                    ThemeFile themeFile = new ThemeFile();
                    themeFile.setThemeId(themeId);
                    themeFile.setType(FileTypeEnum.THEME_COVER.getValue());
                    themeFile.setFileId(item);
                    return themeFile;
                })
                .collect(Collectors.toList())
                ;
    }

    /**
     * 删除活动主题
     *
     * @param id 活动id
     */
    @Transactional(rollbackFor = Exception.class)
    public void del(Long id) {
        ActivityTheme activityTheme = activityThemeService.getById(id);
        if (PublicUtil.isEmpty(activityTheme)) {
            return;
        }
        PublicUtil.assertTrue(new Date().compareTo(activityTheme.getEndTime()) <= 0, "活动已经结束, 无法删除");
        activityThemeService.removeById(id);
        themeFileService.remove(Wrappers.<ThemeFile>lambdaQuery()
                .eq(ThemeFile::getThemeId, id)
        );
        livePoolService.remove(Wrappers.<LivePool>lambdaQuery()
                .eq(LivePool::getThemeId, id)
        );
        videoPoolService.remove(Wrappers.<VideoPool>lambdaQuery()
                .eq(VideoPool::getThemeId, id)
        );
        List<ToDoDto> todos = configCommonBizService.getUserJoinThemeBacklogParams(activityTheme, null);
        if (!CollectionUtils.isEmpty(todos)) {
            todoRpc.batchToDoCancel(TodoCodeEnum.NEED_JOIN_THEME.getCode(), todos);// 取消待办
            todoRpc.batchToDoCancel(TodoCodeEnum.THEME_CHANGE.getCode(), todos);// 取消待办
        }
    }

    /**
     * 查询指定月份活动
     */
    public List<ActivityThemeVo> listByMonth(LoginAuthBean user, long timeStamp, List<Long> shopIds, Long brandId) {
        Set<Long> configIds = new HashSet<>();
        List<ConfigGroup> configGroups = configGroupService.lambdaQuery()
                .eq(ConfigGroup::getGroupId, user.getGroupId())
                .list();// 集团下所有配置组
        Map<Long, ConfigGroup> configMap = configGroups.stream()
                .collect(Collectors.toMap(ConfigGroup::getId, Function.identity()));
        if (Objects.nonNull(brandId)) {
            configIds = configGroups.stream()
                    .filter(item -> Objects.equals(item.getBrandId(), brandId))
                    .map(ConfigGroup::getId)
                    .collect(Collectors.toSet());
            if (CollectionUtils.isEmpty(configIds))
                configIds.add(-1L);
        }
        Date date = new Date(timeStamp);
        Date startTime = DateUtil.monthBegin(date);
        Date endTime = DateUtil.monthEnd(date);
        List<ActivityTheme> themeList = activityThemeService.lambdaQuery()
                .eq(ActivityTheme::getGroupId, user.getGroupId())
                .in(!CollectionUtils.isEmpty(configIds), ActivityTheme::getConfigGroupId, configIds)
                .between(ActivityTheme::getEndTime, startTime, endTime)
                .list();// 查询整月的活动主题
        return themeList.stream()
                .filter(theme -> JSON.parseArray(theme.getShopJson(), ShopDto.class).stream()
                        .anyMatch(shop -> shopIds.contains(shop.getShopId()))
                )
                .map(ActivityTheme::toVO)
                .peek(item -> {
                    ConfigGroup configGroup = configMap.get(item.getConfigGroupId());
                    if (Objects.nonNull(configGroup)) {
                        item.setBrandId(configGroup.getBrandId());
                        item.setBrandName(configGroup.getBrandName());
                    }
                })
                .sorted(Comparator.comparing(ActivityThemeVo::getId)
                        .thenComparing(ActivityThemeVo::getStartTime)
                        .reversed())
                .collect(Collectors.toList())
                ;
    }

    /**
     * 刷新主题封面图缓存
     *
     * @param themeId 主题id
     */
    public void flushThemeCoverCache(Long themeId) {
        List<ThemeFile> files = themeFileService.lambdaQuery()
                .eq(ThemeFile::getThemeId, themeId)
                .eq(ThemeFile::getType, FileTypeEnum.THEME_COVER.getValue())
                .list();
        if (CollectionUtils.isEmpty(files))
            return;
        String tempDir = common.getActivityThemeCoverDir();
        String dirPath = tempDir + "activityTheme" + File.separator + themeId + File.separator + "settingCover";
        File file = new File(dirPath);
        if (file.isDirectory())
            common.deleteAllFile(dirPath, file);
        this.downLoadCover(themeId, files);
    }

    /**
     * 获取用户当前参与的进行中的主题有效作品完成进度详情
     *
     * @param user 当前操作用户
     * @return 用户当前参与的进行中的主题有效作品完成进度详情
     */
    public List<UserThemeCompleteDetailsVo> getUserAllGoingThemeCompleteDetails(LoginAuthBean user) {
        List<Account> accounts = accountService.lambdaQuery()
                .eq(Account::getUserId, user.getUserId())
                .list();
        PublicUtil.assertTrue(!CollectionUtils.isEmpty(accounts), "人员当前暂无自媒体账号");
        List<ConfigGroup> configGroups = configGroupService.lambdaQuery()
                .eq(ConfigGroup::getGroupId, user.getGroupId())
                .eq(ConfigGroup::getEnable, true)
                .list();
        if (CollectionUtils.isEmpty(configGroups))
            return new ArrayList<>();
        Long shopId = accounts.get(0).getShopId();
        List<Long> configIds = configGroups.stream()
                .filter(item -> {
                    String shopRange = item.getShopJson();
                    if (!StringUtils.hasText(shopRange))
                        return false;
                    List<ShopVo> shopVos = JSON.parseArray(shopRange, ShopVo.class);
                    if (CollectionUtils.isEmpty(shopVos))
                        return false;
                    return shopVos.stream()
                            .anyMatch(item1 -> Objects.equals(item1.getShopId(), shopId));
                })
                .map(ConfigGroup::getId)
                .collect(Collectors.toList());
        if (CollectionUtils.isEmpty(configIds))
            return new ArrayList<>();
        Date now = new Date();
        List<ActivityTheme> goingThemes = activityThemeService.lambdaQuery()
                .in(ActivityTheme::getConfigGroupId, configIds)
                .le(ActivityTheme::getStartTime, now)
                .ge(ActivityTheme::getEndTime, now)
                .list();// 获取进行中的主题
        if (CollectionUtils.isEmpty(goingThemes))
            return new ArrayList<>();
        List<Long> accountIds = accounts.stream()
                .map(Account::getId)
                .collect(Collectors.toList());
        Map<Long, List<VideoPool>> validVideoMap = videoPoolService.lambdaQuery()
                .in(VideoPool::getAccountId, accountIds)
                .ge(VideoPool::getValidVideo, 1)
                .list()
                .stream()
                .collect(Collectors.groupingBy(VideoPool::getThemeId));// 有效短视频map
        Map<Long, List<LivePool>> validLiveMap = livePoolService.lambdaQuery()
                .in(LivePool::getAccountId, accountIds)
                .ge(LivePool::getValidLive, 1)
                .list()
                .stream()
                .collect(Collectors.groupingBy(LivePool::getThemeId));// 有效直播map
        return goingThemes.stream()
                .map(item -> {
                    UserThemeCompleteDetailsVo vo = PublicUtil.copy(item, UserThemeCompleteDetailsVo.class);
                    vo.setActivityTheme(item);
                    vo.setShopId(shopId);
                    vo.setThemeId(item.getId());
                    vo.setNeedValidVideoCnt(item.getVideoValidTimes());
                    vo.setNeedValidLiveCnt(item.getLiveValidTimes());
                    List<VideoPool> validVideos = validVideoMap.get(item.getId());
                    vo.setCurrentValidVideoCnt(CollectionUtils.isEmpty(validVideos) ? 0 : validVideos.size());
                    List<LivePool> validLives = validLiveMap.get(item.getId());
                    vo.setCurrentValidLiveCnt(CollectionUtils.isEmpty(validLives) ? 0 : validLives.size());
                    vo.setTopic(JSON.parseArray(item.getTopic(), TopicVo.class).stream()
                            .sorted(Comparator.comparing(TopicVo::getForcedCheck).reversed())
                            .collect(Collectors.toList())
                    );
                    return vo;
                })
                .collect(Collectors.toList())
                ;
    }

    /**
     * 获取用户某主题有效作品完成进度详情
     *
     * @param user    当前操作用户
     * @param themeId 主题id
     * @return 用户某主题有效作品完成进度详情
     */
    public UserThemeCompleteDetailsVo getUserThemeCompleteDetails(LoginAuthBean user, Long themeId) {
        UserThemeCompleteDetailsVo vo = this.getUserAllGoingThemeCompleteDetails(user).stream()
                .filter(item -> Objects.equals(item.getThemeId(), themeId))
                .findFirst()
                .orElse(new UserThemeCompleteDetailsVo());
        if (Objects.equals(vo.getNeedValidLiveCnt(), vo.getCurrentValidLiveCnt()) &&
                Objects.equals(vo.getNeedValidVideoCnt(), vo.getCurrentValidVideoCnt()))
            configCommonBizService.handleBacklog(Collections.singletonList(
                    configCommonBizService.buildUserJoinThemeBacklogParam(vo.getShopId(), user.getUserId(),
                            vo.getActivityTheme(), null, null)), false);// 作品数量达标, 完成待办
        return vo;
    }

    /**
     * 获取活动主题最低能配置的开始时间
     *
     * @param configGroupId 配置组id
     * @return 活动主题最低能配置的开始时间
     */
    public Date getThemeCanStartMinTime(Long configGroupId) {
        List<ActivityTheme> themeList = activityThemeService.lambdaQuery()
                .eq(ActivityTheme::getConfigGroupId, configGroupId)
                .orderByDesc(ActivityTheme::getEndTime)
                .list();
        Date monthBegin = DateUtil.monthBegin(new Date());
        if (CollectionUtils.isEmpty(themeList))
            return monthBegin;
        Date endTime = themeList.get(0).getEndTime();// 最新的结束时间
        Date latestThemeEndPlus1Day = DateUtil.dayBegin(DateUtil.plusDays(endTime, 1));// 最新主题延期一天
        if (DateUtil.sameMonth(endTime, monthBegin))
            return latestThemeEndPlus1Day;
        if (endTime.before(monthBegin))
            return monthBegin;
        return latestThemeEndPlus1Day;
    }

    /**
     * 获取可选的品牌列表
     *
     * @param user 当前用户
     * @return 可选的品牌列表
     */
    public List<BrandVo> getCanSelectBrands(LoginAuthBean user) {
        return oopRpc.getGroupBrands(user.getGroupId()).stream()
                .map(item -> {
                    BrandVo vo = new BrandVo();
                    vo.setBrandId(item.getId());
                    vo.setBrandName(item.getName());
                    return vo;
                })
                .collect(Collectors.toList());
    }

}
