package cn.fw.dalaran.service.biz.config;

import cn.fw.common.exception.BusinessException;
import cn.fw.dalaran.common.domain.dto.ToDoDto;
import cn.fw.dalaran.common.domain.enums.ThemeChangeTypeEnum;
import cn.fw.dalaran.common.domain.enums.TodoCodeEnum;
import cn.fw.dalaran.common.utils.DateUtil;
import cn.fw.dalaran.common.utils.PublicUtil;
import cn.fw.dalaran.domain.db.Account;
import cn.fw.dalaran.domain.db.config.*;
import cn.fw.dalaran.domain.dto.activity.TopicDto;
import cn.fw.dalaran.domain.dto.common.ShopDto;
import cn.fw.dalaran.domain.dto.config.EvlDto;
import cn.fw.dalaran.domain.dto.config.ValidDto;
import cn.fw.dalaran.domain.enums.config.SubTypeEnum;
import cn.fw.dalaran.domain.enums.config.WorksTypeEnum;
import cn.fw.dalaran.domain.vo.common.ShopVo;
import cn.fw.dalaran.domain.vo.config.ConfigGroupDetailVo;
import cn.fw.dalaran.domain.vo.config.EvlVo;
import cn.fw.dalaran.domain.vo.config.ValidVo;
import cn.fw.dalaran.rpc.backlog.TodoRpc;
import cn.fw.dalaran.service.data.*;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.google.common.collect.HashBasedTable;
import com.google.common.collect.Table;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 配置组公共实现
 *
 * @author: lym
 * @date: 2023/4/28
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class ConfigCommonBizService {
    private final TodoRpc todoRpc;
    private final AccountService accountService;
    private final EvlIndexService evlIndexService;
    private final ConfigGroupService configGroupService;
    private final EvlSubIndexService evlSubIndexService;
    private final ActivityThemeService activityThemeService;
    private final ValidConfigNewService validConfigNewService;

    /**
     * 创建指标配置数据
     *
     * @param configGroup
     * @param evlParams
     */
    @Async("asyncTaskExecutor")
    public CompletableFuture<Map<EvlIndex, List<EvlSubIndex>>> createEvlConfigInfo(ConfigGroup configGroup, List<EvlDto> evlParams) {
        Long configGroupId = configGroup.getId();
        List<EvlIndex> oldEvlIndexList = evlIndexService.list(Wrappers.<EvlIndex>lambdaQuery()
                .eq(EvlIndex::getConfigGroupId, configGroupId));
        List<EvlSubIndex> oldSubIndexList = evlSubIndexService.list(Wrappers.<EvlSubIndex>lambdaQuery()
                .eq(EvlSubIndex::getConfigGroupId, configGroupId));
        Map<EvlIndex, List<EvlSubIndex>> result = new HashMap<>();
        //首次初始化
        if (PublicUtil.isEmpty(oldEvlIndexList)) {
            for (EvlDto evlIndexDto : evlParams) {
                WorksTypeEnum worksTypeEnum = WorksTypeEnum.ofValue(evlIndexDto.getType());
                EvlIndex evlIndex = EvlIndex.initBean(worksTypeEnum, configGroupId, evlIndexDto.getRate());
                List<EvlSubIndex> collect = evlIndexDto.getEvlSubItems().stream().map(r -> {
                    SubTypeEnum subTypeEnum = SubTypeEnum.ofValue(r.getSubType());
                    return EvlSubIndex.initBean(subTypeEnum, configGroupId, evlIndex.getId(), r.getRate(), r.getStandard());
                }).collect(Collectors.toList());
                result.put(evlIndex, collect);
            }
            return CompletableFuture.completedFuture(result);
        }
        //回填数据
        Map<Integer, EvlDto> paramMap = evlParams.stream().collect(Collectors.toMap(EvlDto::getType, r -> r));
        Map<Long, List<EvlSubIndex>> oldSubIndexMap = oldSubIndexList.stream().collect(Collectors.groupingBy(EvlSubIndex::getEvlId));
        //处理
        for (EvlIndex evlIndex : oldEvlIndexList) {
            //设置占比
            EvlDto evlIndexDto = paramMap.get(evlIndex.getType().getValue());
            evlIndex.setRate(PublicUtil.isEmpty(evlIndexDto) ? evlIndex.getRate() : evlIndexDto.getRate());
            //设置指标子项占比
            List<EvlSubIndex> evlSubIndexList = oldSubIndexMap.get(evlIndex.getId());
            Map<Integer, EvlDto.EvlSubIndexDto> subIndexDtoMap = evlIndexDto.getEvlSubItems()
                    .stream().collect(Collectors.toMap(EvlDto.EvlSubIndexDto::getSubType, r -> r));
            evlSubIndexList.forEach(r -> {
                EvlDto.EvlSubIndexDto evlSubIndexDto = subIndexDtoMap.get(r.getSubType().getValue());
                r.setRate(evlSubIndexDto.getRate());
                r.setStandard(evlSubIndexDto.getStandard());
            });
            result.put(evlIndex, evlSubIndexList);
        }
        return CompletableFuture.completedFuture(result);
    }

    /**
     * 创建有效性配置数据
     *
     * @param validParams
     * @return
     */
    @Async("asyncTaskExecutor")
    @Transactional(rollbackFor = Exception.class)
    public CompletableFuture<List<ValidConfigNew>> createValidConfigInfo(ConfigGroup configGroup, List<ValidDto> validParams) {
        //处理数据转换
        HashBasedTable<Integer, Integer, ValidDto.ValidSubConfigDto> table = HashBasedTable.create();
        for (ValidDto validItem : validParams) {
            for (ValidDto.ValidSubConfigDto validSubItem : validItem.getValidSubItems()) {
                table.put(validItem.getType(), validSubItem.getIndexType(), validSubItem);
            }
        }
        //历史数据
        List<ValidConfigNew> oldValidConfigList = validConfigNewService.list(Wrappers.<ValidConfigNew>lambdaQuery()
                .eq(ValidConfigNew::getConfigGroupId, configGroup.getId()));
        //首次初始化
        if (PublicUtil.isEmpty(oldValidConfigList)) {
            List<ValidConfigNew> result = new LinkedList<>();
            for (Table.Cell<Integer, Integer, ValidDto.ValidSubConfigDto> cell : table.cellSet()) {
                ValidDto.ValidSubConfigDto configDto = cell.getValue();
                Integer type = cell.getRowKey();
                Integer indexType = cell.getColumnKey();
                result.add(ValidConfigNew.createConfig(indexType, configGroup.getId(), type, configDto.getMinValue(), configDto.getValid()));
            }
            return CompletableFuture.completedFuture(result);
        }
        for (ValidConfigNew configNew : oldValidConfigList) {
            ValidDto.ValidSubConfigDto validSubDto = table.get(configNew.getType(), configNew.getIndexType());
            if (PublicUtil.isNotEmpty(validSubDto)) {
                configNew.setValid(validSubDto.getValid());
                configNew.setMinValue(validSubDto.getMinValue());
            }
        }
        return CompletableFuture.completedFuture(oldValidConfigList);
    }

    /**
     * 获取配置组有效性配置
     *
     * @param configGroupId
     * @return
     */
    public String getValidConfigJson(Long configGroupId) {
        List<ValidConfigNew> configNewList = validConfigNewService.list(Wrappers.<ValidConfigNew>lambdaQuery()
                .eq(ValidConfigNew::getConfigGroupId, configGroupId));
        List<ValidVo> validVoList = ConfigGroupDetailVo.toValidVo(configNewList);
        return JSON.toJSONString(validVoList);
    }

    /**
     * 获取配置组指标评分配置
     *
     * @param configGroupId
     * @return
     */
    public String getEvlConfigJson(Long configGroupId) {
        List<EvlIndex> evlIndexList = evlIndexService.list(Wrappers.<EvlIndex>lambdaQuery()
                .eq(EvlIndex::getConfigGroupId, configGroupId));
        List<EvlSubIndex> evlSubIndexList = evlSubIndexService.list(Wrappers.<EvlSubIndex>lambdaQuery()
                .eq(EvlSubIndex::getConfigGroupId, configGroupId));
        List<EvlVo> evlVos = ConfigGroupDetailVo.toEvlVo(evlIndexList, evlSubIndexList);
        return JSON.toJSONString(evlVos);
    }

    /**
     * 校验本主题门店同时参与的主题话题是有否冲突
     *
     * @param theme 本次主题
     */
    public void validThemeTagConflict(ActivityTheme theme) {
        String shopJson = theme.getShopJson();
        if (!StringUtils.hasText(shopJson))
            return;
        Set<Long> shopIds = JSON.parseArray(shopJson, ShopDto.class)
                .stream()
                .map(ShopDto::getShopId)
                .collect(Collectors.toSet());// 本次配置的主题适应门店id列表
        Set<String> tags = JSON.parseArray(theme.getTopic(), TopicDto.class)
                .stream()
                .map(TopicDto::getTopic)
                .collect(Collectors.toSet());// 本次配置的主题话题列表
        List<ActivityTheme> themes = activityThemeService.lambdaQuery()
                .eq(ActivityTheme::getGroupId, theme.getGroupId())
                .ge(ActivityTheme::getEndTime, theme.getStartTime())
                .ne(Objects.nonNull(theme.getId()), ActivityTheme::getId, theme.getId())
                .list()
                .stream()
                .filter(item -> {
                    if (!StringUtils.hasText(item.getShopJson()))
                        return true;
                    return JSON.parseArray(item.getShopJson(), ShopDto.class)
                            .stream()
                            .anyMatch(item1 -> shopIds.contains(item1.getShopId()));
                })// 门店有交集
                .filter(item -> DateUtil.validDateOverlap(item.getStartTime(), item.getEndTime(),
                        theme.getStartTime(), theme.getEndTime())
                )// 活动时间有交集
                .collect(Collectors.toList());// 和本次配置的主题时间/门店有交集的活动主题
        if (CollectionUtils.isEmpty(themes))
            return;
        for (ActivityTheme item : themes) {
            List<TopicDto> existTags = JSON.parseArray(item.getTopic(), TopicDto.class);
            for (TopicDto existTag : existTags) {
                if (tags.contains(existTag.getTopic()))
                    throw new BusinessException(String.format("话题: %s 与主题: %s(%s)中话题冲突",
                            existTag.getTopic(), item.getTheme(), item.getId())
                    );
            }
        }
    }

    /**
     * 获取'参加活动主题待办'参数
     *
     * @param theme       主题
     * @param changeTypes 变更类型列表
     * @return '参加活动主题待办'参数
     */
    public List<ToDoDto> getUserJoinThemeBacklogParams(ActivityTheme theme, List<ThemeChangeTypeEnum> changeTypes) {
        String shopJson = theme.getShopJson();
        if (!StringUtils.hasText(shopJson))
            return new ArrayList<>();// 找不到门店, 直接返回
        Set<Long> shopIds = JSON.parseArray(shopJson, ShopVo.class).stream()
                .map(ShopVo::getShopId)
                .collect(Collectors.toSet());// 主题参与的门店id集合
        ConfigGroup configGroup = configGroupService.getById(theme.getConfigGroupId());
        return accountService.lambdaQuery()
                .in(Account::getShopId, shopIds)
                .list()
                .stream()
                .collect(Collectors.collectingAndThen(Collectors.toCollection(() ->
                        new TreeSet<>(Comparator.comparing(Account::getUserId))), ArrayList::new))
                .stream()
                .map(item -> this.buildUserJoinThemeBacklogParam(item.getShopId(), item.getUserId(), theme,
                        configGroup, changeTypes))
                .collect(Collectors.toList())
                ;
    }

    /**
     * 构建人员'参加活动主题待办'参数
     *
     * @param account     人员账号
     * @param theme       主题
     * @param configGroup 配置组
     * @param changeTypes 变更类型列表
     * @return 人员'参加活动主题待办'参数
     */
    public ToDoDto buildUserJoinThemeBacklogParam(Long shopId, Long userId, ActivityTheme theme,
                                                  ConfigGroup configGroup, List<ThemeChangeTypeEnum> changeTypes) {
        ToDoDto backlog = new ToDoDto();
        backlog.setDataId(String.valueOf(theme.getId()));
        backlog.setShopId(shopId);
        backlog.setUserId(userId);
        backlog.setTodoCode(TodoCodeEnum.NEED_JOIN_THEME.getCode());// 自媒体要参加的活动主题(UDS3JTyBF6)
        backlog.setPlanTime(new Date());
        backlog.setExpireTime(theme.getEndTime());
        Map<String, String> params = new HashMap<>();
        if (Objects.nonNull(configGroup))
            params.put("brandName", configGroup.getBrandName());
        params.put("themeId", String.valueOf(theme.getId()));
        params.put("themeName", theme.getTheme());
        params.put("themeTimeRange", DateUtil.format(theme.getStartTime(), DateUtil.FORMAT_2) + " ~ " +
                DateUtil.format(theme.getEndTime(), DateUtil.FORMAT_2));
        if (!CollectionUtils.isEmpty(changeTypes))
            params.put("themeChangeType", changeTypes.stream()
                    .map(ThemeChangeTypeEnum::getDesc)
                    .collect(Collectors.joining(", ")));
        backlog.setDynamicMap(params);
        Map<String, Object> extra = new HashMap<>();
        extra.put("themeId", String.valueOf(theme.getId()));
        backlog.setExtraData(extra);
        return backlog;
    }

    /**
     * 获取集团门店指定时间在进行的活动
     *
     * @param groupId 集团id
     * @param shopId  门店id
     * @param date    指定时间
     * @return 集团门店指定时间在进行的活动
     */
    public List<ActivityTheme> getShopGoingThemes(Long groupId, Long shopId, Date date) {
        return activityThemeService.lambdaQuery()
                .eq(ActivityTheme::getGroupId, groupId)
                .ge(ActivityTheme::getEndTime, date)
                .list()
                .stream()
                .filter(item -> {
                    if (!StringUtils.hasText(item.getShopJson()))
                        return true;
                    return JSON.parseArray(item.getShopJson(), ShopDto.class)
                            .stream()
                            .anyMatch(item1 -> Objects.equals(shopId, item1.getShopId()));
                })// 门店有交集
                .filter(item -> DateUtil.timePointInRange(date, item.getStartTime(), item.getEndTime())
                )// 活动时间有交集
                .collect(Collectors.toList());
    }

    /**
     * 增加账号/删除账号, 处理参与主题待办
     *
     * @param groupId       集团id
     * @param shopId        门店id
     * @param userId        用户id
     * @param addType       true:增加账号, false:删除账号
     * @param accountChange true:修改账号
     */
    public void handleUserJoinThemeBacklog(Long groupId, Long shopId, Long userId,
                                           boolean addType, boolean accountChange) {
        if (!accountChange) {
            List<Account> accounts = accountService.lambdaQuery()
                    .eq(Account::getUserId, userId)
                    .list();
            if (!CollectionUtils.isEmpty(accounts))
                return;// 添加账号, 之前已经有账号了, 说明早就发过待办了; 删除账号, 目前还有账号, 不能删除待办
        }
        List<ActivityTheme> shopGoingThemes = this.getShopGoingThemes(groupId, shopId, new Date());
        if (CollectionUtils.isEmpty(shopGoingThemes))
            return;
        Map<Long, ConfigGroup> configGroupMap = configGroupService.lambdaQuery()
                .eq(ConfigGroup::getGroupId, groupId)
                .list()
                .stream()
                .collect(Collectors.toMap(ConfigGroup::getId, Function.identity()));// K:configGroupId, V:配置组
        List<ThemeChangeTypeEnum> changeTypes = Collections.singletonList(ThemeChangeTypeEnum.FIRST_CONFIG);
        List<ToDoDto> todos = shopGoingThemes.stream()
                .map(item -> this.buildUserJoinThemeBacklogParam(shopId, userId, item,
                        configGroupMap.get(item.getConfigGroupId()), changeTypes))
                .collect(Collectors.toList());
        this.handleBacklog(todos, addType);
    }

    /**
     * 决定是发送待办还是取消待办
     *
     * @param todos   待办参数列表
     * @param addType true:发送待办, false:取消待办
     */
    public void handleBacklog(List<ToDoDto> todos, boolean addType) {
        if (CollectionUtils.isEmpty(todos))
            return;
        if (addType) {
            todoRpc.batchToDoAdd(TodoCodeEnum.NEED_JOIN_THEME.getCode(), todos);// 发送待办
            todoRpc.batchToDoAdd(TodoCodeEnum.THEME_CHANGE.getCode(), todos);// 发送待办
        } else {
            todoRpc.batchToDoCancel(TodoCodeEnum.NEED_JOIN_THEME.getCode(), todos);// 取消待办
            todoRpc.batchToDoCancel(TodoCodeEnum.THEME_CHANGE.getCode(), todos);// 取消待办
        }
    }
}
