package cn.fw.dalaran.service.biz.config;

import cn.fw.common.exception.BusinessException;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.dalaran.common.domain.dto.ToDoDto;
import cn.fw.dalaran.common.domain.enums.ThemeChangeTypeEnum;
import cn.fw.dalaran.common.page.PageHelper;
import cn.fw.dalaran.common.utils.DateUtil;
import cn.fw.dalaran.common.utils.PageUtil;
import cn.fw.dalaran.common.utils.PublicUtil;
import cn.fw.dalaran.domain.db.config.*;
import cn.fw.dalaran.domain.dto.common.ShopDto;
import cn.fw.dalaran.domain.dto.config.GroupQueryDto;
import cn.fw.dalaran.domain.dto.config.GroupSaveDto;
import cn.fw.dalaran.domain.enums.config.ShopRangeEnum;
import cn.fw.dalaran.domain.vo.config.ConfigGroupDetailVo;
import cn.fw.dalaran.domain.vo.config.ConfigGroupVo;
import cn.fw.dalaran.rpc.backlog.TodoRpc;
import cn.fw.dalaran.rpc.erp.ErpTaskRpc;
import cn.fw.dalaran.rpc.oop.OopRpc;
import cn.fw.dalaran.rpc.oop.dto.ShopDTO;
import cn.fw.dalaran.service.data.*;
import cn.fw.oop.sdk.enums.BizTypeEnum;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.LockSupport;
import java.util.stream.Collectors;

/**
 * 配置组配置实现
 *
 * @author: lym
 * @date: 2023/4/24
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class ConfigGroupBizService {
    private final OopRpc oopRpc;
    private final TodoRpc todoRpc;
    private final ErpTaskRpc erpTaskRpc;
    private final EvlIndexService evlIndexService;
    private final ConfigGroupService configGroupService;
    private final EvlSubIndexService evlSubIndexService;
    private final ActivityThemeService activityThemeService;
    private final ValidConfigNewService validConfigNewService;
    private final ConfigCommonBizService configCommonBizService;
    private final IConfigGroupDraftService configGroupDraftService;

    /**
     * 查询配置组列表
     *
     * @param user       当前登录用户信息
     * @param pageHelper 分页参数
     * @param queryDto   查询参数
     * @return 符合条件的配置列表
     */
    public PageHelper<ConfigGroupVo> getConfigGroupList(LoginAuthBean user, PageHelper<ConfigGroupVo> pageHelper,
                                                        GroupQueryDto queryDto) {
        //查询草稿
        if (queryDto.getDraft()) {
            IPage<ConfigGroupDraft> iPage = configGroupDraftService.page(PageUtil.helper2Page(pageHelper),
                    Wrappers.<ConfigGroupDraft>lambdaQuery()
                            .eq(ConfigGroupDraft::getGroupId, user.getGroupId())
                            .eq(ConfigGroupDraft::getUserId, user.getUserId())
                            .like(PublicUtil.isNotEmpty(queryDto.getConfigName()), ConfigGroupDraft::getConfigName,
                                    queryDto.getConfigName()));

            return PageUtil.toPage(iPage, ConfigGroupVo::dbToVo);
        }
        //查询生效列表
        IPage<ConfigGroup> iPage = configGroupService.page(PageUtil.helper2Page(pageHelper),
                Wrappers.<ConfigGroup>lambdaQuery()
                        .eq(ConfigGroup::getGroupId, user.getGroupId())
                        .eq(ConfigGroup::getYn, 1)
                        .like(PublicUtil.isNotEmpty(queryDto.getConfigName()), ConfigGroup::getConfigName,
                                queryDto.getConfigName())
                        .orderByDesc(ConfigGroup::getId));
        if (PublicUtil.isEmpty(iPage.getRecords()))
            return new PageHelper<>(pageHelper.getCurrent(), pageHelper.getPageSize());
        return PageUtil.toPage(iPage, ConfigGroupVo::dbToVo);
    }

    /**
     * 查询配置组详情（新增无需传递id）
     *
     * @param draft 是否草稿
     * @param id    查询参数
     * @return 详情
     */
    public ConfigGroupDetailVo getConfigGroupDetail(Boolean draft, Long id) {
        //返回初始数据
        if (PublicUtil.isEmpty(id))
            return ConfigGroupDetailVo.getInitConfig();
        //返回草稿信息
        if (draft) {
            ConfigGroupDraft configGroupDraft = configGroupDraftService.getById(id);
            PublicUtil.assertNotEmpty(configGroupDraft, "草稿信息不存在");
            return ConfigGroupDetailVo.dbToDraftVo(configGroupDraft);
        }
        //返回配置组信息
        ConfigGroup configGroup = configGroupService.getById(id);
        PublicUtil.assertNotEmpty(configGroup, "配置组信息不存在");
        List<ValidConfigNew> list = validConfigNewService.lambdaQuery()
                .eq(ValidConfigNew::getConfigGroupId, id)
                .list();
        List<EvlIndex> evlIndexList = evlIndexService.lambdaQuery()
                .eq(EvlIndex::getConfigGroupId, id)
                .list();
        List<EvlSubIndex> evlSubIndexList = evlSubIndexService.lambdaQuery()
                .eq(EvlSubIndex::getConfigGroupId, id)
                .list();
        return ConfigGroupDetailVo.dbToVo(configGroup, list, evlIndexList, evlSubIndexList);
    }

    /**
     * 保存草稿信息
     *
     * @param user  操作人
     * @param param 草稿信息
     */
    @Transactional(rollbackFor = Exception.class)
    public void saveDraft(LoginAuthBean user, GroupSaveDto param) {
        ConfigGroupDraft draft;
        if (PublicUtil.isEmpty(param.getDraftId())) {
            draft = new ConfigGroupDraft();
            draft.setGroupId(user.getGroupId());
            draft.setUserId(user.getUserId());
        } else {
            ConfigGroupDraft oldDraft = configGroupDraftService.getById(param.getDraftId());
            PublicUtil.assertNotEmpty(oldDraft, "原草稿信息不存在");
            PublicUtil.assertTrue(oldDraft.getUserId().equals(user.getUserId()), "当前草稿非您本人发布，不可操作编辑");
            draft = oldDraft;
        }
        draft.setConfigName(param.getConfigName());
        draft.setBrandId(param.getBrandId());
        draft.setBrandName(param.getBrandName());
        draft.setAllShop(param.getAllShop());
        draft.setShopJson(!CollectionUtils.isEmpty(param.getShopRange()) ?
                JSON.toJSONString(param.getShopRange()) : null);
        draft.setEvlJson(!CollectionUtils.isEmpty(param.getEvlItems()) ?
                JSON.toJSONString(param.getEvlItems()) : null);
        draft.setValidJson(!CollectionUtils.isEmpty(param.getValidItems()) ?
                JSON.toJSONString(param.getValidItems()) : null);
        configGroupDraftService.saveOrUpdate(draft);
    }

    /**
     * 保存配置组
     *
     * @param user  登录用户
     * @param param 新的配置参数
     */
    @Transactional(rollbackFor = Exception.class)
    public void save(LoginAuthBean user, GroupSaveDto param) {
        if (param.getDraft()) {
            this.saveDraft(user, param);
            return;
        }
        boolean add = PublicUtil.isEmpty(param.getId());
        this.preCheckParam(user, param);
        ConfigGroup configGroup = this.createConfigGroup(user, param);
        configGroupService.saveOrUpdate(configGroup);
        configGroupDraftService.removeByDraft(param.getDraftId());
        try {
            CompletableFuture<List<ValidConfigNew>> validResult =
                    configCommonBizService.createValidConfigInfo(configGroup, param.getValidItems());
            CompletableFuture<Map<EvlIndex, List<EvlSubIndex>>> evlResult =
                    configCommonBizService.createEvlConfigInfo(configGroup, param.getEvlItems());
            List<ValidConfigNew> configNewList = validResult.get();
            Map<EvlIndex, List<EvlSubIndex>> evlIndexListMap = evlResult.get();
            List<EvlSubIndex> evlSubIndexList = new ArrayList<>();
            //新增
            if (add) {
                validConfigNewService.insertBatchSomeColumn(configNewList);
                evlIndexService.insertBatchSomeColumn(new ArrayList<>(evlIndexListMap.keySet()));
                for (Map.Entry<EvlIndex, List<EvlSubIndex>> entry : evlIndexListMap.entrySet()) {
                    entry.getValue().forEach(r -> r.setEvlId(entry.getKey().getId()));
                    evlSubIndexList.addAll(entry.getValue());
                }
                evlSubIndexService.insertBatchSomeColumn(evlSubIndexList);
            } else {
                ArrayList<EvlIndex> evlIndices = new ArrayList<>(evlIndexListMap.keySet());
                evlIndexService.updateBatch(evlIndices);
                evlIndexListMap.values().forEach(evlSubIndexList::addAll);
                evlSubIndexService.updateBatch(evlSubIndexList);
                validConfigNewService.updateBatch(configNewList);
            }
        } catch (Exception e) {
            log.error("保存失败" + e);
            throw new BusinessException("保存有效性/指标信息失败");
        }
        // 异步数据更新
        CompletableFuture.runAsync(() -> {
            LockSupport.parkNanos(TimeUnit.SECONDS.toNanos(5));
            this.syncGroupConfigToTheme(configGroup.getId());
        });
    }

    /**
     * 创建配置组信息
     */
    private ConfigGroup createConfigGroup(LoginAuthBean user, GroupSaveDto param) {
        boolean add = PublicUtil.isEmpty(param.getId());
        ConfigGroup configGroup;
        if (add) {
            configGroup = new ConfigGroup();
            configGroup.setGroupId(user.getGroupId());
            configGroup.setEnable(true);
            configGroup.setYn(1);
        } else {
            configGroup = configGroupService.getById(param.getId());
            PublicUtil.assertNotEmpty(configGroup, "配置组信息不存在");
        }
        //基础配置处理
        configGroup.setConfigName(param.getConfigName());
        configGroup.setBrandId(param.getBrandId());
        configGroup.setBrandName(param.getBrandName());
        configGroup.setAllShop(param.getAllShop());
        configGroup.setShopJson(JSON.toJSONString(param.getShopRange()));
        configGroup.setUserId(user.getUserId());
        configGroup.setUserName(user.getUserName());
        return configGroup;
    }

    /**
     * 参数检查
     */
    private void preCheckParam(LoginAuthBean user, GroupSaveDto param) {
        //门店范围校验
        Long configGroupId = param.getId();
        boolean allShop = Objects.equals(ShopRangeEnum.ALL.getValue(), param.getAllShop());
        List<ConfigGroup> existConfigList = configGroupService.list(Wrappers.<ConfigGroup>lambdaQuery()
                .eq(ConfigGroup::getBrandId, param.getBrandId())
                .eq(ConfigGroup::getGroupId, user.getGroupId())
                .ne(Objects.nonNull(configGroupId), ConfigGroup::getId, configGroupId)
                .eq(ConfigGroup::getEnable, true));// 获取该品牌已存在的配置组
        if (allShop) {
            PublicUtil.assertEmpty(existConfigList, "无法选择[适用全部门店], 部分门店已被配置在同品牌的其它配置组中");
            List<ShopDTO> groupShops = oopRpc.getGroupShops(user.getGroupId(), BizTypeEnum.PRE_SALE.getValue());
            param.setShopRange(groupShops.stream()
                    .map(r -> {
                        ShopDto shopDto = new ShopDto();
                        shopDto.setShopId(r.getId());
                        shopDto.setShopName(r.getName());
                        return shopDto;
                    })
                    .collect(Collectors.toList())
            );
        } else {
            PublicUtil.assertTrue(!CollectionUtils.isEmpty(param.getShopRange()), "未选择适用门店范围");
            for (ConfigGroup configGroup : existConfigList) {
                PublicUtil.assertFalse(Objects.equals(ShopRangeEnum.ALL.getValue(), configGroup.getAllShop()),
                        String.format("适用门店范围错误, 配置组[%s](%s)已授权[全部门店]", configGroup.getConfigName(),
                                configGroup.getId())
                );
                //单个门店校验
                Set<Long> existShopSet = JSON.parseArray(configGroup.getShopJson(), ShopDto.class)
                        .stream()
                        .map(ShopDto::getShopId)
                        .collect(Collectors.toSet());// 已存在配置组配置的门店id列表
                for (ShopDto shopDto : param.getShopRange()) {
                    PublicUtil.assertFalse(existShopSet.contains(shopDto.getShopId()),
                            String.format("适用门店范围错误, 门店[%s]已存在于配置组[%s](%s)中", shopDto.getShopName(),
                                    configGroup.getConfigName(), configGroup.getId())
                    );
                }
            }
        }
        if (Objects.nonNull(configGroupId)) {// 校验如果门店配置生效后, 配置组下主题参与门店是否会有话题冲突
            List<ActivityTheme> themes = activityThemeService.lambdaQuery()
                    .eq(ActivityTheme::getConfigGroupId, configGroupId)
                    .ge(ActivityTheme::getEndTime, new Date())
                    .list();// 配置组下进行中及待进行的主题
            if (!CollectionUtils.isEmpty(themes)) {
                String shopJson = JSON.toJSONString(param.getShopRange());
                themes.stream()
                        .peek(item -> item.setShopJson(shopJson))
                        .forEach(configCommonBizService::validThemeTagConflict)
                ;
            }
        }
        param.validCheck();// 有效性配置校验
        param.evlCheck();// 指标配置校验
    }

    /**
     * 删除配置组
     *
     * @param draftId       草稿id
     * @param configGroupId 配置组id
     */
    @Transactional(rollbackFor = Exception.class)
    public void del(Long draftId, Long configGroupId) {
        //删除草稿
        if (PublicUtil.isNotEmpty(draftId))
            configGroupDraftService.removeById(draftId);
        if (PublicUtil.isEmpty(configGroupId))
            return;
        //删除配置组
        List<ActivityTheme> list = activityThemeService.list(Wrappers.<ActivityTheme>lambdaQuery()
                .eq(ActivityTheme::getConfigGroupId, configGroupId));
        PublicUtil.assertEmpty(list, "当前配置组已存在活动主题，无法删除");
        ConfigGroup configGroup = configGroupService.getById(configGroupId);
        if (PublicUtil.isNotEmpty(configGroup)) {
            configGroupService.removeById(configGroupId);
            validConfigNewService.remove(Wrappers.<ValidConfigNew>lambdaQuery()
                    .eq(ValidConfigNew::getConfigGroupId, configGroupId)
            );
            evlIndexService.remove(Wrappers.<EvlIndex>lambdaQuery()
                    .eq(EvlIndex::getConfigGroupId, configGroupId)
            );
            evlSubIndexService.remove(Wrappers.<EvlSubIndex>lambdaQuery()
                    .eq(EvlSubIndex::getConfigGroupId, configGroupId)
            );
        }
    }

    /**
     * 配置组更新，更新配置组缓存信息, 取消/重发主题相关待办
     *
     * @param configGroupId 配置组id
     */
    @Transactional(rollbackFor = Exception.class)
    public void syncGroupConfigToTheme(Long configGroupId) {
        ConfigGroup configGroup = configGroupService.getById(configGroupId);
        // 更新活动信息
//        final Date yesterday = DateUtil.minusDays(new Date(), 1);
        final Date yesterday = new Date();
        String validConfigJson = configCommonBizService.getValidConfigJson(configGroupId);
        String evlConfigJson = configCommonBizService.getEvlConfigJson(configGroupId);
        List<ActivityTheme> themes = activityThemeService.lambdaQuery()
                .eq(ActivityTheme::getConfigGroupId, configGroupId)
                .and(wrapper -> wrapper
                        .lt(ActivityTheme::getStartTime, yesterday)
                        .gt(ActivityTheme::getEndTime, yesterday)
                        .or()
                        .gt(ActivityTheme::getStartTime, yesterday))
                .list();
        if (CollectionUtils.isEmpty(themes))
            return;
        List<ShopDto> oldShops = JSON.parseArray(themes.get(0).getShopJson(), ShopDto.class);
        List<ShopDto> nowShops = JSON.parseArray(configGroup.getShopJson(), ShopDto.class);
        List<ShopDto> removeShops = new ArrayList<>(oldShops);
        List<ShopDto> addShops = new ArrayList<>(nowShops);
        removeShops.removeAll(nowShops);// 计算出移除的门店
        addShops.removeAll(oldShops);// 计算出新增的门店
        themes.forEach(item -> {
            item.setAllShop(configGroup.getAllShop());
            item.setShopJson(configGroup.getShopJson());
            item.setValidConfigJson(validConfigJson);
            item.setEvlConfigJson(evlConfigJson);
            if (activityThemeService.updateById(item)) {// 主题更新成功
                this.addOrCancelTodos(item, addShops, true);
                this.addOrCancelTodos(item, removeShops, false);
            }
        });
    }

    /**
     * 门店变动, 取消/重发主题相关待办
     *
     * @param theme   主题详情
     * @param shops   涉及门店信息
     * @param addType true:增加, false:取消
     */
    private void addOrCancelTodos(ActivityTheme theme, List<ShopDto> shops, boolean addType) {
        if (CollectionUtils.isEmpty(shops))
            return;
        theme.setShopJson(JSON.toJSONString(shops));
        List<ThemeChangeTypeEnum> changeTypes = Collections.singletonList(ThemeChangeTypeEnum.FIRST_CONFIG);
        List<ToDoDto> todos = configCommonBizService.getUserJoinThemeBacklogParams(theme, changeTypes);// 计算相关人员
        if (CollectionUtils.isEmpty(todos))
            return;
        configCommonBizService.handleBacklog(todos, addType);
    }
}
