package cn.fw.dalaran.service.biz.report;

import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.tuple.Pair;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.function.Function;
import java.util.function.ToIntFunction;
import java.util.function.ToLongFunction;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * 计算支持
 * <p>
 * create at 2019-11-22
 *
 * @author zhangjj
 */
public interface CalSupport {
    /**
     * 求最小数字
     *
     * @param numbers 数字列表
     * @param <T>     数字类型
     * @return 最小值
     */
    default <T extends Number> T min(final T... numbers) {
        if (ArrayUtils.isEmpty(numbers)) {
            return null;
        }
        T min = numbers[0];
        for (final T number : numbers) {
            if (Objects.compare(min, number, Comparator.nullsFirst(Comparator.comparing(v -> Optional.of(v).map(Number::doubleValue).orElse(null)))) > 0) {
                min = number;
            }
        }
        return min;
    }

    /**
     * 求最大数字
     *
     * @param numbers 数字列表
     * @param <T>     数字类型
     * @return 最大值
     */
    default <T extends Number> T max(final T... numbers) {
        if (ArrayUtils.isEmpty(numbers)) {
            return null;
        }
        T max = numbers[0];
        for (final T number : numbers) {
            if (Objects.compare(max, number, Comparator.nullsLast(Comparator.comparing(v -> Optional.of(v).map(Number::doubleValue).orElse(null)))) < 0) {
                max = number;
            }
        }
        return max;
    }

    /**
     * 除法
     *
     * @param num 分子
     * @param den 分母
     * @return 结果
     */
    default BigDecimal divide(final Number num, final Number den) {
        return divide(num, den, 4);
    }

    /**
     * 除法
     *
     * @param num   分子
     * @param den   分母
     * @param scale scale
     * @return 结果
     */
    default BigDecimal divide(final Number num, final Number den, final int scale) {
        return divide(num, den, scale, RoundingMode.HALF_UP);
    }

    /**
     * 除法
     *
     * @param num   分子
     * @param den   分母
     * @param scale scale
     * @param mode  rounding mode
     * @return 结果
     */
    default BigDecimal divide(final Number num, final Number den, final int scale, final RoundingMode mode) {
        final BigDecimal n = number2BigDecimal(num, 0);
        final BigDecimal d = number2BigDecimal(den, 0);
        if (n.doubleValue() == 0 || d.doubleValue() == 0) {
            // 分子或分母为0，返回0
            return BigDecimal.ZERO;
        }
        return n.divide(d, scale, mode);
    }

    /**
     * 减法
     *
     * @param min 被减数
     * @param sub 减数
     * @return 结果
     */
    default BigDecimal subtract(final Number min, final Number sub) {
        if (Objects.isNull(min) || Objects.isNull(sub)) {
            // 被减数或减数为空，返回0
            return BigDecimal.ZERO;
        }
        final BigDecimal m = number2BigDecimal(min, 0);
        final BigDecimal s = number2BigDecimal(sub, 0);
        return s.subtract(m);
    }

    /**
     * 乘法
     *
     * @param mud 被乘数
     * @param mur 乘数
     * @return 结果
     */
    default BigDecimal multiply(final Number mud, final Number mur) {
        if (Objects.isNull(mud) || Objects.isNull(mur)) {
            // 被乘数或乘数为空 返回0
            return BigDecimal.ZERO;
        }
        final BigDecimal d = number2BigDecimal(mud, 0);
        final BigDecimal r = number2BigDecimal(mur, 0);
        return r.multiply(d);
    }

    /**
     * 加法
     *
     * @param num 加数
     * @param den 加数
     * @return 结果
     */
    default BigDecimal add(final Number num, final Number den) {
        if (Objects.isNull(num) || Objects.isNull(den)) {
            // 加数，返回0
            return BigDecimal.ZERO;
        }
        final BigDecimal d = number2BigDecimal(den, 0);
        final BigDecimal n = number2BigDecimal(num, 0);
        return d.add(n);
    }

    /**
     * 除法，返回Int结果
     *
     * @param num 分子
     * @param den 分母
     * @return 结果
     */
    default int divide2Int(final Number num, final Number den) {
        return divide(num, den, 0, RoundingMode.HALF_UP).intValue();
    }

    /**
     * 除法，返回Int结果
     *
     * @param num 分子
     * @param den 分母
     * @return 结果
     */
    default int divide2IntUp(final Number num, final Number den) {
        return divide(num, den, 0, RoundingMode.UP).intValue();
    }

    /**
     * Number转BigDecimal
     *
     * @param num Number数值
     * @param def 默认值
     * @return BigDecimal数值
     */
    default BigDecimal number2BigDecimal(final Number num, final Number def) {
        return BigDecimal.valueOf(Optional.ofNullable(num).orElse(def).doubleValue());
    }

    /**
     * 集合中指定Int类型字段求和
     *
     * @param data  数据集合
     * @param field 需要求和的字段
     * @return 结果
     */
    default <T> Integer sumInt(final Collection<T> data, final ToIntFunction<T> field) {
        if (CollectionUtils.isEmpty(data)) {
            return 0;
        }
        return data.stream().mapToInt(field).sum();
    }

    /**
     * 集合中指定Int类型字段最大值
     *
     * @param data  数据集合
     * @param field 需要求和的字段
     * @return 结果
     */
    default <T> Integer max(final Collection<T> data, final ToIntFunction<T> field) {
        if (CollectionUtils.isEmpty(data)) {
            return 0;
        }
        return data.stream().mapToInt(field).max().orElse(0);
    }

    /**
     * 流中指定Int类型字段求和
     *
     * @param stream 数据流
     * @param field  需要求和的字段
     * @return 结果
     */
    default <T> Integer sumInt(final Stream<T> stream, final Function<T, Integer> field) {
        return sumInt(stream.map(field));
    }

    /**
     * Int流累加求和
     *
     * @param stream 数据流
     * @return 结果
     */
    default Integer sumInt(final Stream<Integer> stream) {
        return stream.mapToInt(v -> v).sum();
    }


    /**
     * Int流累加求和
     *
     * @param stream 数据流
     * @return 结果
     */
    default Long sumLong(final Stream<Long> stream) {
        return stream.mapToLong(v -> v).sum();
    }

    /**
     * 集合中指定Int类型字段求和
     *
     * @param data  数据集合
     * @param field 需要求和的字段
     * @return 结果
     */
    default <T> Long sumLong(final Collection<T> data, final ToLongFunction<T> field) {
        if (CollectionUtils.isEmpty(data)) {
            return 0L;
        }
        return data.stream().mapToLong(field).sum();
    }

    /**
     * 流中指定Int类型字段求和
     *
     * @param stream 数据流
     * @param field  需要求和的字段
     * @return 结果
     */
    default <T> Long sumLong(final Stream<T> stream, final Function<T, Long> field) {
        return sumLong(stream.map(field));
    }


    /**
     * 集合中指定Decimal类型字段求和
     *
     * @param data  数据集合
     * @param field 需要求和的字段
     * @return 结果
     */
    default <T> BigDecimal sumDecimal(final Collection<T> data, final Function<T, BigDecimal> field) {
        if (CollectionUtils.isEmpty(data)) {
            return BigDecimal.ZERO;
        }
        return sumDecimal(data.stream().filter(Objects::nonNull), field);
    }

    /**
     * 集合中指定Decimal类型字段求和
     *
     * @param data  数据集合
     * @param field 需要求和的字段
     * @return 结果
     */
    default <T> BigDecimal sumMillionDecimal(final Collection<T> data, final Function<T, BigDecimal> field) {
        if (CollectionUtils.isEmpty(data)) {
            return BigDecimal.ZERO;
        }
        return divide(sumDecimal(data.stream().filter(Objects::nonNull), field), 10000, 2);
    }

    /**
     * 流中指定Decimal类型字段求和
     *
     * @param stream 数据流
     * @param field  需要求和的字段
     * @return 结果
     */
    default <T> BigDecimal sumDecimal(final Stream<T> stream, final Function<T, BigDecimal> field) {
        return sumDecimal(stream.map(field));
    }

    /**
     * Decimal集客累加求和
     *
     * @param data 数据集合
     * @return 结果
     */
    default BigDecimal sumDecimal(final Collection<BigDecimal> data) {
        return sumDecimal(data.stream());
    }

    /**
     * Decimal流累加求和
     *
     * @param stream 数据流
     * @return 结果
     */
    default BigDecimal sumDecimal(final Stream<BigDecimal> stream) {
        return stream.filter(Objects::nonNull).reduce(BigDecimal.ZERO, BigDecimal::add);
    }

    /**
     * 总数
     */
    default Integer totalCount(final Integer... numbers) {
        return Arrays.stream(numbers).mapToInt(item -> Optional.ofNullable(item).orElse(0)).sum();
    }

    /**
     * 分组求和
     */
    default <T, K> Map<K, Integer> groupSummingInt(final List<T> list,
                                                   final Function<T, K> groupField,
                                                   final ToIntFunction<T> field) {
        if (CollectionUtils.isEmpty(list)) {
            return Collections.emptyMap();
        }
        return list.parallelStream()
                .collect(Collectors.groupingBy(groupField, Collectors.summingInt(field)));
    }

    /**
     * 分组求和
     */
    default <T, K> Map<K, BigDecimal> groupSummingDecimal(final List<T> list,
                                                          final Function<T, K> groupField,
                                                          final Function<T, BigDecimal> field) {
        if (CollectionUtils.isEmpty(list)) {
            return Collections.emptyMap();
        }
        final Map<K, List<T>> map = list.stream().collect(Collectors.groupingBy(groupField));
        final Map<K, BigDecimal> result = new HashMap<>(map.size());
        map.forEach((k, l) -> result.put(k, sumDecimal(l, field)));
        return result;
    }

    /**
     * 分组求和
     */
    default <T, K> Map<K, List<Pair<String, BigDecimal>>> groupSummingDecimal2(final List<T> list,
                                                                               final Function<T, K> groupField,
                                                                               final Function<T, String> groupField2,
                                                                               final Function<T, BigDecimal> field) {
        if (CollectionUtils.isEmpty(list)) {
            return Collections.emptyMap();
        }
        final Map<K, List<T>> map = list.stream().collect(Collectors.groupingBy(groupField));
        final Map<K, List<Pair<String, BigDecimal>>> result = new HashMap<>(map.size());
        map.forEach((k, l) -> {
            final Map<String, List<T>> collect = l.stream().collect(Collectors.groupingBy(groupField2));
            List<Pair<String, BigDecimal>> pairs = new ArrayList<>();
            collect.forEach((k1, l1) -> {
                pairs.add(Pair.of(k1, sumDecimal(l1, field)));
            });
            result.put(k, pairs);
        });
        return result;
    }
}
