package cn.fw.dalaran.service.biz.report;

import cn.fw.dalaran.common.domain.enums.RoleMsgEnum;
import cn.fw.dalaran.common.utils.DateUtil;
import cn.fw.dalaran.common.utils.PublicUtil;
import cn.fw.dalaran.domain.db.Account;
import cn.fw.dalaran.domain.db.LivePool;
import cn.fw.dalaran.domain.db.VideoPool;
import cn.fw.dalaran.domain.db.config.ActivityTheme;
import cn.fw.dalaran.domain.db.snapshot.ConsultantSnapshot;
import cn.fw.dalaran.domain.db.snapshot.ManagerSnapshot;
import cn.fw.dalaran.domain.dto.report.score.ScoreResultDto;
import cn.fw.dalaran.domain.enums.PlatformEnum;
import cn.fw.dalaran.domain.vo.account.AccountInfoVo;
import cn.fw.dalaran.domain.vo.activity.TopicVo;
import cn.fw.dalaran.domain.vo.report.UserCalRuleVo;
import cn.fw.dalaran.domain.vo.report.UserFansCountVo;
import cn.fw.dalaran.domain.vo.report.score.TargetResultDto;
import cn.fw.dalaran.domain.vo.report.score.WorksCompleteDetails;
import cn.fw.dalaran.service.data.AccountService;
import cn.fw.dalaran.service.data.ActivityThemeService;
import cn.fw.dalaran.service.data.IConsultantSnapshotService;
import cn.fw.dalaran.service.data.IManagerSnapshotService;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.time.LocalDate;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 短视频报表公共方法
 *
 * @author: lym
 * @date: 2023/5/15
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class ReportCommonBizService {
    private final AccountService accountService;
    private final ActivityThemeService activityThemeService;
    private final IManagerSnapshotService managerSnapshotService;
    private final IConsultantSnapshotService consultantSnapshotService;

    /**
     * 获取主题门店范围
     * (k:主题 v:主题门店范围)
     */
    public Map<ActivityTheme, List<Long>> getThemeShopRange(Long groupId, Date month, List<Long> reportShopRange) {
        Date start = DateUtil.monthBegin(month);
        Date end = DateUtil.monthEnd(month);
        List<ActivityTheme> list = activityThemeService.list(Wrappers.<ActivityTheme>lambdaQuery()
                .eq(ActivityTheme::getGroupId, groupId)
                .ge(ActivityTheme::getStartTime, start)
                .le(ActivityTheme::getStartTime, end));
        Map<ActivityTheme, List<Long>> result = new HashMap<>();
        for (ActivityTheme theme : list) {
            List<Long> shopIds = theme.getShopIdByReport(reportShopRange);
            if (PublicUtil.isNotEmpty(shopIds)) {
                result.put(theme, shopIds);
            }
        }
        return result;
    }

    /**
     * 获取趋势时间范围
     */
    public List<LocalDate> getTrendDate(Date beginTime, Date endTime) {
        final List<LocalDate> result = new LinkedList<>();
        LocalDate now = LocalDate.now();
        LocalDate start = DateUtil.date2LocalDate(beginTime);
        LocalDate endDate = DateUtil.date2LocalDate(endTime);
        LocalDate end = now.isAfter(endDate) ? endDate : now.minusDays(1);
        while (!start.isAfter(end)) {
            result.add(start);
            start = start.plusDays(1);
        }
        return result;
    }

    /**
     * 管理角色范围处理
     */
    public void extracted(RoleMsgEnum roleEnum, ManagerSnapshot snapshot) {
        if (RoleMsgEnum.XSJL.equals(roleEnum)) {
            snapshot.setFirstManagerId(snapshot.getSecondManagerId());
            snapshot.setFirstManagerName(snapshot.getSecondManagerName());
        } else if (RoleMsgEnum.XSZJL.equals(roleEnum)) {
            snapshot.setFirstManagerId(snapshot.getThirdManagerId());
            snapshot.setFirstManagerName(snapshot.getThirdManagerName());
        } else if (RoleMsgEnum.XSSJGL.equals(roleEnum)) {
            snapshot.setFirstManagerId(snapshot.getFourthManagerId());
            snapshot.setFirstManagerName(snapshot.getFourthManagerName());
        }
    }

    /**
     * 人员粉丝数量统计
     */
    public Map<Long, UserFansCountVo> getThemeFansCount(Long themeId) {
        Map<Long, UserFansCountVo> result = new HashMap<>();

        List<ConsultantSnapshot> snapshotList = consultantSnapshotService.list(Wrappers.<ConsultantSnapshot>lambdaQuery()
                .eq(ConsultantSnapshot::getThemeId, themeId));

        for (ConsultantSnapshot snapshot : snapshotList) {
            UserFansCountVo userFansCountVo = new UserFansCountVo();
            userFansCountVo.setUserId(snapshot.getStaffId());
            List<AccountInfoVo> accountInfoList = snapshot.getAccountInfoList();
            if (PublicUtil.isEmpty(accountInfoList)) {
               continue;
            }
            for (AccountInfoVo accountInfoVo : accountInfoList) {
                UserFansCountVo.platformCount platformCount = new UserFansCountVo.platformCount();
                platformCount.setAccount(accountInfoVo.getAccount());
                platformCount.setFansNum(accountInfoVo.getFansNum());
                platformCount.setType(accountInfoVo.getType().getValue());
                if (PlatformEnum.KS.equals(accountInfoVo.getType())) {
                    userFansCountVo.setKs(platformCount);
                }
                if (PlatformEnum.DY.equals(accountInfoVo.getType())) {
                    userFansCountVo.setDy(platformCount);
                }
            }
            userFansCountVo.setTotal(userFansCountVo.getPlatformFans(userFansCountVo.getKs())
                    + userFansCountVo.getPlatformFans(userFansCountVo.getDy()));
            result.put(snapshot.getStaffId(), userFansCountVo);
        }
        return result;
    }

    /**
     * 人员粉丝数量统计
     */
    public Map<Long, UserFansCountVo> getUserFansCount(Set<Long> userIds) {
        Map<Long, UserFansCountVo> result = new HashMap<>();
        if (PublicUtil.isEmpty(userIds)) {
            return result;
        }
        List<Account> accountList = accountService.list(Wrappers.<Account>lambdaQuery()
                .in(Account::getType, PlatformEnum.KS, PlatformEnum.DY)
                .in(Account::getUserId, userIds));
        Map<Long, List<Account>> accountMap = accountList.stream().collect(Collectors.groupingBy(Account::getUserId));
        for (Long userId : userIds) {
            UserFansCountVo userFansCountVo = new UserFansCountVo();
            userFansCountVo.setUserId(userId);
            List<Account> userAccountList = accountMap.getOrDefault(userId, new LinkedList<>());
            //平台粉丝
            for (Account account : userAccountList) {
                UserFansCountVo.platformCount platformCount = new UserFansCountVo.platformCount();
                platformCount.setAccount(account.getAccount());
                platformCount.setFansNum(account.getFansNum());
                platformCount.setType(account.getType().getValue());
                if (PlatformEnum.KS.equals(account.getType())) {
                    userFansCountVo.setKs(platformCount);
                }
                if (PlatformEnum.DY.equals(account.getType())) {
                    userFansCountVo.setDy(platformCount);
                }
            }
            userFansCountVo.setTotal(userFansCountVo.getPlatformFans(userFansCountVo.getKs())
                    + userFansCountVo.getPlatformFans(userFansCountVo.getDy()));
            result.put(userId, userFansCountVo);
        }
        return result;
    }

    /**
     * 获取主题各门店各人员作品完成及是否计分详情
     *
     * @param theme         主题详情
     * @param liveList      直播数据列表
     * @param videoList     短视频数据列表
     * @param shopPublisher 短视频数据列表
     * @return 主题各门店各人员作品完成及是否计分详情
     */
    @Deprecated
    public WorksCompleteDetails getCompleteDetails(ActivityTheme theme, List<LivePool> liveList,
                                                   List<VideoPool> videoList, Map<Long, Set<Long>> shopPublisher) {
        WorksCompleteDetails details = new WorksCompleteDetails();
        details.setThemeId(theme.getId());
        details.setThemeName(theme.getTheme());
        details.setThemeNeedBestLiveCnt(theme.getLiveValidTimes());
        details.setThemeNeedBestVideoCnt(theme.getVideoValidTimes());
        List<WorksCompleteDetails.ShopCompleteDetails> shopDetails = new ArrayList<>();
        shopPublisher.forEach((k, v) -> {
            WorksCompleteDetails.ShopCompleteDetails shop = new WorksCompleteDetails.ShopCompleteDetails();
            shop.setShopId(k);
            shop.setUserDetails(v.stream()
                    .map(item -> {// userId
                        UserCalRuleVo user = new UserCalRuleVo();
                        user.setUserId(item);
                        user.setNeedLiveValidTimes(theme.getLiveValidTimes());
                        user.setNeedVideoValidTimes(theme.getVideoValidTimes());
                        user.setCurrentLiveValidTimes(0);
                        user.setCurrentVideoValidTimes(0);
                        details.getAllUserDetails().add(user);
                        return user;
                    })
                    .collect(Collectors.toList())
            );
            shopDetails.add(shop);
        });// 初始化数据
        details.setShopDetails(shopDetails);
        /* 设置直播数据 */
        if (!CollectionUtils.isEmpty(liveList))
            liveList.stream()
                    .filter(item -> Objects.nonNull(item.getBelongShopId()))
                    //.filter(item -> Objects.equals(item.getValidLive(), 11))
                    .collect(Collectors.groupingBy(LivePool::getBelongShopId))
                    .forEach((k, v) -> {
                        WorksCompleteDetails.ShopCompleteDetails shop = details.getShopDetailByShopId(k);
                        if (Objects.isNull(shop.getShopId())) {
                            shop.setShopId(k);
                            shopDetails.add(shop);
                        }
                        List<UserCalRuleVo> userDetails = shop.getUserDetails();
                        v.stream()
                                .collect(Collectors.groupingBy(LivePool::getBelongUserId))
                                .forEach((k1, v1) -> {
                                    UserCalRuleVo user = details.getUserDetailByUserId(k, k1);
                                    boolean nullFlag = Objects.isNull(user.getUserId());
                                    if (nullFlag) {
                                        log.info("统计直播目标, 门店id: {}, 用户id: {} 不在快照范围内", k, k1);
                                        user.setUserId(k1);
                                        user.setNeedVideoValidTimes(theme.getVideoValidTimes());
                                        user.setCurrentVideoValidTimes(0);
                                    }
                                    user.setNeedLiveValidTimes(theme.getLiveValidTimes());
                                    user.setCurrentLiveValidTimes(v1.size());
                                    if (nullFlag) {
                                        userDetails.add(user);
                                        details.getAllUserDetails().add(user);
                                    }
                                });
                        shop.setUserDetails(userDetails);
                    })
                    ;
        /* 设置短视频数据 */
        if (!CollectionUtils.isEmpty(videoList))
            videoList.stream()
                    .filter(item -> Objects.nonNull(item.getBelongShopId()))
                    //.filter(item -> Objects.equals(item.getValidVideo(), 11))
                    .collect(Collectors.groupingBy(VideoPool::getBelongShopId))
                    .forEach((k, v) -> {
                        WorksCompleteDetails.ShopCompleteDetails shop = details.getShopDetailByShopId(k);
                        if (Objects.isNull(shop.getShopId())) {
                            shop.setShopId(k);
                            shopDetails.add(shop);
                        }
                        List<UserCalRuleVo> userDetails = shop.getUserDetails();
                        v.stream()
                                .collect(Collectors.groupingBy(VideoPool::getBelongUserId))
                                .forEach((k1, v1) -> {
                                    UserCalRuleVo user = details.getUserDetailByUserId(k, k1);
                                    boolean nullFlag = Objects.isNull(user.getUserId());
                                    if (nullFlag) {
                                        log.info("统计短视频目标, 门店id: {}, 用户id: {} 不在快照范围内", k, k1);
                                        user.setUserId(k1);
                                        user.setNeedLiveValidTimes(theme.getLiveValidTimes());
                                        user.setCurrentLiveValidTimes(0);
                                    }
                                    user.setNeedVideoValidTimes(theme.getVideoValidTimes());
                                    user.setCurrentVideoValidTimes(v1.size());
                                    if (nullFlag) {
                                        userDetails.add(user);
                                        details.getAllUserDetails().add(user);
                                    }
                                });
                        shop.setUserDetails(userDetails);
                    })
                    ;
        return details;
    }

    /**
     * 累加int字段
     *
     * @param list  T对象列表
     * @param filed 待累加字段
     * @param <T>   泛型
     * @return 所有T对象对应字段总和
     */
    public static <T> Integer sumInt(List<T> list, Function<T, Integer> filed) {
        return list.stream()
                .filter(Objects::nonNull)
                .map(filed)
                .filter(Objects::nonNull)
                .mapToInt(item -> item)
                .sum()
                ;
    }

    /**
     * 处理作品话题字符串
     *
     * @param topic 话题字符串
     * @return 数组格式
     */
    public List<String> handleWorksTopic(String topic) {
        try {
            if (StringUtils.hasText(topic)) {
                String[] split = topic.split(",");
                return Arrays.stream(split)
                        .map(item1 -> item1.replace("#", ""))
                        .collect(Collectors.toList());
            }
        } catch (Exception e) {
            log.error("处理话题字符串出错!", e);
        }
        return new ArrayList<>();
    }

    /**
     * 处理主题设置话题字符串
     *
     * @param theme 主题详情
     * @return 数组格式
     */
    public List<TopicVo> handleSetTopic(ActivityTheme theme) {
        if (Objects.isNull(theme) || !StringUtils.hasText(theme.getTopic()))
            return new ArrayList<>();
        try {
            return JSON.parseArray(theme.getTopic(), TopicVo.class);
        } catch (Exception e) {
            return new ArrayList<>();
        }
    }

    /**
     * 目标总览
     *
     * @param groupId  集团id
     * @param themeIds 主题id
     * @param shopIds  门店id
     */
    public TargetResultDto getTargetTotal(Long groupId, List<Long> themeIds, List<Long> shopIds) {
        List<ManagerSnapshot> snapshotList = managerSnapshotService.list(Wrappers.<ManagerSnapshot>lambdaQuery()
                .eq(ManagerSnapshot::getGroupId, groupId)
                .in(ManagerSnapshot::getThemeId, themeIds)
                .in(PublicUtil.isNotEmpty(shopIds), ManagerSnapshot::getShopId, shopIds));
        TargetResultDto targetResultDto = new TargetResultDto();
        targetResultDto.setNeedBestVideoCnt(snapshotList.stream().mapToInt(ManagerSnapshot::getNeedVideoNum).sum());
        targetResultDto.setCurrentBestVideoCnt(snapshotList.stream().mapToInt(ManagerSnapshot::getVideoQualifiedNum).sum());
        targetResultDto.setNeedBestLiveCnt(snapshotList.stream().mapToInt(ManagerSnapshot::getNeedLiveNum).sum());
        targetResultDto.setCurrentBestLiveCnt(snapshotList.stream().mapToInt(ManagerSnapshot::getLiveQualifiedNum).sum());
        return targetResultDto;
    }

    /**
     * 查询目标统计
     */
    public List<ScoreResultDto> getScoreCalRange(Integer type, Long groupId, List<Long> themeIds, List<Long> shopIds,
                                                 RoleMsgEnum roleEnum) {
        if (1 == type) {
            return this.getStaff(groupId, themeIds, shopIds);
        }
        if (2 == type) {
            return this.getShop(groupId, themeIds, shopIds);
        }
        return this.getManager(groupId, themeIds, shopIds, roleEnum);
    }

    /**
     * 发布人
     */
    private List<ScoreResultDto> getStaff(Long groupId, List<Long> themeIds, List<Long> shopIds) {
        List<ConsultantSnapshot> list = consultantSnapshotService.list(Wrappers.<ConsultantSnapshot>lambdaQuery()
                .eq(ConsultantSnapshot::getGroupId, groupId)
                .in(ConsultantSnapshot::getThemeId, themeIds)
                .in(PublicUtil.isNotEmpty(shopIds), ConsultantSnapshot::getShopId, shopIds));
        return PublicUtil.copyList(list, ScoreResultDto.class);
    }

    /**
     * 门店
     */
    private List<ScoreResultDto> getShop(Long groupId, List<Long> themeIds, List<Long> shopIds) {
        List<ManagerSnapshot> list = managerSnapshotService.list(Wrappers.<ManagerSnapshot>lambdaQuery()
                .eq(ManagerSnapshot::getGroupId, groupId)
                .in(ManagerSnapshot::getThemeId, themeIds)
                .in(PublicUtil.isNotEmpty(shopIds), ManagerSnapshot::getShopId, shopIds));
        return PublicUtil.copyList(list, ScoreResultDto.class);
    }

    /**
     * 管理人范围
     */
    private List<ScoreResultDto> getManager(Long groupId, List<Long> themeIds, List<Long> shopIds,
                                            RoleMsgEnum roleEnum) {
        List<ManagerSnapshot> list = managerSnapshotService.list(Wrappers.<ManagerSnapshot>lambdaQuery()
                .eq(ManagerSnapshot::getGroupId, groupId)
                .in(ManagerSnapshot::getThemeId, themeIds)
                .in(PublicUtil.isNotEmpty(shopIds), ManagerSnapshot::getShopId, shopIds));
        List<ManagerSnapshot> manageShoplist = list.stream()
                .peek(snapshot -> this.extracted(roleEnum, snapshot))
                .filter(r -> PublicUtil.isNotEmpty(r.getFirstManagerId()))
                .collect(Collectors.toList());
        return manageShoplist.stream().map(r -> {
            ScoreResultDto copy = PublicUtil.copy(r, ScoreResultDto.class);
            copy.setManagerId(r.getFirstManagerId());
            copy.setManagerName(r.getFirstManagerName());
            return copy;
        }).collect(Collectors.toList());
    }

}
