package cn.fw.dalaran.service.biz.report.calculate;

import cn.fw.dalaran.common.utils.PublicUtil;
import cn.fw.dalaran.dao.score.CalThemeScoreParam;
import cn.fw.dalaran.domain.db.Account;
import cn.fw.dalaran.domain.db.LivePool;
import cn.fw.dalaran.domain.db.VideoPool;
import cn.fw.dalaran.domain.db.config.ActivityTheme;
import cn.fw.dalaran.domain.db.snapshot.ConsultantSnapshot;
import cn.fw.dalaran.domain.db.snapshot.ManagerSnapshot;
import cn.fw.dalaran.domain.dto.report.EvlFormulaDto;
import cn.fw.dalaran.domain.enums.config.SubTypeEnum;
import cn.fw.dalaran.domain.enums.config.WorksTypeEnum;
import cn.fw.dalaran.domain.vo.account.AccountInfoVo;
import cn.fw.dalaran.domain.vo.config.EvlVo;
import cn.fw.dalaran.domain.vo.config.FansLimitVo;
import cn.fw.dalaran.service.data.*;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 定时计算主题得分实现
 *
 * @author: lym
 * @date: 2023/5/12
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class CalThemeScoreBizService {
    private final AccountService accountService;
    private final LivePoolService livePoolService;
    private final VideoPoolService videoPoolService;
    private final IManagerSnapshotService managerSnapshotService;
    private final CalThemeTargetBizService calThemeTargetBizService;
    private final IConsultantSnapshotService consultantSnapshotService;

    /**
     * 计算场次得分
     * (以门店、计算门店下每个人发布短视频、直播得分)
     */
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    public void calVideoOrLiveScore(ActivityTheme theme) {
        try {
            List<ConsultantSnapshot> staffList = consultantSnapshotService.list(Wrappers.<ConsultantSnapshot>lambdaQuery()
                    .eq(ConsultantSnapshot::getThemeId, theme.getId()));
            CalThemeScoreParam calParam = this.getCalParam(theme, staffList);
            //计算有效直播、短视频得分
            this.calLiveScore(calParam.getLiveEvl(), calParam.getLivePoolList());
            this.calVideoScore(calParam.getVideoEvl(), calParam.getVideoPoolList());
            //计算最佳场次
            this.findBest(theme, calParam);
            livePoolService.updateBatchById(calParam.getLivePoolList());
            videoPoolService.updateBatchById(calParam.getVideoPoolList());
            //统计目标
            this.calStaffTarget(theme, calParam.getLivePoolList(), calParam.getVideoPoolList(),
                    staffList);
            if (PublicUtil.isNotEmpty(staffList)) {
                consultantSnapshotService.updateBatchById(staffList);
                List<ManagerSnapshot> shopList = this.calShopTarget(theme, staffList);
                managerSnapshotService.updateBatchById(shopList);
            }
        } catch (Exception e) {
            log.error("计算单场得分失败，主题id{}", theme.getId(), e);
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
        }
    }

    /**
     * 查找计算参数
     */
    private CalThemeScoreParam getCalParam(ActivityTheme theme, List<ConsultantSnapshot> staffList) {
        CalThemeScoreParam scoreParam = new CalThemeScoreParam();
        //有效直播场次
        List<LivePool> livePoolList = livePoolService.list(Wrappers.<LivePool>lambdaQuery()
                .eq(LivePool::getThemeId, theme.getId())
                .eq(LivePool::getStatus, 1)
                .gt(LivePool::getValidLive, 0));
        scoreParam.setLivePoolList(livePoolList);
        //有效短视频场次
        List<VideoPool> videoPoolList = videoPoolService.list(Wrappers.<VideoPool>lambdaQuery()
                .eq(VideoPool::getThemeId, theme.getId())
                .eq(VideoPool::getStatus, 1)
                .gt(VideoPool::getValidVideo, 0)
                .eq(VideoPool::getLatestRecord, 1));
        scoreParam.setVideoPoolList(videoPoolList);
        List<EvlVo> evlConfigList = JSON.parseArray(theme.getEvlConfigJson(), EvlVo.class);
        Map<Integer, EvlVo> evlMap = evlConfigList.stream()
                .collect(Collectors.toMap(EvlVo::getType, Function.identity()));
        scoreParam.setEvlMap(evlMap);
        //评分指标
        scoreParam.setVideoEvl(evlMap.get((WorksTypeEnum.VIDEO.getValue())));
        scoreParam.setLiveEvl(evlMap.get(WorksTypeEnum.LIVE.getValue()));
        //粉丝数要求
        if (PublicUtil.isNotEmpty(theme.getFansLimit())) {
            scoreParam.setFansLimit(JSON.parseArray(theme.getFansLimit(), FansLimitVo.class)
                    .stream().collect(Collectors.toMap(FansLimitVo::getPlatformType, r -> r)));
        }
        //账号粉丝数
        for (ConsultantSnapshot snapshot : staffList) {
            for (AccountInfoVo accountInfoVo : snapshot.getAccountInfoList()) {
                scoreParam.getAccountMap().put(accountInfoVo.getAccountId(), accountInfoVo);
            }
        }
        return scoreParam;
    }

    /**
     * 直播得分计算
     */
    private void calLiveScore(EvlVo evlConfig, List<LivePool> livePoolList) {
        if (PublicUtil.isEmpty(livePoolList) || PublicUtil.isEmpty(evlConfig))
            return;
        Map<Integer, EvlVo.EvlSubIndexVo> evlParamMap = evlConfig.getEvlSubItems().stream()
                .collect(Collectors.toMap(EvlVo.EvlSubIndexVo::getSubType, Function.identity()));
        //单场得分计算
        for (LivePool livePool : livePoolList) {
            livePool.setValidLive(1);
            livePool.setThemeScore(BigDecimal.ZERO);
            final EvlFormulaDto formulaDto = new EvlFormulaDto();
            this.calcSingleIndexScore(evlParamMap.get(SubTypeEnum.UV_PEAK.getValue()), livePool.getUvPeak(), formulaDto);
            this.calcSingleIndexScore(evlParamMap.get(SubTypeEnum.UV.getValue()), livePool.getUv(), formulaDto);
            this.calcSingleIndexScore(evlParamMap.get(SubTypeEnum.LIKE.getValue()), livePool.getLikeNum(), formulaDto);
            this.calcSingleIndexScore(evlParamMap.get(SubTypeEnum.SHARE.getValue()), livePool.getShareNum(), formulaDto);
            this.calcSingleIndexScore(evlParamMap.get(SubTypeEnum.COMMENT.getValue()), livePool.getCommentNum(), formulaDto);
            this.calcSingleIndexScore(evlParamMap.get(SubTypeEnum.NEW_FANS.getValue()), livePool.getNewFanNum(), formulaDto);
            this.calcSingleIndexScore(evlParamMap.get(SubTypeEnum.LIVE_DURATION.getValue()), livePool.getLiveDuration(), formulaDto);
            livePool.setScore(formulaDto.getScore());
            livePool.setScoreCalJson(JSON.toJSONString(formulaDto));
        }
    }

    /**
     * 短视频得分计算
     */
    private void calVideoScore(EvlVo evlConfig, List<VideoPool> videoPoolList) {
        if (PublicUtil.isEmpty(videoPoolList) || PublicUtil.isEmpty(evlConfig))
            return;
        Map<Integer, EvlVo.EvlSubIndexVo> evlParamMap = evlConfig.getEvlSubItems().stream()
                .collect(Collectors.toMap(EvlVo.EvlSubIndexVo::getSubType, r -> r));
        //计算单条视频得分
        for (VideoPool videoPool : videoPoolList) {
            videoPool.setValidVideo(1);
            videoPool.setThemeScore(BigDecimal.ZERO);
            final EvlFormulaDto formulaDto = new EvlFormulaDto();
            this.calcSingleIndexScore(evlParamMap.get(SubTypeEnum.UV.getValue()), videoPool.getUv(), formulaDto);
            this.calcSingleIndexScore(evlParamMap.get(SubTypeEnum.FULL_UV.getValue()), videoPool.getFullUv(), formulaDto);
            this.calcSingleIndexScore(evlParamMap.get(SubTypeEnum.SHARE.getValue()), videoPool.getShareNum(), formulaDto);
            this.calcSingleIndexScore(evlParamMap.get(SubTypeEnum.COMMENT.getValue()), videoPool.getCommentNum(), formulaDto);
            this.calcSingleIndexScore(evlParamMap.get(SubTypeEnum.NEW_FANS.getValue()), videoPool.getNewFanNum(), formulaDto);
            this.calcSingleIndexScore(evlParamMap.get(SubTypeEnum.LIKE.getValue()), videoPool.getLikeNum(), formulaDto);
            videoPool.setScore(formulaDto.getScore());
            videoPool.setScoreCalJson(JSON.toJSONString(formulaDto));
        }
    }

    /**
     * 计算单项指标实际得分
     *
     * @param subIndex  指标配置
     * @param actualNum 实际值
     */
    private void calcSingleIndexScore(EvlVo.EvlSubIndexVo subIndex, Integer actualNum, EvlFormulaDto formulaDto) {
        if (PublicUtil.isEmpty(subIndex) || subIndex.getRate().compareTo(BigDecimal.ZERO) <= 0)
            return;
        EvlFormulaDto.SubEvl subEvl = new EvlFormulaDto.SubEvl();
        subEvl.setEvlName(subIndex.getSubTypeName());
        subEvl.setStandard(subIndex.getStandard());
        subEvl.setSubIndexRate(subIndex.getRate().divide(new BigDecimal(100), 4, RoundingMode.HALF_UP));
        subEvl.setCollectNum(actualNum);
        if (subIndex.getStandard() <= 0) {
            subEvl.setScore(subIndex.getRate());
        } else {
            actualNum = PublicUtil.isEmpty(actualNum) ? 0 : actualNum;
            BigDecimal proportion = BigDecimal.valueOf(actualNum)
                    .divide(BigDecimal.valueOf(subIndex.getStandard()), 4, RoundingMode.HALF_DOWN);
            proportion = proportion.compareTo(BigDecimal.ONE) > 0 ? BigDecimal.ONE : proportion;
            subEvl.setScore(subEvl.getSubIndexRate().multiply(proportion).setScale(4, RoundingMode.HALF_UP));
        }
        formulaDto.setScore(formulaDto.getScore().add(subEvl.getScore()).setScale(4, RoundingMode.HALF_UP));
        formulaDto.getScoreItems().add(subEvl);
    }

    /**
     * 计算主题得分，按人员聚合，前置判断场次要求
     */
    private void findBest(ActivityTheme theme, CalThemeScoreParam calParam) {
        Map<Long, List<LivePool>> staffLiveMap = calParam.getLivePoolList().stream()
                .filter(r -> this.fansLimitCheck(r.getAccountId(), calParam))
                .collect(Collectors.groupingBy(LivePool::getBelongUserId));
        Map<Long, List<VideoPool>> staffVideoMap = calParam.getVideoPoolList().stream()
                .filter(r -> this.fansLimitCheck(r.getAccountId(), calParam))
                .collect(Collectors.groupingBy(VideoPool::getBelongUserId));
        //得分前置要求最低有效场次
        int needLiveTimes = PublicUtil.isEmpty(theme.getLiveValidTimes()) ? 0 : theme.getLiveValidTimes();
        int needVideoTimes = PublicUtil.isEmpty(theme.getVideoValidTimes()) ? 0 : theme.getVideoValidTimes();
        //所有发布人员
        Set<Long> staffs = new HashSet<>();
        staffs.addAll(staffLiveMap.keySet());
        staffs.addAll(staffVideoMap.keySet());
        //是否忽略计算
        boolean ignoreLive = calParam.getIsIgnoreLive();
        boolean ignoreVideo = calParam.getIsIgnoreVideo();
        //按人处理
        for (Long staffId : staffs) {
            List<LivePool> staffLivePools = staffLiveMap.getOrDefault(staffId, new ArrayList<>());
            List<VideoPool> staffVideoPools = staffVideoMap.getOrDefault(staffId, new ArrayList<>());
            //如果不满足主题得分场次要求
            boolean liveNotCompliance = !ignoreLive && staffLivePools.size() < needLiveTimes;
            boolean videoNotCompliance = !ignoreVideo && staffVideoPools.size() < needVideoTimes;
            boolean notCompliance = liveNotCompliance || videoNotCompliance;// 短视频/直播有效场次未同时合格
            this.findStaffBestLive(notCompliance, needLiveTimes, staffLivePools);
            this.findBestVideo(notCompliance, needVideoTimes, staffVideoPools);
        }
    }

    /**
     * 账号粉丝数检查
     */
    private boolean fansLimitCheck(Long accountId, CalThemeScoreParam calParam) {
        AccountInfoVo accountInfoVo = calParam.getAccountMap().get(accountId);
        if (PublicUtil.isEmpty(accountInfoVo)) {
            return false;
        }
        FansLimitVo fansLimitVo = calParam.getFansLimit().get(accountInfoVo.getType().getValue());
        return PublicUtil.isEmpty(fansLimitVo) || accountInfoVo.getFansNum() >= fansLimitVo.getMinNum();
    }

    /**
     * 计算最优直播（主题得分计算需要）
     */
    private void findStaffBestLive(Boolean notCompliance, Integer needLiveTimes, List<LivePool> livePoolList) {
        Set<Long> bestLiveIds = livePoolList.stream()
                .sorted(Comparator.comparing(LivePool::getScore).reversed())
                .limit(needLiveTimes)
                .map(LivePool::getId)
                .collect(Collectors.toSet());
        livePoolList.forEach(live -> live.setValidLive(bestLiveIds.contains(live.getId())
                ? notCompliance ? 2 : 11 : 1));
    }

    /**
     * 计算人员最优短视频场次
     */
    private void findBestVideo(Boolean notCompliance, Integer needVideoTimes, List<VideoPool> videoPoolList) {
        Set<Long> bestVideoIds = videoPoolList.stream()
                .sorted(Comparator.comparing(VideoPool::getScore).reversed())
                .limit(needVideoTimes)
                .map(VideoPool::getId)
                .collect(Collectors.toSet());
        videoPoolList.forEach(video -> video.setValidVideo(bestVideoIds.contains(video.getId())
                ? notCompliance ? 2 : 11 : 1));
    }

    /**
     * 计算发布人目标
     *
     * @param theme
     * @param liveList
     * @param videoList
     * @return
     */
    private void calStaffTarget(ActivityTheme theme, List<LivePool> liveList, List<VideoPool> videoList,
                                List<ConsultantSnapshot> staffList) {
        calThemeTargetBizService.calStaffTarget(theme, staffList, liveList, videoList);
    }

    /**
     * 计算门店目标
     *
     * @param theme
     * @param staffList
     * @return
     */
    private List<ManagerSnapshot> calShopTarget(ActivityTheme theme, List<ConsultantSnapshot> staffList) {
        List<ManagerSnapshot> shopList = managerSnapshotService.list(Wrappers.<ManagerSnapshot>lambdaQuery()
                .eq(ManagerSnapshot::getThemeId, theme.getId()));
        calThemeTargetBizService.calShopTarget(theme, shopList, staffList);
        return shopList;
    }

}
