package cn.fw.dalaran.service.biz.report.calculate;

import cn.fw.dalaran.common.domain.enums.RoleMsgEnum;
import cn.fw.dalaran.common.utils.DateUtil;
import cn.fw.dalaran.common.utils.PublicUtil;
import cn.fw.dalaran.domain.db.Account;
import cn.fw.dalaran.domain.db.LivePool;
import cn.fw.dalaran.domain.db.VideoPool;
import cn.fw.dalaran.domain.db.config.ActivityTheme;
import cn.fw.dalaran.domain.db.snapshot.ConsultantSnapshot;
import cn.fw.dalaran.domain.db.snapshot.ManagerSnapshot;
import cn.fw.dalaran.domain.dto.common.ShopDto;
import cn.fw.dalaran.domain.enums.account.AccountTypeEnum;
import cn.fw.dalaran.domain.vo.account.AccountInfoVo;
import cn.fw.dalaran.rpc.dto.StaffBaseInfoRpc;
import cn.fw.dalaran.rpc.ehr.EhrRpc;
import cn.fw.dalaran.rpc.erp.ErpRpc;
import cn.fw.dalaran.rpc.oop.OopRpc;
import cn.fw.dalaran.service.data.*;
import cn.fw.erp.sdk.api.result.UserRoleInfo;
import cn.fw.oop.sdk.result.ShopInfoVo;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.google.common.collect.HashBasedTable;
import com.google.common.collect.Table;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 活动快照生成
 *
 * @author: lym
 * @date: 2023/5/23
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class ThemeSnapshotTaskBizService {
    private final ErpRpc erpRpc;
    private final OopRpc oopRpc;
    private final EhrRpc ehrRpc;
    private final AccountService accountService;
    private final LivePoolService livePoolService;
    private final VideoPoolService videoPoolService;
    private final IManagerSnapshotService managerSnapshotService;
    private final CalThemeTargetBizService calThemeTargetBizService;
    private final IConsultantSnapshotService consultantSnapshotService;

    /**
     * 创建发布人员、管理层快照
     *
     * @param theme 主题
     */
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    public void createThemeSnapshot(ActivityTheme theme) {
        try {
            List<ShopDto> shopList = JSON.parseArray(theme.getShopJson(), ShopDto.class);
            List<Long> shopIds = shopList.stream().map(ShopDto::getShopId).collect(Collectors.toList());
            //落库发布人快照
            HashBasedTable<Long, Long, ConsultantSnapshot> releaseSnapshotTable = this.saveStaffSnapshot(theme, shopIds);
            //创建门店快照
            List<ManagerSnapshot> managerSnapshot = this.createShopSnapshot(theme, shopIds,
                    new ArrayList<>(releaseSnapshotTable.values()));
            //刷新门店管理人员
            this.updateMangerBelong(theme.getId(), managerSnapshot);
            if (PublicUtil.isNotEmpty(releaseSnapshotTable.values())) {
                this.calStaffTarget(theme, new ArrayList<>(releaseSnapshotTable.values()));
                consultantSnapshotService.updateBatchById(releaseSnapshotTable.values());
                this.calShopTarget(theme, managerSnapshot, new ArrayList<>(releaseSnapshotTable.values()));
                managerSnapshotService.updateBatchById(managerSnapshot);
            }
        } catch (Exception e) {
            log.error("定时生成活动快照失败", e);
        }
    }

    /**
     * 落库发布人数据
     */
    private HashBasedTable<Long, Long, ConsultantSnapshot> saveStaffSnapshot(ActivityTheme theme, List<Long> shopIds) {
        List<ConsultantSnapshot> oldSnapshotList = consultantSnapshotService.list(Wrappers.<ConsultantSnapshot>lambdaQuery()
                .eq(ConsultantSnapshot::getThemeId, theme.getId()));
        HashBasedTable<Long, Long, ConsultantSnapshot> oldStaffSnapshot = HashBasedTable.create();
        oldSnapshotList.forEach(snapshot -> oldStaffSnapshot.put(snapshot.getShopId(), snapshot.getStaffId(), snapshot));

        BigDecimal liveRate = theme.getLiveRate();
        BigDecimal videoRate = theme.getVideoRate();
        HashBasedTable<Long, Long, ConsultantSnapshot> newStaffSnapshot = this.createNewStaffSnapshot(theme, shopIds);
        for (ConsultantSnapshot newSnapshot : newStaffSnapshot.values()) {
            ConsultantSnapshot oldSnapshot = oldStaffSnapshot.get(newSnapshot.getShopId(), newSnapshot.getStaffId());
            if (PublicUtil.isNotEmpty(oldSnapshot)) {
                newSnapshot.setId(oldSnapshot.getId());
                newSnapshot.setVideoQualifiedNum(oldSnapshot.getVideoQualifiedNum());
                newSnapshot.setLiveQualifiedNum(oldSnapshot.getLiveQualifiedNum());
            }
            newSnapshot.setLiveRate(liveRate);
            newSnapshot.setVideoRate(videoRate);
            oldStaffSnapshot.remove(newSnapshot.getShopId(), newSnapshot.getStaffId());
        }
        if (PublicUtil.isNotEmpty(oldStaffSnapshot)) {
            List<Long> needRemoveIds = oldStaffSnapshot.values().stream()
                    .map(ConsultantSnapshot::getId).collect(Collectors.toList());
            consultantSnapshotService.removeByIds(needRemoveIds);
        }
        if (PublicUtil.isNotEmpty(newStaffSnapshot)) {
            consultantSnapshotService.saveOrUpdateBatch(newStaffSnapshot.values());
        }
        return newStaffSnapshot;
    }

    /**
     * 获取发布人最新快照内容
     */
    private HashBasedTable<Long, Long, ConsultantSnapshot> createNewStaffSnapshot(ActivityTheme theme, List<Long> shopIds) {
        HashBasedTable<Long, Long, ConsultantSnapshot> snapshotTable = HashBasedTable.create();
        List<Account> accountList = accountService.list(Wrappers.<Account>lambdaQuery()
                .in(Account::getShopId, shopIds)
                .eq(Account::getYn, true));
        List<UserRoleInfo> consultantList = erpRpc.getUserRoleInfoList(RoleMsgEnum.XSGW, shopIds);
        Set<Long> staffIdSet = consultantList.stream().map(UserRoleInfo::getUserId).collect(Collectors.toSet());
        //账号
        for (Account account : accountList) {
            ConsultantSnapshot snapshot = snapshotTable.get(account.getShopId(), account.getUserId());
            if (PublicUtil.isEmpty(snapshot)) {
                snapshot = new ConsultantSnapshot();
                snapshot.setGroupId(account.getGroupId());
                snapshot.setThemeId(theme.getId());
                snapshot.setMonth(DateUtil.format(theme.getStartTime(), DateUtil.FORMAT_1));
                snapshot.setShopId(account.getShopId());
                snapshot.setShopName(account.getShopName());
                snapshot.setStaffId(account.getUserId());
                snapshot.setStaffName(account.getUserName());
                snapshot.setNeedLiveNum(0);
                snapshot.setLiveQualifiedNum(0);
                snapshot.setNeedVideoNum(0);
                snapshot.setVideoQualifiedNum(0);

                staffIdSet.remove(snapshot.getStaffId());
            }
            AccountInfoVo accountInfoVo = PublicUtil.copy(account, AccountInfoVo.class);
            accountInfoVo.setAccountId(account.getId());
            List<AccountInfoVo> accountInfoList = snapshot.getAccountInfoList();
            accountInfoList.add(accountInfoVo);
            snapshot.setAccountInfo(JSON.toJSONString(accountInfoList));
            snapshotTable.put(snapshot.getShopId(), snapshot.getStaffId(), snapshot);
        }
        //销售人员补全
        List<StaffBaseInfoRpc> staffInfoBatch = ehrRpc.getStaffInfoBatch(new ArrayList<>(staffIdSet));
        for (StaffBaseInfoRpc staff : staffInfoBatch) {
            ConsultantSnapshot snapshot = new ConsultantSnapshot();
            snapshot.setGroupId(staff.getGroupId());
            snapshot.setThemeId(theme.getId());
            snapshot.setMonth(DateUtil.format(theme.getStartTime(), DateUtil.FORMAT_1));
            snapshot.setShopId(staff.getShopId());
            snapshot.setShopName(staff.getShopName());
            snapshot.setStaffId(staff.getId());
            snapshot.setStaffName(staff.getName());
            snapshot.setNeedLiveNum(0);
            snapshot.setLiveQualifiedNum(0);
            snapshot.setNeedVideoNum(0);
            snapshot.setVideoQualifiedNum(0);

            snapshotTable.put(snapshot.getShopId(), snapshot.getStaffId(), snapshot);
        }
        return snapshotTable;
    }

    /**
     * 创建门店快照
     */
    private List<ManagerSnapshot> createShopSnapshot(ActivityTheme theme, List<Long> shopIds,
                                                     List<ConsultantSnapshot> snapshotList) {
        Map<Long, List<ConsultantSnapshot>> shopMap = snapshotList.stream().collect(Collectors.groupingBy(ConsultantSnapshot::getShopId));

        List<ManagerSnapshot> oldShopSnapshots = managerSnapshotService.list(Wrappers.<ManagerSnapshot>lambdaQuery()
                .eq(ManagerSnapshot::getThemeId, theme.getId()));
        Map<Long, ManagerSnapshot> oldShopMap = oldShopSnapshots.stream()
                .collect(Collectors.toMap(ManagerSnapshot::getShopId, r -> r));

        BigDecimal liveRate = theme.getLiveRate();
        BigDecimal videoRate = theme.getVideoRate();
        List<ManagerSnapshot> newShopSnapshotList = this.getNewShopSnapshot(theme, shopIds, shopMap);
        for (ManagerSnapshot shopSnapshot : newShopSnapshotList) {
            ManagerSnapshot old = oldShopMap.get(shopSnapshot.getShopId());
            if (PublicUtil.isNotEmpty(old)) {
                shopSnapshot.setId(old.getId());
            }
            shopSnapshot.setLiveRate(liveRate);
            shopSnapshot.setVideoRate(videoRate);
            oldShopMap.remove(shopSnapshot.getShopId());
        }

        if (PublicUtil.isNotEmpty(oldShopMap)) {
            managerSnapshotService.removeByIds(oldShopMap.keySet());
        }
        if (PublicUtil.isNotEmpty(newShopSnapshotList)) {
            managerSnapshotService.saveOrUpdateBatch(newShopSnapshotList);
        }
        return newShopSnapshotList;
    }

    /**
     * 获取最新门店快照
     */
    private List<ManagerSnapshot> getNewShopSnapshot(ActivityTheme theme, List<Long> shopIds,
                                                     Map<Long, List<ConsultantSnapshot>> shopMap) {
        Table<Integer, Long, UserRoleInfo> managerTable = this.getManagerTable(shopIds);

        List<Account> accountList = accountService.list(Wrappers.<Account>lambdaQuery()
                .in(Account::getShopId, shopIds)
                .eq(Account::getAccountType, AccountTypeEnum.SHOP.getValue())
                .eq(Account::getYn, true));
        Map<Long, List<Account>> shopAccountMap = accountList.stream().collect(Collectors.groupingBy(Account::getShopId));

        List<ShopInfoVo> shopInfoList = oopRpc.getShopByIds(shopIds);
        return shopInfoList.stream().map(shop -> {
            ManagerSnapshot snapshot = new ManagerSnapshot();
            snapshot.setGroupId(theme.getGroupId());
            snapshot.setThemeId(theme.getId());
            snapshot.setMonth(DateUtil.format(theme.getStartTime(), DateUtil.FORMAT_1));
            snapshot.setShopId(shop.getId());
            snapshot.setShopName(shop.getShopName());
            List<ConsultantSnapshot> staffs = shopMap.getOrDefault(shop.getId(), Collections.emptyList());
            snapshot.setConsultantNum(staffs.size());
            snapshot.setNeedLiveNum(0);
            snapshot.setLiveQualifiedNum(0);
            snapshot.setNeedVideoNum(0);
            snapshot.setVideoQualifiedNum(0);
            this.setManager(snapshot, managerTable);
            if (shopAccountMap.containsKey(shop.getId())) {
                List<Account> shopAccountList = shopAccountMap.get(shop.getId());
                List<AccountInfoVo> accountInfoVoList = shopAccountList.stream().map(account -> {
                    AccountInfoVo vo = PublicUtil.copy(account, AccountInfoVo.class);
                    vo.setAccountId(account.getId());
                    return vo;
                }).collect(Collectors.toList());
                snapshot.setAccountInfo(JSON.toJSONString(accountInfoVoList));
            }
            return snapshot;
        }).collect(Collectors.toList());
    }

    /**
     * 获得管理层级
     */
    private Table<Integer, Long, UserRoleInfo> getManagerTable(List<Long> shopIds) {
        Table<Integer, Long, UserRoleInfo> table = HashBasedTable.create();
        List<UserRoleInfo> manager1List = erpRpc.getUserRoleInfoList(RoleMsgEnum.XSZG, shopIds);
        manager1List.forEach(manager -> manager.getDataRangeList().forEach(shop -> table.put(1, shop.getRangeValue(), manager)));

        List<UserRoleInfo> manager2List = erpRpc.getUserRoleInfoList(RoleMsgEnum.XSJL, shopIds);
        manager2List.forEach(manager -> manager.getDataRangeList().forEach(shop -> table.put(2, shop.getRangeValue(), manager)));

        List<UserRoleInfo> manager3List = erpRpc.getUserRoleInfoList(RoleMsgEnum.XSZJL, shopIds);
        manager3List.forEach(manager -> manager.getDataRangeList().forEach(shop -> table.put(3, shop.getRangeValue(), manager)));

        List<UserRoleInfo> manager4List = erpRpc.getUserRoleInfoList(RoleMsgEnum.XSSJGL, shopIds);
        manager4List.forEach(manager -> manager.getDataRangeList().forEach(shop -> table.put(4, shop.getRangeValue(), manager)));
        return table;
    }

    /**
     * 设置管理层
     */
    private void setManager(ManagerSnapshot snapshot, Table<Integer, Long, UserRoleInfo> managerTable) {
        UserRoleInfo firstManager = managerTable.get(1, snapshot.getShopId());
        if (PublicUtil.isNotEmpty(firstManager)) {
            snapshot.setFirstManagerId(firstManager.getUserId());
            snapshot.setFirstManagerName(firstManager.getUserName());
        }
        UserRoleInfo secondManager = managerTable.get(2, snapshot.getShopId());
        if (PublicUtil.isNotEmpty(secondManager)) {
            snapshot.setSecondManagerId(secondManager.getUserId());
            snapshot.setSecondManagerName(secondManager.getUserName());
        }
        UserRoleInfo thirdManager = managerTable.get(3, snapshot.getShopId());
        if (PublicUtil.isNotEmpty(thirdManager)) {
            snapshot.setThirdManagerId(thirdManager.getUserId());
            snapshot.setThirdManagerName(thirdManager.getUserName());
        }
        UserRoleInfo fourthManager = managerTable.get(4, snapshot.getShopId());
        if (PublicUtil.isNotEmpty(fourthManager)) {
            snapshot.setFourthManagerId(fourthManager.getUserId());
            snapshot.setFourthManagerName(fourthManager.getUserName());
        }
    }

    /**
     * 更新作品归属
     */
    private void updateMangerBelong(Long themeId, List<ManagerSnapshot> managerSnapshot) {
        for (ManagerSnapshot snapshot : managerSnapshot) {
            livePoolService.update(Wrappers.<LivePool>lambdaUpdate()
                    .eq(LivePool::getThemeId, themeId)
                    .eq(LivePool::getBelongShopId, snapshot.getShopId())
                    .set(LivePool::getFirstManagerId, snapshot.getFirstManagerId())
                    .set(LivePool::getSecondManagerId, snapshot.getSecondManagerId())
                    .set(LivePool::getThirdManagerId, snapshot.getThirdManagerId())
                    .set(LivePool::getFourthManagerId, snapshot.getFourthManagerId()));
            videoPoolService.update(Wrappers.<VideoPool>lambdaUpdate()
                    .eq(VideoPool::getThemeId, themeId)
                    .eq(VideoPool::getBelongShopId, snapshot.getShopId())
                    .set(VideoPool::getFirstManagerId, snapshot.getFirstManagerId())
                    .set(VideoPool::getSecondManagerId, snapshot.getSecondManagerId())
                    .set(VideoPool::getThirdManagerId, snapshot.getThirdManagerId())
                    .set(VideoPool::getFourthManagerId, snapshot.getFourthManagerId()));
        }
    }

    /**
     * 计算人员目标
     *
     * @param theme
     * @param snapshotList
     */
    private void calStaffTarget(ActivityTheme theme, List<ConsultantSnapshot> snapshotList) {
        List<LivePool> liveList = livePoolService.list(Wrappers.<LivePool>lambdaQuery()
                .eq(LivePool::getThemeId, theme.getId())
                .gt(LivePool::getValidLive, 1)
                .eq(LivePool::getStatus, 1));
        List<VideoPool> videoList = videoPoolService.list(Wrappers.<VideoPool>lambdaQuery()
                .eq(VideoPool::getThemeId, theme.getId())
                .gt(VideoPool::getValidVideo, 1)
                .eq(VideoPool::getStatus, 1));
        calThemeTargetBizService.calStaffTarget(theme, snapshotList, liveList, videoList);
    }

    /**
     * 计算门店目标
     *
     * @param theme
     * @param shopList
     * @param staffList
     */
    private void calShopTarget(ActivityTheme theme, List<ManagerSnapshot> shopList, List<ConsultantSnapshot> staffList) {
        calThemeTargetBizService.calShopTarget(theme, shopList, staffList);
    }

    /**
     * 同步前一天视频、直播数据
     */
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    public void syncPreDayVideo(ActivityTheme activityTheme) {
        //前天、昨天
        LocalDate preYesterday = LocalDate.now().minusDays(2);
        LocalDate yesterday = LocalDate.now().minusDays(1);
        List<VideoPool> preYesterdayList = videoPoolService.list(Wrappers.<VideoPool>lambdaQuery()
                .eq(VideoPool::getThemeId, activityTheme.getId())
                .eq(VideoPool::getCollectDay, preYesterday)
                .eq(VideoPool::getLatestRecord, 1));
        if (PublicUtil.isEmpty(preYesterdayList)) {
            return;
        }
        //今日已上报数据
        List<VideoPool> yesterdayList = videoPoolService.list(Wrappers.<VideoPool>lambdaQuery()
                .eq(VideoPool::getThemeId, activityTheme.getId())
                .eq(VideoPool::getCollectDay, yesterday));
        Set<String> hasUpVideoIdSet = yesterdayList.stream().map(VideoPool::getVideoId).collect(Collectors.toSet());
        //同步处理
        List<VideoPool> videoPools = preYesterdayList.stream()
                .filter(r -> !hasUpVideoIdSet.contains(r.getVideoId())).map(r -> {
                    VideoPool videoPool = PublicUtil.copy(r, VideoPool.class, "creatTime", "updateTime");
                    videoPool.setLatestRecord(1);
                    videoPool.setCollectDay(yesterday);
                    return videoPool;
                }).collect(Collectors.toList());
        videoPoolService.update(Wrappers.<VideoPool>lambdaUpdate()
                .eq(VideoPool::getThemeId, activityTheme.getId())
                .eq(VideoPool::getCollectDay, preYesterday)
                .set(VideoPool::getLatestRecord, 0));
        videoPoolService.saveBatch(videoPools);
    }
}
