package cn.fw.dalaran.service.biz.report.live;

import cn.fw.dalaran.common.domain.enums.RoleMsgEnum;
import cn.fw.dalaran.common.utils.PublicUtil;
import cn.fw.dalaran.domain.db.LivePool;
import cn.fw.dalaran.domain.db.config.ActivityTheme;
import cn.fw.dalaran.domain.dto.report.score.ScoreResultDto;
import cn.fw.dalaran.domain.enums.report.ReportListTypeEnum;
import cn.fw.dalaran.domain.param.report.live.ListQueryParam;
import cn.fw.dalaran.service.biz.report.ReportCommonBizService;
import cn.fw.dalaran.service.biz.report.score.ScoreActualCountBizService;
import cn.fw.dalaran.service.data.ActivityThemeService;
import cn.fw.dalaran.service.data.LivePoolService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 直播列表分析单独实现
 *
 * @author: LYM
 * @date: 2023/12/21 18:08
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class LiveListReportBizService {
    private final LivePoolService livePoolService;
    private final ActivityThemeService activityThemeService;
    private final ReportCommonBizService reportCommonBizService;
    private final ScoreActualCountBizService scoreActualCountBizService;

    /**
     * 列表分析
     */
    public Map<Long, ScoreResultDto> getScoreMap(ListQueryParam param, Long rowId, Integer analysisType) {
        this.getReportRange(param);
        if (PublicUtil.isEmpty(param.getShopIds())) {
            return new HashMap<>();
        }
        List<LivePool> liveList = this.getLiveDataRange(param, analysisType, rowId);
        ReportListTypeEnum typeEnum = ReportListTypeEnum.ofValue(Objects.isNull(analysisType)
                ? param.getRowType() : analysisType);
        if (ReportListTypeEnum.USER.equals(typeEnum)) {
            return this.getScoreByConsultant(param, liveList);
        }
        if (ReportListTypeEnum.SHOP.equals(typeEnum)) {
            return this.getScoreByShop(param, liveList);
        }
        if (ReportListTypeEnum.MANAGER_1.equals(typeEnum)) {
            return this.analysisByManager(param, liveList, RoleMsgEnum.XSZG);
        }
        if (ReportListTypeEnum.MANAGER_2.equals(typeEnum)) {
            liveList.forEach(r -> r.setFirstManagerId(r.getSecondManagerId()));
            return this.analysisByManager(param, liveList, RoleMsgEnum.XSJL);
        }
        if (ReportListTypeEnum.MANAGER_3.equals(typeEnum)) {
            liveList.forEach(r -> r.setFirstManagerId(r.getThirdManagerId()));
            return this.analysisByManager(param, liveList, RoleMsgEnum.XSZJL);
        }
        if (ReportListTypeEnum.MANAGER_4.equals(typeEnum)) {
            liveList.forEach(r -> r.setFirstManagerId(r.getFourthManagerId()));
            return this.analysisByManager(param, liveList, RoleMsgEnum.XSSJGL);
        }
        return new HashMap<>();
    }

    /**
     * 处理短视频报表展示范围
     *
     * @param param 查询参数
     */
    private void getReportRange(ListQueryParam param) {
        if (param.getThemeDimension()) {
            ActivityTheme theme = activityThemeService.getById(param.getThemeId());
            param.setShopIds(theme.getShopIdByReport(param.getShopIds()));
            param.setThemeIds(Collections.singletonList(param.getThemeId()));
        } else {
            Map<ActivityTheme, List<Long>> themeRange = reportCommonBizService.getThemeShopRange(param.getGroupId(), param.getBeginTime(), param.getShopIds());
            Set<Long> shopSet = new HashSet<>();
            if (PublicUtil.isNotEmpty(themeRange)) {
                themeRange.values().forEach(shopSet::addAll);
            }
            param.setShopIds(new ArrayList<>(shopSet));
            param.setThemeIds(themeRange.keySet().stream().map(ActivityTheme::getId).collect(Collectors.toList()));
        }
    }

    /**
     * 按门店分析
     */
    private Map<Long, ScoreResultDto> getScoreByShop(ListQueryParam param, List<LivePool> liveList) {
        List<ScoreResultDto> scoreCalRange = reportCommonBizService.getScoreCalRange(2, param.getGroupId(), param.getThemeIds(),
                param.getShopIds(), null);
        List<ScoreResultDto> shopScoreList = scoreActualCountBizService.calShopMonthScoreBatch(scoreCalRange,
                liveList, Collections.emptyList());
        return shopScoreList.stream().collect(Collectors.toMap(ScoreResultDto::getShopId, r -> r));
    }

    /**
     * 按发布人分析
     */
    private Map<Long, ScoreResultDto> getScoreByConsultant(ListQueryParam param, List<LivePool> liveList) {
        List<ScoreResultDto> scoreCalRange = reportCommonBizService.getScoreCalRange(1, param.getGroupId(), param.getThemeIds(),
                param.getShopIds(), null);
        List<ScoreResultDto> staffScoreList = scoreActualCountBizService.calStaffMonthScoreBatch(scoreCalRange,
                liveList, Collections.emptyList());
        return staffScoreList.stream().collect(Collectors.toMap(ScoreResultDto::getStaffId, r -> r));
    }

    /**
     * 按销售一级管理分析
     */
    private Map<Long, ScoreResultDto> analysisByManager(ListQueryParam param, List<LivePool> liveList, RoleMsgEnum role) {
        List<ScoreResultDto> scoreCalRange = reportCommonBizService.getScoreCalRange(3, param.getGroupId(), param.getThemeIds(),
                param.getShopIds(), role);
        List<ScoreResultDto> managerList = scoreActualCountBizService.calManagerMonthScoreBatch(scoreCalRange,
                liveList, Collections.emptyList());
        return managerList.stream().collect(Collectors.toMap(ScoreResultDto::getManagerId, r -> r));
    }

    /**
     * 查询短视频报表分析短视频数据范围
     */
    private List<LivePool> getLiveDataRange(ListQueryParam param, Integer analysisType, Long rowId) {
        LambdaQueryWrapper<LivePool> baseWrapper = Wrappers.<LivePool>lambdaQuery()
                .in(LivePool::getThemeId, param.getThemeIds())
                .in(LivePool::getBelongShopId, param.getShopIds())
                .ge(LivePool::getLiveStartTime, param.getBeginTime())
                .le(LivePool::getLiveStartTime, param.getEndTime())
                .gt(LivePool::getValidLive, 1)
                .eq(LivePool::getStatus, 1);
        //设置分析类型
        if (PublicUtil.isNotEmpty(analysisType)) {
            Integer rowType = param.getRowType();
            if (ReportListTypeEnum.SHOP.getValue().equals(rowType)) {
                baseWrapper.eq(LivePool::getBelongShopId, rowId);
            } else if (ReportListTypeEnum.MANAGER_1.getValue().equals(rowType)) {
                baseWrapper.eq(LivePool::getFirstManagerId, rowId);
            } else if (ReportListTypeEnum.MANAGER_2.getValue().equals(rowType)) {
                baseWrapper.eq(LivePool::getSecondManagerId, rowId);
            } else if (ReportListTypeEnum.MANAGER_3.getValue().equals(rowType)) {
                baseWrapper.eq(LivePool::getThirdManagerId, rowId);
            } else if (ReportListTypeEnum.MANAGER_4.getValue().equals(rowType)) {
                baseWrapper.eq(LivePool::getFourthManagerId, rowId);
            }
        }
        return livePoolService.list(baseWrapper);
    }
}

