package cn.fw.dalaran.service.biz.report.score;

import cn.fw.dalaran.common.domain.enums.RoleMsgEnum;
import cn.fw.dalaran.common.domain.vo.NormTrendItemVO;
import cn.fw.dalaran.common.domain.vo.NormTrendVO;
import cn.fw.dalaran.common.utils.DateUtil;
import cn.fw.dalaran.common.utils.PublicUtil;
import cn.fw.dalaran.domain.db.LivePool;
import cn.fw.dalaran.domain.db.VideoPool;
import cn.fw.dalaran.domain.db.config.ActivityTheme;
import cn.fw.dalaran.domain.dto.report.score.ScoreResultDto;
import cn.fw.dalaran.domain.enums.report.ReportListTypeEnum;
import cn.fw.dalaran.domain.param.report.score.ScoreBaseQueryParam;
import cn.fw.dalaran.domain.param.report.score.ScoreListParam;
import cn.fw.dalaran.domain.param.report.score.ScoreTrendParam;
import cn.fw.dalaran.domain.vo.report.UserFansCountVo;
import cn.fw.dalaran.domain.vo.report.score.ScoreSummaryVo;
import cn.fw.dalaran.domain.vo.report.score.ThemeScoreListVo;
import cn.fw.dalaran.rpc.oop.OopRpc;
import cn.fw.dalaran.service.biz.report.ReportCommonBizService;
import cn.fw.oop.sdk.result.ShopInfoVo;
import com.alibaba.fastjson.JSON;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 月度得分报表
 *
 * @author: lym
 * @date: 2023/6/5
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class ScoreMonthBizService {
    private final OopRpc oopRpc;
    private final ReportCommonBizService reportCommonBizService;
    private final ScoreActualCountBizService scoreActualCountBizService;
    private final ScoreReportCommonBizService scoreReportCommonBizService;

    /**
     * 获取主题得分统计
     *
     * @param param 查询参数
     * @return 主题得分统计
     */
    public ScoreSummaryVo getScoreSummary(ScoreBaseQueryParam param) {
        scoreReportCommonBizService.calMonthReportRange(param, null, null, null);
        ScoreSummaryVo scoreSummaryVo = new ScoreSummaryVo();
        if (PublicUtil.isEmpty(param.getShopIds())) {
            return new ScoreSummaryVo();
        }
        //月度直播、短视频得分
        List<VideoPool> videoList = scoreReportCommonBizService.getVideoList(param, null, null, null);
        List<LivePool> liveList = scoreReportCommonBizService.getLiveList(param, null, null, null);
        List<ScoreResultDto> scoreCalRange = reportCommonBizService.getScoreCalRange(2,
                param.getGroupId(), param.getThemeIds(), param.getShopIds(), null);
        ScoreResultDto monthScore = scoreActualCountBizService.calMonthScore(scoreCalRange, liveList, videoList);
        scoreSummaryVo.setI2(monthScore.getLiveThemeScore());
        scoreSummaryVo.setI3(monthScore.getVideoThemeScore());
        scoreSummaryVo.setI1(monthScore.getThemeScore());
        return scoreSummaryVo;
    }

    /**
     * 获取主题得分趋势
     *
     * @param param 查询参数
     * @return 主题趋势
     */
    public NormTrendVO<BigDecimal> getTrend(ScoreTrendParam param) {
        NormTrendVO<BigDecimal> trendVO = new NormTrendVO<>();
        scoreReportCommonBizService.calMonthReportRange(param, null, null, null);
        if (PublicUtil.isEmpty(param.getShopIds())) {
            return trendVO;
        }
        List<LocalDate> dateRange = reportCommonBizService.getTrendDate(DateUtil.monthBegin(param.getBeginTime()),
                DateUtil.monthEnd(param.getBeginTime()));
        Map<LocalDate, Set<Long>> dayThemeIdMap = this.getDayTheme(dateRange, param.getThemeList());
        List<ScoreResultDto> scoreCalRange = reportCommonBizService.getScoreCalRange(2,
                param.getGroupId(), param.getThemeIds(), param.getShopIds(), null);

        List<LivePool> liveList = scoreReportCommonBizService.getLiveTrendData(param);
        List<VideoPool> videoList = scoreReportCommonBizService.getVideoTrendData(param);

        List<NormTrendItemVO<BigDecimal>> trendList = new LinkedList<>();
        for (LocalDate date : dateRange) {
            trendList.add(this.calDayTrend(date, scoreCalRange, liveList, videoList, dayThemeIdMap));
        }
        trendVO.setTeam(trendList);
        return trendVO;
    }

    /**
     * 获取每日主题数量（趋势使用）
     */
    private Map<LocalDate, Set<Long>> getDayTheme(List<LocalDate> dateRange, List<ActivityTheme> themeList) {
        Map<LocalDate, Set<Long>> result = new HashMap<>();
        for (LocalDate localDate : dateRange) {
            Set<Long> collect = themeList.stream()
                    .filter(r -> !DateUtil.date2LocalDate(r.getStartTime()).isAfter(localDate))
                    .map(ActivityTheme::getId).collect(Collectors.toSet());
            result.put(localDate, collect);
        }
        return result;
    }

    /**
     * 计算趋势
     */
    private NormTrendItemVO<BigDecimal> calDayTrend(LocalDate date, List<ScoreResultDto> scoreCalRange,
                                                    List<LivePool> liveList, List<VideoPool> videoList,
                                                    Map<LocalDate, Set<Long>> dayThemeIdMap) {
        NormTrendItemVO<BigDecimal> itemVO = new NormTrendItemVO<>();
        itemVO.setDate(date.toString());

        Set<Long> themeIds = dayThemeIdMap.get(date);
        if (PublicUtil.isEmpty(themeIds)) {
            itemVO.setValue(BigDecimal.ZERO);
            return itemVO;
        }

        List<ScoreResultDto> filterShops = scoreCalRange.stream()
                .filter(r -> themeIds.contains(r.getThemeId())).collect(Collectors.toList());
        List<LivePool> filterLives = liveList.stream()
                .filter(r -> themeIds.contains(r.getThemeId()) &&
                        !DateUtil.date2LocalDate(r.getLiveStartTime()).isAfter(date))
                .collect(Collectors.toList());
        Map<String, VideoPool> filterVideos = videoList.stream()
                .filter(r -> themeIds.contains(r.getThemeId()) && !r.getCollectDay().isAfter(date))
                .collect(Collectors.toMap(VideoPool::getVideoId, r -> r, (o, n) -> n));

        ScoreResultDto result = scoreActualCountBizService.calMonthScore(filterShops, filterLives,
                new ArrayList<>(filterVideos.values()));
        itemVO.setValue(result.getThemeScore());
        return itemVO;
    }

    /**
     * 获取主题得分分析列表
     *
     * @param param 查询参数
     * @return 主题得分分析列表
     */
    public List<ThemeScoreListVo> getScoreAnalysis(ScoreListParam param) {
        scoreReportCommonBizService.calMonthReportRange(param, param.getRowType(), param.getRowId(), param.getPointType());
        Integer rowType = PublicUtil.isNotEmpty(param.getPointType()) ? param.getPointType() : param.getRowType();
        ReportListTypeEnum typeEnum = ReportListTypeEnum.ofValue(rowType);
        if (PublicUtil.isEmpty(param.getShopIds()) || PublicUtil.isEmpty(typeEnum)) {
            return new ArrayList<>();
        }
        log.info(JSON.toJSONString(param));
        List<LivePool> liveList = scoreReportCommonBizService.getLiveList(param, param.getRowType(), param.getRowId(), param.getPointType());
        List<VideoPool> videoList = scoreReportCommonBizService.getVideoList(param, param.getRowType(), param.getRowId(), param.getPointType());
        switch (typeEnum) {
            case SHOP:
                return this.analysisByShop(param, liveList, videoList);
            case USER:
                return this.analysisByConsultant(param, liveList, videoList);
            case MANAGER_1:
                return this.analysisByManager(param, liveList, videoList, RoleMsgEnum.XSZG);
            case MANAGER_2:
                liveList.forEach(r -> r.setFirstManagerId(r.getSecondManagerId()));
                videoList.forEach(r -> r.setFirstManagerId(r.getSecondManagerId()));
                return this.analysisByManager(param, liveList, videoList, RoleMsgEnum.XSJL);
            case MANAGER_3:
                liveList.forEach(r -> r.setFirstManagerId(r.getSecondManagerId()));
                videoList.forEach(r -> r.setFirstManagerId(r.getSecondManagerId()));
                return this.analysisByManager(param, liveList, videoList, RoleMsgEnum.XSZJL);
            case MANAGER_4:
                liveList.forEach(r -> r.setFirstManagerId(r.getSecondManagerId()));
                videoList.forEach(r -> r.setFirstManagerId(r.getSecondManagerId()));
                return this.analysisByManager(param, liveList, videoList, RoleMsgEnum.XSSJGL);
            case THEME:
                return this.analysisByTheme(param, liveList, videoList);
        }
        return new ArrayList<>();
    }

    /**
     * 按门店计算月度综合得分
     */
    private List<ThemeScoreListVo> analysisByShop(ScoreListParam param, List<LivePool> liveList, List<VideoPool> videoList) {
        List<ScoreResultDto> scoreCalRange = reportCommonBizService.getScoreCalRange(2, param.getGroupId(),
                param.getThemeIds(), param.getShopIds(), null);
        List<ScoreResultDto> shopScore = scoreActualCountBizService.calShopMonthScoreBatch(scoreCalRange, liveList, videoList);
        List<ShopInfoVo> shops = oopRpc.getShopByIds(param.getShopIds());
        Map<Object, String> nameMap = shops.stream().collect(Collectors.toMap(ShopInfoVo::getId, ShopInfoVo::getShopName));
        Map<Object, ScoreResultDto> dataMap = shopScore.stream().collect(Collectors.toMap(ScoreResultDto::getShopId, r -> r));
        return this.buildAnalysisResult(dataMap, nameMap);
    }

    /**
     * 按销售顾问计算月度综合得分
     */
    private List<ThemeScoreListVo> analysisByConsultant(ScoreListParam param, List<LivePool> liveList, List<VideoPool> videoList) {
        List<ScoreResultDto> scoreCalRange = reportCommonBizService.getScoreCalRange(1, param.getGroupId(),
                param.getThemeIds(), param.getShopIds(), null);
        List<ScoreResultDto> staffScore = scoreActualCountBizService.calStaffMonthScoreBatch(scoreCalRange, liveList, videoList);
        Map<Object, String> nameMap = staffScore.stream().collect(Collectors.toMap(ScoreResultDto::getStaffId, ScoreResultDto::getStaffName));
        Map<Object, ScoreResultDto> dataMap = staffScore.stream().collect(Collectors.toMap(ScoreResultDto::getStaffId, r -> r));
        List<ThemeScoreListVo> themeScoreListVos = this.buildAnalysisResult(dataMap, nameMap);
        Map<Long, UserFansCountVo> userFansCountMap = reportCommonBizService.getUserFansCount(themeScoreListVos
                .stream().map(r -> Long.parseLong(r.getId())).collect(Collectors.toSet()));
        themeScoreListVos.forEach(vo -> vo.setUserFans(userFansCountMap.get(Long.parseLong(vo.getId()))));
        return themeScoreListVos;
    }

    /**
     * 按管理层分析
     */

    private List<ThemeScoreListVo> analysisByManager(ScoreListParam param, List<LivePool> liveList, List<VideoPool> videoList,
                                                     RoleMsgEnum role) {
        List<ScoreResultDto> scoreCalRange = reportCommonBizService.getScoreCalRange(3, param.getGroupId(),
                param.getThemeIds(), param.getShopIds(), role);
        List<ScoreResultDto> managerScoreList = scoreActualCountBizService.calManagerMonthScoreBatch(scoreCalRange, liveList, videoList);
        Map<Object, String> nameMap = managerScoreList.stream().collect(Collectors.toMap(ScoreResultDto::getManagerId, ScoreResultDto::getManagerName));
        Map<Object, ScoreResultDto> dataMap = managerScoreList.stream().collect(Collectors.toMap(ScoreResultDto::getManagerId, r -> r));
        return this.buildAnalysisResult(dataMap, nameMap);
    }

    /**
     * 组装分析结果
     *
     * @param dataMap 数据集合
     * @param nameMap 名称集合
     * @return 结果
     */
    private List<ThemeScoreListVo> buildAnalysisResult(Map<Object, ScoreResultDto> dataMap, Map<Object, String> nameMap) {
        return nameMap.keySet().stream()
                .map(dataId -> {
                    ThemeScoreListVo listVo = new ThemeScoreListVo();
                    listVo.setId(dataId.toString());
                    listVo.setName(nameMap.get(dataId));
                    //设置数据
                    ScoreResultDto result = dataMap.getOrDefault(dataId, new ScoreResultDto());
                    listVo.setI1(result.getThemeScore());
                    listVo.setI2(result.getLiveThemeScore());
                    listVo.setI3(result.getVideoThemeScore());
                    listVo.setI6(result.getLiveScore());
                    listVo.setI7(result.getVideoScore());
                    return listVo;
                })
                .sorted(Comparator.comparing(ThemeScoreListVo::getI1, Comparator.reverseOrder())
                        .thenComparing(ThemeScoreListVo::getI2, Comparator.reverseOrder())
                        .thenComparing(ThemeScoreListVo::getI3, Comparator.reverseOrder())
                )
                .collect(Collectors.toList());
    }

    /**
     * 按主题计算得分
     */
    private List<ThemeScoreListVo> analysisByTheme(ScoreListParam param, List<LivePool> liveList, List<VideoPool> videoList) {
        Map<Long, ActivityTheme> collect = param.getThemeList().stream().collect(Collectors.toMap(ActivityTheme::getId, r -> r));

        List<ScoreResultDto> scoreCalRange = reportCommonBizService.getScoreCalRange(2, param.getGroupId(),
                param.getThemeIds(), param.getShopIds(), null);
        if (Objects.nonNull(param.getRowType()) && ReportListTypeEnum.USER.getValue().equals(param.getRowType())) {
            scoreCalRange.forEach(dto -> {
                ActivityTheme theme = collect.get(dto.getThemeId());
                dto.setNeedVideoNum(Objects.isNull(theme) ? 1 : theme.getVideoValidTimes());
                dto.setNeedLiveNum(Objects.isNull(theme) ? 1 : theme.getLiveValidTimes());
                dto.setConsultantNum(1);
            });
        }
        List<ScoreResultDto> themeScoreList = scoreActualCountBizService.calThemeScoreBatch(scoreCalRange, liveList, videoList);
        Map<Long, ScoreResultDto> themeMap = themeScoreList.stream().collect(Collectors.toMap(ScoreResultDto::getThemeId, r -> r));
        //计算各个主题得分
        List<ThemeScoreListVo> list = new LinkedList<>();
        for (Map.Entry<Long, ScoreResultDto> entry : themeMap.entrySet()) {
            Long themeId = entry.getKey();
            ThemeScoreListVo scoreListVo = new ThemeScoreListVo();
            scoreListVo.setId(themeId.toString());
            ActivityTheme theme = collect.get(entry.getKey());
            if (PublicUtil.isNotEmpty(theme)) {
                scoreListVo.setName(theme.getTheme());
                scoreListVo.setThemeBeginTime(theme.getStartTime());
                scoreListVo.setThemeEndTime(theme.getEndTime());
            }

            ScoreResultDto result = themeMap.getOrDefault(themeId, new ScoreResultDto());
            scoreListVo.setI1(result.getThemeScore());
            scoreListVo.setI2(result.getLiveThemeScore());
            scoreListVo.setI3(result.getVideoThemeScore());
            scoreListVo.setI4(result.getLiveRate());
            scoreListVo.setI5(result.getLiveScore());
            scoreListVo.setI6(result.getVideoRate());
            scoreListVo.setI7(result.getVideoScore());
            list.add(scoreListVo);
        }
        return list;
    }

}
