package cn.fw.dalaran.service.biz.report.video;

import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.dalaran.common.utils.PublicUtil;
import cn.fw.dalaran.domain.db.VideoPool;
import cn.fw.dalaran.domain.db.snapshot.ConsultantSnapshot;
import cn.fw.dalaran.domain.enums.PlatformEnum;
import cn.fw.dalaran.domain.enums.config.WorksTypeEnum;
import cn.fw.dalaran.domain.enums.report.InvalidReasonEnum;
import cn.fw.dalaran.domain.enums.report.ReportListTypeEnum;
import cn.fw.dalaran.domain.enums.report.video.VideoRateIndexEnum;
import cn.fw.dalaran.domain.param.report.video.VideoRateParam;
import cn.fw.dalaran.domain.vo.report.video.VideoRateAnalysisListVo;
import cn.fw.dalaran.rpc.oop.OopRpc;
import cn.fw.oop.sdk.result.ShopInfoVo;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 短视频占比分析实现
 *
 * @author: juzi
 * @date: 2023/9/14 14:18
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class VideoRateAnalysisService {
    private final OopRpc oopRpc;
    private final VideoReportCommonBizService videoReportCommonBizService;

    /**
     * 数据占比分析
     *
     * @param user  查询人
     * @param param 分析维度参数
     * @return 占比分析结果
     */
    public List<VideoRateAnalysisListVo> getRateAnalysis(LoginAuthBean user, VideoRateParam param) {
        param.setGroupId(user.getGroupId());
        param.setPointType(null);
        param.setRowId(null);
        videoReportCommonBizService.calMonthReportRange(param, null, null, null);
        if (PublicUtil.isEmpty(param.getShopIds())) {
            return new ArrayList<>();
        }
        List<VideoPool> videoList = videoReportCommonBizService.getVideoDataRange(param, param.getRowType(),
                param.getPointType(), param.getRowId());
        //过滤指定平台
        if (PublicUtil.isNotEmpty(param.getPlatformType()) && -1 != param.getPlatformType()) {
            videoList = videoList.stream()
                    .filter(r -> param.getPlatformType().equals(r.getPlatform().getValue()))
                    .collect(Collectors.toList());
        }
        //占比分析
        ReportListTypeEnum typeEnum = ReportListTypeEnum.ofValue(param.getRowType());
        if (PublicUtil.isNotEmpty(typeEnum)) {
            switch (typeEnum) {
                case SHOP:
                    return this.rateAnalysisByShop(param, videoList);
                case USER:
                    return this.rateAnalysisByUser(param, videoList);
                case PLATFORM:
                    return this.rateAnalysisByPlatform(param, videoList);
                case INVALID_REASON:
                    return this.rateAnalysisByReason(videoList);
            }
        }
        return new ArrayList<>();
    }

    /**
     * 按门店计算占比
     */
    private List<VideoRateAnalysisListVo> rateAnalysisByShop(VideoRateParam param, List<VideoPool> videoList) {
        List<ShopInfoVo> shopInfoVos = oopRpc.getShopByIds(param.getShopIds());
        Map<Object, String> nameMap = shopInfoVos.stream().collect(Collectors.toMap(ShopInfoVo::getId, ShopInfoVo::getShortName));
        Map<Object, List<VideoPool>> dataMap = videoList.stream().collect(Collectors.groupingBy(VideoPool::getBelongShopId));
        return this.buildRateAnalysisResult(dataMap, nameMap, param.getIndexType());
    }

    /**
     * 无效原因占比分析
     */
    private List<VideoRateAnalysisListVo> rateAnalysisByReason(List<VideoPool> videoList) {
        Map<Object, List<VideoPool>> videoInvalidMap = videoReportCommonBizService.getInvalidDataMap(videoList);
        Map<Object, String> nameMap = InvalidReasonEnum.getCheckIndexByWorkType(WorksTypeEnum.VIDEO).stream()
                .collect(Collectors.toMap(InvalidReasonEnum::getValue, InvalidReasonEnum::getDesc));
        return this.buildRateAnalysisResult(videoInvalidMap, nameMap, VideoRateIndexEnum.RELEASE.getValue());

    }

    /**
     * 平台占比分析
     */
    private List<VideoRateAnalysisListVo> rateAnalysisByPlatform(VideoRateParam param, List<VideoPool> videoList) {
        Map<Object, List<VideoPool>> dataMap = videoList.stream().collect(Collectors.groupingBy(VideoPool::getPlatform));
        Map<Object, String> nameMap = Arrays.stream(PlatformEnum.values()).collect(Collectors.toSet())
                .stream().collect(Collectors.toMap(r -> r, PlatformEnum::getName));
        return this.buildRateAnalysisResult(dataMap, nameMap, param.getIndexType());
    }

    /**
     * 发布人占比分析
     */
    private List<VideoRateAnalysisListVo> rateAnalysisByUser(VideoRateParam param, List<VideoPool> videoList) {
        List<ConsultantSnapshot> publisherRange = videoReportCommonBizService.getThemeOrMonthPublisher(param.getGroupId(),
                param.getThemeIds(), param.getShopIds());
        log.info("销售顾问数目：{}", publisherRange.size());
        //个人账号数据
        Map<Object, String> nameMap = new HashMap<>();
        for (ConsultantSnapshot snapshot : publisherRange) {
            nameMap.put(snapshot.getStaffId(), snapshot.getStaffName());
        }
        Map<Object, List<VideoPool>> dataMap = videoList.stream()
                .filter(r -> nameMap.containsKey(r.getBelongUserId())).collect(Collectors.groupingBy(VideoPool::getBelongUserId));
        return this.buildRateAnalysisResult(dataMap, nameMap, param.getIndexType());
    }

    /**
     * 组装占比分析结果
     */
    public List<VideoRateAnalysisListVo> buildRateAnalysisResult(Map<Object, List<VideoPool>> dataMap, Map<Object, String> rangeMap, Integer indexType) {
        Map<Object, Integer> rateSumMap = this.getRateSum(dataMap, indexType);
        //设置
        List<VideoRateAnalysisListVo> analysisVoList = rateSumMap.keySet().stream().map(dataId -> {
            VideoRateAnalysisListVo analysisVo = new VideoRateAnalysisListVo();
            analysisVo.setId(dataId.toString());
            analysisVo.setName(rangeMap.getOrDefault(dataId, "无信息"));
            analysisVo.setIndexValue(rateSumMap.get(dataId));
            rangeMap.remove(dataId);
            return analysisVo;
        }).collect(Collectors.toList());
        //计算占比
        long total = analysisVoList.stream().mapToLong(VideoRateAnalysisListVo::getIndexValue).sum();
        if (total > 0) {
            analysisVoList.forEach(vo -> vo.setRate(BigDecimal.valueOf(vo.getIndexValue())
                    .divide(BigDecimal.valueOf(total), 4, RoundingMode.HALF_UP)));
        }
        return analysisVoList;
    }

    /**
     * 占比统计
     */
    private Map<Object, Integer> getRateSum(Map<Object, List<VideoPool>> dataMap, Integer indexType) {
        Map<Object, Integer> map = new HashMap<>();
        for (Map.Entry<Object, List<VideoPool>> entry : dataMap.entrySet()) {
            List<VideoPool> videoPoolList = entry.getValue();
            int sum = 0;
            if (VideoRateIndexEnum.RELEASE.getValue().equals(indexType)) {
                sum = videoPoolList.size();
            } else if (VideoRateIndexEnum.UV.getValue().equals(indexType)) {
                sum = videoPoolList.stream().mapToInt(VideoPool::getUv).sum();
            } else if (VideoRateIndexEnum.FULL_UV.getValue().equals(indexType)) {
                sum = videoPoolList.stream().mapToInt(VideoPool::getFullUv).sum();
            } else if (VideoRateIndexEnum.LIKE_NUM.getValue().equals(indexType)) {
                sum = videoPoolList.stream().mapToInt(VideoPool::getLikeNum).sum();
            } else if (VideoRateIndexEnum.SHARE_NUM.getValue().equals(indexType)) {
                sum = videoPoolList.stream().mapToInt(VideoPool::getShareNum).sum();
            } else if (VideoRateIndexEnum.COMMENT_NUM.getValue().equals(indexType)) {
                sum = videoPoolList.stream().mapToInt(VideoPool::getCommentNum).sum();
            } else if (VideoRateIndexEnum.NEW_FANS.getValue().equals(indexType)) {
                sum = videoPoolList.stream().mapToInt(VideoPool::getNewFanNum).sum();
            }
            map.put(entry.getKey(), sum);
        }
        return map;
    }
}

