package cn.fw.dalaran.service.biz.report.video;

import cn.fw.dalaran.common.domain.enums.RoleMsgEnum;
import cn.fw.dalaran.common.domain.vo.NormTrendItemVO;
import cn.fw.dalaran.common.domain.vo.NormTrendVO;
import cn.fw.dalaran.common.utils.PublicUtil;
import cn.fw.dalaran.domain.db.VideoPool;
import cn.fw.dalaran.domain.dto.report.VideoTrendDto;
import cn.fw.dalaran.domain.dto.report.score.ScoreResultDto;
import cn.fw.dalaran.domain.enums.PlatformEnum;
import cn.fw.dalaran.domain.enums.config.WorksTypeEnum;
import cn.fw.dalaran.domain.enums.report.InvalidReasonEnum;
import cn.fw.dalaran.domain.enums.report.ReportListTypeEnum;
import cn.fw.dalaran.domain.param.report.video.VideoBaseQueryParam;
import cn.fw.dalaran.domain.param.report.video.VideoListParam;
import cn.fw.dalaran.domain.param.report.video.VideoTrendParam;
import cn.fw.dalaran.domain.vo.report.UserFansCountVo;
import cn.fw.dalaran.domain.vo.report.score.TargetResultDto;
import cn.fw.dalaran.domain.vo.report.video.VideoAnalysisVo;
import cn.fw.dalaran.domain.vo.report.video.VideoSummaryVo;
import cn.fw.dalaran.rpc.oop.OopRpc;
import cn.fw.dalaran.service.biz.report.ReportCommonBizService;
import cn.fw.dalaran.service.biz.report.score.ScoreActualCountBizService;
import cn.fw.oop.sdk.result.ShopInfoVo;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 短视频报表实现
 *
 * @author: lym
 * @date: 2023/5/6
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class VideoReportBizService {
    private final OopRpc oopRpc;
    private final ReportCommonBizService reportCommonBizService;
    private final ScoreActualCountBizService scoreActualCountBizService;
    private final VideoReportCommonBizService videoReportCommonBizService;

    /**
     * 获取短视频汇总统计
     *
     * @param param 查询参数
     * @return 短视频主题考核汇总统计
     */
    public VideoSummaryVo getVideoSummary(VideoBaseQueryParam param) {
        videoReportCommonBizService.calMonthReportRange(param, null, null, null);
        List<VideoPool> dataList = videoReportCommonBizService.getVideoDataRange(param, null, null, null);
        VideoSummaryVo summary = new VideoSummaryVo();
        summary.setI1(dataList.size());
        List<VideoPool> validList = dataList.stream()
                .filter(item -> item.getValidVideo() > 0)
                .collect(Collectors.toList());
        summary.setI2(validList.size());
        summary.setI3(dataList.size() - validList.size());
        summary.setI4(ReportCommonBizService.sumInt(validList, VideoPool::getUv));
        summary.setI5(ReportCommonBizService.sumInt(validList, VideoPool::getFullUv));
        summary.setI6(ReportCommonBizService.sumInt(validList, VideoPool::getLikeNum));
        summary.setI7(ReportCommonBizService.sumInt(validList, VideoPool::getShareNum));
        summary.setI8(ReportCommonBizService.sumInt(validList, VideoPool::getCommentNum));
        summary.setI9(ReportCommonBizService.sumInt(validList, VideoPool::getNewFanNum));
        summary.setTargetResult(reportCommonBizService.getTargetTotal(param.getGroupId(), param.getThemeIds(), param.getShopIds()));
        return summary;
    }

    /**
     * 获取短视频趋势
     *
     * @param param 查询参数
     * @return 短视频趋势
     */
    public NormTrendVO<BigDecimal> getVideoTrend(VideoTrendParam param) {
        videoReportCommonBizService.calMonthReportRange(param, null, null, null);
        NormTrendVO<BigDecimal> normTrendVO = new NormTrendVO<>();
        if (PublicUtil.isEmpty(param.getThemeIds()) || PublicUtil.isEmpty(param.getShopIds())) {
            return normTrendVO;
        }
        List<LocalDate> trendDate = reportCommonBizService.getTrendDate(param.getBeginTime(), param.getEndTime());
        List<NormTrendItemVO<BigDecimal>> trendList = new LinkedList<>();
        List<VideoPool> videoList = videoReportCommonBizService.getVideoTrendData(param);
        for (LocalDate localDate : trendDate) {
            trendList.add(this.calDayTrend(localDate, param, videoList));
        }
        normTrendVO.setTeam(trendList);
        return normTrendVO;
    }

    /**
     * 计算趋势
     */
    private NormTrendItemVO<BigDecimal> calDayTrend(LocalDate date, VideoTrendParam param, List<VideoPool> videoList) {
        Map<String, VideoPool> videoMap = videoList.stream().filter(r -> !r.getCollectDay().isAfter(date))
                .collect(Collectors.toMap(VideoPool::getVideoId, r -> r, (o, n) -> n));
        VideoTrendDto videoTrend = new VideoTrendDto();
        List<VideoPool> videos = new ArrayList<>(videoMap.values());
        videoTrend.setI1(videos.size());
        List<VideoPool> validVideos = videos.stream().filter(r -> r.getValidVideo() > 0).collect(Collectors.toList());
        videoTrend.setI2(validVideos.size());
        videoTrend.setI4(validVideos.stream().mapToInt(VideoPool::getUv).sum());
        videoTrend.setI5(validVideos.stream().mapToInt(VideoPool::getFullUv).sum());
        NormTrendItemVO<BigDecimal> itemVO = new NormTrendItemVO<>();
        itemVO.setDate(date.toString());
        itemVO.setValue(PublicUtil.isEmpty(videoTrend)
                ? BigDecimal.ZERO : param.getTrendType() == 2
                ? videoTrend.getFullPlayRate() : videoTrend.getValidRate());
        return itemVO;
    }

    /**
     * 获取分析列表
     *
     * @param param 查询参数
     * @return 短视频分析列表
     */
    public List<VideoAnalysisVo> getVideoAnalysis(VideoListParam param) {
        videoReportCommonBizService.calMonthReportRange(param, param.getRowType(), param.getRowId(), param.getPointType());
        if (PublicUtil.isEmpty(param.getShopIds())) {
            return new ArrayList<>();
        }
        List<VideoPool> videoList = videoReportCommonBizService.getVideoDataRange(param, param.getRowType(),
                param.getPointType(), param.getRowId());
        //当前分析类型
        Integer analysisType = PublicUtil.isNotEmpty(param.getPointType()) ? param.getPointType() : param.getRowType();
        ReportListTypeEnum typeEnum = ReportListTypeEnum.ofValue(analysisType);
        //按类型处理
        if (ReportListTypeEnum.SHOP.equals(typeEnum)) {
            return this.analysisByShop(param, videoList);
        }
        if (ReportListTypeEnum.USER.equals(typeEnum)) {
            return this.analysisByConsultant(param, videoList);
        }
        if (ReportListTypeEnum.MANAGER_1.equals(typeEnum)) {
            return this.analysisByManager1(param, videoList);
        }
        if (ReportListTypeEnum.MANAGER_2.equals(typeEnum)) {
            videoList.forEach(r -> r.setFirstManagerId(r.getSecondManagerId()));
            return this.analysisByManager2(param, videoList);
        }
        if (ReportListTypeEnum.MANAGER_3.equals(typeEnum)) {
            videoList.forEach(r -> r.setFirstManagerId(r.getSecondManagerId()));
            return this.analysisByManager3(param, videoList);
        }
        if (ReportListTypeEnum.MANAGER_4.equals(typeEnum)) {
            videoList.forEach(r -> r.setFirstManagerId(r.getSecondManagerId()));
            return this.analysisByManager4(param, videoList);
        }
        if (ReportListTypeEnum.PLATFORM.equals(typeEnum)) {
            return this.analysisByPlatform(param, videoList);
        }
        if (ReportListTypeEnum.INVALID_REASON.equals(typeEnum)) {
            return this.analysisByReason(param, videoList);
        }
        return new ArrayList<>();
    }

    /**
     * 按门店分析
     */
    private List<VideoAnalysisVo> analysisByShop(VideoListParam param, List<VideoPool> videoList) {
        List<ScoreResultDto> scoreCalRange = reportCommonBizService.getScoreCalRange(2, param.getGroupId(), param.getThemeIds(),
                param.getShopIds(), null);
        List<VideoPool> collect = videoList.stream().filter(r -> r.getValidVideo() > 1).collect(Collectors.toList());
        List<ScoreResultDto> shopScoreList = scoreActualCountBizService.calShopMonthScoreBatch(scoreCalRange,
                Collections.emptyList(), collect);
        Map<Long, ScoreResultDto> shopScoreMap = shopScoreList.stream().collect(Collectors.toMap(ScoreResultDto::getShopId, r -> r));
        List<ShopInfoVo> shopInfoVos = oopRpc.getShopByIds(new ArrayList<>(shopScoreMap.keySet()));
        Map<Object, String> mameRange = shopInfoVos.stream().collect(Collectors.toMap(ShopInfoVo::getId, ShopInfoVo::getShopName));
        Map<Object, List<VideoPool>> dataMap = videoList.stream().collect(Collectors.groupingBy(VideoPool::getBelongShopId));
        return this.buildAnalysisResult(dataMap, mameRange, shopScoreMap, param.isCompletion(), true);
    }

    /**
     * 按发布人分析
     */
    private List<VideoAnalysisVo> analysisByConsultant(VideoListParam param, List<VideoPool> videoList) {
        List<VideoPool> collect = videoList.stream().filter(r -> r.getValidVideo() > 1).collect(Collectors.toList());
        List<ScoreResultDto> scoreCalRange = reportCommonBizService.getScoreCalRange(1, param.getGroupId(), param.getThemeIds(),
                param.getShopIds(), null);
        List<ScoreResultDto> staffScoreList = scoreActualCountBizService.calStaffMonthScoreBatch(scoreCalRange,
                Collections.emptyList(), collect);

        Map<Object, String> nameMap = staffScoreList.stream().collect(Collectors.toMap(ScoreResultDto::getStaffId, ScoreResultDto::getStaffName));
        Map<Object, List<VideoPool>> dataMap = videoList.stream()
                .filter(r -> nameMap.containsKey(r.getBelongUserId()))
                .collect(Collectors.groupingBy(VideoPool::getBelongUserId));
        Map<Long, ScoreResultDto> staffScoreMap = staffScoreList.stream().collect(Collectors.toMap(ScoreResultDto::getStaffId, r -> r));
        //处理
        List<VideoAnalysisVo> analysisVos = this.buildAnalysisResult(dataMap, nameMap, staffScoreMap,
                param.isCompletion(), true);
        Set<Long> userIds = analysisVos.stream().map(r -> Long.parseLong(r.getId())).collect(Collectors.toSet());
        Map<Long, UserFansCountVo> userFansCountMap = Objects.isNull(param.getThemeId())
                ? reportCommonBizService.getUserFansCount(userIds) : reportCommonBizService.getThemeFansCount(param.getThemeId());
        analysisVos.forEach(vo -> vo.setUserFans(userFansCountMap.get(Long.parseLong(vo.getId()))));
        return analysisVos;
    }

    /**
     * 按销售一级管理分析
     */
    private List<VideoAnalysisVo> analysisByManager1(VideoListParam param, List<VideoPool> videoList) {
        Map<Object, List<VideoPool>> dataMap = videoList.stream()
                .filter(r -> PublicUtil.isNotEmpty(r.getFirstManagerId()))
                .collect(Collectors.groupingBy(VideoPool::getFirstManagerId));
        List<ScoreResultDto> scoreCalRange = reportCommonBizService.getScoreCalRange(3, param.getGroupId(), param.getThemeIds(),
                param.getShopIds(), RoleMsgEnum.XSZG);
        List<VideoPool> collect = videoList.stream().filter(r -> r.getValidVideo() > 1).collect(Collectors.toList());
        List<ScoreResultDto> managerList = scoreActualCountBizService.calManagerMonthScoreBatch(scoreCalRange,
                Collections.emptyList(), collect);
        Map<Object, String> nameMap = managerList.stream()
                .collect(Collectors.toMap(ScoreResultDto::getManagerId, ScoreResultDto::getManagerName));
        Map<Long, ScoreResultDto> managerScoreMap = managerList.stream().collect(Collectors.toMap(ScoreResultDto::getManagerId, r -> r));
        return this.buildAnalysisResult(dataMap, nameMap, managerScoreMap, param.isCompletion(), true);
    }

    /**
     * 按销售二级管理分析
     */
    private List<VideoAnalysisVo> analysisByManager2(VideoListParam param, List<VideoPool> videoList) {
        Map<Object, List<VideoPool>> dataMap = videoList.stream()
                .filter(r -> PublicUtil.isNotEmpty(r.getSecondManagerId()))
                .collect(Collectors.groupingBy(VideoPool::getSecondManagerId));
        List<ScoreResultDto> scoreCalRange = reportCommonBizService.getScoreCalRange(3, param.getGroupId(), param.getThemeIds(),
                param.getShopIds(), RoleMsgEnum.XSJL);
        List<VideoPool> collect = videoList.stream().filter(r -> r.getValidVideo() > 1).collect(Collectors.toList());
        List<ScoreResultDto> managerList = scoreActualCountBizService.calManagerMonthScoreBatch(scoreCalRange,
                Collections.emptyList(), collect);
        Map<Long, ScoreResultDto> managerScoreMap = managerList.stream().collect(Collectors.toMap(ScoreResultDto::getManagerId, r -> r));
        Map<Object, String> nameMap = managerList.stream()
                .collect(Collectors.toMap(ScoreResultDto::getManagerId, ScoreResultDto::getManagerName));
        return this.buildAnalysisResult(dataMap, nameMap, managerScoreMap, param.isCompletion(), true);
    }

    /**
     * 按销售三级管理分析
     */
    private List<VideoAnalysisVo> analysisByManager3(VideoListParam param, List<VideoPool> videoList) {
        Map<Object, List<VideoPool>> dataMap = videoList.stream()
                .filter(r -> PublicUtil.isNotEmpty(r.getThirdManagerId()))
                .collect(Collectors.groupingBy(VideoPool::getThirdManagerId));

        List<ScoreResultDto> scoreCalRange = reportCommonBizService.getScoreCalRange(3, param.getGroupId(), param.getThemeIds(),
                param.getShopIds(), RoleMsgEnum.XSZJL);
        List<VideoPool> collect = videoList.stream().filter(r -> r.getValidVideo() > 1).collect(Collectors.toList());
        List<ScoreResultDto> managerList = scoreActualCountBizService.calManagerMonthScoreBatch(scoreCalRange,
                Collections.emptyList(), collect);
        Map<Long, ScoreResultDto> managerScoreMap = managerList.stream().collect(Collectors.toMap(ScoreResultDto::getManagerId, r -> r));
        Map<Object, String> nameMap = managerList.stream()
                .collect(Collectors.toMap(ScoreResultDto::getManagerId, ScoreResultDto::getManagerName));
        return this.buildAnalysisResult(dataMap, nameMap, managerScoreMap, param.isCompletion(), true);
    }

    /**
     * 按销售四级管理分析
     */
    private List<VideoAnalysisVo> analysisByManager4(VideoListParam param, List<VideoPool> videoList) {
        Map<Object, List<VideoPool>> dataMap = videoList.stream()
                .filter(r -> PublicUtil.isNotEmpty(r.getFourthManagerId()))
                .collect(Collectors.groupingBy(VideoPool::getFourthManagerId));
        List<ScoreResultDto> scoreCalRange = reportCommonBizService.getScoreCalRange(3, param.getGroupId(), param.getThemeIds(),
                param.getShopIds(), RoleMsgEnum.XSSJGL);
        List<VideoPool> collect = videoList.stream().filter(r -> r.getValidVideo() > 1).collect(Collectors.toList());
        List<ScoreResultDto> managerList = scoreActualCountBizService.calManagerMonthScoreBatch(scoreCalRange,
                Collections.emptyList(), collect);
        Map<Long, ScoreResultDto> managerScoreMap = managerList.stream().collect(Collectors.toMap(ScoreResultDto::getManagerId, r -> r));
        Map<Object, String> nameMap = managerList.stream()
                .collect(Collectors.toMap(ScoreResultDto::getManagerId, ScoreResultDto::getManagerName));
        return this.buildAnalysisResult(dataMap, nameMap, managerScoreMap, param.isCompletion(), true);
    }

    /**
     * 按平台分析
     */
    private List<VideoAnalysisVo> analysisByPlatform(VideoListParam param, List<VideoPool> videoList) {
        Map<Object, List<VideoPool>> dataMap = videoList.stream().collect(Collectors.groupingBy(r -> r.getPlatform().getValue()));
        Map<Object, String> nameRange = Arrays.stream(PlatformEnum.values()).collect(Collectors.toSet())
                .stream().filter(PlatformEnum::getLimitFans)
                .collect(Collectors.toMap(PlatformEnum::getValue, PlatformEnum::getName));
        return this.buildAnalysisResult(dataMap, nameRange, new HashMap<>(), param.isCompletion(), false);
    }

    /**
     * 按无效原因分析
     */
    private List<VideoAnalysisVo> analysisByReason(VideoListParam param, List<VideoPool> videoList) {
        Map<Object, String> nameMap = InvalidReasonEnum.getCheckIndexByWorkType(WorksTypeEnum.VIDEO).stream()
                .collect(Collectors.toMap(InvalidReasonEnum::getValue, InvalidReasonEnum::getDesc));
        Map<Object, List<VideoPool>> dataMap = videoReportCommonBizService.getInvalidDataMap(videoList);
        return this.buildAnalysisResult(dataMap, nameMap, new HashMap<>(), param.isCompletion(), false);
    }

    /**
     * 组装结果
     */
    public List<VideoAnalysisVo> buildAnalysisResult(Map<Object, List<VideoPool>> dataMap, Map<Object, String> nameMap,
                                                     Map<Long, ScoreResultDto> shopScoreMap, Boolean completion,
                                                     Boolean target) {
        List<VideoAnalysisVo> analysisVoList = dataMap.keySet().stream().map(dataId -> {
            VideoAnalysisVo analysisVo = new VideoAnalysisVo();
            analysisVo.setId(dataId.toString());
            analysisVo.setName(nameMap.getOrDefault(dataId, "无信息"));
            List<VideoPool> videoPoolList = dataMap.getOrDefault(dataId, new LinkedList<>());
            this.clcSum(analysisVo, videoPoolList);
            if (target) {
                ScoreResultDto resultDto = shopScoreMap.getOrDefault((Long) dataId, new ScoreResultDto());
                analysisVo.setScore(resultDto.getVideoScore());
                analysisVo.setTargetResult(TargetResultDto.initTargetResult(resultDto));
            }
            nameMap.remove(dataId);
            return analysisVo;
        }).collect(Collectors.toList());
        if (completion) {
            nameMap.forEach((k, v) -> {
                VideoAnalysisVo analysisVo = new VideoAnalysisVo();
                analysisVo.setId(k.toString());
                analysisVo.setName(v);
                if (target) {
                    ScoreResultDto resultDto = shopScoreMap.getOrDefault((Long) k, new ScoreResultDto());
                    analysisVo.setTargetResult(TargetResultDto.initTargetResult(resultDto));
                }
                analysisVoList.add(analysisVo);
            });
        }
        return analysisVoList;
    }

    /**
     * 数据统计
     */
    private void clcSum(VideoAnalysisVo videoAnalysisVo, List<VideoPool> videoPoolList) {
        if (PublicUtil.isEmpty(videoPoolList)) {
            return;
        }
        videoAnalysisVo.setI1(videoPoolList.size());
        for (VideoPool videoPool : videoPoolList) {
            if (videoPool.getValidVideo() == 0) {
                videoAnalysisVo.setI3(videoAnalysisVo.getI3() + 1);
                continue;
            }
            videoAnalysisVo.setI2(videoAnalysisVo.getI2() + 1);
            videoAnalysisVo.setI4(videoAnalysisVo.getI4() + videoPool.getUv());
            videoAnalysisVo.setI5(videoAnalysisVo.getI5() + videoPool.getFullUv());
            videoAnalysisVo.setI6(videoAnalysisVo.getI6() + videoPool.getLikeNum());
            videoAnalysisVo.setI7(videoAnalysisVo.getI7() + videoPool.getShareNum());
            videoAnalysisVo.setI8(videoAnalysisVo.getI8() + videoPool.getCommentNum());
            videoAnalysisVo.setI9(videoAnalysisVo.getI9() + videoPool.getNewFanNum());
        }
    }
}
