package cn.fw.dalaran.service.biz.report.video;

import cn.fw.dalaran.common.page.PageHelper;
import cn.fw.dalaran.common.utils.PageUtil;
import cn.fw.dalaran.common.utils.PublicUtil;
import cn.fw.dalaran.domain.db.Account;
import cn.fw.dalaran.domain.db.VideoPool;
import cn.fw.dalaran.domain.db.config.ActivityTheme;
import cn.fw.dalaran.domain.dto.report.EvlFormulaDto;
import cn.fw.dalaran.domain.enums.report.ReportListTypeEnum;
import cn.fw.dalaran.domain.enums.report.video.VideoPoolSortEnum;
import cn.fw.dalaran.domain.param.report.video.VideoPoolParam;
import cn.fw.dalaran.domain.vo.report.video.VideoReportPoolVo;
import cn.fw.dalaran.rpc.dto.StaffBaseInfoRpc;
import cn.fw.dalaran.rpc.ehr.EhrRpc;
import cn.fw.dalaran.service.biz.report.ReportCommonBizService;
import cn.fw.dalaran.service.data.AccountService;
import cn.fw.dalaran.service.data.ActivityThemeService;
import cn.fw.dalaran.service.data.VideoPoolService;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 短视频报表池
 *
 * @author: lym
 * @date: 2023/5/17
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class VideoReportPoolBizService {
    private final EhrRpc ehrRpc;
    private final AccountService accountService;
    private final VideoPoolService videoPoolService;
    private final ActivityThemeService activityThemeService;
    private final ReportCommonBizService reportCommonBizService;
    private final VideoReportCommonBizService videoReportCommonBizService;

    /**
     * 短视频报表池
     *
     * @param pageHelper 分页参数
     * @param param      查询参数
     * @return 池
     */
    public PageHelper<VideoReportPoolVo> getVideoReportPool(PageHelper<VideoPool> pageHelper, VideoPoolParam param) {
        videoReportCommonBizService.calMonthReportRange(param, null, null, null);
        if (PublicUtil.isEmpty(param.getShopIds())) {
            return new PageHelper<>();
        }
        IPage<VideoPool> page = this.getVideo(pageHelper, param);
        if (PublicUtil.isEmpty(page.getRecords())) {
            return new PageHelper<>();
        }
        List<Long> accountIds = page.getRecords().stream().map(VideoPool::getAccountId).collect(Collectors.toList());
        Map<Long, Account> accountMap = accountService.lambdaQuery().in(Account::getId, accountIds).list()
                .stream().collect(Collectors.toMap(Account::getId, Function.identity()));
        List<Long> userIds = page.getRecords().stream().map(VideoPool::getBelongUserId).collect(Collectors.toList());
        List<StaffBaseInfoRpc> staffInfoBatch = ehrRpc.getStaffInfoBatch(userIds);
        Map<Long, String> userMap = staffInfoBatch.stream()
                .collect(Collectors.toMap(StaffBaseInfoRpc::getId, StaffBaseInfoRpc::getName));
        List<Long> themeIds = page.getRecords().stream().map(VideoPool::getThemeId).collect(Collectors.toList());
        Map<Long, ActivityTheme> activityThemeMap = activityThemeService.lambdaQuery()
                .in(ActivityTheme::getId, themeIds).list()
                .stream().collect(Collectors.toMap(ActivityTheme::getId, Function.identity()));
        return PageUtil.toPage(page, video -> getVideoReportPoolVo(video, accountMap, userMap, activityThemeMap));
    }

    /**
     * 查询报表池数据
     */
    private IPage<VideoPool> getVideo(PageHelper<VideoPool> pageHelper, VideoPoolParam param) {
        LambdaQueryWrapper<VideoPool> wrapper = this.getBaseWrapper(param);
        //分析列表类型限制处理
        this.firstListQueryHandle(param, wrapper);
        this.secondListQueryHandle(param, wrapper);
        //排序处理
        if (PublicUtil.isNotEmpty(param.getOrderAtt())) {
            String orderSql = VideoPoolSortEnum.getOrderSql(param.getOrderAtt(), param.getOrder());
            wrapper.last(orderSql);
        }
        return videoPoolService.page(PageUtil.helper2Page(pageHelper), wrapper);
    }

    /**
     * 池查询基础sql
     */
    private LambdaQueryWrapper<VideoPool> getBaseWrapper(VideoPoolParam param) {
        Boolean valid = param.getValid();
        List<Long> shopIds = param.getShopIds();
        //标题前置处理
        if (PublicUtil.isNotEmpty(param.getTitle()) && param.getTitle().contains("-")) {
            param.setTitle(null);
        }
        //基础sql
        return Wrappers.<VideoPool>lambdaQuery()
                .in(VideoPool::getBelongShopId, shopIds)
                .in(VideoPool::getThemeId, param.getThemeIds())
                .like(PublicUtil.isNotEmpty(param.getTitle()), VideoPool::getTitle, param.getTitle())
                .like(PublicUtil.isNotEmpty(param.getUserName()), VideoPool::getBelongUserName, param.getUserName())
                .ge(PublicUtil.isNotEmpty(param.getBeginTime()), VideoPool::getPublishTime, param.getBeginTime())
                .le(PublicUtil.isNotEmpty(param.getEndTime()), VideoPool::getPublishTime, param.getEndTime())
                .eq(PublicUtil.isNotEmpty(param.getPlatform()), VideoPool::getPlatform, param.getPlatform())
                .eq(PublicUtil.isNotEmpty(param.getInvalidReason()), VideoPool::getInvalidReason, param.getInvalidReason())
                .ge((PublicUtil.isNotEmpty(valid) && valid), VideoPool::getValidVideo, 1)
                .eq((PublicUtil.isNotEmpty(valid) && !valid), VideoPool::getValidVideo, 0)
                .eq(VideoPool::getLatestRecord, 1)
                .and(wrapper1 -> wrapper1.isNull(VideoPool::getStatus).or().eq(VideoPool::getStatus, 1));
    }

    /**
     * 一级列表范围限定
     */
    private void firstListQueryHandle(VideoPoolParam param, LambdaQueryWrapper<VideoPool> wrapper) {
        ReportListTypeEnum typeEnum = ReportListTypeEnum.ofValue(param.getRowType());
        if (PublicUtil.isNotEmpty(typeEnum)) {
            switch (typeEnum) {
                case SHOP:
                    wrapper.eq(VideoPool::getBelongShopId, Long.valueOf(param.getRowId()));
                    break;
                case USER:
                    wrapper.eq(VideoPool::getBelongUserId, Long.valueOf(param.getRowId()));
                    break;
                case PLATFORM:
                    wrapper.eq(VideoPool::getPlatform, Long.valueOf(param.getRowId()));
                    break;
                case INVALID_REASON:
                    wrapper.apply("FIND_IN_SET({0}, invalid_reason)", String.valueOf(param.getRowId()));
                    break;
                case MANAGER_1:
                    wrapper.eq(VideoPool::getFirstManagerId, Long.valueOf(param.getRowId()));
                    break;
                case MANAGER_2:
                    wrapper.eq(VideoPool::getSecondManagerId, Long.valueOf(param.getRowId()));
                    break;
                case MANAGER_3:
                    wrapper.eq(VideoPool::getThirdManagerId, Long.valueOf(param.getRowId()));
                    break;
                case MANAGER_4:
                    wrapper.eq(VideoPool::getFourthManagerId, Long.valueOf(param.getRowId()));

            }
        }
    }

    /**
     * 二级列表分析范围限定
     */
    private void secondListQueryHandle(VideoPoolParam param, LambdaQueryWrapper<VideoPool> wrapper) {
        ReportListTypeEnum pointEnum = ReportListTypeEnum.ofValue(param.getPointType());
        if (PublicUtil.isEmpty(pointEnum)) {
            return;
        }
        switch (pointEnum) {
            case SHOP:
                wrapper.eq(VideoPool::getBelongShopId, Long.valueOf(param.getPointId()));
                break;
            case USER:
                wrapper.eq(VideoPool::getBelongUserId, Long.valueOf(param.getPointId()));
                break;
            case PLATFORM:
                wrapper.eq(VideoPool::getPlatform, Long.valueOf(param.getPointId()));
                break;
            case INVALID_REASON:
                wrapper.apply("FIND_IN_SET({0}, invalid_reason)", String.valueOf(param.getPointId()));
                break;
            case MANAGER_1:
                wrapper.eq(VideoPool::getFirstManagerId, Long.valueOf(param.getPointId()));
                break;
            case MANAGER_2:
                wrapper.eq(VideoPool::getSecondManagerId, Long.valueOf(param.getPointId()));
                break;
            case MANAGER_3:
                wrapper.eq(VideoPool::getThirdManagerId, Long.valueOf(param.getPointId()));
                break;
            case MANAGER_4:
                wrapper.eq(VideoPool::getFourthManagerId, Long.valueOf(param.getRowId()));
                break;
        }
    }

    /**
     * 组装短视频池vo
     */
    private VideoReportPoolVo getVideoReportPoolVo(VideoPool video, Map<Long, Account> accountMap, Map<Long, String> userMap,
                                                   Map<Long, ActivityTheme> activityThemeMap) {
        VideoReportPoolVo vo = PublicUtil.copy(video, VideoReportPoolVo.class);
        vo.setId(video.getId());
        vo.setValid(video.getValidVideo());
        vo.setPlatform(video.getPlatform().getValue());
        Account account = accountMap.get(video.getAccountId());
        if (Objects.nonNull(account)) {
            vo.setAccount(account.getAccount());
            vo.setShopName(account.getShopName());
            vo.setUserNick(account.getAccountName());
        }
        vo.setUserName(userMap.get(video.getBelongUserId()));
        ActivityTheme theme = activityThemeMap.get(video.getThemeId());
        vo.setTheme(theme.getTheme());
        vo.setTags(reportCommonBizService.handleWorksTopic(video.getTags()));
        vo.setSetTags(reportCommonBizService.handleSetTopic(theme));
        try {
            vo.setScoreDetails(PublicUtil.isNotEmpty(video.getScoreCalJson())
                    ? JSON.parseObject(video.getScoreCalJson(), EvlFormulaDto.class) : null);
        } catch (Exception e) {
            log.error("反序列化得分详情出错!");
        }
        return vo;
    }

}
