package cn.fw.dalaran.service.data.impl;

import cn.fw.dalaran.common.utils.DateUtil;
import cn.fw.dalaran.common.utils.PublicUtil;
import cn.fw.dalaran.dao.LivePoolDao;
import cn.fw.dalaran.domain.db.Account;
import cn.fw.dalaran.domain.db.LivePool;
import cn.fw.dalaran.domain.db.config.ActivityTheme;
import cn.fw.dalaran.domain.dto.report.score.StaffScoreSumDto;
import cn.fw.dalaran.service.data.AccountService;
import cn.fw.dalaran.service.data.ActivityThemeService;
import cn.fw.dalaran.service.data.LivePoolService;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.HashBasedTable;
import com.google.common.collect.Table;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDate;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author kurisu
 * @date 2021-11-17 14:54
 * @description LivePoolService
 */
@Service
@RequiredArgsConstructor
@SuppressWarnings("Duplicates")
public class LivePoolServiceImpl extends ServiceImpl<LivePoolDao, LivePool> implements LivePoolService {

    private final ActivityThemeService activityThemeService;
    private final AccountService accountService;

    @Override
    public void removeAccountDataByDate(Long accountId, LocalDate date) {
        this.getBaseMapper().deleteByAccountDate(accountId, date);
    }

    /**
     * 获取主题活动员工有效直播数
     *
     * @param userId  用户id
     * @param themeId 活动主题id
     * @return 有效直播数
     */
    @Override
    public Integer getValidLiveCnt(Long userId, Long themeId) {
        final ActivityTheme activityTheme = activityThemeService.getById(themeId);
        if (Objects.isNull(activityTheme))
            return 0;
        final Set<Long> accountIds = accountService.lambdaQuery()
                .eq(Account::getUserId, userId)
                .list()
                .stream()
                .map(Account::getId)
                .collect(Collectors.toSet());
        if (Objects.equals(accountIds.size(), 0))
            return 0;
        final List<LivePool> list = this.lambdaQuery()
                .in(LivePool::getAccountId, accountIds)
                .eq(LivePool::getValidLive, 1)
                .ge(LivePool::getLiveStartTime, activityTheme.getStartTime())
                .le(LivePool::getLiveStartTime, activityTheme.getEndTime())
                .list();
        if (Objects.equals(list.size(), 0))
            return 0;
        return list.stream().collect(Collectors.groupingBy(LivePool::getRoomNo)).size();
    }

    /**
     * 设置最佳直播
     *
     * @param liveId 直播数据id
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean markBestLive(Long liveId) {
        final LivePool live = this.getById(liveId);
        final Long themeId = live.getThemeId();// 获取主题id
        final Account currentAccount = accountService.getById(live.getAccountId());
        final Long userId = currentAccount.getUserId();// 获取用户id
        final List<Account> accounts = accountService.getByUserId(userId);// 根据用户id获取拥有的所有账户
        this.lambdaUpdate()
                .set(LivePool::getValidLive, 1)
                .eq(LivePool::getValidLive, 11)
                .eq(LivePool::getThemeId, themeId)
                .in(LivePool::getAccountId, accounts
                        .stream()
                        .map(Account::getId)
                        .collect(Collectors.toList())
                )
                .update();// 还原以前标记过的最佳直播
        this.lambdaUpdate()
                .set(LivePool::getValidLive, 11)
                .eq(LivePool::getThemeId, themeId)
                .eq(LivePool::getId, liveId)
                .update();// 设置最新的最佳直播
        return true;
    }

    @Override
    public List<StaffScoreSumDto> getScoreSummary(List<Long> themeIds, List<Long> shopIds) {
        return this.baseMapper.getScoreSummary(themeIds, shopIds);
    }

    @Override
    public Map<LocalDate, List<StaffScoreSumDto>> getScoreSummaryTrend(List<Long> themeIds, List<Long> shopIds, List<LocalDate> dateRange) {
        List<LivePool> livePools = this.list(Wrappers.<LivePool>lambdaQuery()
                .eq(LivePool::getThemeId, themeIds)
                .eq(LivePool::getBelongShopId, shopIds)
                .eq(LivePool::getValidLive, 11)
                .eq(LivePool::getStatus, 1));
        //处理
        Map<LocalDate, List<StaffScoreSumDto>> result = new LinkedHashMap<>();
        for (LocalDate currentDay : dateRange) {
            //当前时间段数据
            List<LivePool> currentDayLive = livePools.stream()
                    .filter(r -> !DateUtil.date2LocalDate(r.getLiveStartTime()).isAfter(currentDay))
                    .collect(Collectors.toList());
            //按主题汇总
            Map<Long, List<LivePool>> thememMap = currentDayLive.stream().collect(Collectors.groupingBy(LivePool::getThemeId));
            for (Map.Entry<Long, List<LivePool>> entry : thememMap.entrySet()) {
                Table<Long, Long, StaffScoreSumDto> table = HashBasedTable.create();
                for (LivePool livePool : entry.getValue()) {
                    StaffScoreSumDto staffScoreSumDto = table.get(livePool.getBelongShopId(), livePool.getBelongUserId());
                    if (PublicUtil.isEmpty(staffScoreSumDto)) {
                        staffScoreSumDto = new StaffScoreSumDto();
                        staffScoreSumDto.setThemeId(livePool.getThemeId());
                        staffScoreSumDto.setBelongShopId(livePool.getBelongShopId());
                        staffScoreSumDto.setBelongUserId(livePool.getBelongUserId());
                        table.put(livePool.getBelongShopId(), livePool.getBelongUserId(), staffScoreSumDto);
                    }
                    staffScoreSumDto.setScore(staffScoreSumDto.getScore().add(livePool.getScore()));
                }
                result.put(currentDay, new ArrayList<>(table.values()));
            }
        }
        return result;
    }
}
