package cn.fw.dalaran.service.data.impl;

import cn.fw.dalaran.common.utils.DateUtil;
import cn.fw.dalaran.dao.VideoPoolDao;
import cn.fw.dalaran.domain.db.Account;
import cn.fw.dalaran.domain.db.config.ActivityTheme;
import cn.fw.dalaran.domain.db.VideoPool;
import cn.fw.dalaran.domain.dto.report.score.StaffScoreSumDto;
import cn.fw.dalaran.domain.param.report.video.VideoBaseQueryParam;
import cn.fw.dalaran.domain.param.report.video.VideoTrendParam;
import cn.fw.dalaran.domain.vo.report.video.VideoSummaryVo;
import cn.fw.dalaran.domain.dto.report.VideoTrendDto;
import cn.fw.dalaran.service.data.AccountService;
import cn.fw.dalaran.service.data.ActivityThemeService;
import cn.fw.dalaran.service.data.VideoPoolService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author kurisu
 * @date 2021-11-17 14:54
 * @description VideoPoolService
 */
@Service
@RequiredArgsConstructor
@SuppressWarnings("Duplicates")
public class VideoPoolServiceImpl extends ServiceImpl<VideoPoolDao, VideoPool> implements VideoPoolService {

    private final ActivityThemeService activityThemeService;
    private final AccountService accountService;

    @Override
    public void removeAccountDataByDate(Long accountId, LocalDate date) {
        this.getBaseMapper().deleteByAccountDate(accountId, date);
    }

    /**
     * 获取主题活动员工有效视频数
     *
     * @param userId  用户id
     * @param themeId 活动主题id
     * @return 有效视频数
     */
    @Override
    public Integer getValidVideoCnt(Long userId, Long themeId) {
        final ActivityTheme activityTheme = activityThemeService.getById(themeId);
        if (Objects.isNull(activityTheme))
            return 0;
        final Set<Long> accountIds = accountService.lambdaQuery()
                .eq(Account::getUserId, userId)
                .list()
                .stream()
                .map(Account::getId)
                .collect(Collectors.toSet());
        if (Objects.equals(accountIds.size(), 0))
            return 0;
        final List<VideoPool> list = this.lambdaQuery()
                .in(VideoPool::getAccountId, accountIds)
                .eq(VideoPool::getValidVideo, 1)
                .ge(VideoPool::getPublishTime, activityTheme.getStartTime())
                .le(VideoPool::getPublishTime, activityTheme.getEndTime())
                .list();
        if (Objects.equals(list.size(), 0))
            return 0;
        return list.stream().collect(Collectors.groupingBy(VideoPool::getVideoId)).size();
    }

    /**
     * 找到最新的video信息
     *
     * @param videoId 视频id
     * @return 该视频id对应的最新的视频
     */
    @Override
    public VideoPool getLatestVideo(String videoId) {
        final List<VideoPool> videoList = this.lambdaQuery()
                .eq(VideoPool::getVideoId, videoId)
                .orderByDesc(VideoPool::getId)
                .list();
        if (!videoList.isEmpty())
            return videoList.get(0);
        return null;
    }

    /**
     * 获取最近2个月视频归属情况
     *
     * @return 最近2个月视频归属情况   K: 视频id, V: 视频详情
     */
    @Override
    public Map<String, VideoPool> getRecent2MonthVideo() {
        Date lastMonthStart = DateUtil.monthBegin(DateUtil.minusMonths(new Date(), 1));
        QueryWrapper<VideoPool> wrapper = new QueryWrapper<>();
        wrapper.select("video_id", "min(create_time) create_time", "belong_user_id", "belong_shop_id",
                "first_manager_id", "second_manager_id", "third_manager_id", "fourth_manager_id", "score",
                "score_cal_json", "theme_score", "valid_video", "belong_user_name")
                .lambda()
                .between(VideoPool::getPublishTime, lastMonthStart, new Date())
                .groupBy(VideoPool::getVideoId)
        ;
        return this.list(wrapper).stream()
                .collect(Collectors.toMap(VideoPool::getVideoId, Function.identity()));
    }

    /**
     * 获取最近2个月被标记无效视频id列表
     *
     * @return 最近2个月被标记无效视频id列表
     */
    @Override
    public List<String> getRecent2MonthInvalidStatusVideoIds() {
        QueryWrapper<VideoPool> wrapper = new QueryWrapper<>();
        wrapper.select("DISTINCT(video_id)")
                .lambda()
                .eq(VideoPool::getStatus, 0);
        return this.list(wrapper).stream()
                .map(VideoPool::getVideoId)
                .collect(Collectors.toList());
    }

    /**
     * 根据视频id标记该视频为老数据
     *
     * @param videoId 视频id
     */
    @Override
    public void markVideoIsOld(String videoId) {
        Date now = new Date();
        Date lastMonthStart = DateUtil.monthBegin(DateUtil.minusMonths(now, 1));
        this.lambdaUpdate()
                .eq(VideoPool::getVideoId, videoId)
                .eq(VideoPool::getLatestRecord, 1)
                .between(VideoPool::getPublishTime, lastMonthStart, now)
                .set(VideoPool::getLatestRecord, 0)
                .set(VideoPool::getUpdateTime, now)
                .update()
        ;
    }

    @Override
    public List<StaffScoreSumDto> getScoreSummary(List<Long> themeIds, List<Long> shopIds) {
        return this.getBaseMapper().getScoreSummary(themeIds, shopIds);
    }

    @Override
    public Map<LocalDate, List<StaffScoreSumDto>> getScoreSummaryTrend(List<Long> themeIds, List<Long> shopIds) {
        List<StaffScoreSumDto> scoreSummaryList = this.getBaseMapper().getScoreSummaryTrend(themeIds, shopIds);
        return scoreSummaryList.stream().collect(Collectors.groupingBy(StaffScoreSumDto::getCollectDay));
    }
}
